#include <stdio.h>
#include <stdarg.h>
#include <tiffio.h>


static void img_tiff_warning_cb(
	const char *module,
	const char *fmt,
	va_list ap
)
{
	/* Do nothing */
}

static void img_tiff_error_cb(
	const char *module,
	const char *fmt,
	va_list ap
)
{
	/* Do nothing */
}


static int img_tiff_directory_iterate(
	const char *path,
	TIFF *tp,
	int *frame_num
)
{
	int		x, y,
			pixels_per_line;
	unsigned long npixels;
	char emsg[1024];
	uint8 *pixel_ptr;
	uint32		*traster,
			*traster_line_ptr;
	TIFFRGBAImage frame;

	if(!TIFFRGBAImageOK(tp, emsg))
		return(-1);

	if(!TIFFRGBAImageBegin(
		&frame,
		tp,
		0,				/* Do not stop on errors */
		emsg
	))
		return(-1);

	npixels = (unsigned long)(frame.width * frame.height);
printf("Frame #%i %ldx%ld\n", *frame_num + 1, frame.width, frame.height);
	traster = (uint32 *)_TIFFmalloc(
		npixels * sizeof(uint32)
	);
	if(traster == NULL)
	{
		TIFFRGBAImageEnd(&frame);
		return(-3);
	}

	if(!TIFFRGBAImageGet(
		&frame,
		traster,
		frame.width, frame.height
	))
	{
		_TIFFfree(traster);
		TIFFRGBAImageEnd(&frame);
		return(-1);
	}

	pixels_per_line = (int)frame.width;
	for(y = 0; y < frame.height; y++)
	{
		traster_line_ptr = traster + (y * pixels_per_line);
pixel_ptr = (uint8 *)(traster_line_ptr + (frame.width / 2));
printf("line %i RGBA = 0x%.2x 0x%.2x 0x%.2x 0x%.2x\n",
 y,
 pixel_ptr[0],
 pixel_ptr[1],
 pixel_ptr[2],
 pixel_ptr[3]
);

		for(x = 0; x < frame.width; x++)
		{


		}
	}


	_TIFFfree(traster);
	(void)TIFFRGBAImageEnd(&frame);

	return(0);
}

static void img_tiff_main(const char *path)
{
	int frame_num;
	uint32 twidth, theight;
	TIFF *tp = TIFFOpen(path, "r");
	if(tp == NULL)
		return;

printf("Opened \"%s\"\n", path);

	TIFFGetField(tp, TIFFTAG_IMAGEWIDTH, &twidth);
	TIFFGetField(tp, TIFFTAG_IMAGELENGTH, &theight);

printf("Size: %ldx%ld\n", twidth, theight);
printf("Frames: %i\n", TIFFNumberOfDirectories(tp));

	frame_num = 0;

	do {

		img_tiff_directory_iterate(
			path,
			tp,
			&frame_num
		);

	} while(TIFFReadDirectory(tp));

	TIFFClose(tp);

printf("Closed.\n");
}

int main(int argc, char* argv[])
{
	const char *s;

	if(argc <= 1)
		return(2);

	s = TIFFGetVersion();
if(s != NULL)
 printf("TIFF Version %s\n", s);

	(void)TIFFSetWarningHandler(img_tiff_warning_cb);
	(void)TIFFSetErrorHandler(img_tiff_error_cb);

	img_tiff_main(argv[1]);

	return(0);
}
