#include <string.h>
#if defined(HAVE_IMLIB)
# include <Imlib.h>
#elif defined(HAVE_IMLIB2)
# include <X11/Xlib.h>
# include <Imlib2.h>
#endif
#include <gtk/gtk.h>
#include <gdk/gdkprivate.h>

#include "imgio.h"
#include "guiutils.h"
#include "guirgbimg.h"
#include "imgview.h"
#include "iv_title.h"
#include "config.h"

/* #undef INCLUDE_IV_TITLE_IMAGE */

#ifdef INCLUDE_IV_TITLE_IMAGE
/* Uncomment only one, each xpm image defines title_320x240_xpm */
#include "titles/blue_320x240.xpm"
/* #include "titles/rainbow_320x240.xpm" */
/* #include "titles/classic_320x240.xpm" */
#endif


#ifdef INCLUDE_IV_TITLE_IMAGE
static imgview_image_struct *iv_title_create_title(
#if defined(HAVE_IMLIB)
	gpointer imlib_handle
#elif defined(HAVE_IMLIB2)
	void
#endif
);
#endif
static GdkPixmap *iv_title_create_simple_title(
	GdkWindow *window,
	const gint width, const gint height
);

void iv_title_load_to_image_viewer(
	imgview_struct *iv,
#if defined(HAVE_IMLIB)
	gpointer imlib_handle,
#elif defined(HAVE_IMLIB2)
#endif
	const gboolean auto_resize,		/* Automatically resize the
						 * ImgView to the size of the
						 * title image */
	const gboolean auto_zoom_to_fit,	/* Automatically adjust the
						 * ImgView's zoom to fit the
						 * title image */
	const gboolean simple_title		/* Display a "less graphical"
						 * title image */
);


#define ATOI(s)		(((s) != NULL) ? atoi(s) : 0)
#define ATOL(s)		(((s) != NULL) ? atol(s) : 0)
#define ATOF(s)		(((s) != NULL) ? atof(s) : 0.0f)
#define STRDUP(s)	(((s) != NULL) ? g_strdup(s) : NULL)

#define MAX(a,b)	(((a) > (b)) ? (a) : (b))
#define MIN(a,b)	(((a) < (b)) ? (a) : (b))
#define CLIP(a,l,h)	(MIN(MAX((a),(l)),(h)))
#define STRLEN(s)	(((s) != NULL) ? strlen(s) : 0)
#define STRISEMPTY(s)	(((s) != NULL) ? (*(s) == '\0') : TRUE)


#ifdef INCLUDE_IV_TITLE_IMAGE
/*
 *	Creates the standard title image.
 */
static imgview_image_struct *iv_title_create_title(
#if defined(HAVE_IMLIB)
	gpointer imlib_handle
#elif defined(HAVE_IMLIB2)
	void
#else
	void
#endif
)
{
#if defined(HAVE_IMLIB)
	ImlibImage *imlib_image;
	const gint bpp = 4;			/* RGBA */
	gint		width, height,
			bpl,
			user_aborted = 0;
	guint8 *rgba;
	imgview_image_struct *img;

	if(imlib_handle == NULL)
	    return(NULL);

	/* Load the standard title image using Imlib */
	imlib_image = Imlib_create_image_from_xpm_data(
	    imlib_handle,
	    (char **)title_320x240_xpm
	);
	if(imlib_image == NULL)
	    return(NULL);

	/* Need to realize the changes on the Imlib image */
	Imlib_changed_image(imlib_handle, imlib_image);
	width = (gint)imlib_image->rgb_width;
	height = (gint)imlib_image->rgb_height;
	bpl = width * bpp;

	/* Copy/convert the Imlib image data to RGBA image data */
	rgba = ImgImlibConvertImlibToRGBA(
	    imlib_image->rgb_data, imlib_image->alpha_data,
	    width, height,
	    -1, -1,
	    NULL, NULL,
	    &user_aborted
	);
	if(rgba == NULL)
	{
	    Imlib_destroy_image(imlib_handle, imlib_image);
	    return(NULL);
	}

	/* Create an ImgView image */
	img = ImgViewImageNew(
	    width, height,
	    bpp, bpl
	);
	if(img == NULL)
	{
	    Imlib_destroy_image(imlib_handle, imlib_image);
	    g_free(rgba);
	    return(NULL);
	}

	/* Set the RGBA image data to the ImgView image */
	(void)ImgViewImageAppendFrame(
	    img,
	    rgba,				/* Transfered */
	    0l
	);
/*	rgba = NULL; */

	/* Unref the Imlib image */
	Imlib_destroy_image(imlib_handle, imlib_image);

	return(img);
#elif defined(HAVE_IMLIB2)
/* TODO */
	Imlib_Image imlib_image;
	return(NULL);
#else
	return(NULL);
#endif
}
#endif	/* INCLUDE_IV_TITLE_IMAGE */

/*
 *	Creates a simple title on a GdkPixmap.
 */
static GdkPixmap *iv_title_create_simple_title(
	GdkWindow *window,
	const gint width, const gint height
)
{
	const gint border_major = 5;
	gint		x = 0, y = 0;
	GdkFont *font;
	GdkColor	fg,
			bg;
	GdkColormap *colormap = GDK_COLORMAP_REF(gdk_window_get_colormap(window));
	GdkPixmap *pixmap = gdk_pixmap_new(
	    window,
	    width, height,
	    -1
	);
	GdkDrawable *drawable = pixmap;
	GdkGC *gc = gdk_gc_new(window);

	if((gc == NULL) || (pixmap == NULL) || (colormap == NULL))
	{
	    (void)GDK_GC_UNREF(gc);
	    (void)GDK_PIXMAP_UNREF(pixmap);
	    (void)GDK_COLORMAP_UNREF(colormap);
	    return(NULL);
	}

	/* Set the background color and draw the background */
	GDK_COLOR_SET_COEFF(&bg, 1.0f, 1.0f, 1.0f);
	GDK_COLORMAP_ALLOC_COLOR(colormap, &bg);
	gdk_gc_set_foreground(gc, &bg);
	gdk_draw_rectangle(   
	    drawable,
	    gc,
	    TRUE,				/* Fill */
	    0, 0,
	    width, height
	);
	GDK_COLORMAP_FREE_COLOR(colormap, &bg);

	/* Draw the title banner & text */
	font = gdk_font_load(
"-adobe-helvetica-bold-r-normal-*-34-*-*-*-p-*-iso8859-1"
	);
	if(font == NULL)
	    font = gdk_font_load(
"-adobe-helvetica-bold-r-normal-*-20-*-*-*-p-*-iso8859-1"
	    );
	if(font == NULL)
	    font = gdk_font_load("8x16");
	if(font != NULL)
	{
#ifdef PROG_NAME_FULL
	    const gfloat y_coeff = 0.30f;
	    gint	tar_width, tar_height;
	    GdkTextBounds b;
	    const gchar *s = PROG_NAME_FULL;

	    GDK_COLOR_SET_COEFF(&fg, 1.0f, 1.0f, 1.0f);
	    GDK_COLORMAP_ALLOC_COLOR(colormap, &fg);
	    GDK_COLOR_SET_COEFF(&bg, 0.25f, 0.25f, 1.0f);
	    GDK_COLORMAP_ALLOC_COLOR(colormap, &bg);

	    gdk_string_bounds(font, s, &b);

	    /* Label banner background */
	    tar_width = b.width + (4 * border_major);
	    tar_height = font->ascent + font->descent + (4 * border_major);
	    x = (width / 2) - (tar_width / 2);
	    y = (gint)(height * y_coeff) - (tar_height / 2);
	    gdk_gc_set_foreground(gc, &bg);
	    gdk_draw_rectangle(
		drawable,
		gc,
		TRUE,				/* Fill */
		x, y,
		tar_width, tar_height
	    );

	    /* Label banner outline */
	    tar_width = b.width + (2 * border_major);
	    tar_height = font->ascent + font->descent + (2 * border_major);
	    x = (width / 2) - (tar_width / 2);
	    y = (gint)(height * y_coeff) - (tar_height / 2);
	    gdk_gc_set_foreground(gc, &fg);
	    gdk_draw_rectangle(
		drawable,
		gc,
		FALSE,				/* Outline */
		x, y,
		tar_width - 1, tar_height - 1
	    );

	    /* Label */
	    tar_width = b.width;
	    tar_height = font->ascent + font->descent;
	    x = (width / 2) - (tar_width / 2);
	    y = (gint)(height * y_coeff) - (tar_height / 2);
	    gdk_gc_set_foreground(gc, &fg);
	    gdk_gc_set_font(gc, font);
	    gdk_draw_string(
		drawable, font, gc,
		x + b.lbearing,
		y + font->ascent,
		s
	    );

	    GDK_COLORMAP_FREE_COLOR(colormap, &fg);
	    GDK_COLORMAP_FREE_COLOR(colormap, &bg);

	    y += tar_height + (4 * border_major);
#endif
	}
	font = GDK_FONT_UNREF(font);

	/* Draw the version text */
	font = gdk_font_load(
"-adobe-helvetica-bold-r-normal-*-20-*-*-*-p-*-iso8859-1"
	);
	if(font == NULL)
	    font = gdk_font_load("8x16");
	if(font != NULL)
	{
#ifdef PROG_VERSION
	    gint tar_width, tar_height;
	    GdkTextBounds b;
	    const gchar *s = "Version " PROG_VERSION;

	    GDK_COLOR_SET_COEFF(&fg, 0.0f, 0.0f, 0.0f);
	    GDK_COLORMAP_ALLOC_COLOR(colormap, &fg);

	    gdk_string_bounds(font, s, &b);
	    tar_width = b.width;
	    tar_height = font->ascent + font->descent;
	    x = (width / 2) - (tar_width / 2);

	    gdk_gc_set_foreground(gc, &fg);
	    gdk_gc_set_font(gc, font);
	    gdk_draw_string(
		drawable, font, gc,
		x + b.lbearing,
		y + font->ascent,
		s
	    );

	    GDK_COLORMAP_FREE_COLOR(colormap, &fg);
	    y += tar_height + border_major;
#endif
	}
	font = GDK_FONT_UNREF(font);

	/* Draw the home page URL text */
	font = gdk_font_load(
"-adobe-helvetica-medium-r-normal-*-12-*-*-*-p-*-iso8859-1"
	);
	if(font == NULL)
	    font = gdk_font_load("7x14");
	if(font != NULL)
	{
#ifdef PROG_URL
	    gint tar_width, tar_height;
	    GdkTextBounds b;
	    const gchar *s = PROG_URL;

	    GDK_COLOR_SET_COEFF(&fg, 0.0f, 0.0f, 0.0f);
	    GDK_COLORMAP_ALLOC_COLOR(colormap, &fg);

	    gdk_string_bounds(font, s, &b);
	    tar_width = b.width;
	    tar_height = font->ascent + font->descent;
	    x = (width / 2) - (tar_width / 2);

	    gdk_gc_set_foreground(gc, &fg);
	    gdk_gc_set_font(gc, font);
	    gdk_draw_string(
		drawable, font, gc,
		x + b.lbearing,
		y + font->ascent,
		s
	    );

	    GDK_COLORMAP_FREE_COLOR(colormap, &fg);
#endif
	}
	font = GDK_FONT_UNREF(font);

	/* Draw the "(Press Button3 for menu)" text */
	font = gdk_font_load(
"-adobe-helvetica-bold-r-normal-*-20-*-*-*-p-*-iso8859-1"
	);
	if(font == NULL)
	    font = gdk_font_load("8x16");
	if(font != NULL)
	{
	    gint tar_width, tar_height;
	    GdkTextBounds b;
	    const gchar *s = "(Press Button3 for menu)";

	    gdk_string_bounds(font, s, &b);
	    tar_width = b.width;
	    tar_height = font->ascent + font->descent;
	    x = (width / 2) - (tar_width / 2);
	    y = (gint)(height * 0.80f) - (tar_height / 2);

	    gdk_gc_set_foreground(gc, &fg);
	    gdk_gc_set_font(gc, font);
	    gdk_draw_string(
		drawable, font, gc,
		x + b.lbearing,
		y + font->ascent,
		s
	    );

	    GDK_COLORMAP_FREE_COLOR(colormap, &fg);
	}
	font = GDK_FONT_UNREF(font);

	/* Draw the copyright text */
	font = gdk_font_load(
"-adobe-helvetica-medium-r-normal-*-10-*-*-*-p-*-iso8859-1"
	);
	if(font == NULL)
	    font = gdk_font_load("6x10");
	if(font != NULL)
	{
#ifdef PROG_COPYRIGHT
	    gint tar_width, tar_height;
	    gchar	*s = g_strdup(PROG_COPYRIGHT),
			*s2;
	    GdkTextBounds b;

	    s2 = strchr(s, '\n');
	    if(s2 != NULL)
		*s2 = '\0';
	    s2 = strchr(s, '.');
	    if(s2 != NULL)
		*s2 = '\0';

	    GDK_COLOR_SET_COEFF(&fg, 0.0f, 0.0f, 0.0f);
	    GDK_COLORMAP_ALLOC_COLOR(colormap, &fg);

	    gdk_string_bounds(font, s, &b);
	    tar_width = b.width;
	    tar_height = font->ascent + font->descent;
	    x = width - tar_width - border_major;
	    y = height - tar_height - border_major;

	    gdk_gc_set_foreground(gc, &fg);
	    gdk_gc_set_font(gc, font);
	    gdk_draw_string(
		drawable,
		font,
		gc,
		x + b.lbearing,
		y + font->ascent,
		s
	    );

	    GDK_COLORMAP_FREE_COLOR(colormap, &fg);
	    g_free(s);
#endif
	}
	font = GDK_FONT_UNREF(font);

	(void)GDK_GC_UNREF(gc);
	(void)GDK_COLORMAP_UNREF(colormap);

	return(pixmap);
}


/*
 *	Loads the title image data on to the ImgView.
 *
 *	If INCLUDE_IV_TITLE_IMAGE is defined then it means the image
 *	data is compiled in and it will be the one loaded.
 *
 *	If INCLUDE_IV_TITLE_IMAGE is not defined then a genertic
 *	title image will be generated.
 *
 *	If simple_title is TRUE then a simpler (less graphical) title
 *	image will be used.
 */
void iv_title_load_to_image_viewer(
	imgview_struct *iv,
#if defined(HAVE_IMLIB)
	gpointer imlib_handle,
#elif defined(HAVE_IMLIB2)
#endif
	const gboolean auto_resize,		/* Automatically resize the
						 * ImgView to the size of the
						 * title image */
	const gboolean auto_zoom_to_fit,	/* Automatically adjust the
						 * ImgView's zoom to fit the
						 * title image */
	const gboolean simple_title		/* Display a "less graphical"
						 * title image */
)
{
	GdkWindow *window;
	GtkWidget *toplevel;

	if(iv == NULL)
	    return;

	/* Get the image viewer toplevel widget if it is a GtkWindow */
	if(ImgViewIsToplevelWindow(iv))
	    toplevel = ImgViewGetToplevelWidget(iv);
	else
	    toplevel = NULL;

	window = (toplevel != NULL) ? toplevel->window : NULL;

	/* Display a "less graphical" title? */
#ifdef INCLUDE_IV_TITLE_IMAGE
	if(simple_title)
#else
	if(TRUE)				/* Must use "less graphical"
						 * title since no standard
						 * title was defined */
#endif
	{
	    /* Create the "less graphical" title on a GdkPixmap */
	    GdkPixmap *pixmap = iv_title_create_simple_title(
		window,
		320, 240
	    );
	    if(pixmap != NULL)
	    {
		const gint bpp = 4;		/* RGBA */
		gint	width, height,
			bpl;
		GtkWidget *w = (GtkWidget *)ImgViewGetViewWidget(iv);
		guint8 *rgba = gdk_get_rgba_image(
		    pixmap,
		    NULL,			/* No mask */
		    NULL,			/* Entire area */
		    &width, &height,
		    &bpl
		);
		imgview_image_struct *img = ImgViewImageNew(
		    width, height,
		    bpp, bpl
		);
		if(img != NULL)
		{
		    (void)ImgViewImageAppendFrame(
			img,
			rgba,			/* Trasnfered */
			0l
		    );
/*		    rgba = NULL; */

		    /* Resize the ImgView to the size of the image? */
		    if((w != NULL) && auto_resize)
		    {
			/* Resize the ImgView's view and toplevel widgets */
			gtk_widget_set_usize(
			    w,
			    img->width, img->height
			);
			gtk_widget_queue_resize(w);
			if(toplevel != NULL)
			    gtk_widget_queue_resize(toplevel);
		    }
	     
		    /* Set the new image on to the ImgView */
		    if(auto_zoom_to_fit)
			ImgViewSetImageToFit(
			    iv,
			    img			/* Transfered */
			);
		    else
			ImgViewSetImage(
			    iv,
			    img			/* Transfered */
			);
/*		    img = NULL; */
		}
	    }
	    (void)GDK_PIXMAP_UNREF(pixmap);
	}
	else
	{
#ifdef INCLUDE_IV_TITLE_IMAGE
	    /* Load the standard title */
	    GtkWidget *w = (GtkWidget *)ImgViewGetViewWidget(iv);
	    imgview_image_struct *img = iv_title_create_title(
#if defined(HAVE_IMLIB)
		imlib_handle
#elif defined(HAVE_IMLIB2)
#endif
	    );
	    if(img != NULL)
	    {
		/* Resize the ImgView to the size of the image? */
		if((w != NULL) && auto_resize)
		{
		    gtk_widget_set_usize(
			w,
			img->width, img->height
		    );
		    gtk_widget_queue_resize(w);
		    if(toplevel != NULL)
			gtk_widget_queue_resize(toplevel);
		}

		/* Set the new image on to the ImgView */
		if(auto_zoom_to_fit)
		    ImgViewSetImageToFit(
			iv,
			img			/* Transfered */
		    );
		else
		    ImgViewSetImage(
			iv,
			img			/* Transfered */
		    );
/*		img = NULL; */
	    }
#endif	/* INCLUDE_IV_TITLE_IMAGE */
	}
}
