/*
				  Print
 */

#ifndef PRINT_H
#define PRINT_H

#if !defined(WIN32)

#include <gtk/gtk.h>
#ifdef HAVE_LIBENDEAVOUR2
# include <endeavour2.h>
#endif
#include "imgview.h"
#include "print_values.h"


#if defined(HAVE_XPRINT)
typedef struct _IVPrintXPrintPrinter	IVPrintXPrintPrinter;
#define IV_PRINT_XPRINT_PRINTER(p)	((IVPrintXPrintPrinter *)(p))

typedef struct _IVPrintXPrintMedium	IVPrintXPrintMedium;
#define IV_PRINT_XPRINT_MEDIUM(p)	((IVPrintXPrintMedium *)(p))

typedef struct _IVPrintXPrintResolution	IVPrintXPrintResolution;
#define IV_PRINT_XPRINT_RESOLUTION(p)	((IVPrintXPrintResolution *)(p))


/*
 *	X Print Printer:
 */
struct _IVPrintXPrintPrinter {
	gchar		*name,
			*description;
};

/*
 *	X Print Medium:
 */
struct _IVPrintXPrintMedium {
	gchar		*name,
			*tray;		/* Tray that this medium is on */
	gfloat		x_mm,		/* Offset in millimeters */
			y_mm,
			width_mm,	/* Size in millimeters */
			height_mm;
};

/*
 *	X Print Resolution:
 */
struct _IVPrintXPrintResolution {
	gchar		*name;
	gint		dpi,		/* MAX(x_dpi, y_dpi) */
			x_dpi,
			y_dpi;
};
#endif


#if defined(HAVE_XPRINT)
/* print_xprint.c */
extern IVPrintXPrintPrinter *iv_print_xprint_printer_new(void);
extern IVPrintXPrintPrinter *iv_print_xprint_printer_copy(IVPrintXPrintPrinter *p);
extern void iv_print_xprint_printer_delete(IVPrintXPrintPrinter *p);
extern void iv_print_xprint_printers_list_delete(GList *glist);

extern gfloat iv_print_xprint_medium_mm_to_inches(const gfloat mm);
extern IVPrintXPrintMedium *iv_print_xprint_medium_new(void);
extern IVPrintXPrintMedium *iv_print_xprint_medium_copy(IVPrintXPrintMedium *m);
extern void iv_print_xprint_medium_delete(IVPrintXPrintMedium *m);
extern void iv_print_xprint_mediums_list_delete(GList *glist);

extern IVPrintXPrintResolution *iv_print_xprint_resolution_new(void);
extern IVPrintXPrintResolution *iv_print_xprint_resolution_copy(IVPrintXPrintResolution *r);
extern void iv_print_xprint_resolution_delete(IVPrintXPrintResolution *r);
extern void iv_print_xprint_resolutions_list_delete(GList *glist);

extern GList *iv_print_xprint_get_printers_list(
	const gchar *xp_address
);
extern GList *iv_print_xprint_get_printer_trays(
	const gchar *xp_address,
	const gchar *printer_name
);
extern GList *iv_print_xprint_get_printer_mediums(
	const gchar *xp_address,
	const gchar *printer_name
);
extern GList *iv_print_xprint_get_printer_orientations(
	const gchar *xp_address,
	const gchar *printer_name
);
extern GList *iv_print_xprint_get_printer_resolutions(
	const gchar *xp_address,
	const gchar *printer_name
);
extern IVPrintXPrintResolution *iv_print_xprint_get_default_resolution(
        const gchar *xp_address,
        const gchar *printer_name
);

extern gint iv_print_xprint_get_medium_size(
	const gchar *xp_address,
	const gchar *printer_name,
	const gchar *tray_name,
	const gchar *medium_name,
	const gchar *orientation_name,
	const gint x_dpi, const gint y_dpi,
	gint *width_rtn, gint *height_rtn
);

extern gint iv_print_xprint(
	imgview_image_struct *img,		/* Image to print */
	gint frame_num,				/* Frame on image to print */
	const print_values_struct *pv,		/* Print values */ 
	const guint8 *bg_color,			/* 4 bytes RGBA */
	GtkWidget *toplevel
);
#endif

/* print.c */
extern imgview_image_struct *iv_print_render_image_from_values(
	imgview_image_struct *img,
	const print_values_struct *pv,
	const guint8 *bg_color,			/* 4 bytes RGBA */
	gint (*progress_cb)(const gulong, const gulong, gpointer),
	gpointer progress_data
);
extern void iv_print_adjust_image_color(
	imgview_image_struct *img,
	const print_values_struct *pv,
	gint (*progress_cb)(const gulong, const gulong, gpointer),
	gpointer progress_data
);

#ifdef HAVE_LIBENDEAVOUR2
/* print_lpr.c or print_xprint.c */
extern gint iv_print(
	imgview_image_struct *img,		/* Image to print */
	const print_values_struct *pv,		/* Print values */
	const guint8 *bg_color,			/* 4 bytes RGBA */
	GtkWidget *toplevel,
	EDVContext *edv2_ctx
);
#else
extern gint iv_print(
	imgview_image_struct *img,		/* Image to print */
	const print_values_struct *pv,		/* Print values */
	const guint8 *bg_color,			/* 4 bytes RGBA */
	GtkWidget *toplevel
);
#endif

#endif	/* !WIN32 */

#endif	/* PRINT_H */
