#if !defined(_WIN32) && defined(HAVE_XPRINT)
#include <stdio.h>
#include <stdlib.h>
#include <X11/X.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/extensions/Print.h>
#include <X11/XprintUtil/xprintutil.h>
#include <gtk/gtk.h>
#include <unistd.h>
#include "print_values.h"
#include "print.h"
#include "cdialog.h"
#include "progressdialog.h"
#include "config.h"

#include "images/icon_print2_32x32.xpm"


typedef struct _IVPrintXPrintProgressData	IVPrintXPrintProgressData;
#define IV_PRINT_XPRINT_PROGRESS_DATA(p)	((IVPrintXPrintProgressData *)(p))


/* Pixel Operation Callbacks */
static void pixel_set_rgb_to_rgb(guint8 *tar, const guint8 *src);
static void pixel_set_rgb_to_rgba(guint8 *tar, const guint8 *src);
static void pixel_set_rgb_to_bgr(guint8 *tar, const guint8 *src);
static void pixel_set_rgb_to_bgra(guint8 *tar, const guint8 *src);
static void pixel_set_rgb_to_grey_alpha(guint8 *tar, const guint8 *src);
#if 0
static void pixel_set_rgb_to_alpha_grey(guint8 *tar, const guint8 *src);
#endif
static void pixel_set_rgb_to_grey(guint8 *tar, const guint8 *src);

/* Connect/Disconnect From X Print Server */
static gint iv_print_xprint_connect(
	Display **dpy_rtn,
	int *xp_event_base_rtn, int *xp_error_base_rtn,
	const gchar *address
);
static void iv_print_xprint_disconnect(Display *dpy);

/* X Print Context */
static XPContext iv_print_xprint_context_new(
	Display *dpy,
	const gchar *printer_name
);
static void iv_print_xprint_context_delete(
	Display *dpy,
	XPContext xp_ctx
);

/* Color */
static unsigned long iv_print_xprint_get_pixel_rgb(
	Display *dpy, Colormap cmap,
	guint8 r, guint8 g, guint8 b
);

/* X Visual */
static Visual *iv_print_xprint_get_best_visual(
	Display *dpy,
	const int visual_class,
	int *depth_rtn
);

/* Printers */
IVPrintXPrintPrinter *iv_print_xprint_printer_new(void);
IVPrintXPrintPrinter *iv_print_xprint_printer_copy(IVPrintXPrintPrinter *p);
void iv_print_xprint_printer_delete(IVPrintXPrintPrinter *p);
void iv_print_xprint_printers_list_delete(GList *glist);

/* Mediums */
gfloat iv_print_xprint_medium_mm_to_inches(const gfloat mm);
IVPrintXPrintMedium *iv_print_xprint_medium_new(void);
IVPrintXPrintMedium *iv_print_xprint_medium_copy(IVPrintXPrintMedium *m);
void iv_print_xprint_medium_delete(IVPrintXPrintMedium *m);
void iv_print_xprint_mediums_list_delete(GList *glist);

/* Resolutions */
IVPrintXPrintResolution *iv_print_xprint_resolution_new(void);
IVPrintXPrintResolution *iv_print_xprint_resolution_copy(IVPrintXPrintResolution *r);
void iv_print_xprint_resolution_delete(IVPrintXPrintResolution *r);
void iv_print_xprint_resolutions_list_delete(GList *glist);

/* Get Printer Attributes */
GList *iv_print_xprint_get_printers_list(const gchar *xp_address);
GList *iv_print_xprint_get_printer_trays(
	const gchar *xp_address,
	const gchar *printer_name
);
GList *iv_print_xprint_get_printer_mediums(
	const gchar *xp_address,
	const gchar *printer_name
);
GList *iv_print_xprint_get_printer_orientations(
	const gchar *xp_address,
	const gchar *printer_name
);
GList *iv_print_xprint_get_printer_resolutions(
	const gchar *xp_address,
	const gchar *printer_name
);
IVPrintXPrintResolution *iv_print_xprint_get_default_resolution(
	const gchar *xp_address,
	const gchar *printer_name
);


/* Get Medium Size */
gint iv_print_xprint_get_medium_size(
	const gchar *xp_address,
	const gchar *printer_name,
	const gchar *tray_name,
	const gchar *medium_name,
	const gchar *orientation_name,
	const gint x_dpi, const gint y_dpi,
	gint *width_rtn, gint *height_rtn
);

/* Callbacks */
static int iv_print_x_error_cb(Display *dpy, XErrorEvent *error);
static gint iv_print_xprint_progress_cb(
	const gulong i, const gulong m, gpointer data
);

/* Render Image For Printing */
static XImage *iv_print_xprint_render_ximage(
	Display *dpy, Visual *visual, const int depth,
	imgview_image_struct *img,
	const print_values_struct *pv,
	const guint8 *bg_color,
	gint (*progress_cb)(const gulong, const gulong, gpointer),
	IVPrintXPrintProgressData *progress_data
);

/* Print Main */
static gint iv_print_xprint_main_iteration(
	Display *dpy,
	Visual *visual,
	const int depth,
	Window w,
	GC gc,
	XPContext xp_ctx,
	const int xp_event_base, const int xp_error_base,
	imgview_image_struct *img,
	const print_values_struct *pv,
	const guint8 *bg_color,
	const gboolean verbose,
	const gboolean interactive,
	GtkWidget *toplevel,
	gint (*progress_cb)(const gulong, const gulong, gpointer),
	IVPrintXPrintProgressData *progress_data
);
static gint iv_print_xprint_main(
	imgview_image_struct *img,
	const print_values_struct *pv,
	const guint8 *bg_color,
	const gboolean verbose,
	const gboolean interactive,
	GtkWidget *toplevel
);

#ifdef HAVE_LIBENDEAVOUR2
gint iv_print(
	imgview_image_struct *img,		/* Image to print */
	const print_values_struct *pv,		/* Print values */
	const guint8 *bg_color,			/* 4 bytes RGBA */
	GtkWidget *toplevel,
	EDVContext *edv2_ctx
);
#else
gint iv_print(
	imgview_image_struct *img,		/* Image to print */
	const print_values_struct *pv,		/* Print values */
	const guint8 *bg_color,			/* 4 bytes RGBA */
	GtkWidget *toplevel 
);
#endif


struct _IVPrintXPrintProgressData {
	gulong		cur_stage,
			nstages;
};


#define ATOI(s)		(((s) != NULL) ? atoi(s) : 0)
#define ATOL(s)		(((s) != NULL) ? atol(s) : 0)
#define ATOF(s)		(((s) != NULL) ? atof(s) : 0.0f)
#define STRDUP(s)	(((s) != NULL) ? g_strdup(s) : NULL)

#define MAX(a,b)	(((a) > (b)) ? (a) : (b))
#define MIN(a,b)	(((a) < (b)) ? (a) : (b))
#define CLIP(a,l,h)	(MIN(MAX((a),(l)),(h)))
#define STRLEN(s)	(((s) != NULL) ? strlen(s) : 0)
#define STRISEMPTY(s)	(((s) != NULL) ? (*(s) == '\0') : TRUE)

#define ISBLANK(c)	(((c) == ' ') || ((c) == '\t'))


/*
 *	Pixel operation callbacks.
 *
 *	The src is always assumed to be at least 3 bytes in RGB
 *	format.
 */
static void pixel_set_rgb_to_rgb(guint8 *tar, const guint8 *src)
{
	tar[0] = src[0];
	tar[1] = src[1];
	tar[2] = src[2];
}

static void pixel_set_rgb_to_rgba(guint8 *tar, const guint8 *src)
{
	tar[0] = src[0];
	tar[1] = src[1];
	tar[2] = src[2];
	tar[3] = 0xff;
}

static void pixel_set_rgb_to_bgr(guint8 *tar, const guint8 *src)
{
	tar[0] = src[2];
	tar[1] = src[1];
	tar[2] = src[0];
}

static void pixel_set_rgb_to_bgra(guint8 *tar, const guint8 *src)
{
	tar[0] = src[2];
	tar[1] = src[1];
	tar[2] = src[0];
	tar[3] = 0xff;
}

static void pixel_set_rgb_to_grey_alpha(guint8 *tar, const guint8 *src)
{
	tar[0] = src[0];
	tar[1] = src[1];
}

#if 0
static void pixel_set_rgb_to_alpha_grey(guint8 *tar, const guint8 *src)
{
	tar[0] = src[1];
	tar[1] = src[0];
}
#endif

static void pixel_set_rgb_to_grey(guint8 *tar, const guint8 *src)
{
	tar[0] = src[0];
}


/*
 *	Connects to the X Print server.
 *
 *	The address specifies the address to the X Print server. If
 *	address is NULL then the address specified by the environment
 *	variable "XPSERVERLIST" will be used instead.
 *
 *	Returns 0 on success and non-zero on error.
 */
static gint iv_print_xprint_connect(
	Display **dpy_rtn,
	int *xp_event_base_rtn, int *xp_error_base_rtn,
	const gchar *address
)
{
	Display *dpy;

	/* Connect to the X server */
	if(address != NULL)
	{
	    dpy = XOpenDisplay(address);
	}
	else
	{
	    address = g_getenv("XPSERVERLIST");
	    dpy = XOpenDisplay(address);
	}

	/* Failed to connect to the X Server? */
	if(dpy == NULL)
	{
	    *dpy_rtn = NULL;
	    *xp_event_base_rtn = 0;
	    *xp_error_base_rtn = 0;
	    return(-1);
	}

	*dpy_rtn = dpy;

	/* Set the X error handler */
	XSetErrorHandler(iv_print_x_error_cb);

	/* Check if X Print is supported on this X Server */
	if(!XpQueryExtension(
	    dpy, xp_event_base_rtn, xp_error_base_rtn
	))
	{
	    XCloseDisplay(dpy);
	    *dpy_rtn = NULL;
	    return(-2);
	}

	return(0);
}

/*
 *	Disconnects from the X Print server.
 */
static void iv_print_xprint_disconnect(Display *dpy)
{
	if(dpy == NULL)
	    return;

	XCloseDisplay(dpy);
}

/*
 *	Creates a new X Print context for a printer on the X Print
 *	server.
 *
 *	The dpy specifies the Display connection to the X Print server
 *	to create the new XPContext on.
 *
 *	The printer_name specifies the printer to create the X Print
 *	context for.
 */
static XPContext iv_print_xprint_context_new(
	Display *dpy,
	const gchar *printer_name
)
{
	if((dpy == NULL) || (printer_name == NULL))
	    return(None);

	return(XpCreateContext(
	    dpy,
	    (char *)printer_name
	));
}

/*
 *	Destroys the X Print context.
 */
static void iv_print_xprint_context_delete(
	Display *dpy,
	XPContext xp_ctx
)
{
	if((dpy == NULL) || (xp_ctx == None))
	    return;

	XpDestroyContext(
	    dpy,
	    xp_ctx
	);
}


/*
 *	Gets the pixel from the colormap by RGB values.
 */
static unsigned long iv_print_xprint_get_pixel_rgb(
	Display *dpy, Colormap cmap,
	guint8 r, guint8 g, guint8 b
)
{
	XColor xc;
	xc.flags = DoRed | DoGreen | DoBlue;
	xc.red = ((unsigned short)r << 8) |
	    (unsigned short)r;
	xc.green = ((unsigned short)g << 8) |
	    (unsigned short)g;
	xc.blue = ((unsigned short)b << 8) |
	    (unsigned short)b;
	XAllocColor(dpy, cmap, &xc);
	return(xc.pixel);
}


/*
 *	Gets the best X Visual suitable for printing.
 *
 *	Returns the X Visual or NULL if no suitable X Visual was
 *	found.
 */
static Visual *iv_print_xprint_get_best_visual(
	Display *dpy,
	const int visual_class,
	int *depth_rtn
)
{
	const int screen = DefaultScreen(dpy);
	XVisualInfo visual_info;

	/* Begin checking for a suitable X Visual */

	/* TrueColor */
	if(visual_class == TrueColor)
	{
	    /* 32 bit depth TrueColor */
	    if(XMatchVisualInfo(
		dpy, screen,
		32,			/* 32 bit depth */
		visual_class,
		&visual_info
	    ))
	    {
		*depth_rtn = visual_info.depth;
		return(visual_info.visual);
	    }

	    /* 24 bit depth TrueColor */
	    if(XMatchVisualInfo(
		dpy, screen,
		24,				/* 24 bit depth */
		visual_class,
		&visual_info
	    ))
	    { 
		*depth_rtn = visual_info.depth;
		return(visual_info.visual);
	    }
	}
	/* GrayScale */
	else if(visual_class == GrayScale)
	{
	    /* 16 bit depth GrayScale */
	    if(XMatchVisualInfo(
		dpy, screen,
		16,				/* 16 bit depth */
		visual_class,
		&visual_info
	    ))
	    {
		*depth_rtn = visual_info.depth;
		return(visual_info.visual);
	    }

	    /* 8 bit depth GrayScale */
	    if(XMatchVisualInfo(
		dpy, screen,
		8,				/* 8 bit depth */
		visual_class,
		&visual_info
	    ))
	    { 
		*depth_rtn = visual_info.depth;
		return(visual_info.visual);
	    }

	    /* Resort to TrueColor when a GrayScale X Visual is
	     * not available
	     */

	    /* 32 bit depth TrueColor */
	    if(XMatchVisualInfo(
		dpy, screen,
		32,				/* 32 bit depth */
		TrueColor,
		&visual_info
	    ))
	    {
		*depth_rtn = visual_info.depth;
		return(visual_info.visual);
	    }

	    /* 24 bit depth TrueColor */
	    if(XMatchVisualInfo(
		dpy, screen,
		24,				/* 24 bit depth */
		TrueColor,
		&visual_info
	    ))
	    { 
		*depth_rtn = visual_info.depth;
		return(visual_info.visual);
	    }
	}

	return(NULL);
}


/*
 *	Creates a new X Print printer.
 */
IVPrintXPrintPrinter *iv_print_xprint_printer_new(void)
{
	return(IV_PRINT_XPRINT_PRINTER(g_malloc0(
	    sizeof(IVPrintXPrintPrinter)
	)));
}

/*
 *	Coppies the X Print printer.
 */
IVPrintXPrintPrinter *iv_print_xprint_printer_copy(
	IVPrintXPrintPrinter *p
)
{
	IVPrintXPrintPrinter	*p_tar,
					*p_src = p;

	if(p_src == NULL)
	    return(NULL);

	p_tar = iv_print_xprint_printer_new();
	if(p_tar == NULL)
	    return(NULL);

	p_tar->name = STRDUP(p_src->name);
	p_tar->description = STRDUP(p_src->description);

	return(p_tar);
}

/*
 *	Deletes the X Print printer.
 */
void iv_print_xprint_printer_delete(IVPrintXPrintPrinter *p)
{
	if(p == NULL)
	    return;

	g_free(p->name);
	g_free(p->description);
	g_free(p);
}


/*
 *	Deletes the list of X Print printers.
 */
void iv_print_xprint_printers_list_delete(GList *glist)
{
	if(glist == NULL)
	    return;

	g_list_foreach(glist, (GFunc)iv_print_xprint_printer_delete, NULL);
	g_list_free(glist);
}


/*
 *	Converts the medium millimeters values to inches.
 */
gfloat iv_print_xprint_medium_mm_to_inches(const gfloat mm)
{
	return(mm / 25.40f);
}

/*
 *	Creates a new X Print medium.
 */
IVPrintXPrintMedium *iv_print_xprint_medium_new(void)
{
	return(IV_PRINT_XPRINT_MEDIUM(g_malloc0(
	    sizeof(IVPrintXPrintMedium)
	)));
}

/*
 *	Coppies the X Print medium.
 */
IVPrintXPrintMedium *iv_print_xprint_medium_copy(IVPrintXPrintMedium *m)
{
	IVPrintXPrintMedium	*tar_m,
				*src_m = m;

	if(src_m == NULL)
	    return(NULL);

	tar_m = iv_print_xprint_medium_new();
	if(tar_m == NULL)
	    return(NULL);

	tar_m->name = STRDUP(src_m->name);
	tar_m->tray = STRDUP(src_m->tray);
	tar_m->x_mm = src_m->x_mm;
	tar_m->y_mm = src_m->y_mm;
	tar_m->width_mm = src_m->width_mm;
	tar_m->height_mm = src_m->height_mm;

	return(tar_m);
}

/*
 *	Deletes the X Print medium.
 */
void iv_print_xprint_medium_delete(IVPrintXPrintMedium *m)
{
	if(m == NULL)
	    return;

	g_free(m->name);
	g_free(m->tray);
	g_free(m);
}

/*
 *	Deletes the list of X Print mediums.
 */
void iv_print_xprint_mediums_list_delete(GList *glist)
{
	if(glist == NULL)
	    return;

	g_list_foreach(glist, (GFunc)iv_print_xprint_medium_delete, NULL);
	g_list_free(glist);
}


/*
 *	Creates a new X Print resolution.
 */
IVPrintXPrintResolution *iv_print_xprint_resolution_new(void)
{
	return(IV_PRINT_XPRINT_RESOLUTION(g_malloc0(sizeof(IVPrintXPrintResolution))));
}

/*
 *	Coppies the X Print resolution.
 */
IVPrintXPrintResolution *iv_print_xprint_resolution_copy(IVPrintXPrintResolution *r)
{
	IVPrintXPrintResolution	*tar_r,
				*src_r = r;

	if(src_r == NULL)
	    return(NULL);

	tar_r = iv_print_xprint_resolution_new();
	if(tar_r == NULL)
	    return(NULL);

	tar_r->name = STRDUP(src_r->name);
	tar_r->dpi = src_r->dpi;
	tar_r->x_dpi = src_r->x_dpi;
	tar_r->y_dpi = src_r->y_dpi;

	return(tar_r);
}

/*
 *	Deletes the X Print resolution.
 */
void iv_print_xprint_resolution_delete(IVPrintXPrintResolution *r)
{
	if(r == NULL)
	    return;

	g_free(r->name);
	g_free(r);
}

/*
 *	Deletes the list of X Print resolutions.
 */
void iv_print_xprint_resolutions_list_delete(GList *glist)
{
	if(glist == NULL)
	    return;

	g_list_foreach(glist, (GFunc)iv_print_xprint_resolution_delete, NULL);
	g_list_free(glist);
}


/*
 *	Gets the list of printers from the X Print server.
 *
 *	The xp_address specifies the address to the X Print server. If
 *	xp_address is NULL then the address specified by the environment
 *	variable "XPSERVERLIST" will be used instead.
 *
 *	Returns a glist of IVPrintXPrintPrinter describing
 *	each printer It must be deleted by the calling
 *	iv_print_xprint_printers_list_delete().
 */
GList *iv_print_xprint_get_printers_list(const gchar *xp_address)
{
	int		xp_event_base,
			xp_error_base,
			i,
			nprinters;
	Display *dpy;
	XPPrinterList xp, xp_printers_list;
	GList *printers_list;
	IVPrintXPrintPrinter *p;

	if(iv_print_xprint_connect(
	    &dpy,
	    &xp_event_base, &xp_error_base,
	    xp_address
	))
	    return(NULL);

	/* Get the printers list from the X Print server */
	xp_printers_list = XpGetPrinterList(
	    dpy,
	    NULL,				/* Get all printers */
	    &nprinters
	);
	if(xp_printers_list == NULL)
	{
	    iv_print_xprint_disconnect(dpy);
	    return(NULL);
	}

	/* Parse the printers list */
	printers_list = NULL;
	for(i = 0; i < nprinters; i++)
	{
	    xp = &xp_printers_list[i];
	    p = iv_print_xprint_printer_new();
	    if(p != NULL)
	    {
		p->name = STRDUP(xp->name);
		p->description = STRDUP(xp->desc);
		printers_list = g_list_append(
		    printers_list,
		    p
		);
	    }
	}

	/* Delete the printers list obtained from the X Print server */
	XpFreePrinterList(xp_printers_list);
	iv_print_xprint_disconnect(dpy);

	return(printers_list);
}

/*
 *	Gets the list of trays on the printer.
 *
 *	The xp_address specifies the address to the X Print server. If
 *	xp_address is NULL then the address specified by the environment
 *	variable "XPSERVERLIST" will be used instead.
 *
 *	The printer_name specifies the name of the printer obtained
 *	from iv_print_xprint_get_printers_list(). This value may
 *	not be NULL.
 *
 *	Returns a GList of gchar * strings describing each tray's name.
 */
GList *iv_print_xprint_get_printer_trays(
	const gchar *xp_address,
	const gchar *printer_name
)
{
	int		i,
			xp_event_base, xp_error_base,
			nmediums;
	Display *dpy;
	XPContext xp_ctx;
	XpuMediumSourceSizeRec *xpu_medium;
	XpuMediumSourceSizeList xpu_mediums_list;
	const gchar *tray;
	GList		*glist,
			*trays_list;

	if(printer_name == NULL)
	    return(NULL);

	if(iv_print_xprint_connect(
	    &dpy,
	    &xp_event_base, &xp_error_base,
	    xp_address
	))
	    return(NULL);

	xp_ctx = iv_print_xprint_context_new(
	    dpy,
	    printer_name
	);
	if(xp_ctx == None)
	{
	    iv_print_xprint_disconnect(dpy);
	    return(NULL);
	}

	/* Get the mediums list from the X Print server and derive
	 * the number of trays from it
	 */
	xpu_mediums_list = XpuGetMediumSourceSizeList(
	    dpy,
	    xp_ctx,
	    &nmediums
	);
	if(xpu_mediums_list == NULL)
	{
	    iv_print_xprint_context_delete(dpy, xp_ctx);
	    iv_print_xprint_disconnect(dpy);
	    return(NULL);
	}

	/* Parse the mediums listand derive the number of trays from
	 * it
	 */
	trays_list = NULL;
	for(i = 0; i < nmediums; i++)
	{
	    xpu_medium = &xpu_mediums_list[i];
	    if(xpu_medium == NULL)
		continue;

	    if(STRISEMPTY(xpu_medium->tray_name))
		continue;

	    /* Check if this tray name is already in the list */
	    for(glist = trays_list;
		glist != NULL;
		glist = g_list_next(glist)
	    )
	    {
		tray = (const gchar *)glist->data;
		if(tray == NULL)
		    continue;

		if(!g_strcasecmp(tray, (const gchar *)xpu_medium->tray_name))
		    break;
	    }
	    /* Already in the list? */
	    if(glist != NULL)
		continue;

	    trays_list = g_list_append(
		trays_list,
		g_strdup(xpu_medium->tray_name)
	    );
	}

	XpuFreeMediumSourceSizeList(xpu_mediums_list);
	iv_print_xprint_context_delete(dpy, xp_ctx);
	iv_print_xprint_disconnect(dpy);

	return(trays_list);
}

/*
 *	Gets the list of mediums supported by the printer.
 *
 *	The xp_address specifies the address to the X Print server. If
 *	xp_address is NULL then the address specified by the environment
 *	variable "XPSERVERLIST" will be used instead.
 *
 *	The printer_name specifies the name of the printer obtained
 *	from iv_print_xprint_get_printers_list(). This value may
 *	not be NULL.
 *
 *	Returns a GList of IVPrintXPrintMedium * mediums describing
 *	each medium.
 */
GList *iv_print_xprint_get_printer_mediums(
	const gchar *xp_address,
	const gchar *printer_name
)
{
	int		i,
			xp_event_base, xp_error_base,
			nmediums;
	Display *dpy;
	XPContext xp_ctx;
	XpuMediumSourceSizeRec *xpu_medium;
	XpuMediumSourceSizeList xpu_mediums_list;
	GList *mediums_list;
	IVPrintXPrintMedium *m;

	if(printer_name == NULL)
	    return(NULL);

	if(iv_print_xprint_connect(
	    &dpy,
	    &xp_event_base, &xp_error_base,
	    xp_address
	))
	    return(NULL);

	xp_ctx = iv_print_xprint_context_new(
	    dpy,
	    printer_name
	);
	if(xp_ctx == None)
	{
	    iv_print_xprint_disconnect(dpy);
	    return(NULL);
	}

	/* Get the list of mediums supported by the specified printer
	 * from the X Print server
	 */
	xpu_mediums_list = XpuGetMediumSourceSizeList(
	    dpy,
	    xp_ctx,
	    &nmediums
	);
	if(xpu_mediums_list == NULL)
	{
	    iv_print_xprint_context_delete(dpy, xp_ctx);
	    iv_print_xprint_disconnect(dpy);
	    return(NULL);
	}

	/* Parse the mediums list */
	mediums_list = NULL;
	for(i = 0; i < nmediums; i++)
	{
	    xpu_medium = &xpu_mediums_list[i];
	    if(xpu_medium == NULL)
		continue;

	    /* Create a new medium and append it to the list */
	    m = iv_print_xprint_medium_new();
	    if(m == NULL)
		break;

	    mediums_list = g_list_append(
		mediums_list,
		m
	    );

	    m->tray = STRDUP(xpu_medium->tray_name);
	    m->name = STRDUP(xpu_medium->medium_name);

	    m->x_mm = (gfloat)xpu_medium->ma1;
	    m->width_mm = (gfloat)xpu_medium->ma2;
	    m->y_mm = (gfloat)xpu_medium->ma3;
	    m->height_mm = (gfloat)xpu_medium->ma4;
	}

	XpuFreeMediumSourceSizeList(xpu_mediums_list);
	iv_print_xprint_context_delete(dpy, xp_ctx);
	iv_print_xprint_disconnect(dpy);

	return(mediums_list);
}

/*
 *	Gets the list of orientations supported by the printer.
 *
 *	The xp_address specifies the address to the X Print server. If
 *	xp_address is NULL then the address specified by the environment
 *	variable "XPSERVERLIST" will be used instead.
 *
 *	The printer_name specifies the name of the printer obtained
 *	from iv_print_xprint_get_printers_list(). This value may
 *	not be NULL.
 *
 *	Returns a GList of gchar *strings describing each orientation's
 *	name.
 */
GList *iv_print_xprint_get_printer_orientations(
	const gchar *xp_address,
	const gchar *printer_name
)
{
	int		i,
			xp_event_base, xp_error_base,
			norientations;
	Display *dpy;
	XPContext xp_ctx;
	XpuOrientationRec *xpu_orientation;
	XpuOrientationList xpu_orientations_list;
	GList *orientations_list;

	if(printer_name == NULL)
	    return(NULL);

	if(iv_print_xprint_connect(
	    &dpy,
	    &xp_event_base, &xp_error_base,
	    xp_address
	))
	    return(NULL);

	xp_ctx = iv_print_xprint_context_new(
	    dpy,
	    printer_name
	);
	if(xp_ctx == None)
	{
	    iv_print_xprint_disconnect(dpy);
	    return(NULL);
	}

	/* Get the list of orientations supported by the specified
	 * printer from the X Print server
	 */
	xpu_orientations_list = XpuGetOrientationList(
	    dpy,
	    xp_ctx,
	    &norientations
	);
	if(xpu_orientations_list == NULL)
	{
	    iv_print_xprint_context_delete(dpy, xp_ctx);
	    iv_print_xprint_disconnect(dpy);
	    return(NULL);
	}

	/* Parse the orientations list */
	orientations_list = NULL;
	for(i = 0; i < norientations; i++)
	{
	    xpu_orientation = &xpu_orientations_list[i];
	    orientations_list = g_list_append(
		orientations_list,
		STRDUP(xpu_orientation->orientation)
	    );
	}

	XpuFreeOrientationList(xpu_orientations_list);
	iv_print_xprint_context_delete(dpy, xp_ctx);
	iv_print_xprint_disconnect(dpy);

	return(orientations_list);
}

/*
 *	Gets the list of resolutions supported by the printer.
 *
 *	The xp_address specifies the address to the X Print server. If
 *	xp_address is NULL then the address specified by the environment
 *	variable "XPSERVERLIST" will be used instead.
 *
 *	The printer_name specifies the name of the printer obtained
 *	from iv_print_xprint_get_printers_list(). This value may
 *	not be NULL.
 *
 *	Returns a GList of IVPrintXPrintResolution * resolutions.
 */
GList *iv_print_xprint_get_printer_resolutions(
	const gchar *xp_address,
	const gchar *printer_name
)
{
	int		i,
			xp_event_base, xp_error_base,
			nresolutions;
	Display *dpy;
	XPContext xp_ctx;
	XpuResolutionRec *xpu_resolution;
	XpuResolutionList xpu_resolutions_list;
	GList *resolutions_list;
	IVPrintXPrintResolution *r;

	if(printer_name == NULL)
	    return(NULL);

	if(iv_print_xprint_connect(
	    &dpy,
	    &xp_event_base, &xp_error_base,
	    xp_address
	))
	    return(NULL);

	xp_ctx = iv_print_xprint_context_new(
	    dpy,
	    printer_name
	);
	if(xp_ctx == None)
	{
	    iv_print_xprint_disconnect(dpy);
	    return(NULL);
	}

	/* Get the list of resolutions supported by the specified
	 * printer from the X Print server
	 */
	xpu_resolutions_list = XpuGetResolutionList(
	    dpy,
	    xp_ctx,
	    &nresolutions
	);
	if(xpu_resolutions_list == NULL)
	{
	    iv_print_xprint_context_delete(dpy, xp_ctx);
	    iv_print_xprint_disconnect(dpy);
	    return(NULL);
	}

	/* Parse the resolutions list */
	resolutions_list = NULL;
	for(i = 0; i < nresolutions; i++)
	{
	    xpu_resolution = &xpu_resolutions_list[i];

	    r = iv_print_xprint_resolution_new();
	    if(r == NULL)
		break;

	    resolutions_list = g_list_append(
		resolutions_list,
		r
	    );

	    r->name = STRDUP(xpu_resolution->name);
	    r->x_dpi = (gint)xpu_resolution->x_dpi;
	    r->y_dpi = (gint)xpu_resolution->y_dpi;
	    r->dpi = MAX(r->x_dpi, r->y_dpi);
	}

	XpuFreeResolutionList(xpu_resolutions_list);
	iv_print_xprint_context_delete(dpy, xp_ctx);
	iv_print_xprint_disconnect(dpy);

	return(resolutions_list);
}

/*
 *	Gets the default resolution on the printer
 */
IVPrintXPrintResolution *iv_print_xprint_get_default_resolution(
	const gchar *xp_address,
	const gchar *printer_name
)
{
	int		xp_event_base, xp_error_base;
	long		x_dpi, y_dpi;
	Display *dpy;
	XPContext xp_ctx;

	if(printer_name == NULL)
	    return(NULL);

	if(iv_print_xprint_connect(
	    &dpy,
	    &xp_event_base, &xp_error_base,
	    xp_address
	))
	    return(NULL);

	xp_ctx = iv_print_xprint_context_new(
	    dpy,
	    printer_name
	);
	if(xp_ctx == None)
	{
	    iv_print_xprint_disconnect(dpy);
	    return(NULL);
	}

	/* Get the default resolution */
	if(XpuGetResolution(
	    dpy,
	    xp_ctx,
	    &x_dpi, &y_dpi
	))
	{
	    IVPrintXPrintResolution *r = iv_print_xprint_resolution_new();
	    if(r != NULL)
	    {
		r->x_dpi = (gint)x_dpi;
		r->y_dpi = (gint)y_dpi;
		r->dpi = MAX(r->x_dpi, r->y_dpi);
	    }
	    iv_print_xprint_context_delete(dpy, xp_ctx);
	    iv_print_xprint_disconnect(dpy);
	    return(r);
	}
	else
	{
	    iv_print_xprint_context_delete(dpy, xp_ctx);
	    iv_print_xprint_disconnect(dpy);
	    return(NULL);
	}
}


/*
 *	Gets the size of the medium.
 *
 *	The xp_address specifies the address to the X Print server. If
 *	xp_address is NULL then the address specified by the environment
 *	variable "XPSERVERLIST" will be used instead.
 *
 *	The printer_name specifies the name of the printer obtained
 *	from iv_print_xprint_get_printers_list(). This value may
 *	not be NULL.
 *
 *	The tray_name specifies the name of the tray obtained
 *	from iv_print_xprint_get_trays_list(). If tray_name is NULL
 *	then the tray will not be specified when obtaining the medium
 *	size from the X Print server.
 *
 *	The medium_name specifies the name of the medium obtained from
 *	iv_print_xprint_get_mediums(). If medium_name is NULL then the
 *	medium will not be specified when obtaining the medium size from
 *	the X Print server.
 *
 *	The orientation_name specifies the name of the orientation
 *	obtained from iv_print_xprint_get_orientations(). If
 *	orientation_name is NULL then the orientation will not be
 *	specified when obtaining the medium size from the X Print
 *	server.
 *
 *	The x_dpi and y_dpi specifies the resolution in dots per inch.
 *	If dpi is 0 or negative then the resolution will not be
 *	specified when obtaining the medium size from the X Print server.
 *
 *	Returns 0 on success or non-zero on error.
 */
gint iv_print_xprint_get_medium_size(
	const gchar *xp_address,
	const gchar *printer_name,
	const gchar *tray_name,
	const gchar *medium_name,
	const gchar *orientation_name,
	const gint x_dpi, const gint y_dpi,
	gint *width_rtn, gint *height_rtn
)
{
	int xp_event_base, xp_error_base;
	Display *dpy;
	unsigned short width, height;
	XRectangle rect;
	XPContext xp_ctx;

	if(width_rtn != NULL)
	    *width_rtn = 0;
	if(height_rtn != NULL)
	    *height_rtn = 0;

	if((printer_name == NULL) || (medium_name == NULL) ||
	   (orientation_name == NULL)
	)
	    return(-2);

	if(iv_print_xprint_connect(
	    &dpy,
	    &xp_event_base, &xp_error_base,
	    xp_address
	))
	    return(-1);

	xp_ctx = iv_print_xprint_context_new(
	    dpy,
	    printer_name
	);
	if(xp_ctx == None)
	{
	    iv_print_xprint_disconnect(dpy);
	    return(-1);
	}

#if 0
	/* Set the resolution for XPutImage() requests */
	XpSetImageResolution(dpy, xp_ctx, dpi, &resulting_dpi);
#endif

	/* Set the tray */
	if(!STRISEMPTY(tray_name))
	{
	    gchar *s = g_strconcat(
		"*default-input-tray: ",
		tray_name,
		NULL
	    );
	    XpSetAttributes(dpy, xp_ctx, XPDocAttr, s, XPAttrMerge);
	    g_free(s);
	}

	/* Set the medium */
	if(!STRISEMPTY(medium_name))
	{
	    gchar *s = g_strconcat(
		"*default-medium: ",
		medium_name,
		NULL
	    );
	    XpSetAttributes(dpy, xp_ctx, XPDocAttr, s, XPAttrMerge);
	    g_free(s);
	}

	/* Set the orientation */
	if(!STRISEMPTY(orientation_name))
	{
	    gchar *s = g_strconcat(
		"*content-orientation: ",
		orientation_name,
		NULL
	    );
	    XpSetAttributes(dpy, xp_ctx, XPDocAttr, s, XPAttrMerge);
	    g_free(s);
	}

	/* Set the resolution */
	if((x_dpi > 0) && (y_dpi > 0))
	{
#if 1
	    /* Use XpuSetDocResolution() to set the resolution so
	     * that we can have a wrapper to set a x and a y dpi
	     * resolution since some versions of the X Print server
	     * do not support a x and a y dpi resolution setting
	     */
	    XpuResolutionRec r;
	    r.name = g_strdup_printf(
		"%i",
		MAX(x_dpi, y_dpi)
	    );
	    r.x_dpi = (long)x_dpi;
	    r.y_dpi = (long)y_dpi;
	    XpuSetDocResolution(
		dpy,
		xp_ctx,
		&r
	    );
	    g_free((gchar *)r.name);
#else
	    gchar *s = g_strdup_printf(
		"*default-printer-resolution: %i",
		MAX(x_dpi, y_dpi)
	    );
	    XpSetAttributes(dpy, xp_ctx, XPDocAttr, s, XPAttrMerge);
	    g_free(s);
#endif
	}

	/* Get the resulting size of the medium in pixels */
	if(XpGetPageDimensions(
	    dpy,
	    xp_ctx,
	    &width, &height,
	    &rect
	))
	{
	    if(width_rtn != NULL)
		*width_rtn = (gint)rect.width;
	    if(height_rtn != NULL)
		*height_rtn = (gint)rect.height;

	    iv_print_xprint_context_delete(dpy, xp_ctx);
	    iv_print_xprint_disconnect(dpy);

	    return(0);
	}
	else
	{
	    iv_print_xprint_context_delete(dpy, xp_ctx);
	    iv_print_xprint_disconnect(dpy);

	    return(-1);
	}
}


/*
 *	X error callback.
 */
static int iv_print_x_error_cb(Display *dpy, XErrorEvent *error)
{
	gint buf_len;
	gchar *buf;

	if((dpy == NULL) || (error == NULL))
	    return(0);

	buf_len = 1024;
	buf = (gchar *)g_malloc((buf_len + 1) * sizeof(gchar));
	if(buf == NULL)
	    return(0);

	*buf = '\0';
	XGetErrorText(
	    dpy,
	    error->error_code,
	    (char *)buf, buf_len
	);

	g_printerr(
"X Error: %i  Code: %i (%s)\n",
	    error->type,
	    error->error_code,
	    buf
	);

	g_free(buf);

	return(0);
}

/*
 *	Progress callback.
 */
static gint iv_print_xprint_progress_cb(
	const gulong i, const gulong m, gpointer data
)
{
	IVPrintXPrintProgressData *d = IV_PRINT_XPRINT_PROGRESS_DATA(data);
	if(d == NULL)
	    return(ProgressDialogStopCount());

	if((m > 0l) && (d->nstages > 0))
	{
	    ProgressDialogUpdate(
		NULL, NULL, NULL, NULL,
		((gfloat)d->cur_stage / (gfloat)d->nstages) +
		    ((gfloat)i / (gfloat)m / (gfloat)d->nstages),
		IV_PROGRESS_BAR_DEF_TICKS,
		TRUE
	    );
	}
	else
	{
	    ProgressDialogUpdateUnknown(
		NULL, NULL, NULL, NULL,
		TRUE
	    );
	}
	return(ProgressDialogStopCount());
}


/*
 *	Creates an X Image from the image with resizing and color
 *	modifications applied.
 *
 *	The visual specifies the X Visual which must be of class
 *	TrueColor.
 *
 *	The img specifies the image.
 *
 *	The pv specifies the print values which will be used to resize
 *	and apply color modifications.
 *
 *	The bg_color specifies the background color. It must contain
 *	four bytes describing the RGBA values.
 *
 *	The progress_data specifies the IVPrintXPrintProgressData
 *	progress data which will be incremented 2 stages by this call.
 *
 *	Returns the X Image.
 */
static XImage *iv_print_xprint_render_ximage(
	Display *dpy, Visual *visual, const int depth,
	imgview_image_struct *img,
	const print_values_struct *pv,
	const guint8 *bg_color,
	gint (*progress_cb)(const gulong, const gulong, gpointer),
	IVPrintXPrintProgressData *progress_data
)
{
	XImage *ximg;
	imgview_frame_struct *frame;

	/* Render the output image */
	imgview_image_struct *output_img = iv_print_render_image_from_values(
	    img,
	    pv,
	    bg_color,
	    progress_cb,
	    progress_data
	);
	if(output_img == NULL)
	    return(NULL);

	/* Go on the next progress stage */
	progress_data->cur_stage++;

	/* The output image must be 3 bpp or greater in RGB* format */
	if(output_img->bpp < 3)
	{
	    ImgViewImageDelete(output_img);
	    return(NULL);
	}

	frame = ImgViewImageGetFrame(output_img, 0);
	if(frame == NULL)
	{
	    ImgViewImageDelete(output_img);
	    return(NULL);
	}                

	/* Create the X Image */
	ximg = XCreateImage(
	    dpy, visual, depth,
	    ZPixmap,
	    0,					/* X pixel offset */
	    NULL,
	    output_img->width, output_img->height,
	    BitmapPad(dpy),
	    0
	);
	if(ximg == NULL)
	{
	    ImgViewImageDelete(output_img);
	    return(NULL);
	}

	/* Allocate the X Image's data as needed */
	if((ximg->data == NULL) && (ximg->bytes_per_line > 0) &&
	   (ximg->height > 0)
	)
	    ximg->data = (char *)g_malloc(
		ximg->height * ximg->bytes_per_line
	    );
	if(ximg->data == NULL)
	{
	    XDestroyImage(ximg);
	    ImgViewImageDelete(output_img);
	    return(NULL);
	}

	/* Copy/convert the rendered image's data to the X Image */
	if(frame->data != NULL)
	{
	    gint y;
	    const gint	width = MIN(ximg->width, output_img->width),
			height = MIN(ximg->height, output_img->height),
			output_img_bpp = output_img->bpp,
			output_img_bpl = output_img->bpl,
			ximg_bpp = ximg->bits_per_pixel / 8,
			ximg_bpl = ximg->bytes_per_line;
	    gulong	ximg_red_mask = ximg->red_mask,
			ximg_green_mask = ximg->green_mask,
			ximg_blue_mask = ximg->blue_mask;
	    const guint8	*src_data = frame->data,
				*src_ptr,
				*src_end;
	    guint8 *tar_data = (guint8 *)ximg->data, *tar_ptr;
	    void (*pix_func)(guint8 *, const guint8 *);

	    /* Set the pixel function based on the X Image's color
	     * mask
	     */
	    if((ximg_red_mask   == 0xff0000) &&
	       (ximg_green_mask == 0x00ff00) &&
	       (ximg_blue_mask  == 0x0000ff)
	    )
	    {
		if(ximg_bpp == 4)
		    pix_func = pixel_set_rgb_to_bgra;
		else
		    pix_func = pixel_set_rgb_to_bgr;
	    }
	    else if((ximg_red_mask   == 0x0000ff) &&
		    (ximg_green_mask == 0x00ff00) &&
		    (ximg_blue_mask  == 0xff0000)
	    )
	    {
		if(ximg_bpp == 4)
		    pix_func = pixel_set_rgb_to_rgba;
		else
		    pix_func = pixel_set_rgb_to_rgb;
	    }
	    else
	    {
		if(ximg_bpp >= 2)
		    pix_func = pixel_set_rgb_to_grey_alpha;
		else
		    pix_func = pixel_set_rgb_to_grey;
	    }

	    /* Copy/convert the rendered image's data to the X Image */
	    for(y = 0; y < height; y++)
	    {
		src_ptr = src_data + (y * output_img_bpl);
		src_end = src_ptr + (width * output_img_bpp);
		tar_ptr = tar_data + (y * ximg_bpl);
		while(src_ptr < src_end)
		{
		    pix_func(tar_ptr, src_ptr);
		    src_ptr += output_img_bpp;
		    tar_ptr += ximg_bpp;
		}

		/* Report progress for this */
		if(progress_cb != NULL)
		{
		    if(progress_cb(
			(gulong)y + 1,
			(gulong)height,
			progress_data
		    ))
			break;
		}
	    }
	}

	/* Delete the rendered image */
	ImgViewImageDelete(output_img);

	/* Go on the next progress stage */
	progress_data->cur_stage++;

	return(ximg);
}

/*
 *	Print operation iteration.
 *
 *	Performs one iteration of the print operation by printing the
 *	page with the specified image. The X Window to print to,
 *	X Graphic Context, and the X Print job must already be
 *	created prior to calling this function.
 *
 *	The w specifies the X Window to render the image to.
 *
 *	The gc specifies the X Graphic Context to use to render the
 *	image and to put the image to the X Window.
 *
 *	The img specifies the image to print.
 *
 *	The pv specifies the print values.
 *
 *	The bg_color specifies the background color. It must contain
 *	four bytes describing the RGBA values.
 *
 *	The progress_cb and progress_data specifies the progress
 *	callback. The progress_data->cur_stage will be incremented
 *	5 stages by this call unless aborted.
 */
static gint iv_print_xprint_main_iteration(
	Display *dpy,
	Visual *visual,
	const int depth,
	Window w,
	GC gc,
	XPContext xp_ctx,
	const int xp_event_base, const int xp_error_base,
	imgview_image_struct *img,
	const print_values_struct *pv,
	const guint8 *bg_color,
	const gboolean verbose,
	const gboolean interactive,
	GtkWidget *toplevel,
	gint (*progress_cb)(const gulong, const gulong, gpointer),
	IVPrintXPrintProgressData *progress_data
)
{
	gboolean waiting;
	XImage *ximg;
	const print_value_flags pvflags = pv->flags;
	const gint	output_x = (pvflags & PRINT_VALUE_OUTPUT_X) ?
		pv->output_x : 0,
			output_y = (pvflags & PRINT_VALUE_OUTPUT_Y) ?
		pv->output_y : 0;

#define UPDATE_PROGRESS_MESSAGE(_msg_)	{		\
 if(verbose) {						\
  IVPrintXPrintProgressData *d = progress_data;		\
  ProgressDialogUpdate(					\
   NULL,						\
   (_msg_),						\
   NULL, NULL,						\
   ((gfloat)d->cur_stage / (gfloat)d->nstages) +	\
    (0.0f / (gfloat)d->nstages),			\
   IV_PROGRESS_BAR_DEF_TICKS,				\
   TRUE							\
  );							\
 }							\
}
#define UPDATE_PROGRESS(_coeff_)	{		\
 if(verbose) {						\
  IVPrintXPrintProgressData *d = progress_data;		\
  ProgressDialogUpdate(					\
   NULL, NULL, NULL, NULL,				\
   ((gfloat)d->cur_stage / (gfloat)d->nstages) +	\
    ((gfloat)(_coeff_) / (gfloat)d->nstages),		\
   IV_PROGRESS_BAR_DEF_TICKS,				\
   TRUE							\
  );							\
 }							\
}

	/* Start the page */
	progress_data->cur_stage++;		/* Stage 4 */
	UPDATE_PROGRESS_MESSAGE(
	    "Starting page..."
	);
	XpStartPage(dpy, w);

	/* Wait for the XPStartPageNotify XEvent from the X Print server */
	waiting = True;
	if(waiting)
	{
	    XEvent event;
	    do
	    {
		if(XPending(dpy) > 0)
		{
		    XNextEvent(dpy, &event);
		    if(event.type == (xp_event_base + XPPrintNotify))
		    {
			XPPrintEvent *pev = (XPPrintEvent *)&event;
			switch(pev->detail)
			{
			  case XPStartPageNotify:
			    waiting = False;
			    break;
			}
		    }
		}
		UPDATE_PROGRESS(0.0f);
		usleep(8000);
	    }
	    while(waiting);
	}
	if(verbose)
	{
	    if(ProgressDialogStopCount() > 0)
	    {
		XpCancelPage(dpy, xp_ctx);
		return(-4);
	    }
	}

	/* Create and render the image data to an X Image
	 *
	 * The progress data will be incremented 2 stages by this
	 * call (stages 6 and 7)
	 */
	progress_data->cur_stage++;		/* Stage 5 */
	UPDATE_PROGRESS_MESSAGE(
	    "Rendering the image for printing..."
	);
	ximg = iv_print_xprint_render_ximage(
	    dpy, visual, depth,
	    img,
	    pv,
	    bg_color,
	    progress_cb,
	    progress_data
	);
	if(ximg == NULL)
	{
	    /* Unable to create or render an X Image */
	    if(verbose)
	    {
		CDialogSetTransientFor(toplevel);
		CDialogGetResponse(
		    "Print Failed",
"Unable to render the image for printing.",
		    NULL,
		    CDIALOG_ICON_WARNING,
		    CDIALOG_BTNFLAG_OK,
		    CDIALOG_BTNFLAG_OK
		);
		CDialogSetTransientFor(NULL);
	    }
	    XpCancelPage(dpy, xp_ctx);
	    return(-1);
	}
	if(verbose)
	{
	    if(ProgressDialogStopCount() > 0)
	    {
		XpCancelPage(dpy, xp_ctx);
		XDestroyImage(ximg);
		return(-4);
	    }
	}

	/* Update progress message and check for abort */
	progress_data->cur_stage++;		/* Stage 8 */
	UPDATE_PROGRESS_MESSAGE(
	    "Sending the rendered image to the page..."
	);

	/* Set the resolution for XPutImage() operations */
	if(pvflags & PRINT_VALUE_RESOLUTION)
	{
	    if((pv->resolution_x_dpi > 0) && (pv->resolution_y_dpi > 0))
	    {
#if 0
/* Not needed since the image is already scaled */
		int prev_resolution;
		XpSetImageResolution(
		    dpy, xp_ctx,
		    MAX(pv->resolution_x_dpi, pv->resolution_y_dpi),
		    &prev_resolution
		);
#endif
	    }
	}

	/* Put the X Image to the X Window */
	XPutImage(
	    dpy, (Drawable)w, gc,
	    ximg,
	    0, 0,
	    (int)output_x, (int)output_y,
	    ximg->width, ximg->height
	);
	if(verbose)
	{
	    if(ProgressDialogStopCount() > 0)
	    {
		XpCancelPage(dpy, xp_ctx);
		XDestroyImage(ximg);
		return(-4);
	    }
	}

	/* End the page */
	XpEndPage(dpy);

	/* Wait for the XPEndPageNotify XEvent from the X Print server */
	waiting = True;
	if(waiting)
	{
	    XEvent event;
	    do
	    {
		if(XPending(dpy) > 0)
		{
		    XNextEvent(dpy, &event);
		    if(event.type == (xp_event_base + XPPrintNotify))
		    {
			XPPrintEvent *pev = (XPPrintEvent *)&event;
			switch(pev->detail)
			{
			  case XPEndPageNotify:
			    waiting = False;
			    break;
			}
		    }
		}
		UPDATE_PROGRESS(1.0f);
		usleep(8000);
	    }
	    while(waiting);
	}
	if(verbose)
	{
	    if(ProgressDialogStopCount() > 0)
	    {
		XDestroyImage(ximg);
		return(-4);
	    }
	}

	/* Delete the X Image */
	XDestroyImage(ximg);

#undef UPDATE_PROGRESS_MESSAGE
#undef UPDATE_PROGRESS

	return(0);
}

/*
 *	Performs the print operation.
 */
static gint iv_print_xprint_main(
	imgview_image_struct *img,
	const print_values_struct *pv,
	const guint8 *bg_color,
	const gboolean verbose,
	const gboolean interactive,
	GtkWidget *toplevel
)
{
	unsigned short	page_width, page_height;
	int		visual_class,
			depth,
			xp_event_base, xp_error_base;
	XRectangle page_rect;
	Colormap cmap;
	Screen *screen_ptr;
	Visual *visual;
	Window w, root;
	Display *dpy;
	XSetWindowAttributes w_set_attr;
	GC gc;
	XPContext xp_ctx;
	gboolean waiting;
	gint status;
	const print_value_flags pvflags = pv->flags;
	const gchar *xprint_server_address = (pvflags & PRINT_VALUE_XPRINT_SERVER_ADDRESS) ?
	    pv->xprint_server_address : NULL;
	const gchar *printer_name = (pvflags & PRINT_VALUE_PRINTER) ?
	    pv->printer : NULL;
	gint (*progress_cb)(const gulong, const gulong, gpointer);
	IVPrintXPrintProgressData *progress_data;

	/* Set the progress callback if verbose is TRUE */
	progress_cb = verbose ? iv_print_xprint_progress_cb : NULL;

	/* Allocate the progress data */
	progress_data = IV_PRINT_XPRINT_PROGRESS_DATA(g_malloc0(
	    sizeof(IVPrintXPrintProgressData)
	));
	if(progress_data == NULL)
	    return(-3);

	/* Set up the progress data for 10 stages */
	progress_data->nstages = 10;

#define UPDATE_PROGRESS_MESSAGE(_msg_)	{		\
 if(verbose) {						\
  IVPrintXPrintProgressData *d = progress_data;		\
  ProgressDialogUpdate(					\
   NULL,						\
   (_msg_),						\
   NULL, NULL,						\
   ((gfloat)d->cur_stage / (gfloat)d->nstages) +	\
    (0.0f / (gfloat)d->nstages),			\
   IV_PROGRESS_BAR_DEF_TICKS,				\
   TRUE							\
  );							\
 }							\
}
#define UPDATE_PROGRESS(_coeff_)	{		\
 if(verbose) {						\
  IVPrintXPrintProgressData *d = progress_data;		\
  ProgressDialogUpdate(					\
   NULL, NULL, NULL, NULL,				\
   ((gfloat)d->cur_stage / (gfloat)d->nstages) +	\
    ((gfloat)(_coeff_) / (gfloat)d->nstages),		\
   IV_PROGRESS_BAR_DEF_TICKS,				\
   TRUE							\
  );							\
 }							\
}

	/* Printer not specified? */
	if(STRISEMPTY(printer_name))
	{
	    if(verbose)
	    {
		CDialogSetTransientFor(toplevel);
		CDialogGetResponse(
		    "Print Failed",
		    "Printer not specified.",
		    NULL,
		    CDIALOG_ICON_WARNING,
		    CDIALOG_BTNFLAG_OK,
		    CDIALOG_BTNFLAG_OK
		);
		CDialogSetTransientFor(NULL);
	    }
	    g_free(progress_data);
	    return(-2);
	}

	/* Connect to the X Print server */
	progress_data->cur_stage = 0;		/* Stage 0 */
	UPDATE_PROGRESS_MESSAGE(
	    "Connecting to the X Print Server..."
	);
	if(iv_print_xprint_connect(
	    &dpy,
	    &xp_event_base, &xp_error_base,
	    xprint_server_address
	))
	{
	    if(verbose)
	    {
		gchar *msg = g_strdup_printf(
"Unable to connect to the X Print Server at address:\n\
\n\
    %s",
		    xprint_server_address
		);
		CDialogSetTransientFor(toplevel);
		CDialogGetResponse(
		    "Print Failed",
		    msg,
		    NULL,
		    CDIALOG_ICON_WARNING,
		    CDIALOG_BTNFLAG_OK,
		    CDIALOG_BTNFLAG_OK
		);
		CDialogSetTransientFor(NULL);
		g_free(msg);
	    }
	    g_free(progress_data);
	    return(-1);
	}

	/* Report progress and check for abort */
	if(verbose)
	{
	    UPDATE_PROGRESS(0.3f);
	    if(ProgressDialogStopCount() > 0)
	    {
		iv_print_xprint_disconnect(dpy);
		g_free(progress_data);
		return(-4);
	    }
	}

	/* Create an X Print context for the specified printer */
	xp_ctx = iv_print_xprint_context_new(
	    dpy,
	    printer_name
	);
	if(xp_ctx == None)                                  
	{
	    if(verbose)
	    {
		CDialogSetTransientFor(toplevel);
		CDialogGetResponse(
		    "Print Failed",
"Unable to create a new X Print context.",
		    NULL,
		    CDIALOG_ICON_WARNING,
		    CDIALOG_BTNFLAG_OK,
		    CDIALOG_BTNFLAG_OK
		);
		CDialogSetTransientFor(NULL);
	    }
	    iv_print_xprint_disconnect(dpy);
	    g_free(progress_data);
	    return(-1);
	}

	/* Report progress and check for abort */
	if(verbose)
	{
	    UPDATE_PROGRESS(0.6f);
	    if(ProgressDialogStopCount() > 0)
	    {
		iv_print_xprint_disconnect(dpy);
		g_free(progress_data);
		return(-4);
	    }
	}

	/* Set the print parameters */
	progress_data->cur_stage++;		/* Stage 1 */
	UPDATE_PROGRESS_MESSAGE(
	    "Setting the print parameters..."
	);

	/* Set the tray */
	if(pvflags & PRINT_VALUE_INPUT_TRAY)
	{
	    if(!STRISEMPTY(pv->input_tray))
	    {
		gchar *s = g_strconcat(
		    "*default-input-tray: ",
		    pv->input_tray,
		    NULL
		);
		XpSetAttributes(dpy, xp_ctx, XPDocAttr, s, XPAttrMerge);
		g_free(s);
	    }
	}
	if(verbose)
	{
	    UPDATE_PROGRESS(0.1f);
	    if(ProgressDialogStopCount() > 0)
	    {
		iv_print_xprint_context_delete(dpy, xp_ctx);
		iv_print_xprint_disconnect(dpy);
		g_free(progress_data);
		return(-4);
	    }
	}

	/* Set the medium */
	if(pvflags & PRINT_VALUE_MEDIUM)
	{
	    if(!STRISEMPTY(pv->medium))
	    {
		gchar *s = g_strconcat(
		    "*default-medium: ",
		    pv->medium,
		    NULL
		);
		XpSetAttributes(dpy, xp_ctx, XPDocAttr, s, XPAttrMerge);
		g_free(s);
	    }
	}
	if(verbose)
	{
	    UPDATE_PROGRESS(0.2f);
	    if(ProgressDialogStopCount() > 0)
	    {
		iv_print_xprint_context_delete(dpy, xp_ctx);
		iv_print_xprint_disconnect(dpy);
		g_free(progress_data);
		return(-4);
	    }
	}

	/* Set the orientation */
	if(pvflags & PRINT_VALUE_ORIENTATION)
	{
	    if(!STRISEMPTY(pv->orientation))
	    {
		gchar *s = g_strconcat(
		    "*content-orientation: ",
		    pv->orientation,
		    NULL
		);
		XpSetAttributes(dpy, xp_ctx, XPDocAttr, s, XPAttrMerge);
		g_free(s);
	    }
	}
	if(verbose)
	{
	    UPDATE_PROGRESS(0.3f);
	    if(ProgressDialogStopCount() > 0)
	    {
		iv_print_xprint_context_delete(dpy, xp_ctx);
		iv_print_xprint_disconnect(dpy);
		g_free(progress_data);
		return(-4);
	    }
	}

	/* Set the resolution */
	if(pvflags & PRINT_VALUE_RESOLUTION)
	{
	    if((pv->resolution_x_dpi > 0) && (pv->resolution_y_dpi > 0))
	    {
#if 1
		/* Use XpuSetDocResolution() to set the resolution so
		 * that we can have a wrapper to set a x and a y dpi
		 * resolution since some versions of the X Print server
		 * do not support a x and a y dpi resolution setting
		 */
		XpuResolutionRec r;
		r.name = g_strdup_printf(
		    "%i",
		    MAX(pv->resolution_x_dpi, pv->resolution_y_dpi)
		);
		r.x_dpi = (long)pv->resolution_x_dpi;
		r.y_dpi = (long)pv->resolution_y_dpi;
		XpuSetDocResolution(
		    dpy,
		    xp_ctx,
		    &r
		);
		g_free((gchar *)r.name);
#else
		gchar *s = g_strdup_printf(
		    "*default-printer-resolution: %i",
		    MAX(pv->resolution_x_dpi, pv->resolution_y_dpi)
		);
		XpSetAttributes(dpy, xp_ctx, XPDocAttr, s, XPAttrMerge);
		g_free(s);
#endif
	    }
	}
	if(verbose)
	{
	    UPDATE_PROGRESS(0.4f);
	    if(ProgressDialogStopCount() > 0)
	    {
		iv_print_xprint_context_delete(dpy, xp_ctx);
		iv_print_xprint_disconnect(dpy);
		g_free(progress_data);
		return(-4);
	    }
	}

	/* Set the number of coppies */
	if(pvflags & PRINT_VALUE_OUTPUT_NCOPPIES)
	{
	    if(pv->output_ncoppies > 0)
	    {
		gchar *s = g_strdup_printf(
		    "*copy-count: %i",
		    pv->output_ncoppies
		);
		XpSetAttributes(dpy, xp_ctx, XPDocAttr, s, XPAttrMerge);
		g_free(s);
	    }
	}
	if(verbose)
	{
	    UPDATE_PROGRESS(0.5f);
	    if(ProgressDialogStopCount() > 0)
	    {
		iv_print_xprint_context_delete(dpy, xp_ctx);
		iv_print_xprint_disconnect(dpy);
		g_free(progress_data);
		return(-4);
	    }
	}

	/* Monitor the printing procedure since the process is async */
	XpSelectInput(dpy, xp_ctx, XPPrintMask);
	if(verbose)
	{
	    UPDATE_PROGRESS(0.6f);
	    if(ProgressDialogStopCount() > 0)
	    {
		iv_print_xprint_context_delete(dpy, xp_ctx);
		iv_print_xprint_disconnect(dpy);
		g_free(progress_data);
		return(-4);
	    }
	}

	/* Set the print job title */
	if(pvflags & PRINT_VALUE_JOB_TITLE)
	{
	    if(pv->job_title != NULL)
	    {
		gchar *s = g_strconcat(
		    "*job-name: %s",
		    pv->job_title,
		    NULL
		);
		XpSetAttributes(dpy, xp_ctx, XPDocAttr, s, XPAttrMerge);
		g_free(s);
	    }
	}
	if(verbose)
	{
	    UPDATE_PROGRESS(0.7f);
	    if(ProgressDialogStopCount() > 0)
	    {
		iv_print_xprint_context_delete(dpy, xp_ctx);
		iv_print_xprint_disconnect(dpy);
		g_free(progress_data);
		return(-4);
	    }
	}

	/* Set the X print context */
	XpSetContext(dpy, xp_ctx);
	if(verbose)
	{
	    UPDATE_PROGRESS(0.8f);
	    if(ProgressDialogStopCount() > 0)
	    {
		iv_print_xprint_context_delete(dpy, xp_ctx);
		iv_print_xprint_disconnect(dpy);
		g_free(progress_data);
		return(-4);
	    }
	}

	/* Get the screen associated with this X print context */
	screen_ptr = XpGetScreenOfContext(dpy, xp_ctx);
	root = RootWindowOfScreen(screen_ptr);
	if(verbose)
	{
	    UPDATE_PROGRESS(0.9f);
	    if(ProgressDialogStopCount() > 0)
	    {
		iv_print_xprint_context_delete(dpy, xp_ctx);
		iv_print_xprint_disconnect(dpy);
		g_free(progress_data);
		return(-4);
	    }
	}

	/* Start the print job in spool mode */
	progress_data->cur_stage++;		/* Stage 2 */
	UPDATE_PROGRESS_MESSAGE(
	    "Starting print job..."
	);
	XpStartJob(dpy, XPSpool);

	/* Wait for the XPStartJobNotify XEvent from the XPrint server */
	waiting = True;
	if(waiting)
	{
	    XEvent event;
	    do
	    {
		if(XPending(dpy) > 0)
		{
		    XNextEvent(dpy, &event);
		    if(event.type == (xp_event_base + XPPrintNotify))
		    {
			XPPrintEvent *pev = (XPPrintEvent *)&event;
			switch(pev->detail)
			{
			  case XPStartJobNotify:
			    waiting = False;
			    break;
			}
		    }
		}
		UPDATE_PROGRESS(0.3f);
		usleep(8000);
	    }
	    while(waiting);
	}
	if(verbose)
	{
	    if(ProgressDialogStopCount() > 0)
	    {
		XpCancelJob(dpy, xp_ctx);
		iv_print_xprint_context_delete(dpy, xp_ctx);
		iv_print_xprint_disconnect(dpy);
		g_free(progress_data);
		return(-4);
	    }
	}

#if 0
	/* If printing multiple pages then XpStartDoc() needs to be
	 * called before XpStartPage(), otherwise XpStartPage()
	 * automatically calls XpStarPage() when printing a single
	 * document
	 */
	XpStartDoc(dpy, XPDocNormal);

	/* Wait for the XPStartDocNotify XEvent from the XPrint server */
	waiting = True;
	if(waiting)
	{
	    XEvent event;
	    do
	    {
		if(XPending(dpy) > 0)
		{
		    XNextEvent(dpy, &event);
		    if(event.type == (xp_event_base + XPPrintNotify))
		    {
			XPPrintEvent *pev = (XPPrintEvent *)&event;
			switch(pev->detail)
			{
			  case XPStartDocNotify:
			    waiting = False;
			    break;
			}
		    }
		}
		UPDATE_PROGRESS(0.7f);
		usleep(8000);
	    }
	    while(waiting);
	}
	if(verbose)
	{
	    if(ProgressDialogStopCount() > 0)
	    {
		XpCancelDoc(dpy, TRUE);
		XpCancelJob(dpy, xp_ctx);
		iv_print_xprint_context_delete(dpy, xp_ctx);
		iv_print_xprint_disconnect(dpy);
		g_free(progress_data);
		return(-4);
	    }
	}
#endif

	/* Get the page dimensions in pixels */
	if(!XpGetPageDimensions(  
	    dpy, xp_ctx,
	    &page_width, &page_height,
	    &page_rect
	))
	{
	    if(verbose)
	    {
		CDialogSetTransientFor(toplevel);
		CDialogGetResponse(
		    "Print Failed",
"Unable to get the page dimensions from the X Print Server.",
		    NULL,
		    CDIALOG_ICON_WARNING,
		    CDIALOG_BTNFLAG_OK,
		    CDIALOG_BTNFLAG_OK
		);
		CDialogSetTransientFor(NULL);
	    }
	    XpCancelJob(dpy, xp_ctx);
	    iv_print_xprint_context_delete(dpy, xp_ctx);
	    iv_print_xprint_disconnect(dpy);
	    g_free(progress_data);
	    return(-1);
	}

	/* Get the best X Visual for printing */
	visual_class = TrueColor;
	if(pvflags & PRINT_VALUE_VISUAL)
	{
	    switch(pv->visual)
	    {
	      case PRINT_VISUAL_RGB:
		visual_class = TrueColor;
		break;
	      case PRINT_VISUAL_GREYSCALE:
		visual_class = GrayScale;
		break;
	      case PRINT_VISUAL_BW:
		visual_class = GrayScale;
		break;
	    }
	}
	visual = iv_print_xprint_get_best_visual(
	    dpy,
	    visual_class,			/* X Visual class */
	    &depth
	);
	if(visual == NULL)
	{
	    if(verbose)
	    {
		CDialogSetTransientFor(toplevel);
		CDialogGetResponse(
		    "Print Failed",
"Unable to obtain a suitable X Visual on the X Print\n\
Server for printing.",
		    NULL,
		    CDIALOG_ICON_WARNING,
		    CDIALOG_BTNFLAG_OK,
		    CDIALOG_BTNFLAG_OK
		);
		CDialogSetTransientFor(NULL);
	    }
	    XpCancelJob(dpy, xp_ctx);
	    iv_print_xprint_context_delete(dpy, xp_ctx);
	    iv_print_xprint_disconnect(dpy);
	    g_free(progress_data);
	    return(-1);
	}

	/* Update the progress dialog message */
	progress_data->cur_stage++;		/* Stage 3 */
	UPDATE_PROGRESS_MESSAGE(
	    "Creating print resources..."
	);

	/* Create the Colormap */
	cmap = XCreateColormap(
	    dpy,
	    root,
	    visual,
	    AllocNone
	);
	if(verbose)
	{
	    UPDATE_PROGRESS(0.25f);
	    if(ProgressDialogStopCount() > 0)
	    {
		XpCancelJob(dpy, xp_ctx);
		iv_print_xprint_context_delete(dpy, xp_ctx);
		XFreeColormap(dpy, cmap);
		iv_print_xprint_disconnect(dpy);
		g_free(progress_data);
		return(-4);
	    }
	}

	/* Create the X Window to render the page (image) to */
	w_set_attr.background_pixel = iv_print_xprint_get_pixel_rgb(
	    dpy, cmap,
	    0xff, 0xff, 0xff
	);
	w_set_attr.border_pixel = w_set_attr.background_pixel;
	w_set_attr.colormap = cmap;
	w = XCreateWindow(
	    dpy,
	    root,
	    page_rect.x, page_rect.y,
	    page_rect.width, page_rect.height,
	    0,					/* No border */
	    depth,
	    CopyFromParent,			/* IO Type */
	    visual,
	    CWBackPixel | CWBorderPixel | CWColormap,
	    &w_set_attr
	);
	if(w == None)
	{
	    if(verbose)
	    {
		CDialogSetTransientFor(toplevel);
		CDialogGetResponse(
		    "Print Failed",
"Unable to create a new X Window for rendering the\n\
image to on the X Print Server.",
		    NULL,
		    CDIALOG_ICON_WARNING,
		    CDIALOG_BTNFLAG_OK,
		    CDIALOG_BTNFLAG_OK
		);
		CDialogSetTransientFor(NULL);
	    }
	    XpCancelJob(dpy, xp_ctx);
	    iv_print_xprint_context_delete(dpy, xp_ctx);
	    XFreeColormap(dpy, cmap);
	    iv_print_xprint_disconnect(dpy);
	    g_free(progress_data);
	    return(-1);
	}
	if(verbose)
	{
	    UPDATE_PROGRESS(0.5f);
	    if(ProgressDialogStopCount() > 0)
	    {
		XpCancelJob(dpy, xp_ctx);
		iv_print_xprint_context_delete(dpy, xp_ctx);
		XDestroyWindow(dpy, w);
		XFreeColormap(dpy, cmap);
		iv_print_xprint_disconnect(dpy);
		g_free(progress_data);
		return(-4);
	    }
	}

	/* Create the graphic context */
	gc = XCreateGC(
	    dpy,
	    (Drawable)w,
	    0l,
	    NULL
	);
	if(gc == None)
	{
	    if(verbose)
	    {
		CDialogSetTransientFor(toplevel);
		CDialogGetResponse(
		    "Print Failed",
"Unable to create a new X Graphic Context for\n\
rendering the image with on the X Print Server.",
		    NULL,
		    CDIALOG_ICON_WARNING,
		    CDIALOG_BTNFLAG_OK,
		    CDIALOG_BTNFLAG_OK
		);
		CDialogSetTransientFor(NULL);
	    }
	    XpCancelJob(dpy, xp_ctx);
	    iv_print_xprint_context_delete(dpy, xp_ctx);
	    XDestroyWindow(dpy, w);
	    XFreeColormap(dpy, cmap);
	    iv_print_xprint_disconnect(dpy);
	    return(-1);
	}
	if(verbose)
	{
	    UPDATE_PROGRESS(0.75f);
	    if(ProgressDialogStopCount() > 0)
	    {
		XpCancelJob(dpy, xp_ctx);
		iv_print_xprint_context_delete(dpy, xp_ctx);
		XFreeGC(dpy, gc);
		XDestroyWindow(dpy, w);
		XFreeColormap(dpy, cmap);
		iv_print_xprint_disconnect(dpy);
		g_free(progress_data);
		return(-4);
	    }
	}

	/* Map the Window */
	XMapWindow(dpy, w);

	/* Render the image to the XPrint Window
	 *
	 * The progress data will be incremented 5 stages by this call
	 */
	status = iv_print_xprint_main_iteration(
	    dpy,
	    visual,
	    depth,
	    w,
	    gc,
	    xp_ctx,
	    xp_event_base, xp_error_base,
	    img,
	    pv,
	    bg_color,
	    verbose,
	    interactive,
	    toplevel,
	    progress_cb, progress_data
	);
	if(status != 0)
	{
	    XpCancelJob(dpy, xp_ctx);
	    iv_print_xprint_context_delete(dpy, xp_ctx);
	    XFreeGC(dpy, gc);
	    XDestroyWindow(dpy, w);
	    XFreeColormap(dpy, cmap);
	    iv_print_xprint_disconnect(dpy);
	    g_free(progress_data);
	    return(status);
	}

#if 0
	/* If printing multiple pages then XpEndDoc() needs to be
	 * called after the last XpEndPage()
	 */
	XpEndDoc(dpy);

	/* Wait for the XPEndDocNotify XEvent from the XPrint server */
	waiting = True;
	if(waiting)
	{
	    XEvent event;
	    do
	    {
		if(XPending(dpy) > 0)
		{
		    XNextEvent(dpy, &event);
		    if(event.type == (xp_event_base + XPPrintNotify))
		    {
			XPPrintEvent *pev = (XPPrintEvent *)&event;
			switch(pev->detail)
			{
			  case XPEndDocNotify:
			    waiting = False;
			    break;
			}
		    }
		}
		UPDATE_PROGRESS(1.0f);
		usleep(8000);
	    }
	    while(waiting);
	}
	if(verbose)
	{
	    if(ProgressDialogStopCount() > 0)
	    {
		XpCancelJob(dpy, xp_ctx);
		iv_print_xprint_context_delete(dpy, xp_ctx);
		XFreeGC(dpy, gc);
		XDestroyWindow(dpy, w);
		XFreeColormap(dpy, cmap);
		iv_print_xprint_disconnect(dpy);
		g_free(progress_data);
		return(-4);
	    }
	}
#endif

	/* End the print job */
	progress_data->cur_stage++;		/* Stage 9 */
	UPDATE_PROGRESS_MESSAGE(
	    "Sending the print job to the printer..."
	);
	XpEndJob(dpy);

	/* Wait for the XPEndJobNotify XEvent from the XPrint server */
	waiting = True;
	if(waiting)
	{
	    XEvent event;
	    do
	    {
		if(XPending(dpy) > 0)
		{
		    XNextEvent(dpy, &event);
		    if(event.type == (xp_event_base + XPPrintNotify))
		    {
			XPPrintEvent *pev = (XPPrintEvent *)&event;
			switch(pev->detail)
			{
			  case XPEndJobNotify:
			    waiting = False;
			    break;
			}
		    }
		}
		UPDATE_PROGRESS(0.0f);
		usleep(8000);
	    }
	    while(waiting);
	}
	if(verbose)
	{
	    if(ProgressDialogStopCount() > 0)
	    {
		iv_print_xprint_context_delete(dpy, xp_ctx);
		XFreeGC(dpy, gc);
		XDestroyWindow(dpy, w);
		XFreeColormap(dpy, cmap);
		iv_print_xprint_disconnect(dpy);
		g_free(progress_data);
		return(-4);
	    }
	}

	/* Delete the X Print context and disconnect from the X Print
	 * server
	 */
	iv_print_xprint_context_delete(dpy, xp_ctx);
	XFreeGC(dpy, gc);
	XDestroyWindow(dpy, w);
	XFreeColormap(dpy, cmap);
	iv_print_xprint_disconnect(dpy);

	if(verbose)
	{
	    UPDATE_PROGRESS(1.0f);
	    if(ProgressDialogStopCount() > 0)
	    {
		g_free(progress_data);
		return(-4);
	    }
	}

#undef UPDATE_PROGRESS_MESSAGE
#undef UPDATE_PROGRESS

	/* Delete the progress data */
	g_free(progress_data);

	return(status);
}

/*
 *	Prints the image.
 *
 *	The img specifies the image to print.
 *
 *	The pv specifies the print values that will be used as the
 *	parameters for printing the image.
 *
 *	The bg_color specifies the background color. It must contain
 *	four bytes describing the RGBA values.
 *
 *	The toplevel specifies the reference toplevel GtkWindow.
 *
 *      Returns:
 *
 *	0	Success.
 *	-1	General error.
 *	-2	Invalid value.
 *	-3	Systems error.
 *	-4	User aborted.
 */
#ifdef HAVE_LIBENDEAVOUR2
gint iv_print(
	imgview_image_struct *img,		/* Image to print */
	const print_values_struct *pv,		/* Print values */
	const guint8 *bg_color,			/* 4 bytes RGBA */
	GtkWidget *toplevel,
	EDVContext *edv2_ctx
)
#else
gint iv_print(
	imgview_image_struct *img,		/* Image to print */
	const print_values_struct *pv,		/* Print values */
	const guint8 *bg_color,			/* 4 bytes RGBA */
	GtkWidget *toplevel 
)
#endif
{
	gint status;

	if((img == NULL) || (pv == NULL))
	    return(-2);

	if(ProgressDialogIsQuery())
	    ProgressDialogBreakQuery(TRUE);
	ProgressDialogSetTransientFor(toplevel);
	ProgressDialogMap(
	    "Printing",
	    "Printing image...",
	    (const guint8 **)icon_print2_32x32_xpm,
	    "Stop"
	);
	gdk_flush();

	status = iv_print_xprint_main(
	    img,
	    pv,
	    bg_color,
	    TRUE,			/* Verbose */
	    TRUE,			/* Interactive */
	    toplevel
	);

	ProgressDialogBreakQuery(TRUE);
	ProgressDialogSetTransientFor(NULL);

	return(status);
}


#endif	/* HAVE_XPRINT */
