#include <stdlib.h>
#include <gtk/gtk.h>
#include <gdk/gdkkeysyms.h>

#include "guiutils.h"
#include "save_options_dlg.h"
#include "config.h"

#include "images/icon_save_20x20.xpm"
#include "images/icon_save_32x32.xpm"
#include "images/icon_save_48x48.xpm"
#include "images/icon_cancel_20x20.xpm"


static gint delete_event_cb(GtkWidget *widget, GdkEvent *event, gpointer data);
static void save_btn_cb(GtkWidget *widget, gpointer data);
static void cancel_btn_cb(GtkWidget *widget, gpointer data);

static void toggle_set_widget_sensitive_cb(GtkWidget *widget, gpointer data);
static void toggle_set_widget_insensitive_cb(GtkWidget *widget, gpointer data);
static void toggle_set_toggle_active_cb(GtkWidget *widget, gpointer data);
/* static void toggle_set_toggle_inactive_cb(GtkWidget *widget, gpointer data); */
static void scale_changed_cb(GtkAdjustment *adjustment, gpointer data);
static void entry_changed_adj_cb(GtkWidget *widget, gpointer data);

static void build_std_dialog(
	gint width, gint height,
	const gchar *title,
	GtkWidget *ref_toplevel,
	GtkWidget **toplevel_rtn,
	GtkWidget **client_vbox_rtn,
	GtkAccelGroup **accelgrp_rtn,
	gboolean *got_response
);

/* GIF */
gint iv_save_options_gif_query(
	GtkWidget *ref_toplevel,
	const gchar *path,
	gboolean *interlaced,			/* FALSE = Progressive
						 * TRUE = Interlaced */
	gint *color_type,			/* 0 = Black & White 
						 * 1 = Greyscale
						 * 2 = Color Index */
	gboolean *transparency,			/* FALSE = No transparent color
						 * TRUE = Include transparent color */
	guint8 *alpha_threshold,		/* 0x00 to 0xFF */
	gboolean *looping			/* FALSE = No looping block 
						 * TRUE = Write looping block */
);

/* JPEG */
gint iv_save_options_jpeg_query(
	GtkWidget *ref_toplevel,
	const gchar *path,
	gfloat *quality,			/* 0.0 to 1.0 */
	gboolean *is_color			/* FALSE = Greyscale
						 * TRUE = Color */
);

/* JNG */
gint iv_save_options_jng_query(
	GtkWidget *ref_toplevel,
	const gchar *path,
	gfloat *quality,                        /* 0.0 to 1.0 */
	gboolean *is_color,			/* FALSE = Greyscale [Alpha]
						 * TRUE = RGB[A] */
	gboolean *transparency			/* FALSE = No alpha channel
						 * TRUE = Alpha channel */
);

/* MNG */
gint iv_save_options_mng_query(
	GtkWidget *ref_toplevel,
	const gchar *path,
	gint *compression,			/* 0 to 9 */
	gint *color_type,			/* 0 = Black & White 
						 * 1 = Greyscale
						 * 2 = Greyscale Alpha
						 * 3 = RGB
						 * 4 = RGBA */
	gboolean *interlaced,			/* FALSE = Progressive
						 * TRUE = Interlaced */
	gboolean *looping			/* FALSE = Play once
						 * TRUE = Repeating */
);

/* PNG */
gint iv_save_options_png_query(
	GtkWidget *ref_toplevel,
	const gchar *path,
	gint *compression,			/* 0 to 9 */
	gint *color_type,			/* 0 = Black & White
						 * 1 = Greyscale
						 * 2 = Greyscale Alpha
						 * 3 = RGB
						 * 4 = RGBA */
	gboolean *interlaced			/* FALSE = Progressive
						 * TRUE = Interlaced */
);

/* TGA */
gint iv_save_options_targa_query(
	GtkWidget *ref_toplevel,
	const gchar *path,
	gint *color_type			/* 0 = Greyscale
						 * 1 = RGB
						 * 2 = RGBA */
);

/* TIFF */
gint iv_save_options_tiff_query(
	GtkWidget *ref_toplevel,
	const gchar *path,
	gint *compression_type,			/* 0 = None
                                                 * 1 = GZIP
                                                 * 2 = JPEG
                                                 * 3 = CCITT RLE
                                                 * 4 = CCITT Group 3 Fax
                                                 * 5 = CCITT Group 4 Fax */
	gfloat *jpeg_quality,			/* 0.0 to 1.0 */
	gint *color_type			/* 0 = Black & White
						 * 1 = Greyscale
						 * 2 = Greyscale Alpha
						 * 3 = RGB
						 * 4 = RGBA */
);

/* XPM */
gint iv_save_options_xpm_query(
	GtkWidget *ref_toplevel,
	const gchar *path,
	gchar **c_id,
	gint *color_type,			/* 0 = Black & White
						 * 1 = Greyscale
						 * 2 = Color Index */
	gint *max_colors,			/* -1 for no limit */
	guint8 *alpha_threshold			/* 0x00 to 0xFF */
);


#define ATOI(s)         (((s) != NULL) ? atoi(s) : 0)
#define ATOL(s)         (((s) != NULL) ? atol(s) : 0)
#define ATOF(s)         (((s) != NULL) ? atof(s) : 0.0f)
#define STRDUP(s)       (((s) != NULL) ? g_strdup(s) : NULL)

#define MAX(a,b)        (((a) > (b)) ? (a) : (b))
#define MIN(a,b)        (((a) < (b)) ? (a) : (b))
#define CLIP(a,l,h)     (MIN(MAX((a),(l)),(h)))
#define ABSOLUTE(x)     (((x) < 0) ? ((x) * -1) : (x))
#define STRLEN(s)       (((s) != NULL) ? strlen(s) : 0)
#define STRISEMPTY(s)   (((s) != NULL) ? (*(s) == '\0') : TRUE)


/* Tooltip messages */

#define TOOLTIP_GIF_BW			"Check this to save the image\
 as a black and white image with a colormap (palette) consisting of\
 3 colors, black and white plus one additional color for both the\
 background and (if transparency is checked) the transparent color."
#define TOOLTIP_GIF_GREYSCALE		"Check this to save the image\
 as a greyscale image with a colormap (palette) consisting of 256\
 shades of grey, one of the 256 shades of grey will be used as both the\
 background and (if transparency is checked) the transparent color."
#define TOOLTIP_GIF_CI			"Check this to save the image\
 as a pseudo color (color-index) image with a colormap (palette)\
 consisting of 256 colors, one of the 256 colors will be used as both the\
 background and (if transparency is checked) the transparent color.\
 The GIF format does not support RGB or any higher color resolution storage."
#define TOOLTIP_GIF_INTERLACED		"Check this to save the image\
 using interlacing (no data will be discarded)"
#define TOOLTIP_GIF_TRANSPARENCY	"Check this to save the image\
 with transparent pixels. This will use up one color entry in the\
 palette as the transparent color"
#define TOOLTIP_GIF_LOOPING		"Check this to save the image\
 with the Netscape looping control block (\"NETSCAPE2.0\"), in which case\
 compliant viewers should replay this animation infinately (if the\
 image has only one frame then this has no affect)"
#define TOOLTIP_GIF_ALPHA_THRESHOLD	"Select the alpha value in\
 which alpha values that are less than this value will be made fully\
 transparent and alpha values greater than or equal to this value will\
 be made fully opaque"

#define TOOLTIP_JPEG_QUALITY		"Select the percentage of data\
 retained, higher percentages improve the quality and reduce the amount\
 of data discarded but increases the file size"

#define TOOLTIP_JNG_QUALITY		"Select the percentage of data\
 retained, higher percentages improve the quality and reduce the amount\
 of data discarded but increases the file size"
#define TOOLTIP_JNG_TRANSPARENCY	"Check this to save the image\
 with transparent pixels. This will add an additional alpha mask\
 image to the file"

#define TOOLTIP_MNG_COMPRESSION		"Select the level of\
 compression, the optimal compression level is 6 and the maximum\
 compression level is 9. MNG compression is loss-less so it never\
 discards data"
#define TOOLTIP_MNG_ALPHA		"Check this to save the alpha\
 channel, the alpha channel is used to store MNG transparency and\
 blending information"
#define TOOLTIP_MNG_INTERLACED		"Check this to save the image\
 using Adam 7 interlacing (no data will be discarded)"
#define TOOLTIP_MNG_LOOPING		"Check this to save the image\
 with the MNG TERM chunk's repeat count set to infinite (0x7FFFFFFF),\
 in which case compliant viewers should replay this animation infinately\
 (if the image has only one frame then this has no affect)"

#define TOOLTIP_PNG_COMPRESSION		"Select the level of\
 compression, the optimal compression level is 6 and the maximum\
 compression level is 9. PNG compression is loss-less so it never\
 discards data"
#define TOOLTIP_PNG_ALPHA		"Check this to save the alpha\
 channel, the alpha channel is used to store PNG transparency and\
 blending information"
#define TOOLTIP_PNG_INTERLACED		"Check this to save the image\
 using Adam 7 interlacing (no data will be discarded)"

#define TOOLTIP_TIFF_GZIP		"Z Library (libz) Deflate (non-lossy)"
#define TOOLTIP_TIFF_JPEG		"Joint Photographic Experts\
 Group DCT encoding. This compression method is lossy, meaning that\
 some degree of information may be discarded each time you save the data."
#define TOOLTIP_TIFF_CCITT_RLE		"CCITT modified Huffman Run-Length\
 Encoding (Black & White only)"
#define TOOLTIP_TIFF_CCITT_FAX3		"CCITT Group 3 fax encoding\
 (Black & White only)"
#define TOOLTIP_TIFF_CCITT_FAX4		"CCITT Group 4 fax encoding\
 (Black & White only)"
#define TOOLTIP_TIFF_JPEG_QUALITY	"Select the percentage of data\
 retained for JPEG compression, higher percentages improve the quality\
 and reduce the amount of data discarded but increases the file size"
#define TOOLTIP_TIFF_ALPHA		"Check this to save the alpha\
 channel, the alpha channel is used to store TIFF transparency and\
 blending information"

#define TOOLTIP_XPM_C_ID		"Enter the C identifier for\
 this XPM image, the C identifier is used to refer to this XPM image\
 when it is used in C or C++ source code with the #include directive,\
 a typical C identifier should consist of this XPM image's file name\
 with the '.' character replaced with a '_' character, for example,\
 the C identifier for \"myimage.xpm\" should be \"myimage_xpm\""
#define TOOLTIP_XPM_MAX_COLORS		"Enter the maximum number of\
 colors that will be generated for this XPM image's colormap, the\
 actual number of colors generated may still be less if this XPM\
 image does not have a diverse enough amount of colors"
#define TOOLTIP_XPM_NO_MAX_COLORS	"Check this to generate as\
 many colors as needed in order to preserve all the colors from the\
 original image, if the XPM image has a diverse amount of colors\
 then this may generate a very large number of colors and thus\
 increase the file size"
#define TOOLTIP_XPM_THRESHOLD		"Set the alpha threshold,\
 the alpha threshold determines which colors will be solid and\
 which colors will be transparent. Colors with an alpha value less\
 than the threshold will be completely transparent"


/*
 *	Closes the given toplevel window given by data and breaks
 *	the highest gtk_main level.
 */
static gint delete_event_cb(GtkWidget *widget, GdkEvent *event, gpointer data)
{
	gboolean *got_response = (gboolean *)data;
	if(got_response != NULL)
		*got_response = FALSE;

	gtk_main_quit();

	return(TRUE);
}

/*
 *	Save button callback.
 */
static void save_btn_cb(GtkWidget *widget, gpointer data)
{
	gboolean *got_response = (gboolean *)data;
	if(got_response != NULL)
		*got_response = TRUE;

	gtk_main_quit();
}

/*
 *	Cancel button callback.
 */
static void cancel_btn_cb(GtkWidget *widget, gpointer data)
{
	gboolean *got_response = (gboolean *)data;
	if(got_response != NULL)
		*got_response = FALSE;

	gtk_main_quit();
}


/*
 *	GtkToggleButton "toggled" callback.
 *
 *	Sets the GtkWidget specified by data as sensitive if the
 *	GtkToggleButton is active.
 */
static void toggle_set_widget_sensitive_cb(GtkWidget *widget, gpointer data)
{
	gtk_widget_set_sensitive(
		(GtkWidget *)data,
		(GTK_TOGGLE_BUTTON(widget)->active) ? TRUE : FALSE
	);
}

/*
 *	GtkToggleButton "toggled" callback.
 *
 *	Sets the GtkWidget specified by data as insensitive if the
 *	GtkToggleButton is active.
 */
static void toggle_set_widget_insensitive_cb(GtkWidget *widget, gpointer data)
{
	gtk_widget_set_sensitive(
		(GtkWidget *)data,
		(GTK_TOGGLE_BUTTON(widget)->active) ? FALSE : TRUE
	);
}

/*
 *	GtkToggleButton "toggled" callback.
 *
 *	Sets the GtkWidget specified by data as sensitive if the
 *	GtkToggleButton is active.
 */
static void toggle_set_toggle_active_cb(GtkWidget *widget, gpointer data)
{
	if(GTK_TOGGLE_BUTTON(widget)->active)
		gtk_toggle_button_set_active(
			(GtkToggleButton *)data,
			TRUE
		);
}

#if 0
/*
 *	GtkToggleButton "toggled" callback.
 *
 *	Sets the GtkWidget specified by data as insensitive if the
 *	GtkToggleButton is active.
 */
static void toggle_set_toggle_inactive_cb(GtkWidget *widget, gpointer data)
{
	if(GTK_TOGGLE_BUTTON(widget)->active)
		gtk_toggle_button_set_active(
			(GtkToggleButton *)data,
			FALSE
		);
}
#endif

/*
 *	GtkAdjustment "value_changed" signal callback.
 *
 *	Updates the GtkWidget's value specified by data to the value of
 *	the GtkAdjustment.
 */
static void scale_changed_cb(GtkAdjustment *adjustment, gpointer data)
{
	GtkWidget *w = (GtkWidget *)data;
	if(GTK_IS_ENTRY(w))
	{
		gchar	*s,
					num_str[80];
		GtkEntry *entry = GTK_ENTRY(w);
		g_snprintf(
			num_str, sizeof(num_str),
			"%.0f",
			GTK_ADJUSTMENT_GET_VALUE(adjustment)
		);

		s = gtk_entry_get_text(entry);
		if(s != NULL)
		{
			if(strcmp((const char *)s, (const char *)num_str))
				gtk_entry_set_text(entry, num_str);
		}
		else
		{
			gtk_entry_set_text(entry, num_str);
		}
	}
}

/*
 *	GtkEntry "changed" signal callback/
 *
 *	Updates the GtkAdjustment specified by data to the value of
 *	the GtkEntry.
 */
static void entry_changed_adj_cb(GtkWidget *widget, gpointer data)
{
	const gchar *s = gtk_entry_get_text(GTK_ENTRY(widget));
	if(s != NULL)
		gtk_adjustment_set_value(
			(GtkAdjustment *)data,
			ATOF(s)
		);
}


/*
 *	Builds a standard toplevel dialog and returns the pointers to
 *	the created widgets.
 */
static void build_std_dialog(
	gint width, gint height,
	const gchar *title,
	GtkWidget *ref_toplevel,
	GtkWidget **toplevel_rtn,
	GtkWidget **client_vbox_rtn,
	GtkAccelGroup **accelgrp_rtn,
	gboolean *got_response
)
{
	const gint	bw = GUI_BUTTON_HLABEL_WIDTH_DEF,
					bh = GUI_BUTTON_HLABEL_HEIGHT_DEF,
					border_major = 5;
	GdkWindow *window;
	GtkAccelGroup *accelgrp;
	GtkWidget	*w,
					*parent, *parent2,
					*toplevel,
					*main_vbox;

	/* Create the keyboard accelerator group */
	accelgrp = gtk_accel_group_new();
	if(accelgrp_rtn != NULL)
		*accelgrp_rtn = accelgrp;

	/* Create toplevel */
	toplevel = parent = w = gtk_window_new(GTK_WINDOW_DIALOG);
	gtk_widget_set_usize(w, width, height);
	gtk_window_set_title(GTK_WINDOW(w), title);
#ifdef PROG_NAME
	gtk_window_set_wmclass(
		GTK_WINDOW(w), "dialog", PROG_NAME
	);
#endif
	gtk_widget_realize(w);
	window = w->window;
	if(window != NULL)
	{
		gdk_window_set_decorations(
			window,
			GDK_DECOR_BORDER | GDK_DECOR_TITLE
		);
		gdk_window_set_functions(
			window,
			GDK_FUNC_MOVE | GDK_FUNC_CLOSE
		);
		GUISetWMIcon(window, (guint8 **)icon_save_48x48_xpm);
	}
	gtk_signal_connect(
		GTK_OBJECT(w), "delete_event",
		GTK_SIGNAL_FUNC(delete_event_cb), got_response
	);
	gtk_accel_group_attach(accelgrp, GTK_OBJECT(w));
	if(ref_toplevel != NULL)
	{
		if(GTK_IS_WINDOW(ref_toplevel))
		{
			gtk_window_set_modal(GTK_WINDOW(toplevel), TRUE);
			gtk_window_set_transient_for(
				GTK_WINDOW(toplevel),
				GTK_WINDOW(ref_toplevel)
			);
		}
	}
	if(toplevel_rtn != NULL)
		*toplevel_rtn = toplevel;

	main_vbox = w = gtk_vbox_new(FALSE, 0);
	gtk_container_add(GTK_CONTAINER(parent), w);
	gtk_widget_show(w);
	parent = w;

	w = gtk_vbox_new(FALSE, border_major);
	gtk_container_border_width(GTK_CONTAINER(w), border_major);
	gtk_box_pack_start(GTK_BOX(parent), w, TRUE, TRUE, 0);
	gtk_widget_show(w);
	parent2 = w;
	if(client_vbox_rtn != NULL)
		*client_vbox_rtn = w;

	w = gtk_hseparator_new();
	gtk_box_pack_start(GTK_BOX(main_vbox), w, FALSE, FALSE, 0);
	gtk_widget_show(w);

	/* Buttons */
	w = gtk_hbox_new(TRUE, 0);
	gtk_box_pack_start(GTK_BOX(main_vbox), w, FALSE, FALSE, border_major);
	gtk_widget_show(w);
	parent2 = w;

	/* Save GtkButton */
	w = GUIButtonPixmapLabelH(
		(guint8 **)icon_save_20x20_xpm,
#if defined(PROG_LANGUAGE_SPANISH)
		"Salve",
#elif defined(PROG_LANGUAGE_FRENCH)
		"Epargner",
#elif defined(PROG_LANGUAGE_GERMAN)
		"Sparen",
#elif defined(PROG_LANGUAGE_ITALIAN)
		"Risparmi",
#elif defined(PROG_LANGUAGE_DUTCH)
		"Red",
#elif defined(PROG_LANGUAGE_PORTUGUESE)
		"Poupe",
#elif defined(PROG_LANGUAGE_NORWEGIAN)
		"Untatt",
#else
		"Save",
#endif
		NULL
	);
	gtk_widget_set_usize(w, bw, bh);
	GTK_WIDGET_SET_FLAGS(w, GTK_CAN_DEFAULT);
	gtk_box_pack_start(GTK_BOX(parent2), w, FALSE, FALSE, 0);
	gtk_signal_connect(
		GTK_OBJECT(w), "clicked",
		GTK_SIGNAL_FUNC(save_btn_cb), got_response
	);
	gtk_accel_group_add(
		accelgrp, GDK_s, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE,
		GTK_OBJECT(w), "clicked"
	);
	GUIButtonLabelUnderline(w, GDK_s);
	gtk_widget_grab_focus(w);
	gtk_widget_grab_default(w);
	gtk_widget_show(w);

	/* Cancel GtkButton */
	w = GUIButtonPixmapLabelH(
		(guint8 **)icon_cancel_20x20_xpm,
#if defined(PROG_LANGUAGE_SPANISH)
		"Cancele",
#elif defined(PROG_LANGUAGE_FRENCH)
		"Annuler",
#elif defined(PROG_LANGUAGE_GERMAN)
		"Heben",
#elif defined(PROG_LANGUAGE_ITALIAN)
		"Annullar",
#elif defined(PROG_LANGUAGE_DUTCH)
		"Annuler",
#elif defined(PROG_LANGUAGE_PORTUGUESE)
		"Cancela",
#elif defined(PROG_LANGUAGE_NORWEGIAN)
		"Kanseler",
#else
		"Cancel",
#endif
		NULL
	);
	gtk_widget_set_usize(w, bw, bh);
	GTK_WIDGET_SET_FLAGS(w, GTK_CAN_DEFAULT);
	gtk_box_pack_start(GTK_BOX(parent2), w, FALSE, FALSE, 0);
	gtk_signal_connect(
		GTK_OBJECT(w), "clicked",
		GTK_SIGNAL_FUNC(cancel_btn_cb), got_response
	);
	gtk_accel_group_add(
		accelgrp, GDK_Escape, 0, GTK_ACCEL_VISIBLE,
		GTK_OBJECT(w), "clicked"
	);
	gtk_accel_group_add(
		accelgrp, GDK_c, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE,
		GTK_OBJECT(w), "clicked"
	);
	GUIButtonLabelUnderline(w, GDK_c); 
	gtk_widget_show(w);
}


/*
 *	Queries the user for GIF format save options.
 */
gint iv_save_options_gif_query(
	GtkWidget *ref_toplevel,
	const gchar *path,
	gboolean *interlaced,			/* FALSE = Progressive
						 * TRUE = Interlaced */
	gint *color_type,			/* 0 = Black & White 
						 * 1 = Greyscale
						 * 2 = Color Index */
	gboolean *transparency,			/* FALSE = No transparent color
						 * TRUE = Include transparent color */
	guint8 *alpha_threshold,		/* 0x00 to 0xFF */
	gboolean *looping			/* FALSE = No looping block 
						 * TRUE = Write looping block */
)
{
	gboolean	got_response = FALSE;
	gint		width = 320,
			height = -1,
			border_major = 5,
			border_minor = 2;
	GtkAccelGroup	*accelgrp = NULL;
	GtkAdjustment *alpha_threshold_adj = NULL;
	GtkWidget	*toplevel = NULL,
			*client_vbox = NULL,
			*color_type_bw_radio = NULL,
			*color_type_greyscale_radio = NULL,
			*color_type_color_index_radio = NULL,
			*interlaced_toggle = NULL,
			*transparency_toggle = NULL,
			*looping_toggle = NULL,
			*alpha_threshold_parent = NULL;

	/* Create the dialog base GtkWidgets */
	build_std_dialog(
		width, height,
		"GIF Format Options",
		ref_toplevel,
		&toplevel,
		&client_vbox,
		&accelgrp,
		&got_response
	);

	/* Create the GIF format-specific option GtkWidgets */
	if(client_vbox != NULL)
	{
		GSList *gslist;
		GtkAdjustment *adj;
		GtkWidget	*w,
					*parent = client_vbox,
					*parent2, *parent3;
		GtkEntry *entry;

		w = gtk_hbox_new(FALSE, border_major);
		gtk_box_pack_start(GTK_BOX(parent), w, TRUE, TRUE, 0);
		gtk_widget_show(w);
		parent = w;

		/* Left Column GtkVBox */
		w = gtk_vbox_new(TRUE, 0);
		gtk_box_pack_start(GTK_BOX(parent), w, FALSE, FALSE, 0);
		gtk_widget_show(w);
		parent2 = w;

		/* Icon */
		w = gtk_pixmap_new_from_xpm_d(
			toplevel->window, NULL,
			(guint8 **)icon_save_32x32_xpm
		);
		gtk_box_pack_start(GTK_BOX(parent2), w, FALSE, FALSE, 0);
		gtk_widget_show(w);


		/* Right Column GtkVBox */
		w = gtk_vbox_new(FALSE, border_major);
		gtk_box_pack_start(GTK_BOX(parent), w, TRUE, TRUE, 0);
		gtk_widget_show(w);
		parent2 = w;


		/* Color Type GtkHBox */
		w = gtk_hbox_new(FALSE, border_minor);
		gtk_box_pack_start(GTK_BOX(parent2), w, FALSE, FALSE, 0);
		gtk_widget_show(w);
		parent3 = w;

		/* Black & White GtkRadioButton */
		gslist = NULL;
		color_type_bw_radio = w =
			gtk_radio_button_new_with_label(gslist, "Black & White");
		gtk_box_pack_start(GTK_BOX(parent3), w, FALSE, FALSE, 0);
		GUISetWidgetTip(w, TOOLTIP_GIF_BW);
		gtk_widget_show(w);
		gslist = gtk_radio_button_group(GTK_RADIO_BUTTON(w));

		/* Greyscale GtkRadioButton */
		color_type_greyscale_radio = w =
			gtk_radio_button_new_with_label(gslist, "Greyscale");
		gtk_box_pack_start(GTK_BOX(parent3), w, FALSE, FALSE, 0);
		GUISetWidgetTip(w, TOOLTIP_GIF_GREYSCALE);
		gtk_widget_show(w);
		gslist = gtk_radio_button_group(GTK_RADIO_BUTTON(w));

		/* Color Index GtkRadioButton */
		color_type_color_index_radio = w =
			gtk_radio_button_new_with_label(gslist, "Color Index");
		gtk_box_pack_start(GTK_BOX(parent3), w, FALSE, FALSE, 0);
		GUISetWidgetTip(w, TOOLTIP_GIF_CI);
		gtk_widget_show(w);


		/* Interlaced, Transparency, and Looping GtkHBox */
		w = gtk_hbox_new(FALSE, border_major);
		gtk_box_pack_start(GTK_BOX(parent2), w, FALSE, FALSE, 0);
		gtk_widget_show(w);
		parent3 = w;

		/* Interlaced GtkCheckButton */
		interlaced_toggle = w = gtk_check_button_new_with_label(
			"Interlaced"
		);
		gtk_box_pack_start(GTK_BOX(parent3), w, FALSE, FALSE, 0);
		GUISetWidgetTip(w, TOOLTIP_GIF_INTERLACED);
		gtk_widget_show(w);

		/* Transparency GtkCheckButton */
		transparency_toggle = w = gtk_check_button_new_with_label(
			"Transparency"
		);
		gtk_box_pack_start(GTK_BOX(parent3), w, FALSE, FALSE, 0);
		GUISetWidgetTip(w, TOOLTIP_GIF_TRANSPARENCY);
		gtk_widget_show(w);

		/* Looping GtkCheckButton */
		looping_toggle = w = gtk_check_button_new_with_label(
			"Looping"
		);
		gtk_box_pack_start(GTK_BOX(parent3), w, FALSE, FALSE, 0);
		GUISetWidgetTip(w, TOOLTIP_GIF_LOOPING);
		gtk_widget_show(w);


		/* Alpha Threshold GtkHBox */
		alpha_threshold_parent = w = gtk_hbox_new(FALSE, border_minor);
		gtk_box_pack_start(GTK_BOX(parent2), w, FALSE, FALSE, 0);
		gtk_widget_show(w);
		gtk_signal_connect(
			GTK_OBJECT(transparency_toggle), "toggled",
			GTK_SIGNAL_FUNC(toggle_set_widget_sensitive_cb), alpha_threshold_parent
		);
		parent3 = w;
		/* Alpha Threshold GtkLabel */
		w = gtk_label_new("Alpha Threshold:");
		gtk_label_set_justify(GTK_LABEL(w), GTK_JUSTIFY_RIGHT);
		gtk_box_pack_start(GTK_BOX(parent3), w, FALSE, FALSE, 0);
		gtk_widget_show(w);
		/* Alpha Threshold GtkAdjustment */
		alpha_threshold_adj = adj = (GtkAdjustment *)gtk_adjustment_new(
			(gfloat)0x00,			/* Value */
			(gfloat)0x00,			/* Minimum */
			(gfloat)0xFF,			/* Maximum */
			(gfloat)0x08,			/* Step inc */
			(gfloat)0x20,			/* Page inc */
			0.0f				/* Page size */
		);
		/* Alpha Threshold GtkHScale */
		w = gtk_hscale_new(adj);
		gtk_box_pack_start(GTK_BOX(parent3), w, TRUE, TRUE, 0);
		gtk_scale_set_draw_value(GTK_SCALE(w), FALSE);
		GUISetWidgetTip(w, TOOLTIP_GIF_ALPHA_THRESHOLD);
		gtk_widget_show(w);
		/* Alpha Threshold Entry */
		w = gtk_entry_new_with_max_length(80);
		entry = GTK_ENTRY(w);
		gtk_widget_set_usize(w, 30, -1);
		gtk_box_pack_start(GTK_BOX(parent3), w, FALSE, FALSE, 0);
		GUIEditableEndowPopupMenu(w, 0);
		GUISetWidgetTip(w, TOOLTIP_GIF_ALPHA_THRESHOLD);
		gtk_widget_show(w);
		/* Connect the Alpha Threshold GtkAdjustment and GtkEntry
		 * signals togeather so their values get updated togeather
		 */
		gtk_signal_connect(
			GTK_OBJECT(adj), "value_changed",
			GTK_SIGNAL_FUNC(scale_changed_cb), entry
		);
		gtk_signal_connect(
			GTK_OBJECT(entry), "changed",
			GTK_SIGNAL_FUNC(entry_changed_adj_cb), adj
		);


		/* Set the initial values on the GtkWidgets */

		/* Color Type */
		if(color_type != NULL)
		{
			GtkWidget *w = NULL;
			switch(*color_type)
			{ 
			  case 0:			/* Black & White */
				w = color_type_bw_radio;
				break;
			  case 1:			/* Greyscale */
				w = color_type_greyscale_radio;
				break;
			  case 2:			/* Color Index */
				w = color_type_color_index_radio;
				break;
			}
			if(w != NULL)
			{
				gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(w), TRUE);
				gtk_toggle_button_toggled(GTK_TOGGLE_BUTTON(w));
			}
		}

		/* Interlace */
		if(interlaced != NULL)
		{
			gtk_toggle_button_set_active(
				GTK_TOGGLE_BUTTON(interlaced_toggle),
				(*interlaced) ? TRUE : FALSE
			);
			gtk_toggle_button_toggled(GTK_TOGGLE_BUTTON(interlaced_toggle));
		}

		/* Transparency */
		if(transparency != NULL)
		{
			gtk_toggle_button_set_active(
				GTK_TOGGLE_BUTTON(transparency_toggle),
				(*transparency) ? TRUE : FALSE
			);
			gtk_toggle_button_toggled(GTK_TOGGLE_BUTTON(transparency_toggle));
		}

		/* Looping */
		if(looping != NULL)
		{
			gtk_toggle_button_set_active(
				GTK_TOGGLE_BUTTON(looping_toggle),
				(*looping) ? TRUE : FALSE
			);
			gtk_toggle_button_toggled(GTK_TOGGLE_BUTTON(looping_toggle));
		}

		/* Alpha Threshold */
		if(alpha_threshold != NULL)
		{
			GTK_ADJUSTMENT_SET_VALUE(
				alpha_threshold_adj,
				(gfloat)(*alpha_threshold)
			);
			gtk_adjustment_changed(alpha_threshold_adj);
		}
	}


	/* Map the dialog */
	gtk_widget_show_raise(toplevel);


	/* Wait for user response */
	gtk_main();


	/* User clicked on save? */
	if(got_response)
	{
		/* Get the values */
		if(color_type != NULL)
		{
			if(GTK_TOGGLE_BUTTON_GET_ACTIVE(color_type_bw_radio))
				*color_type = 0;
			else if(GTK_TOGGLE_BUTTON_GET_ACTIVE(color_type_greyscale_radio))
				*color_type = 1;
			else if(GTK_TOGGLE_BUTTON_GET_ACTIVE(color_type_color_index_radio))
				*color_type = 2;
			else
				*color_type = 0;
		}

		if(interlaced != NULL)
			*interlaced = GTK_TOGGLE_BUTTON_GET_ACTIVE(interlaced_toggle);

		if(transparency != NULL)
			*transparency = GTK_TOGGLE_BUTTON_GET_ACTIVE(transparency_toggle);

		if(looping != NULL)
			*looping = GTK_TOGGLE_BUTTON_GET_ACTIVE(looping_toggle);

		if(alpha_threshold != NULL)
			*alpha_threshold = (guint8)GTK_ADJUSTMENT_GET_VALUE(alpha_threshold_adj);
	}


	/* Destroy the dialog */
	gtk_widget_hide(toplevel);
	gtk_window_set_modal(GTK_WINDOW(toplevel), FALSE);
	gtk_window_set_transient_for(GTK_WINDOW(toplevel), NULL);

	GTK_WIDGET_DESTROY(toplevel);
	GTK_ACCEL_GROUP_UNREF(accelgrp);

	return(got_response);
}


/*
 *	Queries the user for JPEG format save options.
 */
gint iv_save_options_jpeg_query(
	GtkWidget *ref_toplevel,
	const gchar *path,
	gfloat *quality,			/* 0.0 to 1.0 */
	gboolean *is_color			/* FALSE = Greyscale
						 * TRUE = Color */
)
{
	gboolean	got_response = FALSE;
	gint		width = 320,
			height = -1,
			border_major = 5,
			border_minor = 2;
	GtkAccelGroup	*accelgrp = NULL;
	GtkAdjustment	*quality_adj = NULL;
	GtkWidget	*toplevel = NULL,
			*client_vbox = NULL,
			*save_as_greyscale_radio = NULL,
			*is_color_radio = NULL;

	/* Create the base dialog GtkWidgets */
	build_std_dialog(
		width, height,
		"JPEG Format Options",
		ref_toplevel,
		&toplevel,
		&client_vbox,
		&accelgrp,
		&got_response
	);

	/* Create the JPEG format-specific option GtkWidgets */
	if(client_vbox != NULL)
	{
		GtkWidget	*w,
				*parent = client_vbox,
				*parent2, *parent3;
		GtkEntry *entry;
		GtkAdjustment *adj;
		GSList *gslist;

		w = gtk_hbox_new(FALSE, border_major);
		gtk_box_pack_start(GTK_BOX(parent), w, TRUE, TRUE, 0);
		gtk_widget_show(w);
		parent = w;

		/* Left Column GtkVBox */
		w = gtk_vbox_new(TRUE, 0);
		gtk_box_pack_start(GTK_BOX(parent), w, FALSE, FALSE, 0);
		gtk_widget_show(w);
		parent2 = w;

		/* Icon */
		w = gtk_pixmap_new_from_xpm_d(
			toplevel->window, NULL,
			(guint8 **)icon_save_32x32_xpm
		);
		gtk_box_pack_start(GTK_BOX(parent2), w, FALSE, FALSE, 0);
		gtk_widget_show(w);


		/* Right Column GtkVBox */
		w = gtk_vbox_new(FALSE, border_major);
		gtk_box_pack_start(GTK_BOX(parent), w, TRUE, TRUE, 0);
		gtk_widget_show(w);
		parent2 = w;

		/* Quality GtkHBox */
		w = gtk_hbox_new(FALSE, border_minor);
		gtk_box_pack_start(GTK_BOX(parent2), w, FALSE, FALSE, 0);
		gtk_widget_show(w);
		parent3 = w;
		/* Quality GtkLabel */
		w = gtk_label_new("Quality:");
		gtk_label_set_justify(GTK_LABEL(w), GTK_JUSTIFY_RIGHT);
		gtk_box_pack_start(GTK_BOX(parent3), w, FALSE, FALSE, 0);
		gtk_widget_show(w);
		/* Quality GtkAdjustment */
		quality_adj = adj = (GtkAdjustment *)gtk_adjustment_new(
			0.0f,				/* Value */
			0.0f,				/* Minimum */
			100.0f,				/* Maximum */
			1.0f,				/* Step inc */
			1.0f,				/* Page inc */
			0.0f				/* Page size */
		);
		/* Quality GtkHScale */
		w = gtk_hscale_new(adj);
		gtk_box_pack_start(GTK_BOX(parent3), w, TRUE, TRUE, 0);
		gtk_scale_set_draw_value(GTK_SCALE(w), FALSE);
		GUISetWidgetTip(w, TOOLTIP_JPEG_QUALITY);
		gtk_widget_show(w);
		/* Quality GtkEntry */
		w = gtk_entry_new_with_max_length(80);
		entry = GTK_ENTRY(w);
		gtk_widget_set_usize(w, 30, -1);
		gtk_box_pack_start(GTK_BOX(parent3), w, FALSE, FALSE, 0);
		GUIEditableEndowPopupMenu(w, 0);
		GUISetWidgetTip(w, TOOLTIP_JPEG_QUALITY);
		gtk_widget_show(w);
		/* Quality Units GtkLabel */
		w = gtk_label_new("%");
		gtk_label_set_justify(GTK_LABEL(w), GTK_JUSTIFY_LEFT);
		gtk_box_pack_start(GTK_BOX(parent3), w, FALSE, FALSE, 0);
		gtk_widget_show(w);
		/* Connect the Quality GtkAdjustment and GtkEntry
		 * signals togeather so their values get updated togeather
		 */
		gtk_signal_connect(
			GTK_OBJECT(adj), "value_changed",
			GTK_SIGNAL_FUNC(scale_changed_cb), entry
		);
		gtk_signal_connect(
			GTK_OBJECT(entry), "changed",
			GTK_SIGNAL_FUNC(entry_changed_adj_cb), adj
		);


		/* Greyscale and Color GtkHBox */
		w = gtk_hbox_new(FALSE, border_minor);
		gtk_box_pack_start(GTK_BOX(parent2), w, FALSE, FALSE, 0);
		gtk_widget_show(w);
		parent3 = w;

		/* Greyscale GtkRadioButton */
		gslist = NULL;
		save_as_greyscale_radio = w =
			gtk_radio_button_new_with_label(gslist, "Greyscale");
		gtk_box_pack_start(GTK_BOX(parent3), w, FALSE, FALSE, 0);
		gtk_widget_show(w);
		gslist = gtk_radio_button_group(GTK_RADIO_BUTTON(w));

		/* Color GtkRadioButton */
		is_color_radio = w = gtk_radio_button_new_with_label(gslist, "Color");
		gtk_box_pack_start(GTK_BOX(parent3), w, FALSE, FALSE, 0);
		gtk_widget_show(w);


		/* Set the initial values on the GtkWidgets */

		/* Quality */
		if(quality != NULL)
		{
			GTK_ADJUSTMENT_SET_VALUE(
				quality_adj,
				(*quality) * 100.0f
			);
			gtk_adjustment_changed(quality_adj);
		}

		/* Color Type */
		if(is_color != NULL)
		{
			GtkWidget *w;
			if(*is_color)
				w = is_color_radio;
			else
				w = save_as_greyscale_radio;
			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(w), TRUE);
			gtk_toggle_button_toggled(GTK_TOGGLE_BUTTON(w));
		}
	}


	/* Map the dialog */
	gtk_widget_show_raise(toplevel);


	/* Wait for user response */
	gtk_main();


	/* User clicked on save? */
	if(got_response)
	{
		/* Get the values */
		if(quality != NULL)
			*quality = GTK_ADJUSTMENT_GET_VALUE(quality_adj) / 100.0f;

		if(is_color != NULL)
			*is_color = GTK_TOGGLE_BUTTON_GET_ACTIVE(is_color_radio);
	}


	/* Destroy the dialog */
	gtk_widget_hide(toplevel);
	gtk_window_set_modal(GTK_WINDOW(toplevel), FALSE);
	gtk_window_set_transient_for(GTK_WINDOW(toplevel), NULL);

	GTK_WIDGET_DESTROY(toplevel);
	GTK_ACCEL_GROUP_UNREF(accelgrp);

	return(got_response);
}

/*
 *	Queries the user for JNG format save options.
 */
gint iv_save_options_jng_query(
	GtkWidget *ref_toplevel,
	const gchar *path,
	gfloat *quality,                        /* 0.0 to 1.0 */
	gboolean *is_color,			/* FALSE = Greyscale [Alpha]
						 * TRUE = RGB[A] */
	gboolean *transparency			/* FALSE = No alpha channel
						 * TRUE = Alpha channel */
)
{
	gboolean	got_response = FALSE;
	gint		width = 320,
			height = -1,
			border_major = 5,
			border_minor = 2;
	GtkAccelGroup	*accelgrp = NULL;
	GtkAdjustment	*quality_adj = NULL;
	GtkWidget	*toplevel = NULL,
			*client_vbox = NULL,
			*save_as_greyscale_radio = NULL,
			*is_color_radio = NULL,
			*transparency_toggle = NULL;

	/* Create the base dialog GtkWidgets */
	build_std_dialog(
		width, height,
		"JNG Format Options",
		ref_toplevel,
		&toplevel,
		&client_vbox,
		&accelgrp,
		&got_response
	);

	/* Create the JNG format-specific option GtkWidgets */
	if(client_vbox != NULL)
	{
		GtkWidget	*w,
					*parent = client_vbox,
					*parent2, *parent3;
		GtkEntry *entry;
		GtkAdjustment *adj;
		GSList *gslist;

		w = gtk_hbox_new(FALSE, border_major);
		gtk_box_pack_start(GTK_BOX(parent), w, TRUE, TRUE, 0);
		gtk_widget_show(w);
		parent = w;

		/* Left Column GtkVBox */
		w = gtk_vbox_new(TRUE, 0);
		gtk_box_pack_start(GTK_BOX(parent), w, FALSE, FALSE, 0);
		gtk_widget_show(w);
		parent2 = w;

		/* Icon */
		w = gtk_pixmap_new_from_xpm_d(
			toplevel->window, NULL,
			(guint8 **)icon_save_32x32_xpm
		);
		gtk_box_pack_start(GTK_BOX(parent2), w, FALSE, FALSE, 0);
		gtk_widget_show(w);


		/* Right Column GtkVBox */
		w = gtk_vbox_new(FALSE, border_major);
		gtk_box_pack_start(GTK_BOX(parent), w, TRUE, TRUE, 0);
		gtk_widget_show(w);
		parent2 = w;

		/* Quality GtkHBox */
		w = gtk_hbox_new(FALSE, border_minor);
		gtk_box_pack_start(GTK_BOX(parent2), w, FALSE, FALSE, 0);
		gtk_widget_show(w);
		parent3 = w;
		/* Quality GtkLabel */
		w = gtk_label_new("Quality:");
		gtk_label_set_justify(GTK_LABEL(w), GTK_JUSTIFY_RIGHT);
		gtk_box_pack_start(GTK_BOX(parent3), w, FALSE, FALSE, 0);
		gtk_widget_show(w);
		/* Quality GtkAdjustment */
		quality_adj = adj = (GtkAdjustment *)gtk_adjustment_new(
			0.0f,				/* Value */
			0.0f,				/* Minimum */
			100.0f,				/* Maximum */
			1.0f,				/* Step inc */
			1.0f,				/* Page inc */
			0.0f				/* Page size */
		);
		/* Quality GtkHScale */
		w = gtk_hscale_new(adj);
		gtk_box_pack_start(GTK_BOX(parent3), w, TRUE, TRUE, 0);
		gtk_scale_set_draw_value(GTK_SCALE(w), FALSE);
		GUISetWidgetTip(w, TOOLTIP_JNG_QUALITY);
		gtk_widget_show(w);
		/* Quality GtkEntry */
		w = gtk_entry_new_with_max_length(80);
		entry = GTK_ENTRY(w);
		gtk_widget_set_usize(w, 30, -1);
		gtk_box_pack_start(GTK_BOX(parent3), w, FALSE, FALSE, 0);
		GUIEditableEndowPopupMenu(w, 0);
		GUISetWidgetTip(w, TOOLTIP_JNG_QUALITY);
		gtk_widget_show(w);
		/* Quality Units GtkLabel */
		w = gtk_label_new("%");
		gtk_label_set_justify(GTK_LABEL(w), GTK_JUSTIFY_LEFT);
		gtk_box_pack_start(GTK_BOX(parent3), w, FALSE, FALSE, 0);
		gtk_widget_show(w);
		/* Connect the Quality GtkAdjustment and GtkEntry
		 * signals togeather so their values get updated togeather
		 */
		gtk_signal_connect(
			GTK_OBJECT(adj), "value_changed",
			GTK_SIGNAL_FUNC(scale_changed_cb), entry
		);
		gtk_signal_connect(
			GTK_OBJECT(entry), "changed",
			GTK_SIGNAL_FUNC(entry_changed_adj_cb), adj
		);


		/* Greyscale and Color GtkHBox */
		w = gtk_hbox_new(FALSE, border_minor);
		gtk_box_pack_start(GTK_BOX(parent2), w, FALSE, FALSE, 0);
		gtk_widget_show(w);
		parent3 = w;

		/* Greyscale GtkRadioButton */
		gslist = NULL;
		save_as_greyscale_radio = w =
			gtk_radio_button_new_with_label(gslist, "Greyscale");
		gtk_box_pack_start(GTK_BOX(parent3), w, FALSE, FALSE, 0);
		gtk_widget_show(w);
		gslist = gtk_radio_button_group(GTK_RADIO_BUTTON(w));

		/* Color GtkRadioButton */
		is_color_radio = w = gtk_radio_button_new_with_label(gslist, "Color");
		gtk_box_pack_start(GTK_BOX(parent3), w, FALSE, FALSE, 0);
		gtk_widget_show(w);


		/* Transparency GtkHBox */
		w = gtk_hbox_new(FALSE, border_minor);
		gtk_box_pack_start(GTK_BOX(parent2), w, FALSE, FALSE, 0);
		gtk_widget_show(w);
		parent3 = w;

		/* Transparency GtkCheckButton */
		transparency_toggle = w = gtk_check_button_new_with_label(
			"Transparency"
		);
		gtk_box_pack_start(GTK_BOX(parent3), w, FALSE, FALSE, 0);
		GUISetWidgetTip(w, TOOLTIP_JNG_TRANSPARENCY);
		gtk_widget_show(w);


		/* Set the initial values on the GtkWidgets */

		/* Quality */
		if(quality != NULL)
		{
			GTK_ADJUSTMENT_SET_VALUE(
				quality_adj,
				(*quality) * 100.0f
			);
			gtk_adjustment_changed(quality_adj);
		}

		/* Color Type */
		if(is_color != NULL)
		{
			GtkWidget *w;
			if(*is_color)
				w = is_color_radio;
			else
				w = save_as_greyscale_radio;
			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(w), TRUE);
			gtk_toggle_button_toggled(GTK_TOGGLE_BUTTON(w));
		}

		/* Transparency */
		if(transparency != NULL)
		{
			gtk_toggle_button_set_active(
				GTK_TOGGLE_BUTTON(transparency_toggle),
				(*transparency) ? TRUE : FALSE
			);
			gtk_toggle_button_toggled(GTK_TOGGLE_BUTTON(transparency_toggle));
		}
	}


	/* Map the dialog */
	gtk_widget_show_raise(toplevel);


	/* Wait for user response */
	gtk_main();


	/* User clicked on save? */
	if(got_response)
	{
		/* Get the values */
		if(quality != NULL)
			*quality = GTK_ADJUSTMENT_GET_VALUE(quality_adj) / 100.0f;

		if(is_color != NULL)
			*is_color = GTK_TOGGLE_BUTTON_GET_ACTIVE(is_color_radio);

		if(transparency != NULL)
			*transparency = GTK_TOGGLE_BUTTON_GET_ACTIVE(transparency_toggle);
	}


	/* Destroy the dialog */
	gtk_widget_hide(toplevel);
	gtk_window_set_modal(GTK_WINDOW(toplevel), FALSE);
	gtk_window_set_transient_for(GTK_WINDOW(toplevel), NULL);

	GTK_WIDGET_DESTROY(toplevel);
	GTK_ACCEL_GROUP_UNREF(accelgrp);

	return(got_response);
}

/*
 *	Queries the user for MNG format save options.
 */
gint iv_save_options_mng_query(
	GtkWidget *ref_toplevel,
	const gchar *path,
	gint *compression,			/* 0 to 9 */
	gint *color_type,			/* 0 = Black & White 
						 * 1 = Greyscale
						 * 2 = Greyscale Alpha
						 * 3 = RGB
						 * 4 = RGBA */
	gboolean *interlaced,			/* FALSE = Progressive
						 * TRUE = Interlaced */
	gboolean *looping			/* FALSE = Play once
						 * TRUE = Repeating */
)
{
	gboolean	got_response = FALSE;
	gint		width = 320,
			height = -1,
			border_minor = 2,
			border_major = 5;
	GtkAdjustment	*compression_adj = NULL;
	GtkAccelGroup	*accelgrp = NULL;
	GtkWidget	*toplevel = NULL,
			*client_vbox = NULL,
			*color_type_greyscale_radio = NULL,
			*color_type_rgb_radio = NULL,
			*alpha_toggle = NULL,
			*interlaced_toggle = NULL,
			*looping_toggle = NULL;

	/* Create the base dialog GtkWidgets */
	build_std_dialog(
		width, height,
		"MNG Format Options",
		ref_toplevel,
		&toplevel,
		&client_vbox,
		&accelgrp,
		&got_response
	);

	/* Create the MNG format-specific option GtkWidgets */
	if(client_vbox != NULL)
	{
		GSList *gslist;
		GtkAdjustment *adj;
		GtkWidget	*w,
					*parent = client_vbox,
					*parent2,
					*parent3,
					*parent4;
		GtkEntry *entry;

		w = gtk_hbox_new(FALSE, border_major);
		gtk_box_pack_start(GTK_BOX(parent), w, TRUE, TRUE, 0);
		gtk_widget_show(w);
		parent = w;

		/* Left Column GtkVBox */
		w = gtk_vbox_new(TRUE, 0);
		gtk_box_pack_start(GTK_BOX(parent), w, FALSE, FALSE, 0);
		gtk_widget_show(w);
		parent2 = w;

		/* Icon */
		w = gtk_pixmap_new_from_xpm_d(
			toplevel->window, NULL,
			(guint8 **)icon_save_32x32_xpm
		);
		gtk_box_pack_start(GTK_BOX(parent2), w, FALSE, FALSE, 0);
		gtk_widget_show(w);


		/* Right Column GtkVBox */
		w = gtk_vbox_new(FALSE, border_major);
		gtk_box_pack_start(GTK_BOX(parent), w, TRUE, TRUE, 0);
		gtk_widget_show(w);
		parent2 = w;

		/* Compression GtkHBox */
		w = gtk_hbox_new(FALSE, border_minor);
		gtk_box_pack_start(GTK_BOX(parent2), w, FALSE, FALSE, 0);
		gtk_widget_show(w);
		parent3 = w;
		/* Compression GtkLabel */
		w = gtk_label_new("Compression:");
		gtk_label_set_justify(GTK_LABEL(w), GTK_JUSTIFY_RIGHT);
		gtk_box_pack_start(GTK_BOX(parent3), w, FALSE, FALSE, 0);
		gtk_widget_show(w);
		/* Compression GtkAdjustment */
		compression_adj = adj = (GtkAdjustment *)gtk_adjustment_new(
			0.0f,				/* Value */
			0.0f,				/* Minimum */
			9.0f,				/* Maximum */
			1.0f,				/* Step inc */
			1.0f,				/* Page inc */
			0.0f				/* Page size */
		);
		/* Compression GtkHScale */
		w = gtk_hscale_new(adj);
		gtk_box_pack_start(GTK_BOX(parent3), w, TRUE, TRUE, 0);
		gtk_scale_set_draw_value(GTK_SCALE(w), FALSE);
		GUISetWidgetTip(w, TOOLTIP_MNG_COMPRESSION);
		gtk_widget_show(w);
		/* Compression GtkEntry */
		w = gtk_entry_new_with_max_length(80);
		entry = GTK_ENTRY(w);
		gtk_widget_set_usize(w, 25, -1);
		gtk_box_pack_start(GTK_BOX(parent3), w, FALSE, FALSE, 0);
		GUIEditableEndowPopupMenu(w, 0);
		GUISetWidgetTip(w, TOOLTIP_MNG_COMPRESSION);
		gtk_widget_show(w);
		/* Connect the Compression GtkAdjustment and GtkEntry
		 * signals togeather so their values get updated togeather
		 */
		gtk_signal_connect(
			GTK_OBJECT(adj), "value_changed",
			GTK_SIGNAL_FUNC(scale_changed_cb), entry
		);
		gtk_signal_connect(
			GTK_OBJECT(entry), "changed",
			GTK_SIGNAL_FUNC(entry_changed_adj_cb), adj
		);


		/* Greyscale, RGB, and Alpha GtkHBox */
		w = gtk_hbox_new(FALSE, border_major);
		gtk_box_pack_start(GTK_BOX(parent2), w, FALSE, FALSE, 0);
		gtk_widget_show(w);
		parent3 = w;

		/* Greyscale and RGB GtkHBox */
		w = gtk_hbox_new(FALSE, border_minor);
		gtk_box_pack_start(GTK_BOX(parent3), w, FALSE, FALSE, 0);
		gtk_widget_show(w);
		parent4 = w;
		/* Greyscale GtkRadioButton */
		gslist = NULL;
		color_type_greyscale_radio = w =
			gtk_radio_button_new_with_label(gslist, "Greyscale");
		gtk_box_pack_start(GTK_BOX(parent4), w, FALSE, FALSE, 0);
		gtk_widget_show(w);
		gslist = gtk_radio_button_group(GTK_RADIO_BUTTON(w));
		/* RGB GtkRadioButton */
		color_type_rgb_radio = w =
			gtk_radio_button_new_with_label(gslist, "RGB");
		gtk_box_pack_start(GTK_BOX(parent4), w, FALSE, FALSE, 0);
		gtk_widget_show(w);

		/* Alpha GtkHBox */
		w = gtk_hbox_new(FALSE, border_minor);
		gtk_box_pack_start(GTK_BOX(parent3), w, FALSE, FALSE, 0);
		gtk_widget_show(w);
		parent4 = w;
		/* Alpha GtkCheckButton */
		alpha_toggle = w = gtk_check_button_new_with_label("Alpha");
		gtk_box_pack_start(GTK_BOX(parent4), w, FALSE, FALSE, 0);
		GUISetWidgetTip(w, TOOLTIP_MNG_ALPHA);
		gtk_widget_show(w);


		/* Interlaced and Looping GtkHbox */
		w = gtk_hbox_new(FALSE, border_minor);
		gtk_box_pack_start(GTK_BOX(parent2), w, FALSE, FALSE, 0);
		gtk_widget_show(w);
		parent3 = w;

		/* Interlaced GtkCheckButton */
		interlaced_toggle = w = gtk_check_button_new_with_label(
			"Interlaced"
		);
		gtk_box_pack_start(GTK_BOX(parent3), w, FALSE, FALSE, 0);
		GUISetWidgetTip(w, TOOLTIP_MNG_INTERLACED);
		gtk_widget_show(w);

		/* Looping GtkCheckButton */
		looping_toggle = w = gtk_check_button_new_with_label(
			"Looping"
		);
		gtk_box_pack_start(GTK_BOX(parent3), w, FALSE, FALSE, 0);
		GUISetWidgetTip(w, TOOLTIP_MNG_LOOPING);
		gtk_widget_show(w);


		/* Set the initial values on the GtkWidgets */

		/* Compression */
		if(compression != NULL)
		{
			GTK_ADJUSTMENT_SET_VALUE(
				compression_adj,
				(gfloat)*compression
			);
			gtk_adjustment_changed(compression_adj);
		}

		/* Color Type & Alpha */
		if(color_type != NULL)
		{
			gboolean b = FALSE;
			GtkWidget *w = NULL;
			switch(*color_type)
			{
			  case 0:			/* Black & White */
			  case 1:			/* Greyscale */
				w = color_type_greyscale_radio;
				break;
			  case 2:			/* Greyscale Alpha */
				w = color_type_greyscale_radio;
				b = TRUE;
				break;
			  case 3:			/* RGB */
				w = color_type_rgb_radio;
				break;
			  case 4:			/* RGBA */
				w = color_type_rgb_radio;
				b = TRUE;
				break;
			}
			if(w != NULL)
			{
				gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(w), TRUE);
				gtk_toggle_button_toggled(GTK_TOGGLE_BUTTON(w));
			}
			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(alpha_toggle), b);
			gtk_toggle_button_toggled(GTK_TOGGLE_BUTTON(alpha_toggle));
		}

		/* Interlaced */
		if(interlaced != NULL)
		{
			gtk_toggle_button_set_active(
				GTK_TOGGLE_BUTTON(interlaced_toggle),
				(*interlaced) ? TRUE : FALSE
			);
			gtk_toggle_button_toggled(GTK_TOGGLE_BUTTON(interlaced_toggle));
		}

		/* Looping */
		if(looping != NULL)
		{
			gtk_toggle_button_set_active(
				GTK_TOGGLE_BUTTON(looping_toggle),
				(*looping) ? TRUE : FALSE
			);
			gtk_toggle_button_toggled(GTK_TOGGLE_BUTTON(looping_toggle));
		}
	}


	/* Map the dialog */
	gtk_widget_show_raise(toplevel);


	/* Wait for user response */
	gtk_main();


	/* User clicked on save? */
	if(got_response)
	{
		/* Get the values */
		if(compression != NULL)
			*compression = (gint)GTK_ADJUSTMENT_GET_VALUE(compression_adj);

		if(color_type != NULL)
		{
			/* RGB[A]? */
			if(GTK_TOGGLE_BUTTON_GET_ACTIVE(color_type_rgb_radio))
			{
				if(GTK_TOGGLE_BUTTON_GET_ACTIVE(alpha_toggle))
					*color_type = 4;	/* RGBA */
				else
					*color_type = 3;	/* RGB */
			}
			/* Greyscale [Alpha] */
			else
			{
				if(GTK_TOGGLE_BUTTON_GET_ACTIVE(alpha_toggle))
					*color_type = 2;	/* Greyscale Alpha */
				else
					*color_type = 1;	/* Greyscale */
			}
		}

		if(interlaced != NULL)
			*interlaced = GTK_TOGGLE_BUTTON_GET_ACTIVE(interlaced_toggle);

		if(looping != NULL)
			*looping = GTK_TOGGLE_BUTTON_GET_ACTIVE(looping_toggle);
	}


	/* Destroy the dialog */
	gtk_widget_hide(toplevel);
	gtk_window_set_modal(GTK_WINDOW(toplevel), FALSE);
	gtk_window_set_transient_for(GTK_WINDOW(toplevel), NULL);

	GTK_WIDGET_DESTROY(toplevel);
	GTK_ACCEL_GROUP_UNREF(accelgrp);

	return(got_response);
}

/*
 *	Queries the user for PNG format save options.
 */
gint iv_save_options_png_query(
	GtkWidget *ref_toplevel,
	const gchar *path,
	gint *compression,			/* 0 to 9 */
	gint *color_type,			/* 0 = Black & White
						 * 1 = Greyscale
						 * 2 = Greyscale Alpha
						 * 3 = RGB
						 * 4 = RGBA */
	gboolean *interlaced			/* FALSE = Progressive
						 * TRUE = Interlaced */
)
{
	gboolean	got_response = FALSE;
	gint		width = 320,
			height = -1,
			border_minor = 2,
			border_major = 5;
	GtkAdjustment	*compression_adj = NULL;
	GtkAccelGroup	*accelgrp = NULL;
	GtkWidget	*toplevel = NULL,
			*client_vbox = NULL,
			*color_type_greyscale_radio = NULL,
			*color_type_rgb_radio = NULL,
			*alpha_toggle = NULL,
			*interlaced_toggle = NULL;

	/* Create the base dialog GtkWidgets */
	build_std_dialog(
		width, height,
		"PNG Format Options",
		ref_toplevel,
		&toplevel,
		&client_vbox,
		&accelgrp,
		&got_response
	);

	/* Create the PNG format-specific option GtkWidgets */
	if(client_vbox != NULL)
	{
		GSList *gslist;
		GtkAdjustment *adj;
		GtkWidget	*w,
					*parent = client_vbox,
					*parent2, *parent3, *parent4;
		GtkEntry *entry;

		w = gtk_hbox_new(FALSE, border_major);
		gtk_box_pack_start(GTK_BOX(parent), w, TRUE, TRUE, 0);
		gtk_widget_show(w);
		parent = w;

		/* Left Column GtkVBox */
		w = gtk_vbox_new(TRUE, 0);
		gtk_box_pack_start(GTK_BOX(parent), w, FALSE, FALSE, 0);
		gtk_widget_show(w);
		parent2 = w;

		/* Icon */
		w = gtk_pixmap_new_from_xpm_d(
			toplevel->window, NULL,
			(guint8 **)icon_save_32x32_xpm
		);
		gtk_box_pack_start(GTK_BOX(parent2), w, FALSE, FALSE, 0);
		gtk_widget_show(w);


		/* Right Column GtkVBox */
		w = gtk_vbox_new(FALSE, border_major);
		gtk_box_pack_start(GTK_BOX(parent), w, TRUE, TRUE, 0);
		gtk_widget_show(w);
		parent2 = w;

		/* Compression GtkHBox */
		w = gtk_hbox_new(FALSE, border_minor);
		gtk_box_pack_start(GTK_BOX(parent2), w, FALSE, FALSE, 0);
		gtk_widget_show(w);
		parent3 = w;
		/* Compression GtkLabel */
		w = gtk_label_new("Compression:");
		gtk_label_set_justify(GTK_LABEL(w), GTK_JUSTIFY_RIGHT);
		gtk_box_pack_start(GTK_BOX(parent3), w, FALSE, FALSE, 0);
		gtk_widget_show(w);
		/* Compression GtkAdjustment */
		compression_adj = adj = (GtkAdjustment *)gtk_adjustment_new(
			0.0f,                           /* Value */
			0.0f,                           /* Minimum */
			9.0f,                           /* Maximum */
			1.0f,                           /* Step inc */
			1.0f,                           /* Page inc */
			0.0f                            /* Page size */
		);
		/* Compression GtkHScale */
		w = gtk_hscale_new(adj);
		gtk_box_pack_start(GTK_BOX(parent3), w, TRUE, TRUE, 0);
		gtk_scale_set_draw_value(GTK_SCALE(w), FALSE);
		GUISetWidgetTip(w, TOOLTIP_PNG_COMPRESSION);
		gtk_widget_show(w);
		/* Compression GtkEntry */
		w = gtk_entry_new_with_max_length(80);
		entry = GTK_ENTRY(w);
		gtk_widget_set_usize(w, 25, -1);
		gtk_box_pack_start(GTK_BOX(parent3), w, FALSE, FALSE, 0);
		GUIEditableEndowPopupMenu(w, 0);
		GUISetWidgetTip(w, TOOLTIP_PNG_COMPRESSION);
		gtk_widget_show(w);
		/* Connect the Compression GtkAdjustment and GtkEntry
		 * signals togeather so their values get updated togeather
		 */
		gtk_signal_connect(
			GTK_OBJECT(adj), "value_changed",
			GTK_SIGNAL_FUNC(scale_changed_cb), entry
		);
		gtk_signal_connect(
			GTK_OBJECT(entry), "changed",
			GTK_SIGNAL_FUNC(entry_changed_adj_cb), adj
		);


		/* Greyscale, RGB, and Alpha GtkHBox */
		w = gtk_hbox_new(FALSE, border_major);
		gtk_box_pack_start(GTK_BOX(parent2), w, FALSE, FALSE, 0);
		gtk_widget_show(w);
		parent3 = w;

		/* Greyscale and RGB GtkHBox */
		w = gtk_hbox_new(FALSE, border_minor);
		gtk_box_pack_start(GTK_BOX(parent3), w, FALSE, FALSE, 0);
		gtk_widget_show(w);
		parent4 = w;
		/* Greyscale GtkRadioButton */
		gslist = NULL;
		color_type_greyscale_radio = w =
			gtk_radio_button_new_with_label(gslist, "Greyscale");
		gtk_box_pack_start(GTK_BOX(parent4), w, FALSE, FALSE, 0);
		gtk_widget_show(w);
		gslist = gtk_radio_button_group(GTK_RADIO_BUTTON(w));
		/* RGB GtkRadioButton */
		color_type_rgb_radio = w =
			gtk_radio_button_new_with_label(gslist, "RGB");
		gtk_box_pack_start(GTK_BOX(parent4), w, FALSE, FALSE, 0);
		gtk_widget_show(w);

		/* Alpha GtkHBox */
		w = gtk_hbox_new(FALSE, border_minor);
		gtk_box_pack_start(GTK_BOX(parent3), w, FALSE, FALSE, 0);
		gtk_widget_show(w);
		parent4 = w;
		/* Alpha GtkCheckButton */
		alpha_toggle = w = gtk_check_button_new_with_label("Alpha");
		gtk_box_pack_start(GTK_BOX(parent4), w, FALSE, FALSE, 0);
		GUISetWidgetTip(w, TOOLTIP_PNG_ALPHA);
		gtk_widget_show(w);


		/* Interlaced GtkHbox */
		w = gtk_hbox_new(FALSE, border_minor);
		gtk_box_pack_start(GTK_BOX(parent2), w, FALSE, FALSE, 0);
		gtk_widget_show(w);
		parent3 = w;

		/* Interlaced GtkCheckButton */
		interlaced_toggle = w = gtk_check_button_new_with_label(
			"Interlaced"
		);
		gtk_box_pack_start(GTK_BOX(parent3), w, FALSE, FALSE, 0);
		GUISetWidgetTip(w, TOOLTIP_PNG_INTERLACED);
		gtk_widget_show(w);


		/* Set the initial values on the GtkWidgets */

		/* Compression */
		if(compression != NULL)
		{
			GTK_ADJUSTMENT_SET_VALUE(
				compression_adj,
				(gfloat)*compression
			);
			gtk_adjustment_changed(compression_adj);
		}

		/* Color Type & Alpha */
		if(color_type != NULL)
		{
			gboolean b = FALSE;
			GtkWidget *w = NULL;
			switch(*color_type)
			{
			  case 0:                       /* Black & White */
			  case 1:                       /* Greyscale */
				w = color_type_greyscale_radio;
				break;
			  case 2:                       /* Greyscale Alpha */
				w = color_type_greyscale_radio;
				b = TRUE;
				break;
			  case 3:                       /* RGB */
				w = color_type_rgb_radio;
				break;
			  case 4:                       /* RGBA */
				w = color_type_rgb_radio;
				b = TRUE;
				break;
			}
			if(w != NULL)
			{
				gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(w), TRUE);
				gtk_toggle_button_toggled(GTK_TOGGLE_BUTTON(w));
			}
			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(alpha_toggle), b);
			gtk_toggle_button_toggled(GTK_TOGGLE_BUTTON(alpha_toggle));
		}

		/* Interlaced */
		if(interlaced != NULL)
		{
			gtk_toggle_button_set_active(
				GTK_TOGGLE_BUTTON(interlaced_toggle),
				(*interlaced) ? TRUE : FALSE
			);
			gtk_toggle_button_toggled(GTK_TOGGLE_BUTTON(interlaced_toggle));
		}
	}


	/* Map the dialog */
	gtk_widget_show_raise(toplevel);


	/* Wait for user response */
	gtk_main();


	/* User clicked on save? */
	if(got_response)
	{
		/* Get the values */
		if(compression != NULL)
			*compression = (gint)GTK_ADJUSTMENT_GET_VALUE(compression_adj);

		if(color_type != NULL)
		{
			/* RGB or RGBA? */
			if(GTK_TOGGLE_BUTTON_GET_ACTIVE(color_type_rgb_radio))
			{
				/* RGBA? */
				if(GTK_TOGGLE_BUTTON_GET_ACTIVE(alpha_toggle))
					*color_type = 4;
				else
					*color_type = 3;
			}
			/* Greyscale or Greyscale Alpha */
			else
			{
				/* Greyscale Alpha? */
				if(GTK_TOGGLE_BUTTON_GET_ACTIVE(alpha_toggle))
					*color_type = 2;
				else
					*color_type = 1;
			}
		}

		if(interlaced != NULL)
			*interlaced = GTK_TOGGLE_BUTTON_GET_ACTIVE(interlaced_toggle);
	}


	/* Destroy the dialog */
	gtk_widget_hide(toplevel);
	gtk_window_set_modal(GTK_WINDOW(toplevel), FALSE);
	gtk_window_set_transient_for(GTK_WINDOW(toplevel), NULL);

	GTK_WIDGET_DESTROY(toplevel);
	GTK_ACCEL_GROUP_UNREF(accelgrp);

	return(got_response);
}

/*
 *	Queries the user for TGA format save options.
 */
gint iv_save_options_targa_query(
	GtkWidget *ref_toplevel,
	const gchar *path,
	gint *color_type			/* 0 = Greyscale
						 * 1 = RGB
						 * 2 = RGBA */
)
{
	gboolean	got_response = FALSE;
	gint		width = 320,
			height = -1,
			border_minor = 2,
			border_major = 5;
	GtkAccelGroup	*accelgrp = NULL;
	GtkWidget	*toplevel = NULL,
			*client_vbox = NULL,
			*color_type_greyscale_radio = NULL,
			*color_type_rgb_radio = NULL,
			*color_type_rgba_radio = NULL;

	/* Create the base dialog GtkWidgets */
	build_std_dialog(
		width, height,
		"TGA Format Options",
		ref_toplevel,
		&toplevel,
		&client_vbox,
		&accelgrp,
		&got_response
	);

	/* Create TGA format-specific option GtkWidgets */
	if(client_vbox != NULL)
	{
		GtkWidget	*w,
				*parent = client_vbox,
				*parent2, *parent3, *parent4;
		GSList *gslist;

		w = gtk_hbox_new(FALSE, border_major);
		gtk_box_pack_start(GTK_BOX(parent), w, TRUE, TRUE, 0);
		gtk_widget_show(w);
		parent = w;

		/* Left Column GtkVBox */
		w = gtk_vbox_new(TRUE, 0);
		gtk_box_pack_start(GTK_BOX(parent), w, FALSE, FALSE, 0);
		gtk_widget_show(w);
		parent2 = w;

		/* Icon */
		w = gtk_pixmap_new_from_xpm_d(
			toplevel->window, NULL,
			(guint8 **)icon_save_32x32_xpm
		);
		gtk_box_pack_start(GTK_BOX(parent2), w, FALSE, FALSE, 0);
		gtk_widget_show(w);


		/* Right Column GtkVBox */
		w = gtk_vbox_new(FALSE, border_major);
		gtk_box_pack_start(GTK_BOX(parent), w, TRUE, TRUE, 0);
		gtk_widget_show(w);
		parent2 = w;

		w = gtk_hbox_new(FALSE, border_major);
		gtk_box_pack_start(GTK_BOX(parent2), w, FALSE, FALSE, 0);
		gtk_widget_show(w);
		parent3 = w;

		/* Greyscale, RGB, and RGBA GtkHBox */
		w = gtk_hbox_new(FALSE, border_minor);
		gtk_box_pack_start(GTK_BOX(parent3), w, FALSE, FALSE, 0);
		gtk_widget_show(w);
		parent4 = w;
		/* Greyscale GtkRadioButton */
		gslist = NULL;
		color_type_greyscale_radio = w =
			gtk_radio_button_new_with_label(gslist, "Greyscale");
		gtk_box_pack_start(GTK_BOX(parent4), w, FALSE, FALSE, 0);
		gtk_widget_show(w);
		gslist = gtk_radio_button_group(GTK_RADIO_BUTTON(w));
		/* RGB GtkRadioButton */
		color_type_rgb_radio = w =
			gtk_radio_button_new_with_label(gslist, "RGB");
		gtk_box_pack_start(GTK_BOX(parent4), w, FALSE, FALSE, 0);
		gtk_widget_show(w);
		gslist = gtk_radio_button_group(GTK_RADIO_BUTTON(w));
		/* RGBA GtkRadioButton */
		color_type_rgba_radio = w =
			gtk_radio_button_new_with_label(gslist, "RGBA");
		gtk_box_pack_start(GTK_BOX(parent4), w, FALSE, FALSE, 0);
		gtk_widget_show(w);


		/* Set the initial values on the GtkWidgets */

		/* Color Type */
		if(color_type != NULL)
		{
			GtkWidget *w = NULL;
			switch(*color_type)
			{
			  case 0:			/* Greyscale */
				w = color_type_greyscale_radio;
				break;
			  case 1:			/* RGB */
				w = color_type_rgb_radio;
				break;
			  case 2:			/* RGBA */
				w = color_type_rgba_radio;
				break;
			}
			if(w != NULL)
			{
				gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(w), TRUE);
				gtk_toggle_button_toggled(GTK_TOGGLE_BUTTON(w));
			}
		}
	}


	/* Map the dialog */
	gtk_widget_show_raise(toplevel);


	/* Wait for user response */
	gtk_main();


	/* User clicked on save? */
	if(got_response)
	{
		/* Get the values */
		if(color_type != NULL)
		{
			if(GTK_TOGGLE_BUTTON_GET_ACTIVE(color_type_rgba_radio))
				*color_type = 2;		/* RGBA */
			else if(GTK_TOGGLE_BUTTON_GET_ACTIVE(color_type_rgb_radio))
				*color_type = 1;		/* RGB */
			else
				*color_type = 0;		/* Greyscale */
		}
	}


	/* Destroy the dialog */
	gtk_widget_hide(toplevel);
	gtk_window_set_modal(GTK_WINDOW(toplevel), FALSE);
	gtk_window_set_transient_for(GTK_WINDOW(toplevel), NULL);

	GTK_WIDGET_DESTROY(toplevel);
	GTK_ACCEL_GROUP_UNREF(accelgrp);

	return(got_response);
}

/*
 *	Queries the user for TIFF format save options.
 */
gint iv_save_options_tiff_query(
	GtkWidget *ref_toplevel,
	const gchar *path,
	gint *compression_type,			/* 0 = None
                                                 * 1 = GZIP
                                                 * 2 = JPEG
                                                 * 3 = CCITT RLE
                                                 * 4 = CCITT Group 3 Fax
                                                 * 5 = CCITT Group 4 Fax */
	gfloat *jpeg_quality,			/* 0.0 to 1.0 */
	gint *color_type			/* 0 = Black & White
						 * 1 = Greyscale
						 * 2 = Greyscale Alpha
						 * 3 = RGB
						 * 4 = RGBA */
)
{
	gboolean	got_response = FALSE;
	gint		width = 320,
			height = -1,
			border_minor = 2,
			border_major = 5;
	GtkAdjustment *jpeg_quality_adj = NULL;
	GtkAccelGroup	*accelgrp = NULL;
	GtkWidget	*toplevel = NULL,
			*client_vbox = NULL,
			*compression_type_none_radio = NULL,
			*compression_type_gzip_radio = NULL,
			*compression_type_jpeg_radio = NULL,
			*compression_type_ccitt_rle_radio = NULL,
			*compression_type_ccitt_fax3_radio = NULL,
			*compression_type_ccitt_fax4_radio = NULL,
			*jpeg_quality_toplevel = NULL,
			*color_type_bw_radio = NULL,
			*color_type_greyscale_radio = NULL,
			*color_type_rgb_radio = NULL,
			*alpha_toggle = NULL;

	/* Create the base dialog GtkWidgets */
	build_std_dialog(
		width, height,
		"TIFF Format Options",
		ref_toplevel,
		&toplevel,
		&client_vbox,
		&accelgrp,
		&got_response
	);

	/* Create the MNG format-specific option GtkWidgets */
	if(client_vbox != NULL)
	{
		GSList *gslist;
		GtkAdjustment *adj;
		GtkWidget	*w,
				*parent = client_vbox,
				*parent2,
				*parent3,
				*parent4;
		GtkEntry *entry;

		w = gtk_hbox_new(FALSE, border_major);
		gtk_box_pack_start(GTK_BOX(parent), w, TRUE, TRUE, 0);
		gtk_widget_show(w);
		parent = w;

		/* Left Column GtkVBox */
		w = gtk_vbox_new(TRUE, 0);
		gtk_box_pack_start(GTK_BOX(parent), w, FALSE, FALSE, 0);
		gtk_widget_show(w);
		parent2 = w;

		/* Icon */
		w = gtk_pixmap_new_from_xpm_d(
			toplevel->window, NULL,
			(guint8 **)icon_save_32x32_xpm
		);
		gtk_box_pack_start(GTK_BOX(parent2), w, FALSE, FALSE, 0);
		gtk_widget_show(w);


		/* Right Column GtkVBox */
		w = gtk_vbox_new(FALSE, border_major);
		gtk_box_pack_start(GTK_BOX(parent), w, TRUE, TRUE, 0);
		gtk_widget_show(w);
		parent2 = w;

		/* Compression GtkVBox */
		w = gtk_vbox_new(FALSE, border_minor);
		gtk_box_pack_start(GTK_BOX(parent2), w, FALSE, FALSE, 0);
		gtk_widget_show(w);
		parent3 = w;

		/* Compression GtkHBox */
		w = gtk_hbox_new(FALSE, border_minor);
		gtk_box_pack_start(GTK_BOX(parent3), w, FALSE, FALSE, 0);
		gtk_widget_show(w);
		parent4 = w;

		/* Compression GtkLabel */
		w = gtk_label_new("Compression:");
		gtk_label_set_justify(GTK_LABEL(w), GTK_JUSTIFY_RIGHT);
		gtk_box_pack_start(GTK_BOX(parent4), w, FALSE, FALSE, 0);
		gtk_widget_show(w);

		/* None GtkRadioButton */
		gslist = NULL;
		compression_type_none_radio = w =
			gtk_radio_button_new_with_label(gslist, "None");
		gtk_box_pack_start(GTK_BOX(parent4), w, FALSE, FALSE, 0);
		gtk_widget_show(w);

		/* GZip GtkRadioButton */
		gslist = gtk_radio_button_group(GTK_RADIO_BUTTON(w));
		compression_type_gzip_radio = w =
			gtk_radio_button_new_with_label(gslist, "GZip");
		gtk_box_pack_start(GTK_BOX(parent4), w, FALSE, FALSE, 0);
		GUISetWidgetTip(w, TOOLTIP_TIFF_GZIP);
		gtk_widget_show(w);

		/* JPEG GtkRadioButton */
		gslist = gtk_radio_button_group(GTK_RADIO_BUTTON(w));
		compression_type_jpeg_radio = w =
			gtk_radio_button_new_with_label(gslist, "JPEG");
		gtk_box_pack_start(GTK_BOX(parent4), w, FALSE, FALSE, 0);
		GUISetWidgetTip(w, TOOLTIP_TIFF_JPEG);
		gtk_widget_show(w);

		/* Compression Line 2 GtkHBox */
		w = gtk_hbox_new(FALSE, border_minor);
		gtk_box_pack_start(GTK_BOX(parent3), w, FALSE, FALSE, 0);
		gtk_widget_show(w);
		parent4 = w;

		/* CCITT RLE GtkRadioButton */
		w = compression_type_jpeg_radio;
		gslist = gtk_radio_button_group(GTK_RADIO_BUTTON(w));
		compression_type_ccitt_rle_radio = w =
			gtk_radio_button_new_with_label(gslist, "RLE");
		gtk_box_pack_start(GTK_BOX(parent4), w, FALSE, FALSE, 0);
		GUISetWidgetTip(w, TOOLTIP_TIFF_CCITT_RLE);
		gtk_widget_show(w);
		/* CCITT FAX3 GtkRadioButton */
		gslist = gtk_radio_button_group(GTK_RADIO_BUTTON(w));
		compression_type_ccitt_fax3_radio = w =
			gtk_radio_button_new_with_label(gslist, "FAX3");
		gtk_box_pack_start(GTK_BOX(parent4), w, FALSE, FALSE, 0);
		GUISetWidgetTip(w, TOOLTIP_TIFF_CCITT_FAX3);
		gtk_widget_show(w);
		/* CCITT FAX4 GtkRadioButton */
		gslist = gtk_radio_button_group(GTK_RADIO_BUTTON(w));
		compression_type_ccitt_fax4_radio = w =
			gtk_radio_button_new_with_label(gslist, "FAX4");
		gtk_box_pack_start(GTK_BOX(parent4), w, FALSE, FALSE, 0);
		GUISetWidgetTip(w, TOOLTIP_TIFF_CCITT_FAX4);
		gtk_widget_show(w);

#if 0
		/* Compression Line 3 GtkHBox */
		w = gtk_hbox_new(FALSE, border_minor);
		gtk_box_pack_start(GTK_BOX(parent3), w, FALSE, FALSE, 0);
		gtk_widget_show(w);
		parent4 = w;

		/* ??? GtkRadioButton */
		w = compression_type_ccitt_fax4_radio;
		gslist = gtk_radio_button_group(GTK_RADIO_BUTTON(w));
		compression_type_jpeg_radio = w =
			gtk_radio_button_new_with_label(gslist, "???");
		gtk_box_pack_start(GTK_BOX(parent4), w, FALSE, FALSE, 0);
		GUISetWidgetTip(w, TOOLTIP_TIFF_???);
		gtk_widget_show(w);
#endif


		/* JPEG Quality GtkHBox */
		jpeg_quality_toplevel = w = gtk_hbox_new(FALSE, border_minor);
		gtk_box_pack_start(GTK_BOX(parent2), w, FALSE, FALSE, 0);
		gtk_widget_set_sensitive(w, FALSE);
		gtk_widget_show(w);
		parent3 = w;
		/* JPEG Quality GtkLabel */
		w = gtk_label_new("JPEG Quality:");
		gtk_label_set_justify(GTK_LABEL(w), GTK_JUSTIFY_RIGHT);
		gtk_box_pack_start(GTK_BOX(parent3), w, FALSE, FALSE, 0);
		gtk_widget_show(w);
		/* JPEG Quality GtkAdjustment */
		jpeg_quality_adj = adj = (GtkAdjustment *)gtk_adjustment_new(
			0.0f,			/* Value */
			0.0f,			/* Minimum */
			100.0f,			/* Maximum */
			1.0f,			/* Step inc */
			1.0f,			/* Page inc */
			0.0f			/* Page size */
		);
		/* JPEG Quality GtkHScale */
		w = gtk_hscale_new(adj);
		gtk_box_pack_start(GTK_BOX(parent3), w, TRUE, TRUE, 0);
		gtk_scale_set_draw_value(GTK_SCALE(w), FALSE);
		GUISetWidgetTip(w, TOOLTIP_TIFF_JPEG_QUALITY);
		gtk_widget_show(w);
		/* JPEG Quality GtkEntry */
		w = gtk_entry_new_with_max_length(80);
		entry = GTK_ENTRY(w);
		gtk_widget_set_usize(w, 30, -1);
		gtk_box_pack_start(GTK_BOX(parent3), w, FALSE, FALSE, 0);
		GUIEditableEndowPopupMenu(w, 0);
		GUISetWidgetTip(w, TOOLTIP_JPEG_QUALITY);
		gtk_widget_show(w);
		/* JPEG Quality Units GtkLabel */
		w = gtk_label_new("%");
		gtk_label_set_justify(GTK_LABEL(w), GTK_JUSTIFY_LEFT);
		gtk_box_pack_start(GTK_BOX(parent3), w, FALSE, FALSE, 0);
		gtk_widget_show(w);
		/* Connect the Quality GtkAdjustment and GtkEntry
		 * signals togeather so their values get updated togeather
		 */
		gtk_signal_connect(
			GTK_OBJECT(adj), "value_changed",
			GTK_SIGNAL_FUNC(scale_changed_cb), entry
		);
		gtk_signal_connect(
			GTK_OBJECT(entry), "changed",
			GTK_SIGNAL_FUNC(entry_changed_adj_cb), adj
		);


		/* Color Type GtkHBox */
		w = gtk_hbox_new(FALSE, border_minor);
		gtk_box_pack_start(GTK_BOX(parent2), w, FALSE, FALSE, 0);
		gtk_widget_show(w);
		parent3 = w;

		/* Black & White GtkRadioButton */
		gslist = NULL;
		color_type_bw_radio = w =
			gtk_radio_button_new_with_label(gslist, "Black & White");
		gtk_box_pack_start(GTK_BOX(parent3), w, FALSE, FALSE, 0);
		gtk_widget_show(w);
		/* Greyscale GtkRadioButton */
		gslist = gtk_radio_button_group(GTK_RADIO_BUTTON(w));
		color_type_greyscale_radio = w =
			gtk_radio_button_new_with_label(gslist, "Greyscale");
		gtk_box_pack_start(GTK_BOX(parent3), w, FALSE, FALSE, 0);
		gtk_widget_show(w);
		/* RGB GtkRadioButton */
		gslist = gtk_radio_button_group(GTK_RADIO_BUTTON(w));
		color_type_rgb_radio = w =
			gtk_radio_button_new_with_label(gslist, "RGB");
		gtk_box_pack_start(GTK_BOX(parent3), w, FALSE, FALSE, 0);
		gtk_widget_show(w);

		/* Alpha GtkHBox */
		w = gtk_hbox_new(FALSE, border_minor);
		gtk_box_pack_start(GTK_BOX(parent2), w, FALSE, FALSE, 0);
		gtk_widget_show(w);
		parent3 = w;

		/* Alpha GtkCheckButton */
		alpha_toggle = w = gtk_check_button_new_with_label("Alpha");
		gtk_box_pack_start(GTK_BOX(parent3), w, FALSE, FALSE, 0);
		GUISetWidgetTip(w, TOOLTIP_TIFF_ALPHA);
		gtk_widget_show(w);


		/* Set GtkWidget value sensitivity signals */
		gtk_signal_connect(
			GTK_OBJECT(color_type_bw_radio), "toggled",
			GTK_SIGNAL_FUNC(toggle_set_widget_insensitive_cb), alpha_toggle
		);

		w = compression_type_jpeg_radio;
		gtk_signal_connect(
			GTK_OBJECT(w), "toggled",
			GTK_SIGNAL_FUNC(toggle_set_widget_insensitive_cb), color_type_bw_radio
		);
		gtk_signal_connect(
			GTK_OBJECT(w), "toggled",
			GTK_SIGNAL_FUNC(toggle_set_widget_sensitive_cb), jpeg_quality_toplevel
		);

		w = compression_type_ccitt_rle_radio;
		gtk_signal_connect(
			GTK_OBJECT(w), "toggled",
			GTK_SIGNAL_FUNC(toggle_set_toggle_active_cb), color_type_bw_radio
		);
		gtk_signal_connect(
			GTK_OBJECT(w), "toggled",
			GTK_SIGNAL_FUNC(toggle_set_widget_insensitive_cb), color_type_greyscale_radio
		);
		gtk_signal_connect(
			GTK_OBJECT(w), "toggled",
			GTK_SIGNAL_FUNC(toggle_set_widget_insensitive_cb), color_type_rgb_radio
		);

		w = compression_type_ccitt_fax3_radio;
		gtk_signal_connect(
			GTK_OBJECT(w), "toggled",
			GTK_SIGNAL_FUNC(toggle_set_toggle_active_cb), color_type_bw_radio
		);
		gtk_signal_connect(
			GTK_OBJECT(w), "toggled",
			GTK_SIGNAL_FUNC(toggle_set_widget_insensitive_cb), color_type_greyscale_radio
		);
		gtk_signal_connect(
			GTK_OBJECT(w), "toggled",
			GTK_SIGNAL_FUNC(toggle_set_widget_insensitive_cb), color_type_rgb_radio
		);

		w = compression_type_ccitt_fax4_radio;
		gtk_signal_connect(
			GTK_OBJECT(w), "toggled",
			GTK_SIGNAL_FUNC(toggle_set_toggle_active_cb), color_type_bw_radio
		);
		gtk_signal_connect(
			GTK_OBJECT(w), "toggled",
			GTK_SIGNAL_FUNC(toggle_set_widget_insensitive_cb), color_type_greyscale_radio
		);
		gtk_signal_connect(
			GTK_OBJECT(w), "toggled",
			GTK_SIGNAL_FUNC(toggle_set_widget_insensitive_cb), color_type_rgb_radio
		);


		/* Set the initial values on the GtkWidgets */

		/* Compression Type */
		if(compression_type != NULL)
		{
			GtkWidget *w = NULL;
			switch(*compression_type)
			{
			    case 0:			/* None */
				w = compression_type_none_radio;
				break;
			    case 1:			/* GZIP */
				w = compression_type_gzip_radio;
				break;
			    case 2:			/* JPEG */
				w = compression_type_jpeg_radio;
				break;
			    case 3:			/* CCITT RLE */
				w = compression_type_ccitt_rle_radio;
				break;
			    case 4:			/* CCITT FAX3 */
				w = compression_type_ccitt_fax3_radio;
				break;
			    case 5:			/* CCITT FAX4 */
				w = compression_type_ccitt_fax4_radio;
				break;
			}
			if(w != NULL)
			{
				GtkToggleButton *tb = GTK_TOGGLE_BUTTON(w);
				gtk_toggle_button_set_active(tb, TRUE);
				gtk_toggle_button_toggled(tb);
			}
		}

                /* Quality */
                if(jpeg_quality != NULL)
                {
                        GTK_ADJUSTMENT_SET_VALUE(
                                jpeg_quality_adj,
                                (*jpeg_quality) * 100.0f
                        );
                        gtk_adjustment_changed(jpeg_quality_adj);
                }

		/* Color Type & Alpha */
		if(color_type != NULL)
		{
			gboolean b = FALSE;
			GtkWidget *w = NULL;
			switch(*color_type)
			{
			    case 0:			/* Black & White */
				w = color_type_bw_radio;
				break;
			    case 1:			/* Greyscale */
				w = color_type_greyscale_radio;
				break;
			    case 2:			/* Greyscale Alpha */
				w = color_type_greyscale_radio;
				b = TRUE;
				break;
			    case 3:			/* RGB */
				w = color_type_rgb_radio;
				break;
			    case 4:			/* RGBA */
				w = color_type_rgb_radio;
				b = TRUE;
				break;
			}
			if(w != NULL)
			{
				GtkToggleButton *tb = GTK_TOGGLE_BUTTON(w);
				gtk_toggle_button_set_active(tb, TRUE);
				gtk_toggle_button_toggled(tb);
			}
			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(alpha_toggle), b);
			gtk_toggle_button_toggled(GTK_TOGGLE_BUTTON(alpha_toggle));
		}
	}


	/* Map the dialog */
	gtk_widget_show_raise(toplevel);


	/* Wait for user response */
	gtk_main();


	/* User clicked on save? */
	if(got_response)
	{
		/* Get the values */
		if(compression_type != NULL)
		{
			/* None? */
			if(GTK_TOGGLE_BUTTON_GET_ACTIVE(compression_type_none_radio))
				*compression_type = 0;
			/* GZIP? */
			else if(GTK_TOGGLE_BUTTON_GET_ACTIVE(compression_type_gzip_radio))
				*compression_type = 1;
			/* JPEG? */
			else if(GTK_TOGGLE_BUTTON_GET_ACTIVE(compression_type_jpeg_radio))
				*compression_type = 2;
			/* CCITT RLE? */
			else if(GTK_TOGGLE_BUTTON_GET_ACTIVE(compression_type_ccitt_rle_radio))
				*compression_type = 3;
			/* CCITT FAX3? */
			else if(GTK_TOGGLE_BUTTON_GET_ACTIVE(compression_type_ccitt_fax3_radio))
				*compression_type = 4;
			/* CCITT FAX4? */
			else if(GTK_TOGGLE_BUTTON_GET_ACTIVE(compression_type_ccitt_fax4_radio))
				*compression_type = 5;
		}
                if(jpeg_quality != NULL)
		{
                        *jpeg_quality = GTK_ADJUSTMENT_GET_VALUE(jpeg_quality_adj) / 100.0f;
		}
		if(color_type != NULL)
		{
			/* RGB[A]? */
			if(GTK_TOGGLE_BUTTON_GET_ACTIVE(color_type_rgb_radio))
			{
				if(GTK_TOGGLE_BUTTON_GET_ACTIVE(alpha_toggle))
					*color_type = 4;	/* RGBA */
				else
					*color_type = 3;	/* RGB */
			}
			/* Greyscale [Alpha] */
			else if(GTK_TOGGLE_BUTTON_GET_ACTIVE(color_type_greyscale_radio))
			{
				if(GTK_TOGGLE_BUTTON_GET_ACTIVE(alpha_toggle))
					*color_type = 2;	/* Greyscale Alpha */
				else
					*color_type = 1;	/* Greyscale */
			}
			/* Black & White */
			else if(GTK_TOGGLE_BUTTON_GET_ACTIVE(color_type_bw_radio))
			{
				*color_type = 0;
			}

			/* If the compression type is CCITT RLE, CCITT
			 * FAX3, or CCITT FAX4 then the color type must
			 * be Black & White
			 */
			if(compression_type != NULL)
			{
				switch(*compression_type)
				{
				    case 3:	/* CCITT RLE */
				    case 4:	/* CCITT FAX3 */
				    case 5:	/* CCITT FAX4 */
					*color_type = 0;	/* Black & White */
					break;
				}
			}
		}
	}


	/* Destroy the dialog */
	gtk_widget_hide(toplevel);
	gtk_window_set_modal(GTK_WINDOW(toplevel), FALSE);
	gtk_window_set_transient_for(GTK_WINDOW(toplevel), NULL);

	GTK_WIDGET_DESTROY(toplevel);
	GTK_ACCEL_GROUP_UNREF(accelgrp);

	return(got_response);
}


/*
 *	Queries the user for XPM format save options.
 */
gint iv_save_options_xpm_query(
	GtkWidget *ref_toplevel,
	const gchar *path,
	gchar **c_id,
	gint *color_type,			/* 0 = Black & White
						 * 1 = Greyscale
						 * 2 = Color Index */
	gint *max_colors,			/* -1 for no limit */
	guint8 *alpha_threshold			/* 0x00 to 0xFF */
)
{
	gboolean	got_response = FALSE;
	gint		width = 320,
			height = -1,
			border_major = 5,
			border_minor = 2;
	GtkAdjustment	*alpha_threshold_adj = NULL;
	GtkAccelGroup	*accelgrp = NULL;
	GtkWidget	*toplevel = NULL,
			*client_vbox = NULL,
			*c_id_entry = NULL,
			*color_type_bw_radio = NULL,
			*color_type_greyscale_radio = NULL,
			*color_type_color_index_radio = NULL,
			*max_colors_spin = NULL,
			*max_colors_parent = NULL,
			*no_max_colors_check = NULL,
			*alpha_threshold_scale = NULL,
			*alpha_threshold_entry = NULL;

	/* Create the base dialog GtkWidgets */
	build_std_dialog(
		width, height,
		"XPM Format Options",
		ref_toplevel,
		&toplevel,
		&client_vbox,
		&accelgrp,
		&got_response
	);

	/* Create XPM format-specific option GtkWidgets */
	if(client_vbox != NULL)
	{
		GSList *gslist;
		GtkAdjustment *adj;
		GtkWidget	*w,
					*parent = client_vbox,
					*parent2, *parent3, *parent4;

		w = gtk_hbox_new(FALSE, border_major);
		gtk_box_pack_start(GTK_BOX(parent), w, TRUE, TRUE, 0);
		gtk_widget_show(w);
		parent = w;

		/* Left Column GtkVBox */
		w = gtk_vbox_new(TRUE, 0);
		gtk_box_pack_start(GTK_BOX(parent), w, FALSE, FALSE, 0);
		gtk_widget_show(w);
		parent2 = w;

		/* Icon */
		w = gtk_pixmap_new_from_xpm_d(
			toplevel->window, NULL,
			(guint8 **)icon_save_32x32_xpm
		);
		gtk_box_pack_start(GTK_BOX(parent2), w, FALSE, FALSE, 0);
		gtk_widget_show(w);


		/* Right Column GtkVBox */
		w = gtk_vbox_new(FALSE, border_major);
		gtk_box_pack_start(GTK_BOX(parent), w, TRUE, TRUE, 0);
		gtk_widget_show(w);
		parent2 = w;

		/* C ID GtkHBox */
		w = gtk_hbox_new(FALSE, border_minor);
		gtk_box_pack_start(GTK_BOX(parent2), w, FALSE, FALSE, 0);
		gtk_widget_show(w);
		parent3 = w;

		/* C ID GtkLabel */
		w = gtk_label_new("C ID:");
		gtk_label_set_justify(GTK_LABEL(w), GTK_JUSTIFY_RIGHT);
		gtk_box_pack_start(GTK_BOX(parent3), w, FALSE, FALSE, 0);
		gtk_widget_show(w);
		/* C ID GtkEntry */
		c_id_entry = w = gtk_entry_new();
		gtk_box_pack_start(GTK_BOX(parent3), w, TRUE, TRUE, 0);
		GUIEditableEndowPopupMenu(w, 0);
		GUISetWidgetTip(w, TOOLTIP_XPM_C_ID);
		gtk_widget_show(w);


		/* Color Type GtkHBox */
		w = gtk_hbox_new(FALSE, border_minor);
		gtk_box_pack_start(GTK_BOX(parent2), w, FALSE, FALSE, 0);
		gtk_widget_show(w);
		parent3 = w;
			   
		/* Black & White GtkRadioButton */
		gslist = NULL;
		color_type_bw_radio = w =
			gtk_radio_button_new_with_label(gslist, "Black & White");
		gtk_box_pack_start(GTK_BOX(parent3), w, FALSE, FALSE, 0);
		gtk_widget_show(w);
		gslist = gtk_radio_button_group(GTK_RADIO_BUTTON(w));

		/* Greyscale GtkRadioButton */
		color_type_greyscale_radio = w =
			gtk_radio_button_new_with_label(gslist, "Greyscale");
		gtk_box_pack_start(GTK_BOX(parent3), w, FALSE, FALSE, 0);
		gtk_widget_show(w);
		gslist = gtk_radio_button_group(GTK_RADIO_BUTTON(w));

		/* Color GtkRadioButton */
		color_type_color_index_radio = w =
			gtk_radio_button_new_with_label(gslist, "Color");
		gtk_box_pack_start(GTK_BOX(parent3), w, FALSE, FALSE, 0);
		gtk_widget_show(w);


		w = gtk_hbox_new(FALSE, border_major);
		gtk_box_pack_start(GTK_BOX(parent2), w, FALSE, FALSE, 0);
		gtk_widget_show(w);
		parent3 = w;

		/* Maximum Colors GtkHBox */
		max_colors_parent = w = gtk_hbox_new(FALSE, border_minor);
		gtk_box_pack_start(GTK_BOX(parent3), w, FALSE, FALSE, 0);
		gtk_widget_show(w);
		parent4 = w;

		/* Maximum Colors GtkLabel */
		w = gtk_label_new("Maximum Colors:");
		gtk_label_set_justify(GTK_LABEL(w), GTK_JUSTIFY_RIGHT);
		gtk_box_pack_start(GTK_BOX(parent4), w, FALSE, FALSE, 0);
		gtk_widget_show(w);
		/* Maximum Colors GtkAdjustment */
		adj = (GtkAdjustment *)gtk_adjustment_new(
			0.0f,				/* Value */
			0.0f,				/* Lower */
			(gfloat)((guint32)-1),		/* Upper */
			1.0f,				/* Step increment */
			10.0f,				/* Page increment */
			0.0f				/* Page size */
		);
		/* Maximum Colors GtkSpinButton */
		max_colors_spin = w = gtk_spin_button_new(adj, 1.0, 0);
		gtk_widget_set_usize(w, 80, -1);
		gtk_box_pack_start(GTK_BOX(parent4), w, FALSE, FALSE, 0);
		GUIEditableEndowPopupMenu(w, 0);
		GUISetWidgetTip(w, TOOLTIP_XPM_MAX_COLORS);
		gtk_widget_show(w);

		/* No Maximum Colors GtkCheckButton */ 
		no_max_colors_check = w = gtk_check_button_new_with_label(
			"No Limit"
		);
		gtk_box_pack_start(GTK_BOX(parent3), w, FALSE, FALSE, 0);
		gtk_signal_connect(
			GTK_OBJECT(w), "toggled",
			GTK_SIGNAL_FUNC(toggle_set_widget_insensitive_cb), max_colors_parent
		);
		GUISetWidgetTip(w, TOOLTIP_XPM_NO_MAX_COLORS);
		gtk_widget_show(w);


		/* Alpha Threshold GtkHBox */
		w = gtk_hbox_new(FALSE, border_minor);
		gtk_box_pack_start(GTK_BOX(parent2), w, FALSE, FALSE, 0);
		gtk_widget_show(w);
		parent3 = w;
		/* Alpha Threshold GtkLabel */
		w = gtk_label_new("Threshold:");
		gtk_label_set_justify(GTK_LABEL(w), GTK_JUSTIFY_RIGHT);
		gtk_box_pack_start(GTK_BOX(parent3), w, FALSE, FALSE, 0);
		gtk_widget_show(w);
		/* Alpha Threshold GtkAdjustment */ 
		alpha_threshold_adj = adj = (GtkAdjustment *)gtk_adjustment_new(
			(gfloat)0x00,			/* Value */
			(gfloat)0x00,			/* Lower */
			(gfloat)0xFF,			/* Upper */
			(gfloat)0x08,			/* Step inc */
			(gfloat)0x20,			/* Page inc */
			0.0f				/* Page size */
		);
		/* Alpha Threshold GtkHScale */ 
		alpha_threshold_scale = w = gtk_hscale_new(adj);
		gtk_box_pack_start(GTK_BOX(parent3), w, TRUE, TRUE, 0);
		gtk_scale_set_draw_value(GTK_SCALE(w), FALSE);
		GUISetWidgetTip(w, TOOLTIP_XPM_THRESHOLD);
		gtk_widget_show(w);
		/* Alpha Threshold GtkEntry */
		alpha_threshold_entry = w = gtk_entry_new_with_max_length(80);
		gtk_widget_set_usize(w, 30, -1);
		gtk_box_pack_start(GTK_BOX(parent3), w, FALSE, FALSE, 0);
		GUIEditableEndowPopupMenu(w, 0);
		GUISetWidgetTip(w, TOOLTIP_XPM_THRESHOLD);
		gtk_widget_show(w);
		/* Connect the Alpha Threshold GtkAdjustment and GtkEntry
		 * signals togeather so their values get updated togeather
		 */
		gtk_signal_connect(
			GTK_OBJECT(alpha_threshold_adj), "value_changed",
			GTK_SIGNAL_FUNC(scale_changed_cb), alpha_threshold_entry
		);
		gtk_signal_connect(
			GTK_OBJECT(alpha_threshold_entry), "changed",
			GTK_SIGNAL_FUNC(entry_changed_adj_cb), alpha_threshold_adj
		);


		/* Set the initial values on the GtkWidgets */

		/* C ID */
		if(c_id != NULL)
		{
			const gchar *s = *c_id;
			GtkEntry *entry = GTK_ENTRY(c_id_entry);
			if(!STRISEMPTY(s))
			{
				gtk_entry_set_text(entry, s);
				gtk_entry_set_position(entry, -1);
			}
		}

		/* Color Type */
		if(color_type != NULL)
		{
			GtkWidget *w = NULL;
			switch(*color_type)
			{
			  case 0:			/* Black & White */
				w = color_type_bw_radio;
				break;
			  case 1:			/* Greyscale */
				w = color_type_greyscale_radio;
				break;
			  case 2:			/* Color Index */
				w = color_type_color_index_radio;
				break;
			}
			if(w != NULL)
			{
				gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(w), TRUE);
				gtk_toggle_button_toggled(GTK_TOGGLE_BUTTON(w));
			}
		}

		/* Max Colors */
		if(max_colors != NULL)
		{
			gtk_toggle_button_set_active(
				GTK_TOGGLE_BUTTON(no_max_colors_check),
				(*max_colors > 0) ? FALSE : TRUE
			);
			gtk_toggle_button_toggled(GTK_TOGGLE_BUTTON(no_max_colors_check));

			gtk_spin_button_set_value(
				GTK_SPIN_BUTTON(max_colors_spin),
				(gfloat)(*max_colors)
			);
			gtk_spin_button_update(GTK_SPIN_BUTTON(max_colors_spin));
		}

		/* Threshold */
		if(alpha_threshold != NULL)
		{
			GTK_ADJUSTMENT_SET_VALUE(
				alpha_threshold_adj,
				(gfloat)(*alpha_threshold)
			);
			gtk_adjustment_changed(alpha_threshold_adj);
		}
	}


	/* Map the dialog */
	gtk_widget_show_raise(toplevel);


	/* Push main loop to wait for user response */
	gtk_main();


	/* User clicked on save? */
	if(got_response)
	{
		/* Get the values */
		if(c_id != NULL)
		{
			g_free(*c_id);
			*c_id = STRDUP(gtk_entry_get_text(GTK_ENTRY(c_id_entry)));
		}

		if(color_type != NULL)
		{
			if(GTK_TOGGLE_BUTTON_GET_ACTIVE(color_type_bw_radio))
				*color_type = 0;
			else if(GTK_TOGGLE_BUTTON_GET_ACTIVE(color_type_greyscale_radio))
				*color_type = 1;
			else if(GTK_TOGGLE_BUTTON_GET_ACTIVE(color_type_color_index_radio))
				*color_type = 2;
			else
				*color_type = 2;
		}

		if(max_colors != NULL)
		{
			if(GTK_TOGGLE_BUTTON_GET_ACTIVE(no_max_colors_check))
				*max_colors = -1;
			else
				*max_colors = GTK_ENTRY_GET_VALUEI(max_colors_spin);
		}

		if(alpha_threshold != NULL)
			*alpha_threshold = (guint8)GTK_ADJUSTMENT_GET_VALUE(alpha_threshold_adj);
	}


	/* Destroy the dialog */
	gtk_widget_hide(toplevel);
	gtk_window_set_modal(GTK_WINDOW(toplevel), FALSE);
	gtk_window_set_transient_for(GTK_WINDOW(toplevel), NULL);

	GTK_WIDGET_DESTROY(toplevel);
	GTK_ACCEL_GROUP_UNREF(accelgrp);

	return(got_response);
}
