#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <ctype.h>
#include <sys/stat.h>
#include <signal.h>
#include <math.h>
#if defined(HAVE_IMLIB)
# include <Imlib.h>
#elif defined(HAVE_IMLIB2)
# include <X11/Xlib.h>
# include <Imlib2.h>
#endif
#include <gtk/gtk.h>
#include <gdk/gdkkeysyms.h>
#if defined(_WIN32)
# include <gdk/gdkwin32.h>
# include <direct.h>
#else
# include <gdk/gdkx.h>
# define HAVE_X
# include <unistd.h>
#endif
#ifdef HAVE_LIBENDEAVOUR2
# include <endeavour2.h>
#endif

#include "../include/string.h"

#include "libps.h"
#include "imgio.h"

#include "guiutils.h"
#include "guirgbimg.h"
#include "cdialog.h"
#include "fprompt.h"
#include "csd.h"
#include "fsd.h"
#include "fb.h"
#include "pdialog.h"
#include "progressdialog.h"

#include "fullscreen_img.h"
#include "desktop.h"
#if !defined(_WIN32)
# include "screenshot.h"
# include "print_values.h"
# include "print.h"
#endif

#include "tview.h"
#include "hview.h"
#include "imgview.h"

#include "resize_dlg.h"
#include "color_levels_dlg.h"
#include "text_dlg.h"
#include "header_dlg.h"
#include "frames_dlg.h"
#if !defined(_WIN32)
# include "screenshot_dlg.h"
#endif
#include "save_options_dlg.h"
#include "img_info_dlg.h"
#if !defined(_WIN32)
# include "print_dlg.h"
#endif

#include "iv_title.h"

#include "config.h"
#include "compiletimeinfo.h"

#include "images/icon_edit_20x20.xpm"
#include "images/icon_resize_20x20.xpm"
#include "images/icon_color_levels_20x20.xpm"
#include "images/icon_fonts_20x20.xpm"
#include "images/icon_animate_20x20.xpm"
#include "images/icon_camera_20x20.xpm"
#include "images/icon_open_20x20.xpm"
#include "images/icon_save_as_20x20.xpm"
#include "images/icon_info_20x20.xpm"
#include "images/icon_monitor2_20x20.xpm"
#include "images/icon_desktop_20x20.xpm"
#include "images/icon_root_fit_20x20.xpm"
#include "images/icon_root_centered_20x20.xpm"
#include "images/icon_root_tiled_20x20.xpm"
#include "images/icon_root_cover_20x20.xpm"
#include "images/icon_clear_20x20.xpm"
#include "images/icon_close_20x20.xpm"
#include "images/icon_print2_20x20.xpm"
#include "images/icon_iv_20x20.xpm"

#include "images/icon_error_32x32.xpm"
#include "images/icon_edit_48x48.xpm"


/*
 *	Options:
 */
typedef enum {
	IV_OPTION_SHOW_TOOLTIPS		= (1 << 0),
	IV_OPTION_SHOW_TOOL_BAR		= (1 << 1),
	IV_OPTION_SHOW_STATUS_BAR	= (1 << 2),
	IV_OPTION_SHOW_VALUES		= (1 << 3),
	IV_OPTION_SHOW_IMAGE_ON_WM_ICON	= (1 << 4),
	IV_OPTION_HIDE_IV_NON_IMAGE	= (1 << 5),	/* Hide the Image Viewer
													 * after opening a
													 * non-image file */
	IV_OPTION_SIMPLE_TITLE		= (1 << 6),	/* Show the "less
													 * graphical" title
													 * instead of the
													 * standard IV title
													 * image */
	IV_OPTION_RESIZE_ON_OPEN	= (1 << 7),	/* Automatically resize
													 * the Image Viewer when
													 * opening a new image */
	IV_OPTION_ZOOM_TO_FIT_ON_OPEN	= (1 << 8)	/* Automatically zoom
													 * to fit the image
													 * when opening a new
													 * image */
} IVOptionFlags;
#define IV_OPTIONS_DEFAULT		(IV_OPTION_SHOW_TOOLTIPS |	\
									 IV_OPTION_SHOW_TOOL_BAR |	\
									 IV_OPTION_SHOW_STATUS_BAR |	\
									 IV_OPTION_SHOW_VALUES |	\
									 IV_OPTION_RESIZE_ON_OPEN)


/*
 *	Has changes string, displayed on title
 */
#define IV_HAS_CHANGES_STR		"(*)"


typedef struct _IVImgIOCBData		IVImgIOCBData;
#define IV_IMGIO_CB(p)			((IVImgIOCBData *)(p))

typedef struct _IVInfo			IVInfo;
#define IV_INFO(p)			((IVInfo *)(p))

typedef struct _IVImageViewer		IVImageViewer;
#define IV_IMAGE_VIEWER(p)		((IVImageViewer *)(p))

typedef struct _IVDataViewer		IVDataViewer;
#define IV_DATA_VIEWER(p)		((IVDataViewer *)(p))

typedef struct _IVCore			IVCore;
#define IV_CORE(p)			((IVCore *)(p))


/*
 *	ImgIO Callback Data:
 */
struct _IVImgIOCBData {

	IVOptionFlags	options;
	GtkWidget	*ref_toplevel;

	IVImageViewer	*viewer;
	imgview_image_struct	*img;
	guint8		last_bg_color[4];

	gfloat		last_progress_coeff;

	gchar		*path;
};

/*
 *	Image Information:
 */
struct _IVInfo {

	gchar		*path;			/* Can be NULL if never been saved */
	gint		width,			/* Size in pixels */
					height;
	gint		nframes,
					nloops;			/* Number of loops or -1 for
											 * infinite */
	gulong		play_time_ms,		/* Play time for one loop in
											 * milliseconds */
					total_play_time_ms;	/* Total play time for all
											 * loops in milliseconds or
											 * (gulong)-1 for infinite */
	gint		x, y;			/* Translation from upper-left in pixels */
	gint		base_width,		/* Base/boundingbox/screen size in pixels */
					base_height;
	gulong		file_size;		/* Size in bytes on disk, can be 0
											 * if not been saved yet */
	gboolean	has_alpha;		/* TRUE if alpha channel exists */
	gchar		*creator;		/* Program that created the image */
	gchar		*title;
	gchar		*author;
	gchar		*comments;
	gulong		time_modified;		/* Last modified time in seconds
											 * since EPOCH */

};

/*
 *	IV Image Viewer:
 */
struct _IVImageViewer {

	imgview_struct	*iv;
	IVCore		*core;
	gboolean	processing,
					has_changes;
	int		stop_count;

	/* Submenus */
	GtkWidget	*edit_submenu,
					*desktop_submenu;

	/* Menu items (added to the ImgView's menu) */
	GtkWidget	*set_bg_mi,
					*edit_mi,
					*resize_mi,
					*color_levels_mi,
					*add_text_mi,
					*edit_header_mi,
					*edit_frames_mi,
					*open_mi,
					*save_as_mi,
					*screenshot_mi,
					*information_mi,
					*fullscreen_mi,
					*desktop_mi,
					*clear_mi,
					*print_mi,
					*new_viewer_mi;

	/* Image Information */
	IVInfo		info;

	/* Foreground & Background Colors
	 *
	 * 4 bytes in RGBA format
	 */
	guint8		fg_color[4],
					bg_color[4];

};

/*
 *	IV Data Viewer:
 */
struct _IVDataViewer {

	GtkWidget	*toplevel;
	GtkAccelGroup	*accelgrp;
	gint		freeze_count;
	IVCore	*core;

	tview_struct	*tv;
	hview_struct	*hv;

	GtkWidget	*text_tb,
					*hex_tb,
					*close_btn;
};

/*
 *	UV Core:
 */
struct _IVCore {

	/* Current directory */
	gchar		*current_directory_path;

	/* Image Viewers */
	GList		*iv_viewer;

	/* Data Viewers */
	GList		*iv_fviewer;

	/* Image Info Dialogs */
	GList		*imginfodlg;

	/* Main timeout callback */
	guint		main_timeout_id;

	/* Close all windows marker */
	gboolean	close_all_windows;

	/* Untitled image counter, increments when a new image is
	 * created or obtained from a screenshot */
	gint		untitled_count;

	/* Cursors */
	GdkCursor	*grab_cur;

	/* GtkWindow used for screenshot operations, pointer events
	 * will be specified to be relative to this GtkWindow
	 */
	GtkWidget	*grab_window;

	/* Options set from the command line at startup, they do not
	 * reflect current values, so do not modify
	 *
	 * Check imgview for current values
	 */
	IVOptionFlags	options;
	gint		quality;		/* 0 to 2 */
	guint8		bg_color[4];		/* 4 bytes RGBA */
	gboolean	bg_color_set;		/* Background color was set
						 * from the command line */
 	gchar		*font_name,		/* User defined font name */
 			*wmname,		/* User defined WM name */
			*wmclass,		/* User defined WM class */
			*title;			/* User defined title */
	GdkGeometryFlags geometry_flags;	/* User defined geometry flags */
	GdkRectangle	*geometry;		/* User defined geometry */
	gint		imlib_fallback;		/* 0 = No Fallbacks
						 * 1 = Use ImageMagick & NETPBM */

	/* File Format Save Options */
	gboolean	gif_interlaced;		/* FALSE = Progressive
						 * TRUE = Interlaced */
	gint		gif_color_type;		/* 0 = B&W
						 * 1 = Greyscale
						 * 2 = Color */
	gboolean	gif_transparency;
	guint8		gif_alpha_threshold;
	gboolean	gif_looping;		/* FALSE = Play once
						 * TRUE = Repeating */

	gfloat		jpeg_quality;		/* 0.0 to 1.0 */
	gboolean	jpeg_is_color;		/* FALSE = Greyscale
						 * TRUE = Color */

	gfloat		jng_quality;		/* 0.0 to 1.0 */
	gboolean	jng_is_color,		/* TRUE = Color
						 * FALSE = Greyscale */
			jng_transparency;

	gint		mng_compression;	/* 0 to 9 */
	gint		mng_format;		/* 0 = B&W
						 * 1 = Greyscale
						 * 2 = Greyscale Alpha
						 * 3 = RGB
						 * 4 = RGBA */
	gboolean	mng_interlaced;
	gboolean	mng_looping;

	gint		png_compression;	/* 0 to 9 */
	gint		png_format;		/* 0 = B&W
						 * 1 = Greyscale
						 * 2 = Greyscale Alpha
						 * 3 = RGB
						 * 4 = RGBA */
	gboolean	png_interlaced;

	gint		tga_format;		/* 0 = Greyscale
						 * 1 = RGB
						 * 2 = RGBA */
	gint		tiff_compression_type;	/* 0 = None
						 * 1 = GZIP
						 * 2 = JPEG
						 * 3 = CCITT RLE
						 * 4 = CCITT FAX3
						 * 5 = CCITT FAX4 */
	gfloat		tiff_jpeg_quality;	/* 0.0 to 1.0 */
	gint		tiff_color_type;	/* 0 = B&W
						 * 1 = Greyscale
						 * 2 = Greyscale Alpha
						 * 3 = RGB
						 * 4 = RGBA */

	gint		xpm_format;		/* 0 = B&W
						 * 1 = Greyscale
						 * 2 = Color */
	gint		xpm_max_colors;		/* -1 for no limit */
	guint8		xpm_threshold;		/* 0x00 to 0xff */

	/* Screenshot Options */
	gulong		screenshot_delay_ms;	/* In milliseconds */
	gboolean	screenshot_hide_windows;
	gboolean	screenshot_include_decorations;
	gboolean	screenshot_record_date_in_comments;
	gboolean	screenshot_auto_save;
	gchar		*screenshot_save_dir_path;
	gchar		*screenshot_save_ext;

	/* Resize Options */
	gboolean	resize_maintain_aspect;

	/* Color Levels Options */
	gboolean	color_levels_all_frames;

	/* Add Text Options */
	gchar		*add_text_font_name;
	GdkColor	add_text_color;
	gboolean	add_text_outline;
	GdkColor	add_text_outline_color;
	gfloat		add_text_opacity;
	gboolean	add_text_all_frames;

	/* Print Options */
#if !defined(_WIN32)
	print_values_struct	*print_values;
#endif

#ifdef HAVE_IMLIB
	/* Imlib Handle */
	gpointer	imlib_h;
#endif

#ifdef HAVE_LIBENDEAVOUR2
	/* Endeavour 2 Context */
	EDVContext	*edv2_ctx;
#endif

};


static IVCore		*iv_core;


/* System Signal Callback */
static void iv_signal_cb(int s);

/* Image Viewer General Callbacks */
static gint iv_viewer_delete_event_cb(
	GtkWidget *widget, GdkEvent *event, gpointer data
);
static gint iv_viewer_key_event_cb(
	GtkWidget *widget, GdkEventKey *key, gpointer data
);
static void iv_viewer_dnd_recieved_cb(
	GtkWidget *widget, GdkDragContext *dc, gint x, gint y,
	GtkSelectionData *selection_data, guint info, guint t,
	gpointer data
);

/* Data Viewer General Callbacks */
static gint iv_fviewer_delete_event_cb(
	GtkWidget *widget, GdkEvent *event, gpointer data
);
static void iv_fviewer_display_format_toggle_cb(
	GtkWidget *widget, gpointer data
);
static void iv_fviewer_close_cb(GtkWidget *widget, gpointer data);

/* Main Timeout Callback */
static gint iv_main_timeout_cb(gpointer data);

/* File Selector Callbacks */
static void iv_file_selector_created_cb(
	const gchar *path, gpointer data
);
static void iv_file_selector_modified_cb(
	const gchar *old_path,
	const gchar *new_path,
	gpointer data
);
static void iv_file_selector_deleted_cb(
	const gchar *path, gpointer data
);

/* Core */
static void iv_core_reset(IVCore *core);
static void iv_core_delete(IVCore *core);

/* Image Viewer New/Delete */
static IVImageViewer *iv_viewer_new(
	IVCore *core,
	const IVOptionFlags options,
	const gint quality,			/* 0 to 2 */
	const guint8 *bg_color,			/* 4 bytes RGBA */
	const GdkGeometryFlags geometry_flags,
	const GdkRectangle *geometry,
	const gboolean load_title
);
static void iv_viewer_delete(IVImageViewer *v);

/* Data Viewer New/Delete */
static IVDataViewer *iv_data_viewer_new(
	IVCore *core,
	const GdkGeometryFlags geometry_flags,
	const GdkRectangle *geometry
);
static void iv_fviewer_delete(IVDataViewer *fv);

/* Image Viewer Operations */
static void iv_viewer_update_image_info(
	IVImageViewer *v,
	const gchar *path,
	const struct stat *stat_buf,
	const gint width, const gint height,
	const gint bpp, const gint bpl,
	const gint nframes,
	const gint nloops,
	const gulong play_time_ms,		/* In milliseconds */
	const gulong total_play_time_ms,	/* In milliseconds */
	const gboolean has_alpha,
	const gint x, const gint y,
	const gint base_width,
	const gint base_height,
	const gchar *creator,
	const gchar *title,
	const gchar *author,
	const gchar *comments,
	const gulong time_modified		/* In seconds since EPOCH */
);
static void iv_viewer_open(
	IVImageViewer *v,
	const gchar *path,
	const gint frame_num
);
static void iv_viewer_save(
	IVImageViewer *v,
	const gchar *path,
	const gchar *extensions
);
#if !defined(_WIN32)
static void iv_viewer_screenshot(IVImageViewer *v);
#endif

/* Command Line Only Operations */
static gint iv_open_fullscreen(
	const gchar *path,
	const gint frame_num,			/* Can be -1 for unspecified */
	guint8 *bg_color,			/* 4 bytes in RGBA format */
	const gint quality			/* Quality, 0 to 2 */
);
static gint iv_open_to_root_centered(
	const gchar *path,
	const gint frame_num,			/* Can be -1 for unspecified */
	guint8 *bg_color,			/* 4 bytes in RGBA format */
	const gint quality			/* Quality, 0 to 2 */
);
static gint iv_open_to_root_tofit(
	const gchar *path,
	const gint frame_num,			/* Can be -1 for unspecified */
	guint8 *bg_color,			/* 4 bytes in RGBA format */
	const gint quality			/* Quality, 0 to 2 */
);
static gint iv_open_to_root_tocover(
	const gchar *path,
	const gint frame_num,			/* Can be -1 for unspecified */
	guint8 *bg_color,			/* 4 bytes in RGBA format */
	const gint quality			/* Quality, 0 to 2 */
);
static gint iv_open_to_root_tiled(
	const gchar *path,
	const gint frame_num,			/* Can be -1 for unspecified */
	guint8 *bg_color,			/* 4 bytes in RGBA format */
	const gint quality			/* Quality, 0 to 2 */
);
static gint iv_open_print(
	const gchar *path,
	const gint frame_num,			/* Can be -1 for unspecified */
	guint8 *bg_color			/* 4 bytes in RGBA format */
);

/* Open With New Data Viewer */
static IVDataViewer *iv_data_viewer_new_open_file(
	IVCore *core,
	const gchar *path,
	GtkWidget *toplevel
);

/* Image Viewer Check Query Save Changes */
static gboolean iv_viewer_check_query_save_changes(IVImageViewer *v);

/* Image Viewer Callbacks */
static void iv_imgview_changed_cb(
	imgview_struct *iv,
	imgview_image_struct *image,
	gpointer data
);
static void iv_viewer_set_bg_color_cb(GtkWidget *widget, gpointer data);
static void iv_viewer_resize_cb(GtkWidget *widget, gpointer data);
static void iv_viewer_color_levels_cb(GtkWidget *widget, gpointer data);
static void iv_viewer_add_text_cb(GtkWidget *widget, gpointer data);
static void iv_viewer_header_cb(GtkWidget *widget, gpointer data);
static void iv_viewer_frames_cb(GtkWidget *widget, gpointer data);
#if !defined(_WIN32)
static void iv_viewer_screenshot_cb(GtkWidget *widget, gpointer data);
#endif
static gint iv_open_progress_cb(
	gpointer progress_data,
	const gint i, const gint n,
	const gint width, const gint height,
	const gint bpl, const gint bpp,
	const guint8 *image_data
);
static gint iv_save_progress_cb(
	gpointer progress_data,
	const gint i, const gint n,
	const gint width, const gint height,
	const gint bpl, const gint bpp,
	const guint8 *image_data
);
static void iv_viewer_open_image_cb(GtkWidget *widget, gpointer data);
static void iv_viewer_open_other_cb(GtkWidget *widget, gpointer data);
static void iv_viewer_save_cb(GtkWidget *widget, gpointer data);
static void iv_viewer_display_on_root_centered_cb(
	GtkWidget *widget, gpointer data
);
static void iv_viewer_display_on_root_tofit_cb(
	GtkWidget *widget, gpointer data
);
static void iv_viewer_display_on_root_tocover_cb(
	GtkWidget *widget, gpointer data
);
static void iv_viewer_display_on_root_tiled_cb(
	GtkWidget *widget, gpointer data
);
static void iv_viewer_clear_cb(GtkWidget *widget, gpointer data);
static void iv_viewer_information_cb(GtkWidget *widget, gpointer data);
static void iv_viewer_fullscreen_cb(GtkWidget *widget, gpointer data);
#if !defined(_WIN32)
static void iv_viewer_print_cb(GtkWidget *widget, gpointer data);
#endif
static void iv_viewer_new_viewer_cb(GtkWidget *widget, gpointer data);

/* Image Viewer Updating */
static void iv_viewer_update_title(
	IVImageViewer *v,
	const gchar *new_path
);
static void iv_viewer_update(IVImageViewer *v);
static void iv_viewer_resize(
	IVImageViewer *v,
	const gint width, const gint height,
	const gboolean keep_visible_on_desktop
);
static void iv_clear_info(IVInfo *info);

/* Command Line Utilities */
static void get_args_value(
	char *arg, char **parm_rtn, char **val_rtn
);
static gboolean stristrue(const gchar *s);


#define ATOI(s)		(((s) != NULL) ? atoi(s) : 0)
#define ATOL(s)		(((s) != NULL) ? atol(s) : 0)
#define ATOF(s)		(((s) != NULL) ? atof(s) : 0.0f)
#define STRDUP(s)	(((s) != NULL) ? g_strdup(s) : NULL)

#define MAX(a,b)	(((a) > (b)) ? (a) : (b))
#define MIN(a,b)	(((a) < (b)) ? (a) : (b))
#define CLIP(a,l,h)	(MIN(MAX((a),(l)),(h)))
#define STRLEN(s)	(((s) != NULL) ? strlen(s) : 0)
#define STRISEMPTY(s)	(((s) != NULL) ? (*(s) == '\0') : TRUE)

#define ABSOLUTE_VALUE(x)	(((x) < 0) ? ((x) * -1) : (x))

#define UNLINK(p)	((!STRISEMPTY(p)) ? unlink(p) : -1)


#define PRINT_ERROR_TO_DESKTOP(_s_)	{		\
 GdkWindow *root_win = gdk_window_get_root();		\
 GdkBitmap *mask;					\
 GdkPixmap *pixmap;					\
 GdkGC *gc = gdk_gc_new(root_win);			\
 GtkStyle *style = gtk_widget_get_default_style();	\
													\
 pixmap = gdk_pixmap_create_from_xpm_d(			\
  root_win, &mask,					\
  (style != NULL) ? &style->bg[GTK_STATE_NORMAL] : NULL,\
  (gchar **)icon_error_32x32_xpm			\
 );							\
													\
 iv_desktop_mesg(					\
  root_win, gc, style, GTK_JUSTIFY_LEFT,		\
  PROG_NAME_FULL " Version " PROG_VERSION,		\
  pixmap, mask, (_s_)					\
 );							\
													\
 GDK_GC_UNREF(gc);					\
 GDK_PIXMAP_UNREF(pixmap);				\
 GDK_BITMAP_UNREF(mask);				\
}


/*
 *	System signal callback.
 */
static void iv_signal_cb(int s)
{
	switch(s)
	{
#ifdef SIGHUP
	  case SIGHUP:
		if(iv_core != NULL)
		{



		}
		break;
#endif
#ifdef SIGINT
	  case SIGINT:
		if(iv_core != NULL)
		{
			IVCore *core = iv_core;

			/* Is there a screenshot in progress? */
			if(core->grab_window != NULL)
			{
				/* Destroy the grab window to break the grab */
				gtk_widget_hide(core->grab_window);
				GTK_WIDGET_DESTROY(core->grab_window);
				core->grab_window = NULL;
			}

			core->close_all_windows = TRUE;
		}
		break;
#endif
#ifdef SIGTERM
	  case SIGTERM:
		exit(0);
		break;
#endif
#ifdef SIGQUIT
	  case SIGQUIT:
		if(iv_core != NULL)
		{
			IVCore *core = iv_core;

			/* Is there a screenshot in progress? */
			if(core->grab_window != NULL)
			{
				/* Destroy the grab window to break the grab */
				gtk_widget_hide(core->grab_window);
				GTK_WIDGET_DESTROY(core->grab_window);
				core->grab_window = NULL;
			}

			core->close_all_windows = TRUE;
		}
		break;
#endif
#ifdef SIGSEGV
	  case SIGSEGV:
		exit(1);
		break;
#endif
#ifdef SIGPIPE
	  case SIGPIPE:
		break;
#endif
	}
}


/*
 *	Image Viewer toplevel GtkWindow "delete_event" signal callback.
 */
static gint iv_viewer_delete_event_cb(
	GtkWidget *widget, GdkEvent *event, gpointer data
)
{
	imgview_struct *iv;
	IVImageViewer *v = IV_IMAGE_VIEWER(data);
	IVCore *core;
	if(v == NULL)
		return(FALSE);

	iv = v->iv;
	core = v->core;
	if((iv == NULL) || (core == NULL))
		return(TRUE);

	if(v->processing)
	{
		v->stop_count++;
		return(TRUE);
	}

	/* Check if there are any unsaved changes and query user to
	 * save those changes
	 */
	if(!iv_viewer_check_query_save_changes(v))
		return(TRUE);

	/* Unmap the ImgView */
	ImgViewUnmap(iv);

	return(TRUE);
}

/*
 *	Image View "key_press_event" or "key_release_event" signal
 *	callback.
 */
static gint iv_viewer_key_event_cb(
	GtkWidget *widget, GdkEventKey *key, gpointer data
)
{
	gint status = FALSE;
	gboolean keypress;
	guint		keyval,
					keystate;
	imgview_struct *iv;
	IVImageViewer *v = IV_IMAGE_VIEWER(data);
	IVCore *core;
	if(v == NULL)
		return(status);

	iv = v->iv;
	core = v->core;
	if((iv == NULL) || (core == NULL))
		return(status);

	keypress = (key->type == GDK_KEY_PRESS) ? TRUE : FALSE;
	keyval = key->keyval;
	keystate = key->state;

	/* Is the viewer processing? */
	if(v->processing)
	{
		/* Increment the stop counter if the escape key was pressed */
		if((keyval == GDK_Escape) && keypress)
		{
			v->stop_count++;
			status = TRUE;
		}

		return(status);
	}

	/* Handle by key value */
	switch(keyval)
	{
	  case GDK_q:
		/* Exit/Quit */
		if(keystate & GDK_CONTROL_MASK)
		{
			if(keypress)
			{
				GList *glist;

				/* Check for any Image Viewer that still has
				 * unsaved changes
				 */
				for(glist = core->iv_viewer;
					glist != NULL;
					glist = g_list_next(glist)
				)
				{
					if(glist->data != NULL)
					{
						IVImageViewer *v = IV_IMAGE_VIEWER(glist->data);

						/* Check if there are any unsaved changes
						 * and query user to save those changes
						 */
						if(!iv_viewer_check_query_save_changes(v))
							break;
					}
				}
				/* Set close_all_windows to TRUE only if there were
				 * no Image Viewers that have unsaved changes
				 */
				core->close_all_windows = (glist == NULL) ? TRUE : FALSE;
			}
			status = TRUE;
		}
		break;

	  case GDK_Escape:
		/* Close */
		if(keypress)
			iv_viewer_delete_event_cb(
				ImgViewGetToplevelWidget(iv), NULL, data
			);
		status = TRUE;
		break;

	  case GDK_w:
		/* Close */
		if(keystate & GDK_CONTROL_MASK)
		{
			if(!keypress)
				iv_viewer_delete_event_cb(
					ImgViewGetToplevelWidget(iv), NULL, data
				);
			status = TRUE;
		}
		break;

	  case GDK_F3:
		if(keystate & GDK_SHIFT_MASK)
		{
			/* Open File */
			if(keypress)
				iv_viewer_open_other_cb(v->open_mi, data);
			status = TRUE;
		}
		else
		{
		    /* Open the image */
		    if(keypress)
				iv_viewer_open_image_cb(v->open_mi, data);
			status = TRUE;
		}
		break;

	  case GDK_o:
		if(keystate & GDK_SHIFT_MASK)
		{
			/* Open Other File */
			if(keypress)
				iv_viewer_open_other_cb(v->open_mi, data);
			status = TRUE;
		}
		break;

	  case GDK_a:
		/* Save As Image */
		if(keystate & GDK_CONTROL_MASK)
		{
			if(keypress)
				iv_viewer_save_cb(v->save_as_mi, data);
			status = TRUE;
		}
		break;

	  case GDK_p:
		/* Play or pause */
		if(keypress)
		{
			const gint nframes = ImgViewGetTotalFrames(iv);
			if(ImgViewIsPlaying(iv))
				ImgViewPause(iv);
			else if(nframes > 1)
				ImgViewPlay(iv);
		}
		status = TRUE;
		break;

	  case GDK_s:
		/* Stop */
		if(keypress)
		{
			if(ImgViewIsPlaying(iv))
				ImgViewPause(iv);
		}
		status = TRUE;
		break;

	  case '[':
	  case '{':
	  case '<':
	  case ',':
	  case GDK_b:
		/* Seek previous */
		if(keypress)
		{
			const gint nframes = ImgViewGetTotalFrames(iv);
			if(nframes > 1)
			{
				gint frame_num = ImgViewGetCurrentFrame(iv);
				frame_num--;
				if(frame_num < 0)
					frame_num = nframes - 1;
				ImgViewSeek(iv, frame_num);
			}
		}
		status = TRUE;
		break;

	  case ']':
	  case '}':
	  case '>':
	  case '.':
	  case GDK_n:
	  case GDK_space:
		/* Seek next */
		if(keypress)
		{
			const gint nframes = ImgViewGetTotalFrames(iv);
			if(nframes > 1)
			{ 
				gint frame_num = ImgViewGetCurrentFrame(iv);
				frame_num++;
				if(frame_num >= nframes)
					frame_num = 0;
				ImgViewSeek(iv, frame_num);
			}
		}
		status = TRUE;
		break;

	  case GDK_1:
		/* New Image Viewer */
		if(keystate & GDK_CONTROL_MASK)
		{
			if(!keypress)
				iv_viewer_new_viewer_cb(widget, data);
			status = TRUE;
		}
		break;
	}

	return(status);
}

/*
 *	Image View "drag_data_recieved" signal callback.
 */
static void iv_viewer_dnd_recieved_cb(
	GtkWidget *widget, GdkDragContext *dc, gint x, gint y,
	GtkSelectionData *selection_data, guint info, guint t,
	gpointer data
)
{
	GtkWidget *w;
	imgview_struct *iv;
	IVCore *core;
	IVImageViewer *v = IV_IMAGE_VIEWER(data);
	if((widget == NULL) || (dc == NULL) || (v == NULL))
		return;

	if(v->processing)
		return;

	iv = v->iv;
	core = v->core;

	/* Mark viewer as processing */
	ImgViewSetBusy(iv, TRUE);
	v->processing = TRUE;
	v->stop_count = 0;

	/* Get ImgView's view widget */
	w = (GtkWidget *)ImgViewGetViewWidget(iv);

	/* View widget the same as the callback widget? */
	if(w == widget)
	{
		/* Check if the DND target type is one that we support */
		if((info == DND_INFO_TEXT_PLAIN) ||
		   (info == DND_INFO_TEXT_URI_LIST) ||
		   (info == DND_INFO_STRING)
		)
		{
			const gchar	*s,
							*url = (const gchar *)selection_data->data;
			gchar *path;

			/* The DND data should be a URL string, we need to parse
			 * it and get the full path portion of that URL string.
			 */
			s = (const gchar *)strstr((const char *)url, "://");
			if(s != NULL)
				s += strlen("://");
			else
				s = url;

			s = (const gchar *)strchr((const char *)s, '/');
			path = STRDUP(s);

			/* Got the full path to object to be loaded? */
			if(path != NULL)
			{
				gboolean response;

				/* Set processing to FALSE so that the user
				 * can be queried to save changes
				 */
				v->processing = FALSE;

				/* Check if there are any unsaved changes and
				 * if so then query the user to save those changes
				 */
				response = iv_viewer_check_query_save_changes(v);

				v->processing = TRUE;	/* Restore processing */
				v->stop_count = 0;

				if(response)
				{
					/* Record the current working directory */
					gchar *parent = g_dirname(path);
					if(parent != NULL)
					{
						g_free(core->current_directory_path);
						core->current_directory_path = g_strdup(parent);
						(void)chdir(parent);
						g_free(parent);
					}

					/* Open the image */
					iv_viewer_open(
						v,
						path,
						-1			/* Frame unspecified */
					);
				}

				g_free(path);
			}
		}
	}

	/* Mark the viewer as no longer processing */
	v->processing = FALSE;
	ImgViewSetBusy(iv, FALSE);

	/* Update Image Viewer's widgets */
	iv_viewer_update(v);
}


/*
 *	Data Viewer toplevel GtkWindow "delete_event" signal callback.
 */
static gint iv_fviewer_delete_event_cb(
	GtkWidget *widget, GdkEvent *event, gpointer data
)
{
	IVDataViewer *fv = IV_DATA_VIEWER(data);
	if(fv == NULL)
		return(FALSE);

	if(fv->freeze_count > 0)
		return(FALSE);

	/* Unmap the Data Viewer */
	gtk_widget_hide(fv->toplevel);

	return(TRUE);
}

/*
 *	Data Viewer Display Format GtkToggleButton "toggled" signal
 *	callback.
 */
static void iv_fviewer_display_format_toggle_cb(
	GtkWidget *widget, gpointer data
)
{
	GtkToggleButton *tb;
	tview_struct *tv;
	hview_struct *hv;
	IVDataViewer *fv = IV_DATA_VIEWER(data);
	if((widget == NULL) || (fv == NULL))
		return;

	if(!GTK_IS_TOGGLE_BUTTON(widget) ||
	   (fv->freeze_count > 0)
	)
		return;

	fv->freeze_count++;

	tb = GTK_TOGGLE_BUTTON(widget);

	/* Ignore untoggled signals */
	if(!gtk_toggle_button_get_active(tb))
	{
		gtk_toggle_button_set_active(tb, TRUE);
		fv->freeze_count--;
		return;
	}

	tv = fv->tv;
	hv = fv->hv;

	/* Check which widget was toggled */
	if(widget == fv->text_tb)
	{
		HViewUnmap(hv);
		TViewMap(tv);
		gtk_toggle_button_set_active(
			GTK_TOGGLE_BUTTON(fv->hex_tb), FALSE
		);
	}
	else if(widget == fv->hex_tb)
	{
		TViewUnmap(tv);
		HViewMap(hv);
		gtk_toggle_button_set_active(
			GTK_TOGGLE_BUTTON(fv->text_tb), FALSE
		);
	}

	fv->freeze_count--;
}

/*
 *	Data Viewer close callback.
 */
static void iv_fviewer_close_cb(GtkWidget *widget, gpointer data)
{
	IVDataViewer *fv = IV_DATA_VIEWER(data);
	if(fv == NULL)
		return;

	if(fv->freeze_count > 0)
		return;

	gtk_widget_hide(fv->toplevel);
}


/*
 *	Main timeout callback.
 */
static gint iv_main_timeout_cb(gpointer data)
{
	gboolean close_all_windows;
	gint nwindows = 0;
	GList *glist;
	GtkWidget *grab_window;
	IVCore *core = IV_CORE(data);
	if(core == NULL)
		return(FALSE);

	close_all_windows = core->close_all_windows;

	/* Count Dialogs */
	if(CDialogIsQuery())
		nwindows++;
	if(CSDIsQuery())
		nwindows++;
	if(FSDIsQuery())
		nwindows++;
	if(FileBrowserIsQuery())
		nwindows++;
	if(PDialogIsQuery())
		nwindows++;
	if(ProgressDialogIsQuery())
		nwindows++;

	/* Count the grab GtkWindow if it exists (mapped or not) */
	grab_window = core->grab_window;
	if(grab_window != NULL)
		nwindows++;

	/* Manage the Image Viewers */
	for(glist = core->iv_viewer;
		glist != NULL;
		glist = g_list_next(glist)
	)
	{
		if(glist->data != NULL)
		{
			IVImageViewer *v = IV_IMAGE_VIEWER(glist->data);
			imgview_struct *iv = v->iv;

			if(close_all_windows)
			{
				iv_viewer_delete(v);
				glist->data = NULL;
				continue;
			}

			if((iv != NULL) ? iv->map_state : FALSE)
			{
				nwindows++;
			}
			/* Delete the unmapped Image Viewer only when there
			 * is no grab window so that no Image Viewers get
			 * deleted during a screenshot
			 */
			else if(grab_window == NULL)
			{
				iv_viewer_delete(v);
				glist->data = NULL;
				continue;
			}
		}
	}

	/* Manage Data Viewers */
	for(glist = core->iv_fviewer;
		glist != NULL;
		glist = g_list_next(glist)
	)
	{
		if(glist->data != NULL)
		{
			IVDataViewer *fv = IV_DATA_VIEWER(glist->data);

			if(close_all_windows)
			{
				iv_fviewer_delete(fv);
				glist->data = NULL;
				continue;
			}

			if(GTK_WIDGET_MAPPED(fv->toplevel))
			{
				nwindows++;
			}
			else
			{
				iv_fviewer_delete(fv);
				glist->data = NULL;
				continue;
			}
		}
	}

	/* Count number of Image Info Dialogs */
	for(glist = core->imginfodlg;
		glist != NULL;
		glist = g_list_next(glist)
	)
	{
		if(glist->data != NULL)
		{
			ImgInfoDlg *d = IMG_INFO_DLG(glist->data);

			if(close_all_windows)
			{
				img_info_dlg_unmap(d);
				img_info_dlg_delete(d);
				glist->data = NULL;
				continue;
			}

			if(img_info_dlg_is_mapped(d))
			{
				nwindows++;
			}
			else
			{
				img_info_dlg_unmap(d);
				img_info_dlg_delete(d);
				glist->data = NULL;
				continue;
			}
		}
	}

	/* If there are no more windows left then break out of the
	 * main loop
	 */
	if(nwindows == 0)
	{
		gtk_main_quit();
		core->main_timeout_id = 0;
		return(FALSE);
	}

	return(TRUE);
}


/*
 *	File selector created callback.
 */
static void iv_file_selector_created_cb(
	const gchar *path, gpointer data
)
{
	IVCore *core = IV_CORE(data);
#ifdef HAVE_LIBENDEAVOUR2
	edv_notify_queue_vfs_object_added(core->edv2_ctx, path);
	edv_context_flush(core->edv2_ctx);
#endif
}

/*
 *	File selector changed callback.
 */
static void iv_file_selector_modified_cb(
	const gchar *old_path,
	const gchar *new_path,
	gpointer data
)
{
	IVCore *core = IV_CORE(data);
#ifdef HAVE_LIBENDEAVOUR2
	edv_notify_queue_vfs_object_modified(core->edv2_ctx, old_path, new_path);
	edv_context_flush(core->edv2_ctx);
#endif
}

/*
 *	File selector deleted callback.
 */
static void iv_file_selector_deleted_cb(
	const gchar *path, gpointer data
)
{
	IVCore *core = IV_CORE(data);
#ifdef HAVE_LIBENDEAVOUR2
	edv_notify_queue_vfs_object_removed(core->edv2_ctx, path);
	edv_context_flush(core->edv2_ctx);
#endif
}


/*
 *	Resets the IV Core's configuration to default values.
 */
static void iv_core_reset(IVCore *core)
{
	guint8 bg_color[4] = { 0x00, 0x00, 0x00, 0xff };
#if !defined(_WIN32)
	print_values_struct *pv;
#endif

	if(core == NULL)
		return;

	g_free(core->current_directory_path);
	core->current_directory_path = STRDUP(g_get_current_dir());
	core->close_all_windows = FALSE;
	core->untitled_count = 0;

	/* Options */
	core->options = IV_OPTIONS_DEFAULT;
	core->quality = 1;
	(void)memcpy(
		core->bg_color,
		bg_color,
		sizeof(bg_color)
	);
	core->bg_color_set = FALSE;
	g_free(core->font_name);
	core->font_name = NULL;
	g_free(core->wmname);
	core->wmname = NULL;
	g_free(core->wmclass);
	core->wmclass = NULL;
	g_free(core->title);
	core->title = NULL;
	core->geometry_flags = 0;
	g_free(core->geometry);
	core->geometry = NULL;

	core->imlib_fallback = 1;

	/* Save Options */
	core->gif_interlaced = IV_GIF_DEF_INTERLACED;
	core->gif_color_type = IV_GIF_DEF_COLOR_TYPE;
	core->gif_transparency = IV_GIF_DEF_TRANSPARENCY;
	core->gif_alpha_threshold = IV_GIF_DEF_ALPHA_THRESHOLD;
	core->gif_looping = IV_GIF_DEF_LOOPING;
	core->jpeg_quality = IV_JPEG_DEF_QUALITY;
	core->jpeg_is_color = IV_JPEG_DEF_IS_COLOR;
	core->jng_quality = IV_JNG_DEF_QUALITY;
	core->jng_is_color = IV_JNG_DEF_IS_COLOR;
	core->jng_transparency = IV_JNG_DEF_TRANSPARENCY;
	core->mng_compression = IV_MNG_DEF_COMPRESSION;
	core->mng_format = IV_MNG_DEF_FORMAT;
	core->mng_interlaced = IV_MNG_DEF_INTERLACED;
	core->mng_looping = IV_MNG_DEF_LOOPING;
	core->png_compression = IV_PNG_DEF_COMPRESSION;
	core->png_format = IV_PNG_DEF_FORMAT;
	core->png_interlaced = IV_PNG_DEF_INTERLACED;
	core->tga_format = IV_TGA_DEF_FORMAT;
	core->tiff_compression_type = IV_TIFF_DEF_COMPRESSION_TYPE;
	core->tiff_jpeg_quality = IV_TIFF_DEF_JPEG_QUALITY;
	core->tiff_color_type = IV_TIFF_DEF_COLOR_TYPE;
	core->xpm_format = IV_XPM_DEF_FORMAT;
	core->xpm_max_colors = IV_XPM_DEF_MAX_COLORS;
	core->xpm_threshold = IV_XPM_DEF_THRESHOLD;

	/* Screenshot Options */
	core->screenshot_delay_ms = IV_SCREENSHOT_DEF_DELAY;
	core->screenshot_hide_windows = IV_SCREENSHOT_DEF_HIDE_WINDOWS;
	core->screenshot_include_decorations = IV_SCREENSHOT_DEF_INCLUDE_DECORATIONS;
	core->screenshot_record_date_in_comments = IV_SCREENSHOT_DEF_RECORD_DATE_IN_COMMENTS;
	core->screenshot_auto_save = FALSE;
	g_free(core->screenshot_save_dir_path);
	core->screenshot_save_dir_path = STRDUP(core->current_directory_path);
	g_free(core->screenshot_save_ext);
	core->screenshot_save_ext = STRDUP(IMAGE_FORMAT_EXTENSION_JPG);

	/* Resize Options */
	core->resize_maintain_aspect = TRUE;

	/* Color Levels Options */
	core->color_levels_all_frames = IV_COLOR_LEVELS_DEF_ALL_FRAMES;

	/* Add Text Options */
	g_free(core->add_text_font_name);
	core->add_text_font_name = STRDUP(IV_ADD_TEXT_DEF_FONT_NAME);
	{ GdkColor c = IV_ADD_TEXT_DEF_COLOR;
	  (void)memcpy(&core->add_text_color, &c, sizeof(GdkColor));
	}
	core->add_text_outline = IV_ADD_TEXT_DEF_OUTLINE;
	{ GdkColor c = IV_IV_ADD_TEXT_DEF_OUTLINE_COLOR;
	  (void)memcpy(&core->add_text_outline_color, &c, sizeof(GdkColor));
	}
	core->add_text_opacity = IV_ADD_TEXT_DEF_OPACITY;
	core->add_text_all_frames = IV_ADD_TEXT_DEF_ALL_FRAMES;

#if !defined(_WIN32)
	/* Print Options */
	PrintValuesDelete(core->print_values);
	core->print_values = pv = PrintValuesNew();
	PrintValuesReset(pv);
#endif	/* !_WIN32 */
}

/*
 *	Deletes the IV Core.
 */
static void iv_core_delete(IVCore *core)
{
	if(core == NULL)
		return;

	g_free(core->current_directory_path);

	/* Delete all the Image Viewers */
	g_list_foreach(
		core->iv_viewer, (GFunc)iv_viewer_delete, NULL
	);
	g_list_free(core->iv_viewer);

	/* Delete all the Data Viewers */
	g_list_foreach(
		core->iv_fviewer, (GFunc)iv_fviewer_delete, NULL
	);
	g_list_free(core->iv_fviewer);

	/* Delete all the Image Info Dialogs */
	g_list_foreach(
		core->imginfodlg, (GFunc)img_info_dlg_delete, NULL
	);
	g_list_free(core->imginfodlg);

	/* Grab GtkWindow */
	if(core->grab_window != NULL)
	{
		gtk_widget_hide(core->grab_window);
		GTK_WIDGET_DESTROY(core->grab_window);
	}

	/* Destroy all GdkCursors */
	GDK_CURSOR_DESTROY(core->grab_cur);

	g_free(core->screenshot_save_dir_path);
	g_free(core->screenshot_save_ext);

	g_free(core->add_text_font_name);

	PrintValuesDelete(core->print_values);

	g_free(core->font_name);
	g_free(core->wmname);
	g_free(core->wmclass);
	g_free(core->title);
	g_free(core->geometry);

#ifdef HAVE_LIBENDEAVOUR2
	edv_context_delete(core->edv2_ctx);
#endif

	g_free(core);
}


/*
 *	Creates a new Image Viewer and adds it to the windows list.
 */
static IVImageViewer *iv_viewer_new(
	IVCore *core,
	const IVOptionFlags options,
	const gint quality,			/* 0 to 2 */
	const guint8 *bg_color,			/* 4 bytes RGBA */
	const GdkGeometryFlags geometry_flags,
	const GdkRectangle *geometry,
	const gboolean load_title
)
{
	GList *glist;
	GtkAccelGroup *accelgrp;
	GtkWidget	*w,
			*parent,
			*toplevel = NULL,
			*iv_toplevel,
			*menu;
	imgview_struct *iv = NULL;
	IVImageViewer *v = NULL;

	/* Create a new Image Viewer */
	v = IV_IMAGE_VIEWER(g_malloc0(sizeof(IVImageViewer)));
	if(v == NULL)
		return(NULL);

	v->core = core;
	v->processing = FALSE;
	v->has_changes = FALSE;
	v->fg_color[0] = 0x00;
	v->fg_color[1] = 0x00;
	v->fg_color[2] = 0x00;
	v->fg_color[3] = 0xFF;
	if(bg_color != NULL)
	{
		(void)memcpy(
			v->bg_color,
			bg_color,
			4 * sizeof(guint8)
		);
	}
	else
	{
		v->bg_color[0] = 0xFF;
		v->bg_color[1] = 0xFF;
		v->bg_color[2] = 0xFF;
		v->bg_color[3] = 0xFF;
	}
	iv_clear_info(&v->info);

	/* Begin creating image viewer */

	/* Create ImgView as a toplevel GtkWindow */
	v->iv = iv = ImgViewNew(
		(options & IV_OPTION_SHOW_TOOL_BAR) ? TRUE : FALSE,
		(options & IV_OPTION_SHOW_VALUES) ? TRUE : FALSE,
		(options & IV_OPTION_SHOW_STATUS_BAR) ? TRUE : FALSE,
		(options & IV_OPTION_SHOW_IMAGE_ON_WM_ICON) ? TRUE : FALSE,
		quality,			/* Quality, 0 to 2 */
		TRUE,			/* Toplevel is to be a GtkWindow */
		&iv_toplevel
	);
	if(iv == NULL)
	{
		g_free(v);
		return(NULL);
	}

	/* Get accelgrp */
	accelgrp = ImgViewGetAccelGroup(iv);

	/* Get the ImgView's toplevel GtkWidget */
	parent = w = toplevel = iv_toplevel;

	gtk_widget_set_name(w, IV_WIDGET_NAME_IMAGE_VIEWER_TOPLEVEL);

	/* Allow cropping */
	ImgViewAllowCrop(iv, TRUE);

	/* Set the ImgView's changed callback */
	ImgViewSetChangedCB(
		iv,
		iv_imgview_changed_cb, v
	);

	/* Set the background color */
	if(v->bg_color != NULL)
	{
		gint i;
		GdkColor c[5];
		for(i = 0; i < 5; i++)
		{
			GDK_COLOR_SET_BYTE(
				&(c[i]),
				v->bg_color[0],
				v->bg_color[1],
				v->bg_color[2]
			);
		}
		ImgViewSetViewBG(iv, c);
	}

	/* Set up the ImgView's toplevel GtkWindow */
	w = iv_toplevel;
	if(w != NULL)
	{
		/* Font */
		if(core->font_name != NULL)
		{
			GtkRcStyle *rcstyle = gtk_rc_style_new();
			rcstyle->font_name = STRDUP(core->font_name);
			gtk_widget_modify_style_recursive(w, rcstyle);
			GTK_RC_STYLE_UNREF(rcstyle);
		}

		/* Geometry */
		if(geometry != NULL)
		{
			if((geometry_flags & GDK_GEOMETRY_X) &&
			   (geometry_flags & GDK_GEOMETRY_Y)
			)
				gtk_widget_set_uposition(
					w,
					geometry->x, geometry->y
				);
			if((geometry_flags & GDK_GEOMETRY_WIDTH) &&
			   (geometry_flags & GDK_GEOMETRY_HEIGHT)
			)
				gtk_widget_set_usize(
					w,
					geometry->width, geometry->height
				);
		}

		/* Title */
		gtk_window_set_title(
			GTK_WINDOW(w),
			(core->title != NULL) ?
				core->title :
				PROG_NAME_FULL " Version " PROG_VERSION
		);

		/* Events */
		gtk_signal_connect(
			GTK_OBJECT(w), "delete_event",
			GTK_SIGNAL_FUNC(iv_viewer_delete_event_cb), v
		);
	}

	/* Set up ImgView's view GtkDrawingArea */
	w = (GtkWidget *)ImgViewGetViewWidget(iv);
	if(w != NULL)
	{
		/* Drag & Drop */
		const GtkTargetEntry dnd_tar_types[] = {
			{"text/plain",		0,	DND_INFO_TEXT_PLAIN},
			{"text/uri-list",	0,	DND_INFO_TEXT_URI_LIST},
			{"STRING",		0,	DND_INFO_STRING}
		};
		GUIDNDSetTar(
			w,
			dnd_tar_types,
			sizeof(dnd_tar_types) / sizeof(GtkTargetEntry),
			GDK_ACTION_COPY,		/* Drag actions */
			GDK_ACTION_MOVE,		/* Default action if same */
			GDK_ACTION_COPY,		/* Default action */
			iv_viewer_dnd_recieved_cb,
			v,
			FALSE				/* Do not highlight */
		);

		/* Events */
		gtk_signal_connect(
			GTK_OBJECT(w), "key_press_event",
			GTK_SIGNAL_FUNC(iv_viewer_key_event_cb), v
		);
		gtk_signal_connect(
			GTK_OBJECT(w), "key_release_event",
			GTK_SIGNAL_FUNC(iv_viewer_key_event_cb), v
		);
	}


	/* Get Image Viewer's GtkMenu */
	menu = (GtkWidget *)ImgViewGetMenuWidget(iv);
	if(menu != NULL)
	{
		GtkWidget *submenu;
		guint accel_key, accel_mods;
		guint8 **icon;
		const gchar *label;
		gpointer mclient_data = v;
		void (*func_cb)(GtkWidget *w, gpointer) = NULL;

#define ADD_MENU_ITEM_LABEL	{		\
 w = GUIMenuItemCreate(				\
  menu,						\
  GUI_MENU_ITEM_TYPE_LABEL,			\
  accelgrp,					\
  icon,						\
  label,					\
  accel_key, accel_mods,			\
  func_cb, mclient_data				\
 );						\
}
#define ADD_MENU_ITEM_SUBMENU	{		\
 w = GUIMenuItemCreate(				\
  menu,						\
  GUI_MENU_ITEM_TYPE_SUBMENU,			\
  accelgrp,					\
  icon,						\
  label,					\
  accel_key, accel_mods,			\
  func_cb, mclient_data				\
 );						\
 if(w != NULL)					\
  GUIMenuItemSetSubMenu(w, submenu);		\
}
#define ADD_MENU_SEP			{	\
 w = GUIMenuItemCreate(				\
  menu,						\
  GUI_MENU_ITEM_TYPE_SEPARATOR,			\
  NULL,						\
  NULL,						\
  NULL,						\
  0, 0,						\
  NULL, NULL					\
 );						\
}

		icon = NULL;
		label = "Background Color...";
		accel_key = GDK_b;
		accel_mods = GDK_CONTROL_MASK;
		func_cb = iv_viewer_set_bg_color_cb;
		ADD_MENU_ITEM_LABEL
		v->set_bg_mi = w;

		/* Edit Submenu */
		v->edit_submenu = submenu = GUIMenuCreate();
		if(submenu != NULL)
		{
			GtkWidget *menu;	/* Overload */

			menu = submenu;

		    icon = (guint8 **)icon_resize_20x20_xpm;
		    label = "Resize...";
		    accel_key = GDK_r;
		    accel_mods = GDK_CONTROL_MASK;
		    func_cb = iv_viewer_resize_cb;
		    ADD_MENU_ITEM_LABEL
		    v->resize_mi = w;

			icon =  (guint8 **)icon_color_levels_20x20_xpm;
			label = "Color Levels...";
			accel_key = GDK_l;
			accel_mods = GDK_CONTROL_MASK;
			func_cb = iv_viewer_color_levels_cb;
			ADD_MENU_ITEM_LABEL
			v->color_levels_mi = w;

			icon = (guint8 **)icon_fonts_20x20_xpm;
			label = "Add Text...";
			accel_key = GDK_t;
			accel_mods = GDK_CONTROL_MASK;
			func_cb = iv_viewer_add_text_cb;
			ADD_MENU_ITEM_LABEL
			v->add_text_mi = w;

		    icon = (guint8 **)icon_edit_20x20_xpm;
		    label = "Header...";
		    accel_key = GDK_h;
		    accel_mods = GDK_CONTROL_MASK;
		    func_cb = iv_viewer_header_cb;
		    ADD_MENU_ITEM_LABEL
		    v->edit_header_mi = w;

			icon = (guint8 **)icon_animate_20x20_xpm;
			label = "Frames...";
			accel_key = GDK_f;
			accel_mods = GDK_CONTROL_MASK;
			func_cb = iv_viewer_frames_cb;
			ADD_MENU_ITEM_LABEL
			v->edit_frames_mi = w;
		}
		icon = (guint8 **)icon_edit_20x20_xpm;
		label = "Edit";
		accel_key = 0;
		accel_mods = 0;
		func_cb = NULL;
		ADD_MENU_ITEM_SUBMENU
		v->edit_mi = w;

		ADD_MENU_SEP

		icon = (guint8 **)icon_open_20x20_xpm;
		label = "Open...";
		accel_key = GDK_o;
		accel_mods = GDK_CONTROL_MASK;
		func_cb = iv_viewer_open_image_cb;
		ADD_MENU_ITEM_LABEL
		v->open_mi = w;

		icon = (guint8 **)icon_save_as_20x20_xpm;
		label = "Save As...";
		accel_key = GDK_s;
		accel_mods = GDK_CONTROL_MASK;
		func_cb = iv_viewer_save_cb;
		ADD_MENU_ITEM_LABEL
		v->save_as_mi = w;

		ADD_MENU_SEP

#if !defined(_WIN32)
		icon = (guint8 **)icon_camera_20x20_xpm;
		label = "Screenshot...";
		accel_key = GDK_g;
		accel_mods = GDK_CONTROL_MASK;
		func_cb = iv_viewer_screenshot_cb;
		ADD_MENU_ITEM_LABEL
		v->screenshot_mi = w;

		ADD_MENU_SEP
#endif

		icon = (guint8 **)icon_info_20x20_xpm;
		label = "Information";
		accel_key = GDK_i;
		accel_mods = GDK_CONTROL_MASK;
		func_cb = iv_viewer_information_cb;
		ADD_MENU_ITEM_LABEL
		v->information_mi = w;

		icon = (guint8 **)icon_monitor2_20x20_xpm;
		label = "Fullscreen";
		accel_key = GDK_F11;
		accel_mods = GDK_CONTROL_MASK;
		func_cb = iv_viewer_fullscreen_cb;
		ADD_MENU_ITEM_LABEL 
		v->fullscreen_mi = w; 

		/* Desktop submenu */
		v->desktop_submenu = submenu = GUIMenuCreate();
		if(submenu != NULL)
		{
			GtkWidget *menu;	/* Overload */

			menu = submenu;

			icon = (guint8 **)icon_root_centered_20x20_xpm;
			label = "Centered";
			accel_key = GDK_d;
			accel_mods = GDK_CONTROL_MASK;
			func_cb = iv_viewer_display_on_root_centered_cb;
			ADD_MENU_ITEM_LABEL

			icon = (guint8 **)icon_root_fit_20x20_xpm;
			label = "To Fit";
			accel_key = GDK_d;
			accel_mods = GDK_MOD1_MASK | GDK_CONTROL_MASK;
			func_cb = iv_viewer_display_on_root_tofit_cb;
			ADD_MENU_ITEM_LABEL

			icon = (guint8 **)icon_root_cover_20x20_xpm;
			label = "To Cover";
			accel_key = GDK_d;
			accel_mods = GDK_SHIFT_MASK | GDK_CONTROL_MASK;
			func_cb = iv_viewer_display_on_root_tocover_cb;
			ADD_MENU_ITEM_LABEL

			icon = (guint8 **)icon_root_tiled_20x20_xpm;
			label = "Tiled";
			accel_key = GDK_d;
			accel_mods = GDK_CONTROL_MASK;
			func_cb = iv_viewer_display_on_root_tiled_cb;
			ADD_MENU_ITEM_LABEL
		}
		icon = (guint8 **)icon_desktop_20x20_xpm;
		label = "Desktop";
		accel_key = 0;
		accel_mods = 0;
		func_cb = NULL;
		ADD_MENU_ITEM_SUBMENU
		v->desktop_mi = w;

		icon = (guint8 **)icon_clear_20x20_xpm;
		label = "Clear";
		accel_key = GDK_k;
		accel_mods = GDK_CONTROL_MASK;
		func_cb = iv_viewer_clear_cb;
		ADD_MENU_ITEM_LABEL
		v->clear_mi = w;

		ADD_MENU_SEP

#if !defined(_WIN32)
		icon = (guint8 **)icon_print2_20x20_xpm;
		label = "Print...";
		accel_key = GDK_p;
		accel_mods = GDK_CONTROL_MASK;
		func_cb = iv_viewer_print_cb;
		ADD_MENU_ITEM_LABEL
		v->print_mi = w;

		ADD_MENU_SEP
#endif

		icon = (guint8 **)icon_iv_20x20_xpm;
		label = "New Image Viewer";
		accel_key = GDK_1;
		accel_mods = GDK_CONTROL_MASK;
		func_cb = iv_viewer_new_viewer_cb;
		ADD_MENU_ITEM_LABEL
		v->new_viewer_mi = w;

#undef ADD_MENU_ITEM_LABEL
#undef ADD_MENU_ITEM_SUBMENU
#undef ADD_MENU_SEP
	}


	/* Record the window */
	for(glist = core->iv_viewer;
		glist != NULL;
		glist = g_list_next(glist)
	)
	{
		if(glist->data == NULL)
		{
			glist->data = v;
			break;
		}
	}
	if(glist == NULL)
		core->iv_viewer = g_list_append(
			core->iv_viewer,
			v
		);

	/* Display the initial values on the GtkWidgets */
	iv_viewer_update(v);

	/* Map the ImgView and the toplevel GtkWindow */
	ImgViewMap(iv);
	gtk_widget_show(toplevel);

	/* Have the ImgView's view widget grab focus */
	w = (GtkWidget *)ImgViewGetViewWidget(iv);
	if(w != NULL)
		gtk_widget_grab_focus(w);

	/* Load the title image? */
	if(load_title)
	{
		iv_title_load_to_image_viewer(
			iv,
#ifdef HAVE_IMLIB
			core->imlib_h,
#endif
			(options & IV_OPTION_RESIZE_ON_OPEN) ? TRUE : FALSE,
			(options & IV_OPTION_ZOOM_TO_FIT_ON_OPEN) ? TRUE : FALSE,
			(options & IV_OPTION_SIMPLE_TITLE) ? TRUE : FALSE
		);
		iv_viewer_update(v);
	}

	return(v);
}

/*
 *	Deletes the Image Viewer.
 */
static void iv_viewer_delete(IVImageViewer *v)
{
	imgview_struct *iv;

	if(v == NULL)
		return;

	iv = v->iv;
	if(ImgViewIsPlaying(iv))
		ImgViewPause(iv);

	/* Begin destroying widgets that we created */

	v->set_bg_mi = NULL;
	v->edit_mi = NULL;
	v->resize_mi = NULL;
	v->color_levels_mi = NULL;
	v->add_text_mi = NULL;
	v->edit_header_mi = NULL;
	v->edit_frames_mi = NULL;
	v->open_mi = NULL;
	v->save_as_mi = NULL;
	v->information_mi = NULL;
	v->desktop_mi = NULL;
	v->clear_mi = NULL;
	v->screenshot_mi = NULL;
	v->print_mi = NULL;
	v->new_viewer_mi = NULL;

	/* Submenus */
	GTK_WIDGET_DESTROY(v->edit_submenu);
	v->edit_submenu = NULL;
	GTK_WIDGET_DESTROY(v->desktop_submenu);
	v->desktop_submenu = NULL;

	/* Delete the ImgView */
	ImgViewUnmap(iv);
	ImgViewDelete(iv);
	v->iv = iv = NULL;

	/* Delete the image info values */
	iv_clear_info(&v->info);

	g_free(v);
}


/*
 *	Creates a new Data Viewer and adds it to the windows list.
 */
static IVDataViewer *iv_data_viewer_new(
	IVCore *core,
	const GdkGeometryFlags geometry_flags,
	const GdkRectangle *geometry
)
{
	const gint border_major = 5;
	GList *glist;
	GdkWindow *window;
	GtkAccelGroup *accelgrp;
	GtkWidget	*w,
			*parent, *parent2, *parent3, *parent4,
			*toplevel;
	tview_struct *tv;
	hview_struct *hv;
	IVDataViewer *fv = IV_DATA_VIEWER(
		g_malloc0(sizeof(IVDataViewer))
	);
	if(fv == NULL)
		return(NULL);

	fv->toplevel = toplevel = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	fv->accelgrp = accelgrp = gtk_accel_group_new();
	fv->freeze_count = 0;
	fv->core = core;

	fv->freeze_count++;

	/* Toplevel GtkWindow */
	w = toplevel;
#ifdef PROG_NAME
	gtk_window_set_wmclass(
		GTK_WINDOW(w),
		(core->wmname != NULL) ? core->wmname : "data-viewer",
		(core->wmclass != NULL) ? core->wmclass : PROG_NAME
	);
#endif
	gtk_widget_set_name(w, IV_WIDGET_NAME_DATA_VIEWER_TOPLEVEL);
	gtk_window_set_policy(GTK_WINDOW(w), TRUE, TRUE, TRUE);
	if(geometry != NULL)
	{
		if((geometry_flags & GDK_GEOMETRY_X) &&
		   (geometry_flags & GDK_GEOMETRY_Y)
		)
			gtk_widget_set_uposition(
				w,
				geometry->x, geometry->y
			);
		if((geometry_flags & GDK_GEOMETRY_WIDTH) &&
		   (geometry_flags & GDK_GEOMETRY_HEIGHT)
		)
			gtk_widget_set_usize(
				w,
				geometry->width, geometry->height
			);
	}
	else
	{
		gtk_widget_set_usize(
			w,
			IV_DATA_VIEWER_DEF_WIDTH,
			IV_DATA_VIEWER_DEF_HEIGHT
		);
	}
#ifdef PROG_NAME
	gtk_window_set_title(
		GTK_WINDOW(w),
		(core->title != NULL) ? core->title : PROG_NAME
	);
#endif
	gtk_widget_realize(w);
	window = w->window;
	if(window != NULL) 
	{
		GdkGeometry geo;
		geo.min_width = 100;
		geo.min_height = 70;
		geo.base_width = 0;
		geo.base_height = 0;
		geo.width_inc = 1;
		geo.height_inc = 1;
		gdk_window_set_geometry_hints(
			window,
			&geo,
			GDK_HINT_MIN_SIZE |
			GDK_HINT_BASE_SIZE |
			GDK_HINT_RESIZE_INC
		);
		GUISetWMIcon( 
			window,
			(guint8 **)icon_edit_48x48_xpm
		);
	}
	gtk_signal_connect(
		GTK_OBJECT(w), "delete_event",
		GTK_SIGNAL_FUNC(iv_fviewer_delete_event_cb), fv
	);
	gtk_window_add_accel_group(GTK_WINDOW(w), accelgrp);
	gtk_container_border_width(GTK_CONTAINER(w), 0);
	parent = w;

	/* Main GtkVBox */
	w = gtk_vbox_new(FALSE, 0);
	gtk_container_add(GTK_CONTAINER(parent), w);
	gtk_widget_show(w);
	parent = w;

	/* Text View & Hex View GtkVBox */
	w = gtk_vbox_new(FALSE, 0);
	gtk_box_pack_start(GTK_BOX(parent), w, TRUE, TRUE, 0);
	gtk_widget_show(w);
	parent2 = w;

	/* Create the Text View */
	fv->tv = tv = TViewNew(parent2);
	gtk_widget_set_name(tv->text, IV_WIDGET_NAME_DATA_VIEWER_TEXT);
	TViewSetReadOnly(tv, TRUE);

	/* Create the Hex View */
	fv->hv = hv = HViewNew(parent2);
	gtk_widget_set_name(hv->view_da, IV_WIDGET_NAME_DATA_VIEWER_HEX);
	HViewShowStatusBar(hv, FALSE);
	HViewSetReadOnly(hv, TRUE);


	/* Buttons GtkHBox */
	w = gtk_hbox_new(FALSE, 0);
	gtk_container_border_width(GTK_CONTAINER(w), border_major);
	gtk_box_pack_start(GTK_BOX(parent), w, FALSE, FALSE, 0);
	gtk_widget_show(w);
	parent2 = w;

	/* Text & Hex toggle buttons GtkVBox */
	w = gtk_vbox_new(TRUE, 0);
	gtk_box_pack_start(GTK_BOX(parent2), w, FALSE, FALSE, 0);
	gtk_widget_show(w);
	parent3 = w;

	/* Text & Hex toggle buttons GtkHBox */
	w = gtk_hbox_new(FALSE, border_major);
	gtk_box_pack_start(GTK_BOX(parent3), w, FALSE, FALSE, 0);
	gtk_widget_show(w);
	parent4 = w;

	/* Text GtkToggleButton */
	fv->text_tb = w = GUIToggleButtonPixmapLabelH(
		NULL, "ASCII", NULL
	);
	gtk_widget_set_usize(
		w,
		GUI_BUTTON_HLABEL_WIDTH, GUI_BUTTON_HLABEL_HEIGHT
	);
	gtk_box_pack_start(GTK_BOX(parent4), w, FALSE, FALSE, 0);
	gtk_signal_connect(
		GTK_OBJECT(w), "toggled",
		GTK_SIGNAL_FUNC(iv_fviewer_display_format_toggle_cb), fv
	);
	gtk_widget_show(w);
	/* Hex GtkToggleButton */
	fv->hex_tb = w = GUIToggleButtonPixmapLabelH(
		NULL, "Hex", NULL
	);
	gtk_widget_set_usize(
		w,
		GUI_BUTTON_HLABEL_WIDTH, GUI_BUTTON_HLABEL_HEIGHT
	);
	gtk_box_pack_start(GTK_BOX(parent4), w, FALSE, FALSE, 0);
	gtk_signal_connect(
		GTK_OBJECT(w), "toggled",
		GTK_SIGNAL_FUNC(iv_fviewer_display_format_toggle_cb), fv
	);
	gtk_widget_show(w);

	/* Close Button */
	fv->close_btn = w = GUIButtonPixmapLabelH(
		(guint8 **)icon_close_20x20_xpm,
		"Close",
		NULL
	);
	GTK_WIDGET_SET_FLAGS(w, GTK_CAN_DEFAULT);
	gtk_widget_set_usize(
		w,
		GUI_BUTTON_HLABEL_WIDTH_DEF, GUI_BUTTON_HLABEL_HEIGHT_DEF
	);
	gtk_box_pack_end(GTK_BOX(parent2), w, FALSE, FALSE, 0);
	gtk_signal_connect(
		GTK_OBJECT(w), "clicked",
		GTK_SIGNAL_FUNC(iv_fviewer_close_cb), fv
	);
	gtk_accel_group_add(
		accelgrp, GDK_Escape, 0, GTK_ACCEL_VISIBLE,
		GTK_OBJECT(w), "clicked"
	);
	gtk_widget_show(w);


	/* Record window */
	for(glist = core->iv_fviewer;
		glist != NULL;
		glist = g_list_next(glist)
	)
	{
		if(glist->data == NULL)
		{
			glist->data = fv;
			break;
		}
	}
	if(glist == NULL)
		core->iv_fviewer = g_list_append(core->iv_fviewer, fv);

	/* Map the Data Viewer */
	gtk_widget_show_raise(toplevel);

	fv->freeze_count--;

	return(fv);
}

/*
 *	Deletes the Data Viewer.
 */
static void iv_fviewer_delete(IVDataViewer *fv)
{
	if(fv == NULL)
		return;

	if(fv->toplevel != NULL)
		gtk_widget_hide(fv->toplevel);

	TViewDelete(fv->tv);
	HViewDelete(fv->hv);
	GTK_WIDGET_DESTROY(fv->text_tb);
	GTK_WIDGET_DESTROY(fv->hex_tb);
	GTK_WIDGET_DESTROY(fv->close_btn);
	GTK_WIDGET_DESTROY(fv->toplevel);
	GTK_ACCEL_GROUP_UNREF(fv->accelgrp);
	g_free(fv);
}


/*
 *	Update Image Viewer's image info.
 *
 *	Any value can be 0 or NULL for none.
 */
static void iv_viewer_update_image_info(
	IVImageViewer *v,
	const gchar *path,
	const struct stat *stat_buf,
	const gint width, const gint height,
	const gint bpp, const gint bpl,
	const gint nframes,
	const gint nloops,
	const gulong play_time_ms,		/* In milliseconds */
	const gulong total_play_time_ms,	/* In milliseconds */
	const gboolean has_alpha,
	const gint x, const gint y,
	const gint base_width,
	const gint base_height,
	const gchar *creator,
	const gchar *title,
	const gchar *author,
	const gchar *comments,
	const gulong time_modified		/* In seconds since EPOCH */
)
{
	IVInfo *info = &v->info;

	/* Clear the existing values */
	iv_clear_info(info);

	/* Set the path */
	g_free(info->path);
	info->path = STRDUP(path);

	/* Set the file information */
	if(stat_buf != NULL)
	{
		info->file_size = (gulong)stat_buf->st_size;
		/* Do not get time modified from the file stats */
	}

	/* Set the rest of the info values */
	info->width = width;
	info->height = height;
	info->nframes = nframes;
	info->nloops = nloops;
	info->play_time_ms = play_time_ms;
	info->total_play_time_ms = total_play_time_ms;
	info->has_alpha = has_alpha;
	info->x = x;
	info->y = y;
	info->base_width = base_width;
	info->base_height = base_height;

	/* Set the creator only if changed */
	if(creator != info->creator)
	{
		g_free(info->creator);
		info->creator = STRDUP(creator);
	}
	/* Set the title only if changed */
	if(title != info->title)
	{
		g_free(info->title);
		info->title = STRDUP(title);
	}
	/* Set the author only if changed */
	if(author != info->author)
	{
		g_free(info->author);
		info->author = STRDUP(author);
	}
	/* Set the comments only if changed */
	if(comments != info->comments)
	{
		g_free(info->comments);
		info->comments = STRDUP(comments);
	}

	info->time_modified = time_modified;
}


/*
 *	Opens the image to the Image Viewer.
 *
 *	The path specifies the full path to the image.
 *
 *	The frame_num specifies the frame to initially display or -1
 *	for unspecified (in which case if there are multiple frames
 *	in the image then the Image Viewer will automatically start
 *	playing).
 *
 *	If resize_on_open is TRUE then the Image Viewer will be
 *	resized to fit the opened image.
 */
static void iv_viewer_open(
	IVImageViewer *v,
	const gchar *path,
	const gint frame_num
)
{
	struct stat stat_buf;
	gboolean	image_geometries_match,
			created_new_image,
			use_file_viewer = FALSE;
	const gint	bpp = 4;		/* RGBA */
	gint		status,
			x, y,
			width, height,
			bpl,
			nframes,
			nloops,
			base_width, base_height;
	gulong		play_time_ms,
			total_play_time_ms;
	gchar		*creator,
			*title,
			*author,
			*comments;
	guint8 **rgba_list;
	gulong		*delay_list,
			modified_time_sec;
	GtkWidget	*w,
			*toplevel;
	imgview_struct *iv;
	imgview_image_struct *img;
	IVImgIOCBData *cb_data;
	IVCore *core;
	IVOptionFlags options;

	if((v == NULL) || STRISEMPTY(path))
		return;

	iv = v->iv;
	core = v->core;
	options = core->options;

	/* Get the ImgView's toplevel widget if it is a GtkWindow */
	if(ImgViewIsToplevelWindow(iv))
		toplevel = ImgViewGetToplevelWidget(iv);
	else
		toplevel = NULL;

	/* Get the ImgView's view widget */
	w = (GtkWidget *)ImgViewGetViewWidget(iv);
	if(w == NULL)
		return;

	/* Check if the image file does not exist */
	if(stat((const char *)path, &stat_buf))
	{
		const gint error_code = (gint)errno;
		gchar *msg = g_strdup_printf(
"%s:\n\
\n\
    %s",
			g_strerror(error_code),
			path
		);
		CDialogSetTransientFor(toplevel);
		CDialogGetResponse(
			"Open Failed",
			msg,
			NULL,
			CDIALOG_ICON_WARNING,
			CDIALOG_BTNFLAG_OK,
			CDIALOG_BTNFLAG_OK
		);
		g_free(msg);
		CDialogSetTransientFor(NULL);
		return;
	}
#ifdef S_ISDIR
	/* Check if the specified path is a directory */
	if(S_ISDIR(stat_buf.st_mode))
	{
		gchar *msg = g_strdup_printf(
"The specified file is a directory:\n\
\n\
    %s",
			path
		);
		CDialogSetTransientFor(toplevel);
		CDialogGetResponse(
			"Open Failed",
			msg,
			NULL,
			CDIALOG_ICON_WARNING,
			CDIALOG_BTNFLAG_OK,
			CDIALOG_BTNFLAG_OK
		);
		g_free(msg);
		CDialogSetTransientFor(NULL);
		return;
	}
#endif  /* S_ISDIR */

	/* Clear the existing image (if any) on Image Viewer so that
	 * the ImgView is cleared and redrawn
	 *
	 * Note that the redraw event will be handled when the events
	 * are handled in the progress callback while opening the
	 * image below
	 */
	ImgViewClear(iv);

	/* Clear image info */
	iv_clear_info(&v->info);

	/* Reset has changes mark */
	v->has_changes = FALSE;


	/* Create the progress callback data */
	cb_data = IV_IMGIO_CB(g_malloc0(sizeof(IVImgIOCBData)));
	if(cb_data == NULL)
		return;

	cb_data->ref_toplevel = toplevel;
	cb_data->viewer = v;
	cb_data->options = options;
	cb_data->path = g_strdup(path);

	/* Record the current background color on the callback data so
	 * that a change in the background color can be detected in
	 * iv_open_progress_cb()
	 */
	(void)memcpy(
		cb_data->last_bg_color,
		v->bg_color,
		sizeof(cb_data->last_bg_color)
	);

	/* Reset the stop count before opening the image since the open
	 * progress callbacks will be testing it
	 */
	v->stop_count = 0;

	/* Open the image from file to RGBA */
	status = ImgFileOpenRGBA(
		path,
		&width, &height,
		&bpl,
		&rgba_list,
		&delay_list,
		&nframes,
		v->bg_color,			/* 4 bytes RGBA (will be modified) */
		&x, &y,
		&base_width, &base_height,
		&creator, &title, &author, &comments,
		&modified_time_sec,			/* Modified time in seconds
											 * since EPOCH GMT */
		0xFF,				/* Default alpha value */
		iv_open_progress_cb, cb_data
	);

	/* Display error message for certain error codes */
	if((status == -1) || (status == -2) || (status == -3) ||
	   (status == -6) || (status == -7)
	)
	{
		const gchar *error_msg = ImgOpenGetError();
		if(!STRISEMPTY(error_msg))
		{
			gchar *msg = g_strdup_printf(
"%s:\n\
\n\
    %s",
				error_msg,
				path
			);
			CDialogSetTransientFor(toplevel);
			CDialogGetResponse(
				"Open Error",
				msg,
				NULL,
				CDIALOG_ICON_WARNING,
				CDIALOG_BTNFLAG_OK,
				CDIALOG_BTNFLAG_OK
			);
			g_free(msg);
			CDialogSetTransientFor(NULL);
		}
	}

	/* Calculate the number of loops */
	nloops = (nframes > 1) ? -1 : 0;

	/* Calculate the play time per loop */
	play_time_ms = 0l;
	if((delay_list != NULL) && (nframes > 0))
	{
		int i;
		for(i = 0; i < nframes; i++)
			play_time_ms += delay_list[i];
	}

	/* Calculate the total play time */
	if(play_time_ms > 0l)
	{
		/* Infinite? */
		if(nloops < 0)
			total_play_time_ms = (gulong)-1;
		else
			total_play_time_ms = nloops * play_time_ms;
	}
	else
		total_play_time_ms = 0l;

	/* Reset the ImgView's progress bar */
	ImgViewProgressUpdate(iv, 0.0f, FALSE);

	/* Get the ImgView's current image and check if its geometry
	 * matches the opened image's geometry
	 *
	 * If and only if the geometries are the same then it
	 * implies that the ImgView's image was created and set
	 * during the open progress callback and we can simply
	 * update the current ImgView's image without having to
	 * recreate it
	 */
	img = ImgViewGetImage(iv);
	if(img != NULL)
	{
		if((img->width == width) && (img->height == height) &&
		   (img->bpp == bpp) && (img->bpl == bpl)
		)
			image_geometries_match = TRUE;
		else
			image_geometries_match = FALSE;
	}
	else
	{
		image_geometries_match = FALSE;
	}

	created_new_image = FALSE;

	/* Was there image data returned from ImgFileOpenRGBA()? */
	if(rgba_list != NULL)
	{
		/* Does the geometry of the returned image data match
		 * the geometry of the ImgView image already on the ImgView?
		 */
		if(image_geometries_match)
		{
			/* Image geometries match, just update the ImgView's
			 * current image
			 */
			img = ImgViewGetImage(iv);
			if(img != NULL)
			{
				const int len = bpl * height;
				gint i;
				GList *glist;
				imgview_frame_struct *frame;

				/* Copy the frames to the existing frames on the
				 * ImgView's image
				 */
				for(i = 0,
					glist = img->frames_list;
					(i < nframes) && (glist != NULL);
					i++,
					glist = g_list_next(glist)
				)
				{
					if(rgba_list[i] == NULL)
						continue;

					/* Get this frame and copy the opened image's
					 * frame data and delay over to it
					 */
					frame = IMGVIEW_FRAME(glist->data);
					if(frame != NULL)
					{
						if(frame->data != NULL)
							(void)memcpy(
								frame->data,
								rgba_list[i],
								len * sizeof(guint8)
							);
						frame->delay = (delay_list != NULL) ?
							delay_list[i] : 0l;
					}
				}

				/* Append/transfer the rest of the frames */
				for(; i < nframes; i++)
				{
					(void)ImgViewImageAppendFrame(
						img,
						rgba_list[i],
						(delay_list != NULL) ? delay_list[i] : 0l
					);
					rgba_list[i] = NULL;
				}

				ImgViewQueueDrawView(iv);

				/* Need to update the ImgView's WM icon since the
				 * ImgView's current image was just modified
				 * above
				 */
				ImgViewUpdateWMIcon(iv);
			}
		}
		else
		{
			/* Image geometries do not match, create a new ImgView
			 * image and transfer it to the ImgView
			 */
			img = ImgViewImageNew(
				width, height,
				bpp, bpl
			);
			if(img != NULL)
			{
				gint i;
				for(i = 0; i < nframes; i++)
				{
					(void)ImgViewImageAppendFrame(
						img,
						rgba_list[i],
						(delay_list != NULL) ? delay_list[i] : 0l
					);
					rgba_list[i] = NULL;
				}
				created_new_image = TRUE;
			}
		}

		/* Update the image info */
		iv_viewer_update_image_info(
			v,
			path,
			&stat_buf,
			width, height,
			bpp, bpl,
			nframes,
			nloops,
			play_time_ms,
			total_play_time_ms,
			TRUE,				/* Has alpha */
			x, y,
			base_width, base_height,
			creator, title, author, comments,
			(modified_time_sec > 0l) ?
				modified_time_sec : (gulong)stat_buf.st_mtime
		);

		/* Update the ImgView's background color if it has changed */
		if(memcmp(cb_data->last_bg_color, v->bg_color, sizeof(cb_data->last_bg_color)))
		{
			gint i;
			GdkColor c[5];
			for(i = 0; i < 5; i++)
			{
				GDK_COLOR_SET_BYTE(
					&(c[i]),
					v->bg_color[0],
					v->bg_color[1],
					v->bg_color[2]
				);
			}
			ImgViewSetViewBG(iv, c);
		}

		/* Resize and reposition the ImgView if specified and if
		 * it was not done so in the progress callback
		 */
		if(cb_data->options & IV_OPTION_RESIZE_ON_OPEN)
			iv_viewer_resize(
				v,
				width, height,
				TRUE			/* Keep visible on desktop */
			);

		/* If a new ImgView image was created from the opened
		 * image data and no ImgView image was set during the
		 * progress callback then set the opened image to the
		 * ImgView
		 */
		if(created_new_image)
		{
			if(cb_data->options & IV_OPTION_ZOOM_TO_FIT_ON_OPEN)
				ImgViewSetImageToFit(
					iv,
					img			/* Transfered */
				);
			else
				ImgViewSetImage(
					iv,
					img			/* Transfered */
				);
/*		img = NULL; */
		}

		/* More than one frame on this image? */
		if(nframes > 1)
		{
			/* If there was a specific frame specified to be
			 * displayed then seek to it, otherwise start
			 * playing
			 */
			if(frame_num > -1)
				ImgViewSeek(iv, frame_num);
			else
				ImgViewPlay(iv);
		}

		/* Update the ImgView's title */
		iv_viewer_update_title(v, path);
	}
	/* No image data obtained from ImgFileOpenRGBA(), check if
	 * there was image data obtained during the progress callbacks
	 * (usually the result of an error or user abort during opening)
	 */
	else if(img != NULL)
	{
		/* Update the image info with any partial image information
		 * obtained
		 */
		iv_viewer_update_image_info(
			v,
			path,
			&stat_buf,
			width, height,
			bpp, bpl,
			nframes,
			nloops,
			play_time_ms,
			total_play_time_ms,
			TRUE,
			x, y,
			base_width, base_height,
			creator, title, author, comments,
			(modified_time_sec > 0l) ?
				modified_time_sec : (gulong)stat_buf.st_mtime
		);

		/* Update the ImgView's background color if it has changed */
		if(memcmp(cb_data->last_bg_color, v->bg_color, sizeof(cb_data->last_bg_color)))
		{
			gint i;
			GdkColor c[5];
			for(i = 0; i < 5; i++)
			{
				GDK_COLOR_SET_BYTE(
					&(c[i]),
					v->bg_color[0],
					v->bg_color[1],
					v->bg_color[2]
				);
			}
			ImgViewSetViewBG(iv, c);
		}

		/* Resize and reposition the ImgView if specified and if
		 * it was not done so in the progress callback
		 */
		if(cb_data->options & IV_OPTION_RESIZE_ON_OPEN)
			iv_viewer_resize(
				v,
				width, height,
				TRUE			/* Keep visible on desktop */
			);

		/* Update the ImgView's title */
		iv_viewer_update_title(v, path);
	}
	else
	{
		/* Failed to open or obtain any image data
		 *
		 * Try to open the image as a binary or text file in a new
		 * Data Viewer
		 */
		use_file_viewer = TRUE;
		iv_data_viewer_new_open_file(
			core,
			path,
			toplevel
		);
	}

	/* Delete the open image callback data */
	g_free(cb_data->path);
	g_free(cb_data);
	cb_data = NULL;

	/* Delete the opened image data and other resources */
	if(rgba_list != NULL)
	{
		gint i;
		for(i = 0; i < nframes; i++)
			g_free(rgba_list[i]);
		g_free(rgba_list);
	}
	g_free(delay_list);
	g_free(creator);
	g_free(title);
	g_free(author);
	g_free(comments);

	/* Update the Image Viewer's widgets */
	iv_viewer_update(v);

	/* Hide the Image Viewer when a non-image file was opened? */
	if(use_file_viewer && (options & IV_OPTION_HIDE_IV_NON_IMAGE))
		ImgViewUnmap(iv);
}

/*
 *	Saves the image to a different format.
 *
 *	The path specifies the full path to the image to save to.
 *
 *	The extensions specifies a string describing a space-separated
 *	list of extension which determines the file format to save as.
 *	If extensions is; NULL, "*", or "*.*" then the format will be
 *	determined by the extension obtained from the path.
 */
static void iv_viewer_save(
	IVImageViewer *v,
	const gchar *path,
	const gchar *extensions
)
{
	struct stat stat_buf;
	gboolean	file_exists;
	gint		i,
			status,
			width, height,
			bpp, bpl,
			nframes,
			nloops,
			x, y,
			base_width, base_height;
	gulong		play_time_ms,
			total_play_time_ms;
	const gchar *ext;
	gchar		*orig_path = NULL,
			*new_path = NULL,
			*format_desc = NULL;
	guint8		*rgba,
			**rgba_list = NULL;
	gulong *delay_list = NULL;
	GList *glist;
	GtkWidget	*w,
			*toplevel;
	IVInfo *info;
	imgview_format format = IMGVIEW_FORMAT_RGBA;
	imgview_frame_struct *frame;
	imgview_image_struct *img;
	imgview_struct *iv;
	IVCore *core;
	IVImgIOCBData *cb_data = NULL;

#define CLEANUP_RETURN	{		\
 if(cb_data != NULL) {			\
  g_free(cb_data->path);		\
  g_free(cb_data);			\
 }					\
					\
 g_free(rgba_list);			\
 g_free(delay_list);			\
 g_free(orig_path);			\
 g_free(new_path);			\
 g_free(format_desc);			\
					\
 return;				\
}

	if((v == NULL) || STRISEMPTY(path))
	{
		CLEANUP_RETURN;
	}

	info = &v->info;
	iv = v->iv;
	core = v->core;
	if((iv == NULL) || (core == NULL))
	{
		CLEANUP_RETURN;
	}

	/* Reset the stop counter */
	v->stop_count = 0;

	/* No image loaded? */
	if(!ImgViewIsLoaded(iv))
	{
		CLEANUP_RETURN;
	}

	/* Get the image data to be saved */
	img = ImgViewGetImage(iv);
	if(img == NULL)
	{
		CLEANUP_RETURN;
	}
	nframes = img->nframes;
	if((nframes <= 0) || (img->frames_list == NULL))
	{
		CLEANUP_RETURN;
	}

	nloops = (nframes > 1) ? -1 : 0;

	play_time_ms = 0;

	/* Create/convert the GList RGBA image data and delay list to
	 * a pointer array for the image library functions' input
	 */
	rgba_list = (guint8 **)g_malloc0(nframes * sizeof(guint8 *));
	delay_list = (gulong *)g_malloc0(nframes * sizeof(gulong));
	if((rgba_list == NULL) || (delay_list == NULL))
	{
		CLEANUP_RETURN;
	}
	for(i = 0,
	    glist = img->frames_list;
	    glist != NULL;
	    i++,
	    glist = g_list_next(glist)
	)
	{
		frame = IMGVIEW_FRAME(glist->data);
		if(frame == NULL)
			continue;

		if(i < nframes)
		{
			rgba_list[i] = frame->data;
			delay_list[i] = frame->delay;
			play_time_ms += frame->delay;
		}
	}
	width = img->width;
	height = img->height;
	bpp = img->bpp;
	bpl = img->bpl;
	rgba = rgba_list[0];
	x = info->x;
	y = info->y;
	base_width = info->base_width;
	base_height = info->base_height;

	/* Calculate the total play time */
	if(play_time_ms > 0l)
	{
		/* Infinite? */
		if(nloops < 0)
			total_play_time_ms = (gulong)-1;
		else
			total_play_time_ms = nloops * play_time_ms;
	}
	else
		total_play_time_ms = 0l;

	/* If no extensions were specified then assume "any format" */
	if(STRISEMPTY(extensions))
		extensions = "*.*";

	/* Format the new path from the specified path and extension */
	new_path = g_strdup(path);

	/* Check if the extensions were specified as "any format" */
	if(!g_strcasecmp(extensions, "*.*") ||
	   !g_strcasecmp(extensions, "*")
	)
	{
		/* Get the extensions as a single extension from the
		 * specified path
		 */
		extensions = g_basename(path);
		if(extensions != NULL)
			extensions = (const gchar *)strchr(
				(const char *)extensions,
				'.'
			);
	}
	else
	{
		/* Add the extension to the new path if the specified path
		 * did not have an extension
		 */
		fb_type_struct *t = FileBrowserTypeNew();
		if(t != NULL)
		{
			g_free(t->extensions);
			t->extensions = g_strdup(extensions);

			g_free(new_path);
			new_path = FileBrowserTypeCompleteExtension(
				path,
				t,
				FALSE			/* Do not replace if not specified */
			);

			FileBrowserTypeDelete(t);
		}
	}
	/* Unable to generate the new path? */
	if(new_path == NULL)
	{
		CLEANUP_RETURN;
	}

	/* Get the extension from the new path */
	ext = g_basename(new_path);
	if(ext != NULL)
		ext = (const gchar *)strchr(
			ext,
			'.'
		);

	/* Get the format's description */
	if(ext != NULL)
	{
		const gchar *imgio_file_types_list[] = IMAGE_FORMAT_FILE_TYPES_LIST;
		gint i;
		for(i = 0; imgio_file_types_list[i] != NULL; i += 2)
		{
			if(strcasestr(imgio_file_types_list[i], ext) != NULL)
			{
				g_free(format_desc);
				format_desc = STRDUP(imgio_file_types_list[i + 1]);
				break;
			}
		}
	}

	/* Get the ImgView's toplevel widget if it is a GtkWindow */
	if(ImgViewIsToplevelWindow(iv))
		w = ImgViewGetToplevelWidget(iv);
	else
		w = NULL;
	toplevel = w;

	/* Check if the file already exists */
	if(stat((const char *)new_path, &stat_buf))
	{
		const gint error_code = (gint)errno;
		if((error_code == ENOENT) || (error_code == ENOTDIR))
			file_exists = FALSE;
		else
			file_exists = TRUE;
	}
	else
	{
		file_exists = TRUE;
	}
	if(file_exists)
	{
		/* Confirm overwrite */
		gint response;
		gchar *msg = g_strdup_printf(
"Overwrite existing file:\n\
\n\
    %s",
			new_path
		);
		CDialogSetTransientFor(toplevel);
		response = CDialogGetResponse(
			"Confirm Overwrite",
			msg,
			NULL,
			CDIALOG_ICON_WARNING,
			CDIALOG_BTNFLAG_YES | CDIALOG_BTNFLAG_NO,
			CDIALOG_BTNFLAG_NO
		);
		g_free(msg);
		CDialogSetTransientFor(NULL);
		switch(response)
		{
		  case CDIALOG_RESPONSE_YES:
		  case CDIALOG_RESPONSE_YES_TO_ALL:
		  case CDIALOG_RESPONSE_OK:
			break;
		  default:
			CLEANUP_RETURN;
			break;
		}
	}


	/* Get the original file name (if any), this may be needed
	 * for some image libraries
	 */
	orig_path = STRDUP(info->path);

	/* Allocate the save callback data */
	cb_data = IV_IMGIO_CB(g_malloc0(sizeof(IVImgIOCBData)));
	if(cb_data == NULL)
	{
		CLEANUP_RETURN;
	}
	cb_data->ref_toplevel = toplevel;
	cb_data->viewer = v;
	cb_data->path = g_strdup(new_path);

	/* Reset status to -1, it will be updated when (and if) an
	 * image saving routine is called
	 */
	status = -1;

	/* Reset the stop count before saving the image since the
	 * save progress callbacks will test it
	 */
	v->stop_count = 0;

	/* Save the image by the extension */
	if(ext != NULL)
	{
		gboolean save_attempted = FALSE;

#define WARN_NO_MULTIPLE_FRAMES_SAVED(_ext_)	{	\
 gint response;						\
 gchar *msg = g_strdup_printf(				\
"The image contains more than one frame and the\n\
specified image format (%s) is unable to store\n\
multiple frames. Only the current frame will be saved.\n\
\n\
If you want to save all the frames then try using\n\
the %s or %s image formats.\n\
\n\
Are you sure you want to save only the current frame?",	\
  ext,							\
  IMAGE_FORMAT_EXTENSION_GIF,				\
  IMAGE_FORMAT_EXTENSION_MNG				\
 );							\
 CDialogSetTransientFor(toplevel);			\
 response = CDialogGetResponse(				\
  "Save Warning",					\
  msg,							\
  NULL,							\
  CDIALOG_ICON_WARNING,					\
  CDIALOG_BTNFLAG_YES | CDIALOG_BTNFLAG_NO,		\
  CDIALOG_BTNFLAG_NO					\
 );							\
 g_free(msg);						\
 CDialogSetTransientFor(NULL);				\
 switch(response)					\
 {							\
  case CDIALOG_RESPONSE_YES:				\
  case CDIALOG_RESPONSE_YES_TO_ALL:			\
  case CDIALOG_RESPONSE_OK:				\
   break;						\
  default:						\
   CLEANUP_RETURN;					\
   break;						\
 }							\
}

		/* Save by extension */
#if defined(HAVE_LIBGIF)
		/* GIF */
		if(!g_strcasecmp(ext, IMAGE_FORMAT_EXTENSION_GIF))
		{
			save_attempted = TRUE;

			/* Query the user for GIF save options */
			if(!iv_save_options_gif_query(
				toplevel,
				new_path,
				&core->gif_interlaced,
				&core->gif_color_type,
				&core->gif_transparency,
				&core->gif_alpha_threshold,
				&core->gif_looping
			))
			{
				CLEANUP_RETURN;
			}

			/* Save by color format */
			if(format == IMGVIEW_FORMAT_RGBA)
			{
				status = ImgFileSaveGIFRGBA(
					new_path,
					width, height,
					bpl,
					(const guint8 **)rgba_list,
					delay_list,
					nframes,
 			v->bg_color,
					info->comments,
					core->gif_interlaced,
					core->gif_color_type,	/* 0 = B&W
											 * 1 = Greyscale
											 * 2 = Color */
					core->gif_transparency,
					core->gif_alpha_threshold,
					core->gif_looping,	/* 0 = Play once
											 * 1 = Repeating */
					iv_save_progress_cb, cb_data
				);
				/* Save failed? */
				if((status != 0) && (status != -4))
				{
					const gchar *error_msg = ImgSaveGetError();
					gchar *msg = g_strdup_printf(
"Failed to save image:\n\
\n\
    %s\n\
\n\
As a %s (%s) image.\n\
\n\
%s",
						new_path,
						format_desc,
						ext,
						STRISEMPTY(error_msg) ?
							"Unknown error occured." :
							error_msg
					);
					CDialogSetTransientFor(w);
					CDialogGetResponse(
						"Save Failed",
						msg,
						NULL,
						CDIALOG_ICON_ERROR,
						CDIALOG_BTNFLAG_OK,
						CDIALOG_BTNFLAG_OK
					);
					g_free(msg);
					CDialogSetTransientFor(NULL);
				}
			}
		}
#endif
#if defined(HAVE_LIBJPEG)
		/* JPEG */
		if(!g_strcasecmp(ext, IMAGE_FORMAT_EXTENSION_JFIF) ||
		   !g_strcasecmp(ext, IMAGE_FORMAT_EXTENSION_JPE) ||
		   !g_strcasecmp(ext, IMAGE_FORMAT_EXTENSION_JPEG) ||
		   !g_strcasecmp(ext, IMAGE_FORMAT_EXTENSION_JPG)
		)
		{
			save_attempted = TRUE;

			/* If there are multiple frames then query the user
			 * to continue saving or not
			 */
			if(nframes > 1)
			{
				WARN_NO_MULTIPLE_FRAMES_SAVED(ext);
			}

			/* Query the user for JPEG save options */
			if(!iv_save_options_jpeg_query(
				toplevel,
				new_path,
				&core->jpeg_quality,	/* 0.0 to 1.0 */
				&core->jpeg_is_color
			))
			{
				CLEANUP_RETURN;
			}

			/* Save by color format */
			if(format == IMGVIEW_FORMAT_RGBA)
			{
				status = ImgFileSaveJPEGRGBA(
					new_path,
					width, height,
					bpl,
					rgba,	/* Source RGBA data */
					v->bg_color,
#if defined(_WIN32)
					(gint)(core->jpeg_quality * 100.0f),
#else
					rint(core->jpeg_quality * 100.0f), /* Quality 0 to 100 */
#endif
					core->jpeg_is_color ? 1 : 0,	/* Format:
						 * 0 = Greyscale
						 * 1 = Color */
					iv_save_progress_cb, cb_data
				);
				/* Save failed? */
				if((status != 0) && (status != -4))
				{
					const gchar *error_msg = ImgSaveGetError();
					gchar *msg = g_strdup_printf(
"Failed to save image:\n\
\n\
    %s\n\
\n\
As a %s (%s) image.\n\
\n\
%s",
						new_path,
						format_desc,
						ext,
						STRISEMPTY(error_msg) ?
							"Unknown error occured." :
							error_msg
					);
					CDialogSetTransientFor(toplevel);
					CDialogGetResponse(
						"Save Failed", 
						msg,
						NULL,
						CDIALOG_ICON_ERROR,
						CDIALOG_BTNFLAG_OK,
						CDIALOG_BTNFLAG_OK
					);
					g_free(msg);
					CDialogSetTransientFor(NULL);
				}
			}
		}
#endif
#if defined(HAVE_LIBMNG)
		/* JNG */
		if(!g_strcasecmp(ext, IMAGE_FORMAT_EXTENSION_JNG))
		{
			save_attempted = TRUE;

			/* Query the user for JNG save options */
			if(!iv_save_options_jng_query(
				toplevel,
				new_path,
				&core->jng_quality,		/* 0.0 to 1.0 */
				&core->jng_is_color,
				&core->jng_transparency
			))
			{
				CLEANUP_RETURN;
			}

			/* Save by color format */
			if(format == IMGVIEW_FORMAT_RGBA)
			{
				/* Save JNG */
				status = ImgFileSaveJNGRGBA(
					new_path,
					width, height,
					bpl,
					rgba,	/* Source RGBA data */
		 			v->bg_color,	/* Background color */
					PROG_NAME_FULL " " PROG_VERSION,	/* Creator */
					info->title,
					info->author,
					info->comments,
					(unsigned long)time(NULL),	/* Modified time in
						 * seconds since EPOCH
						 * GMT */
#if defined(_WIN32)
					(gint)(core->jng_quality * 100.0f),
#else
					rint(core->jng_quality * 100.0f), /* Quality 0 to 100 */
#endif
					core->jng_is_color ? 1 : 0,	/* Format:
							 * 0 = Greyscale
							 * 1 = Color */
					core->jng_transparency ? 1 : 0,
					iv_save_progress_cb, cb_data
				);
				/* Save failed? */
				if((status != 0) && (status != -4))
				{
					const gchar *error_msg = ImgSaveGetError();
					gchar *msg = g_strdup_printf(
"Failed to save image:\n\
\n\
    %s\n\
\n\
As a %s (%s) image.\n\
\n\
%s",
						new_path,
						format_desc,
						ext,
						STRISEMPTY(error_msg) ?
							"Unknown error occured." :
							error_msg
					);
					CDialogSetTransientFor(toplevel);
					CDialogGetResponse(
						"Save Failed",
						msg,
						NULL,
						CDIALOG_ICON_ERROR,
						CDIALOG_BTNFLAG_OK,
						CDIALOG_BTNFLAG_OK
					);
					g_free(msg);
					CDialogSetTransientFor(NULL);
				}
			}
		}
		/* MNG */
		if(!g_strcasecmp(ext, IMAGE_FORMAT_EXTENSION_MNG))
		{
			save_attempted = TRUE;

			/* Query the user for MNG save options */
			if(!iv_save_options_mng_query(
				toplevel,
				new_path,
				&core->mng_compression,	/* 0 to 9 */
				&core->mng_format,/* 0 = B&W
						 * 1 = Greyscale
						 * 2 = Greyscale Alpha
						 * 3 = RGB
						 * 4 = RGBA */
				&core->mng_interlaced,
				&core->mng_looping
			))
			{
				CLEANUP_RETURN;
			}

			/* Save by color format */
			if(format == IMGVIEW_FORMAT_RGBA)
			{
				/* Save MNG */
				status = ImgFileSaveMNGRGBA(
					new_path,
					width, height,
					bpl,
					(const guint8 **)rgba_list,
					delay_list,
					nframes,
		 			v->bg_color,
					info->x, info->y,
					info->base_width, info->base_height,
					PROG_NAME_FULL " " PROG_VERSION,	/* Creator */
					info->title,		/* Title */
					info->author,		/* Author */
					info->comments,		/* Comments */
					(unsigned long)time(NULL),	/* Modified time in
													 * seconds since EPOCH
													 * GMT */
					core->mng_compression,	/* 0 to 9 */
					core->mng_interlaced,
					core->mng_format,
					core->mng_looping,
					iv_save_progress_cb, cb_data
				);
				/* Save failed? */
				if((status != 0) && (status != -4))
				{
					const gchar *error_msg = ImgSaveGetError();
					gchar *msg = g_strdup_printf(
"Failed to save image:\n\
\n\
    %s\n\
\n\
As a %s (%s) image.\n\
\n\
%s",
						new_path,
						format_desc,
						ext,
						STRISEMPTY(error_msg) ?
							"Unknown error occured." :
							error_msg
					);
					CDialogSetTransientFor(toplevel);
					CDialogGetResponse(
						"Save Failed",
						msg,
						NULL,
						CDIALOG_ICON_ERROR,
						CDIALOG_BTNFLAG_OK,
						CDIALOG_BTNFLAG_OK
					);
					g_free(msg);
					CDialogSetTransientFor(NULL);
				}
			}
		}
#endif
#if defined(HAVE_LIBPNG)
		/* PNG */
		if(!g_strcasecmp(ext, IMAGE_FORMAT_EXTENSION_PNG))

		{
			save_attempted = TRUE;

			/* If there are multiple frames then query the user
			 * to continue saving or not
			 */
			if(nframes > 1)
			{
				WARN_NO_MULTIPLE_FRAMES_SAVED(ext);
			}

			/* Query the user for PNG save options */
			if(!iv_save_options_png_query(
				toplevel,
				new_path,
				&core->png_compression,	/* 0 to 9 */
				&core->png_format,		/* 0 = B&W
											 * 1 = Greyscale
											 * 2 = Greyscale Alpha
											 * 3 = RGB
											 * 4 = RGBA */
				&core->png_interlaced
			))
			{
				CLEANUP_RETURN;
			}

			/* Save by color format */
			if(format == IMGVIEW_FORMAT_RGBA)
			{
				status = ImgFileSavePNGRGBA(
					new_path,
					width, height,
					bpl,
					rgba,			/* Source RGBA data */
					v->bg_color,		/* Background color */
					info->x, info->y,
					info->base_width, info->base_height,
					PROG_NAME_FULL " " PROG_VERSION,	/* Creator */
					info->title,		/* Title */
					info->author,		/* Author */
					info->comments,		/* Comments */
					(unsigned long)time(NULL),	/* Modified time in
													 * seconds since EPOCH
													 * GMT */
					core->png_compression,	/* 0 to 9 */
					core->png_interlaced,
					core->png_format,
					iv_save_progress_cb, cb_data
				);
				/* Save failed? */
				if((status != 0) && (status != -4))
				{
					const gchar *error_msg = ImgSaveGetError();
					gchar *msg = g_strdup_printf(
"Failed to save image:\n\
\n\
    %s\n\
\n\
As a %s (%s) image.\n\
\n\
%s",
						new_path,
						format_desc,
						ext,
						STRISEMPTY(error_msg) ?
							"Unknown error occured." :
							error_msg
					);
					CDialogSetTransientFor(toplevel);
					CDialogGetResponse(
						"Save Failed", 
						msg,
						NULL,
						CDIALOG_ICON_ERROR,
						CDIALOG_BTNFLAG_OK,
						CDIALOG_BTNFLAG_OK
					);
					g_free(msg);
					CDialogSetTransientFor(NULL);
				}
			}
		}
#endif
#if !defined(_WIN32)
		/* PS */
		if(!g_strcasecmp(ext, IMAGE_FORMAT_EXTENSION_PS) ||
		   !g_strcasecmp(ext, IMAGE_FORMAT_EXTENSION_PS2) ||
		   !g_strcasecmp(ext, IMAGE_FORMAT_EXTENSION_PS3)
		)
		{
			/* Set up the print options and call the print callback
			 * to save Postscript images
			 */
			gboolean prev_processing = v->processing;
			print_values_struct *pv = core->print_values;

			save_attempted = TRUE;

			/* If there are multiple frames then query the user
			 * to continue saving or not
			 */
			if(nframes > 1)
			{
				WARN_NO_MULTIPLE_FRAMES_SAVED(ext);
			}

			/* Need to remove the existing file (if any) since the
			 * print callback will confirm overwrite again
			 */
			UNLINK(new_path);

			/* Modify the print values */
#if defined(HAVE_XPRINT)

#else
			pv->output = PRINT_OUTPUT_TO_FILE;
			if(info->base_width > 0)
				pv->paper_width = info->base_width;
			if(info->base_height > 0)
				pv->paper_height = info->base_height;
#endif
			pv->output_x = info->x;
			pv->output_y = info->y;

			/* Unset processing marker before calling the print
			 * callback or else the print callback will not function
			 * when it detects the image viewer as processing
			 */
			if(prev_processing)
				v->processing = FALSE;
			iv_viewer_print_cb(v->print_mi, v);
			if(prev_processing)
				v->processing = TRUE;

			status = 0;
		}
#endif
		/* TGA */
		if(!g_strcasecmp(ext, IMAGE_FORMAT_EXTENSION_TGA))
		{
			save_attempted = TRUE;

			/* If there are multiple frames then query the user
			 * to continue saving or not
			 */
			if(nframes > 1)
			{
				WARN_NO_MULTIPLE_FRAMES_SAVED(ext);
			}

			/* Query the user for TGA save options */
			if(!iv_save_options_targa_query(
				toplevel,
				new_path,
				&core->tga_format		/* 0 = Greyscale
											 * 1 = RGB
											 * 2 = RGBA */
			))
			{
				CLEANUP_RETURN;
			}

			/* Save by color format */
			if(format == IMGVIEW_FORMAT_RGBA)
			{
				status = ImgFileSaveTGARGBA(
					new_path,
					width, height,
					bpl,
					rgba,			/* Source RGBA data */
					v->bg_color,
					info->x, info->y,
					info->base_width, info->base_height,
					info->comments,
					core->tga_format,
					iv_save_progress_cb, cb_data
				);
				/* Save failed? */
				if((status != 0) && (status != -4))
				{
					const gchar *error_msg = ImgSaveGetError();
					gchar *msg = g_strdup_printf(
"Failed to save image:\n\
\n\
    %s\n\
\n\
As a %s (%s) image.\n\
\n\
%s",
						new_path,
						format_desc,
						ext,
						STRISEMPTY(error_msg) ?
							"Unknown error occured." :
							error_msg
					);
					CDialogSetTransientFor(toplevel);
					CDialogGetResponse(
						"Save Failed",
						msg,
						NULL, 
						CDIALOG_ICON_ERROR,
						CDIALOG_BTNFLAG_OK,
						CDIALOG_BTNFLAG_OK
					);
					g_free(msg);
					CDialogSetTransientFor(NULL);
				}
			}
		}
#ifdef HAVE_LIBTIFF
		/* TIFF */
		if(!g_strcasecmp(ext, IMAGE_FORMAT_EXTENSION_TIFF) ||
		   !g_strcasecmp(ext, IMAGE_FORMAT_EXTENSION_TIF)
		)
		{
			save_attempted = TRUE;

			/* Query the user for TIFF save options */
			if(!iv_save_options_tiff_query(
				toplevel,
				new_path,
				&core->tiff_compression_type,/* 0 = None
						 * 1 = GZIP
						 * 2 = JPEG
						 * 3 = CCITT RLE
						 * 4 = CCITT FAX3
						 * 5 = CCITT FAX4 */
				&core->tiff_jpeg_quality,/* 0.0 to 1.0 */
				&core->tiff_color_type	/* 0 = B&W
						 * 1 = Greyscale
						 * 2 = Greyscale Alpha
						 * 3 = RGB
						 * 4 = RGBA */
			))
			{
				CLEANUP_RETURN;
			}

			/* Save by color format */
			if(format == IMGVIEW_FORMAT_RGBA)
			{
				/* Save TIFF */
				status = ImgFileSaveTIFFRGBA(
					new_path,
					width, height,
					bpl,
					(const guint8 **)rgba_list,
					delay_list,
					nframes,
		 			v->bg_color,
					info->x, info->y,
					info->base_width, info->base_height,
					PROG_NAME_FULL " " PROG_VERSION,/* Creator */
					info->title,
					info->author,
					info->comments,
					core->tiff_compression_type,
					core->tiff_jpeg_quality,
					core->tiff_color_type,
					iv_save_progress_cb, cb_data
				);
				/* Save failed? */
				if((status != 0) && (status != -4))
				{
					const gchar *error_msg = ImgSaveGetError();
					gchar *msg = g_strdup_printf(
"Failed to save image:\n\
\n\
    %s\n\
\n\
As a %s (%s) image.\n\
\n\
%s",
						new_path,
						format_desc,
						ext,
						STRISEMPTY(error_msg) ?
							"Unknown error occured." :
							error_msg
					);
					CDialogSetTransientFor(toplevel);
					CDialogGetResponse(
						"Save Failed",
						msg,
						NULL,
						CDIALOG_ICON_ERROR,
						CDIALOG_BTNFLAG_OK,
						CDIALOG_BTNFLAG_OK
					);
					g_free(msg);
					CDialogSetTransientFor(NULL);
				}
			}
		}
#endif
#if defined(HAVE_LIBXPM)
		/* XPM */
		if(!g_strcasecmp(ext, IMAGE_FORMAT_EXTENSION_XPM))
		{
			gchar	*s,
					*c_id;

			save_attempted = TRUE;

			/* If there are multiple frames then query the user
			 * to continue saving or not
			 */
			if(nframes > 1)
			{
				WARN_NO_MULTIPLE_FRAMES_SAVED(ext);
			}

			/* Get the C ID from file name */
			s = g_basename(new_path);
			c_id = (s != NULL) ? g_strdup(s) : g_strdup(new_path);

			/* Replace unacceptable characters in the C ID */
			for(s = c_id; *s != '\0'; s++)
			{
				if(!isalnum(*s))
					*s = '_';
			}

			/* Query the user for XPM save options */
			if(!iv_save_options_xpm_query(
				toplevel,
				new_path,
				&c_id,
				&core->xpm_format,/* 0 = B&W
						 * 1 = Greyscale
						 * 2 = Color */
				&core->xpm_max_colors,	/* -1 for no limit */
				&core->xpm_threshold	/* 0x00 to 0xff */
			))
			{
				g_free(c_id);
				CLEANUP_RETURN;
			}

			/* Save by color format */
			if(format == IMGVIEW_FORMAT_RGBA)
			{
				status = ImgFileSaveXPMRGBA(
					new_path,
					width, height,
					bpl,
					rgba,
					v->bg_color,
					info->x, info->y,
					info->base_width, info->base_height,
					c_id, info->comments,
					core->xpm_format,/* 0 = B&W
						 * 1 = Greyscale
						 * 2 = Color */
					core->xpm_max_colors,/* -1 for no limit */
					core->xpm_threshold,/* 0x00 to 0xff */
					iv_save_progress_cb, cb_data
				);
				/* Save failed? */
				if((status != 0) && (status != -4))
				{
					const gchar *error_msg = ImgSaveGetError();
					gchar *msg = g_strdup_printf(
"Failed to save image:\n\
\n\
    %s\n\
\n\
As a %s (%s) image.\n\
\n\
%s",
						new_path,
						format_desc,
						ext,
						STRISEMPTY(error_msg) ?
							"Unknown error occured." :
							error_msg
					);
					CDialogSetTransientFor(toplevel);
					CDialogGetResponse(
						"Save Failed",
						msg,
						NULL,
						CDIALOG_ICON_ERROR,
						CDIALOG_BTNFLAG_OK,
						CDIALOG_BTNFLAG_OK
					);
					g_free(msg);
					CDialogSetTransientFor(NULL);
				}
			}

			g_free(c_id);
		}
#endif
		/* Was a save attempted using a specific image library? */
		if(save_attempted)
		{
			/* Save was attempted */
		}
		else	/* All else try to save using Imlib */
		{
			/* A save was not attempted using a specific image
			 * library, so now we try to use Imlib to save all
			 * other image formats
			 */
			save_attempted = TRUE;
			if(format == IMGVIEW_FORMAT_RGBA)
			{
				/* Save image using Imlib */
				status = ImgFileSaveImlibRGBA(
					new_path,
					width, height,
					bpl,
					rgba,
					v->bg_color,
					info->comments,
					1.0f,	/* Quality, 0.0 to 1.0 */
					1,	/* Save as color (1 = color) */
					0xFF,	/* Default alpha value */
					core->imlib_fallback,
					iv_save_progress_cb, cb_data
				);
				/* Save failed? */
				if((status != 0) && (status != -4))
				{
					const gchar *error_msg = ImgSaveGetError();
					gchar *msg = g_strdup_printf(
"Failed to save image:\n\
\n\
    %s\n\
\n\
With %s.\n\
\n\
%s",
						new_path,
#if defined(HAVE_IMLIB)
						"Imlib",
#elif defined(HAVE_IMLIB2)
						"Imlib2",
#else
						"unknown library",
#endif
						STRISEMPTY(error_msg) ?
"Please check if the image format and the specified save\n\
options (if any) are supported by this application." :
						error_msg
					);
					CDialogSetTransientFor(toplevel);
					CDialogGetResponse(
						"Save Failed",
						msg,
						NULL,
						CDIALOG_ICON_ERROR,
						CDIALOG_BTNFLAG_OK,
						CDIALOG_BTNFLAG_OK
					);
					g_free(msg);
					CDialogSetTransientFor(NULL);
				}
			}
		}

#undef WARN_NO_MULTIPLE_FRAMES_SAVED
	}	/* if(ext != NULL) */
	else
	{
		/* No extension specified */
		gchar *msg = g_strdup_printf(
"No extension:\n\
\n\
    %s\n\
\n\
You must specify a file name with an extension\n\
that describes an image format supported by this\n\
program, example:\n\
\n\
    \"%s.png\"",
			new_path,
			new_path
		);
		CDialogSetTransientFor(toplevel);
		CDialogGetResponse(
			"Save Failed",
			msg,
			NULL,
			CDIALOG_ICON_WARNING,
			CDIALOG_BTNFLAG_OK,
			CDIALOG_BTNFLAG_OK
		);
		g_free(msg);
		CDialogSetTransientFor(NULL);
	}

	/* Save successful? */
	if(status == 0)
	{
		const gulong last_modified = (gulong)time(NULL);
		gchar	*creator,
					*title,
					*author,
					*comments;

		/* Reset the has changes marker */
		v->has_changes = FALSE;

		/* Get the stats for newly saved image file */
		if(stat((const char *)new_path, &stat_buf))
			(void)memset(&stat_buf, 0x00, sizeof(struct stat));

		/* Update the image information with the information from
		 * the newly saved image
		 *
		 * Note that the creator is set to this program's name and
		 * version
		 */
		creator = STRDUP(PROG_NAME_FULL " " PROG_VERSION);
		title = STRDUP(info->title);
		author = STRDUP(info->author);
		comments = STRDUP(info->comments);
		iv_viewer_update_image_info(
			v,
			new_path,
			&stat_buf,
			width, height,
			bpp, bpl,
			nframes,
			nloops,
			play_time_ms,
			total_play_time_ms,
			TRUE,			/* Has alpha */
			x, y,
			base_width, base_height,
			creator, title, author, comments,
			last_modified
		);
		g_free(creator);
		g_free(title);
		g_free(author);
		g_free(comments);

		/* Update the ImgView's title */
		iv_viewer_update_title(v, new_path);
	}
	/* User aborted? */
	else if(status == -4)
	{
		gchar *msg = g_strdup_printf(
"User aborted the saving of:\n\
\n\
    %s\n\
\n\
This image may be corrupt and unreadable.",
			new_path
		);
		CDialogSetTransientFor(toplevel);
		CDialogGetResponse(
			"Save Aborted",
			msg,
			NULL,
			CDIALOG_ICON_WARNING,
			CDIALOG_BTNFLAG_OK,
			CDIALOG_BTNFLAG_OK
		);
		g_free(msg);
		CDialogSetTransientFor(NULL);
	}

	/* Reset the image viewer's progress bar */
	ImgViewProgressUpdate(iv, 0.0f, FALSE);

	/* Update the Image Viewer's widgets */
	iv_viewer_update(v);

#ifdef HAVE_LIBENDEAVOUR2
	/* Report the saved file */
	if(file_exists)
		edv_notify_queue_vfs_object_modified(
			core->edv2_ctx,
			new_path,
			new_path
		);
	else
		edv_notify_queue_vfs_object_added(
			core->edv2_ctx,
			new_path
		);
	edv_context_sync(core->edv2_ctx);
#endif	/* HAVE_LIBENDEAVOUR2 */

	CLEANUP_RETURN;
#undef CLEANUP_RETURN
}

#if !defined(_WIN32)
/*
 *	Screenshot.
 */
static void iv_viewer_screenshot(IVImageViewer *v)
{
	gint		status,
			x, y,
			base_width, base_height;
	GtkWidget	*toplevel,
			*grab_window;
	imgview_struct *iv;
	imgview_image_struct *img;
	IVOptionFlags options;
	IVCore *core;

	if(v == NULL)
		return;

	iv = v->iv;
	core = v->core;
	options = core->options;

	/* Get the ImgView's toplevel widget if it is a GtkWindow */
	if(ImgViewIsToplevelWindow(iv))
		toplevel = ImgViewGetToplevelWidget(iv);
	else
		toplevel = NULL;

	/* Since taking a screenshot will place the current image we
	 * need to check if there are any unsaved changes and if so
	 * then query the user to save those changes
	 */
	if(!iv_viewer_check_query_save_changes(v))
		return;

	/* Get the options for the screenshot */
	if(!iv_screenshot_query(
		toplevel,
		&core->screenshot_delay_ms,
		&core->screenshot_hide_windows,
		&core->screenshot_include_decorations,
		&core->screenshot_record_date_in_comments,
		&core->screenshot_auto_save,
		&core->screenshot_save_dir_path,
		&core->screenshot_save_ext
	))
		return;

	/* Create the grab GtkWindow */
	grab_window = core->grab_window;
	if(grab_window == NULL)
	{
		GtkWidget *w = gtk_window_new(GTK_WINDOW_POPUP);
		gtk_window_set_policy(GTK_WINDOW(w), FALSE, FALSE, FALSE);
		gtk_widget_set_uposition(w, 0, 0);
		gtk_widget_set_usize(w, 1, 1);
		gtk_widget_add_events(
			w,
			GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK |
			GDK_POINTER_MOTION_MASK | GDK_POINTER_MOTION_HINT_MASK
		);
		gtk_widget_realize(w);
		core->grab_window = grab_window = w;
	}
	if(grab_window == NULL)
		return;

	/* Need to unmap the ImgView for the grab? */
	if(iv->map_state && core->screenshot_hide_windows)
		ImgViewUnmap(iv);

	/* Map the grab GtkWindow */
	gtk_widget_show(grab_window);

	/* Need to flush and process any events so that the new grab
	 * window creation and unmapping of other windows take place
	 */
	gdk_flush();
	gtk_events_process();

	/* Query the user to take a screenshot and get the grabbed
	 * image
	 */
	status = iv_screenshot_start(
		iv,
		core->screenshot_delay_ms,
		core->screenshot_include_decorations,
		GTK_WIDGET(grab_window),
		core->grab_cur,
		&img,				/* Grabbed image return */
		&x, &y,
		&base_width, &base_height
	);

	/* Remap the ImgView? */
	if(!iv->map_state)
		ImgViewMap(iv);

	/* Destroy the grab GtkWindow */
	core->grab_window = NULL;
	if(grab_window != NULL)
	{
		gtk_widget_hide(grab_window);
		GTK_WIDGET_DESTROY(grab_window);
	}

	/* Screenshot user aborted? */
	if(status == -4)
	{
		/* User aborted, do nothing */
	}
	/* Screenshot other error? */
	else if(status != 0)
	{
		CDialogSetTransientFor(toplevel);
		CDialogGetResponse(
			"Screenshot Failed",
			iv_screenshot_get_error(),
			NULL,
			CDIALOG_ICON_ERROR,
			CDIALOG_BTNFLAG_OK,
			CDIALOG_BTNFLAG_OK
		);
		CDialogSetTransientFor(NULL);
	}
	/* Screenshot successful */
	else
	{
		/* Got a screenshot image? */
		if(img != NULL)
		{
			/* Get ImgView's view widget */
			const gint	width = img->width,
					height = img->height,
					nframes = img->nframes;
			GtkWidget *w = (GtkWidget *)ImgViewGetViewWidget(iv);
			IVInfo *info = &v->info;

			/* Increment the untitled count since a screenshot is
			 * considered a new image
			 */
			core->untitled_count++;

			/* Resize the ImgView? */
			if(options & IV_OPTION_RESIZE_ON_OPEN)
			{
				gtk_widget_set_usize(w, width, height);
				gtk_widget_queue_resize(w);
				if(toplevel != NULL)
					gtk_widget_queue_resize(toplevel);
			}

			/* Set the screenshot image to the ImgView */
			if(options & IV_OPTION_ZOOM_TO_FIT_ON_OPEN)
				ImgViewSetImageToFit(
					iv,
					img	/* Transfered */
				);
			else
				ImgViewSetImage(
					iv,
					img	/* Transfered */
				);
	 		img = NULL;		/* Mark as transfered */

			/* Mark that the new image has not be saved yet */
			v->has_changes = TRUE;

			/* Clear the image information */
			iv_clear_info(info);

			/* Set the new image information */
			info->width = width;
			info->height = height;
			info->nframes = nframes;
			info->x = x;
			info->y = y;
			info->base_width = base_width;
			info->base_height = base_height;
			info->has_alpha = FALSE;
			g_free(info->creator);
			info->creator = g_strdup(
				PROG_NAME_FULL " " PROG_VERSION
			);
	
			/* Record the date in the image header's comments? */
#ifdef HAVE_LIBENDEAVOUR2
			if(core->screenshot_record_date_in_comments &&
			   (core->edv2_ctx != NULL)
			)
			{
				gchar	*date_str = STRDUP(edv_date_format_absolute(
					core->edv2_ctx,
					(gulong)time(NULL)
				)),
					*user_str = edv_uid_uid_to_name(
					core->edv2_ctx,
					(gint)geteuid()
				),
					host_name_str[NAME_MAX],
					*display_str = STRDUP((const gchar *)gdk_get_display());

				if(gethostname((char *)host_name_str, sizeof(host_name_str)))
					*host_name_str = '\0';
				else
					host_name_str[sizeof(host_name_str) - 1] = '\0';

				g_free(info->comments);
				info->comments = g_strdup_printf(
"Screenshot taken on %s by %s@%s from display \"%s\".",
					date_str,
					user_str,
					host_name_str,
					display_str
				);
				g_free(date_str);
				g_free(user_str);
				g_free(display_str);
			}
#else
			if(core->screenshot_record_date_in_comments)
			{
				time_t t = time(NULL);
				const gchar *s = (const char *)ctime(&t);
				gchar *date_str = STRDUP(s);
				if(date_str != NULL)
				{
					gchar *s = (gchar *)strpbrk(
						(const char *)date_str,
						"\n\r"
					);
					if(s != NULL)
						*s = '\0';
				}
				g_free(info->comments);
				info->comments = g_strdup_printf(
"Screenshot taken on %s.",
					date_str
				);
				g_free(date_str);
			}
#endif

			/* Update the ImgView's title */
			iv_viewer_update_title(v, NULL);

			/* Automatically save the image after the screenshot? */
			if(core->screenshot_auto_save)
			{
/* TODO */



			}
		}
	}

	/* Update the Image Viewer's widgets */
	iv_viewer_update(v);

	/* Delete the returned screenshot image if it was not transfered */
	ImgViewImageDelete(img);
}
#endif	/* !_WIN32 */


/*
 *	Opens the image, displays it fullscreen, and waits for user
 *	response before returning.
 *
 *	The path specifies the full path to the image to be printed.
 *
 *	The frame_num specifies the index of the image's frame to be
 *	displayed. If frame_num is -1 then the default frame will be
 *	displayed.
 *
 *	The bg_color specifies the background color in 4 bytes RGBA
 *	format.
 *
 *	The quality specifies the image data to window rendering
 *	dither level. Where 2 is maximum dithering, 1 is normal
 *	dithering, and 0 is no dithering.
 */
static gint iv_open_fullscreen(
	const gchar *path,
	const gint frame_num,
	guint8 *bg_color,
	const gint quality
)
{
	gint		i,
					status,
					nframes,
					width, height,
					bpl,
					x, y,
					base_width, base_height;
	gchar		*creator,
					*title,
					*author,
					*comments;
	guint8		**rgba_list,
					lbg_color[4];
	gulong		*delay_list,
					modified_time_sec;

	/* Use local default background color? */
	if(bg_color == NULL)
	{
		bg_color = lbg_color;
		bg_color[0] = 0x00;
		bg_color[1] = 0x00;
		bg_color[2] = 0x00;
		bg_color[3] = 0xff;
	}

	/* Open the image from file to RGBA */
	status = ImgFileOpenRGBA(
		path,
		&width, &height,
		&bpl,
		&rgba_list,
		&delay_list,
		&nframes,
		bg_color,				/* 4 bytes RGBA (will be modified) */
		&x, &y,
		&base_width, &base_height,
		&creator, &title, &author, &comments,
		&modified_time_sec,			/* Modified time in seconds
											 * since EPOCH GMT */
		0xFF,				/* Default alpha value */
		NULL, NULL				/* No progress callback */
	);

	/* Image opened successfully? */
	if((status == 0) && (rgba_list != NULL) && (nframes > 0))
	{
		/* Display a specific frame? */
		if(frame_num > -1)
		{
			const gint s_frame_num = MIN(frame_num, (nframes - 1));
			guint8 *s_rgba_list[1];
			gulong s_delay_list[1];

			s_rgba_list[0] = rgba_list[s_frame_num];
			s_delay_list[0] = delay_list[s_frame_num];

			/* Display the specified frame of the image fullscreen
			 * and wait for the user to respond
			 */
			FSImgNew(
				s_rgba_list,
				s_delay_list,
				1,				/* 1 frame */
				width, height, bpl,
				bg_color,			/* 4 bytes RGBA */
				quality,
				TRUE,			/* Block until the user responds */
				NULL
			);
		}
		else
		{
			/* Display the image fullscreen and wait for the user
			 * to respond
			 */
			FSImgNew(
				rgba_list,
				delay_list,
				nframes,
				width, height, bpl,
				bg_color,			/* 4 bytes RGBA */
				quality,
				TRUE,			/* Block until the user responds */
				NULL
			);
		}
	}

	for(i = 0; i < nframes; i++)
		g_free(rgba_list[i]);
	g_free(rgba_list);
	g_free(delay_list);
	g_free(creator);
	g_free(title);
	g_free(author);
	g_free(comments);

	return(status);
}

/*
 *	Opens the image and displays it on the desktop centered.
 *
 *	The path specifies the full path to the image to be printed.
 *
 *	The frame_num specifies the index of the image's frame to be
 *	displayed. If frame_num is -1 then the default frame will be
 *	displayed.
 *
 *	The bg_color specifies the background color in 4 bytes RGBA
 *	format.
 *
 *	The quality specifies the image data to window rendering
 *	dither level. Where 2 is maximum dithering, 1 is normal
 *	dithering, and 0 is no dithering.
 */
static gint iv_open_to_root_centered(
	const gchar *path,
	const gint frame_num,
	guint8 *bg_color,
	const gint quality
)
{
	const gint	bpp = 4;		/* RGBA */
	gint		i,
					status,
					nframes,
					width, height,
					bpl,
					x, y,
					base_width, base_height;
	gchar		*creator,
					*title,
					*author,
					*comments;
	guint8		**rgba_list,
					lbg_color[4];
	gulong		*delay_list,
					modified_time_sec;

	/* Use the default background color? */
	if(bg_color == NULL)
	{
		bg_color = lbg_color;
		bg_color[0] = 0x00;
		bg_color[1] = 0x00;
		bg_color[2] = 0x00;
		bg_color[3] = 0xff;
	}

	/* Open the image from file to RGBA */
	status = ImgFileOpenRGBA(
		path,
		&width, &height,
		&bpl,
		&rgba_list,
		&delay_list,
		&nframes,
		bg_color,				/* 4 bytes RGBA (will be modified) */
		&x, &y,
		&base_width, &base_height,
		&creator, &title, &author, &comments,
		&modified_time_sec,			/* Modified time in seconds
											 * since EPOCH GMT */
		0xFF,				/* Default alpha value */
		NULL, NULL				/* No progress callback */
	);

	/* Image opened successfully? */
	if((status == 0) && (rgba_list != NULL) && (nframes > 0))
	{
		const gint s_frame_num = (frame_num > -1) ?
			MIN(frame_num, (nframes - 1)) : 0;
		guint8 *rgba = rgba_list[s_frame_num];
		GdkWindow *root_win = gdk_window_get_root();

		/* Display the image on the desktop centered */
		iv_desktop_put_image_centered(
			root_win,
			quality,
			rgba,
			width, height,
			bpp, bpl,
			bg_color
		);

		/* Manage any pending events to ensure that the image is
		 * displayed properly before proceeding with additional
		 * operations 
		 */
		gtk_events_process();
	}
	else
	{
		/* Failed to open the image, print error message on to the
		 * desktop   
		 */
		const gchar *error_msg = ImgOpenGetError();
		gchar *s;
		if(error_msg != NULL)
			s = g_strdup_printf(
"Unable to open the image:\n\
\n\
    %s\n\
\n\
%s",
				path,
				error_msg
			);
		else
			s = g_strdup_printf(
"Unable to open the image:\n\
\n\
    %s",
				path
			);
		PRINT_ERROR_TO_DESKTOP(s);
		g_free(s);
	}

	for(i = 0; i < nframes; i++)
		g_free(rgba_list[i]);
	g_free(rgba_list);
	g_free(delay_list);
	g_free(creator);
	g_free(title);
	g_free(author);
	g_free(comments);

	return(status);
}

/*
 *	Opens the image and displays it scaled to fit (preserving
 *	aspect ratio) on the desktop.
 *
 *	The path specifies the full path to the image to be printed.
 *
 *	The frame_num specifies the index of the image's frame to be
 *	displayed. If frame_num is -1 then the default frame will be
 *	displayed.
 *
 *	The bg_color specifies the background color in 4 bytes RGBA
 *	format.
 *
 *	The quality specifies the image data to window rendering
 *	dither level. Where 2 is maximum dithering, 1 is normal
 *	dithering, and 0 is no dithering.
 */
static gint iv_open_to_root_tofit(
	const gchar *path,
	const gint frame_num,		/* Can be -1 for unspecified */
	guint8 *bg_color,		/* 4 bytes in RGBA format */
	const gint quality		/* Quality, 0 to 2 */
)
{
	const gint	bpp = 4;		/* RGBA */
	gint		i,
					status,
					nframes,
					width, height,
					bpl,
					x, y,
					base_width, base_height;
	gchar		*creator,
					*title,
					*author,
					*comments;
	guint8		**rgba_list,
					lbg_color[4];
	gulong		*delay_list,
					modified_time_sec;

	/* Use the default background color? */
	if(bg_color == NULL)
	{
		bg_color = lbg_color;
		bg_color[0] = 0x00;
		bg_color[1] = 0x00;
		bg_color[2] = 0x00;
		bg_color[3] = 0xff;
	}

	/* Open the image from file to RGBA */
	status = ImgFileOpenRGBA(
		path,
		&width, &height,
		&bpl,
		&rgba_list,
		&delay_list,
		&nframes,
		bg_color,				/* 4 bytes RGBA (will be modified) */
		&x, &y,
		&base_width, &base_height,
		&creator, &title, &author, &comments,
		&modified_time_sec,			/* Modified time in seconds
											 * since EPOCH GMT */
		0xFF,				/* Default alpha value */
		NULL, NULL				/* No progress callback */
	);

	/* Image opened successfully? */
	if((status == 0) && (rgba_list != NULL) && (nframes > 0))
	{
		const gint s_frame_num = (frame_num > -1) ?
			MIN(frame_num, (nframes - 1)) : 0;
		guint8 *rgba = rgba_list[s_frame_num];
		GdkWindow *root_win = gdk_window_get_root();

		/* Display the image on the desktop to fit */
		iv_desktop_put_image_tofit(
			root_win,
			quality,
			rgba,
			width, height,
			bpp, bpl,
			bg_color
		);

		/* Manage any pending events to ensure that the image is
		 * displayed properly before proceeding with additional
		 * operations 
		 */
		gtk_events_process();
	}
	else
	{
		/* Failed to open the image, print error message on to the
		 * desktop   
		 */
		const gchar *error_msg = ImgOpenGetError();
		gchar *s;
		if(error_msg != NULL)
			s = g_strdup_printf(
"Unable to open the image:\n\
\n\
    %s\n\
\n\
%s",
				path,
				error_msg
			);
		else
			s = g_strdup_printf(
"Unable to open the image:\n\
\n\
    %s",
				path
			);
		PRINT_ERROR_TO_DESKTOP(s)
		g_free(s);
	}

	for(i = 0; i < nframes; i++)
		g_free(rgba_list[i]);
	g_free(rgba_list);
	g_free(delay_list);
	g_free(creator);
	g_free(title);
	g_free(author);
	g_free(comments);

	return(status);
}

/*
 *	Opens the image and displays it scaled to cover (preserving 
 *	aspect ratio) on the desktop.
 *
 *	The path specifies the full path to the image to be printed.
 *
 *	The frame_num specifies the index of the image's frame to be
 *	displayed. If frame_num is -1 then the default frame will be
 *	displayed.
 *
 *	The bg_color specifies the background color in 4 bytes RGBA
 *	format.
 *
 *	The quality specifies the image data to window rendering
 *	dither level. Where 2 is maximum dithering, 1 is normal
 *	dithering, and 0 is no dithering.
 */
static gint iv_open_to_root_tocover(
	const gchar *path,
	const gint frame_num,		/* Can be -1 for unspecified */
	guint8 *bg_color,		/* 4 bytes in RGBA format */
	const gint quality		/* Quality, 0 to 2 */
)
{
	const gint	bpp = 4;		/* RGBA */
	gint		i,
					status,
					nframes,
					width, height,
					bpl,
					x, y,
					base_width, base_height;
	gchar		*creator,
					*title,
					*author,
					*comments;
	guint8		**rgba_list,
					lbg_color[4];
	gulong		*delay_list,
					modified_time_sec;

	/* Use the default background color? */
	if(bg_color == NULL)
	{
		bg_color = lbg_color;
		bg_color[0] = 0x00;
		bg_color[1] = 0x00;
		bg_color[2] = 0x00;
		bg_color[3] = 0xff;
	}

	/* Open the image from file to RGBA */
	status = ImgFileOpenRGBA(
		path,
		&width, &height,
		&bpl,
		&rgba_list,
		&delay_list,
		&nframes,
		bg_color,				/* 4 bytes RGBA (will be modified) */
		&x, &y,
		&base_width, &base_height,
		&creator, &title, &author, &comments,
		&modified_time_sec,			/* Modified time in seconds
											 * since EPOCH GMT */
		0xFF,				/* Default alpha value */
		NULL, NULL				/* No progress callback */
	);

	/* Image opened successfully? */
	if((status == 0) && (rgba_list != NULL) && (nframes > 0))
	{
		const gint s_frame_num = (frame_num > -1) ?
			MIN(frame_num, (nframes - 1)) : 0;
		guint8 *rgba = rgba_list[s_frame_num];
		GdkWindow *root_win = gdk_window_get_root();

		/* Display the image on the desktop to cover */
		iv_desktop_put_image_tocover(
			root_win,
			quality,
			rgba,
			width, height,
			bpp, bpl,
			bg_color
		);

		/* Manage any pending events to ensure that the image is
		 * displayed properly before proceeding with additional
		 * operations 
		 */
		gtk_events_process();
	}
	else
	{
		/* Failed to open the image, print error message on to the
		 * desktop
		 */
		const gchar *error_msg = ImgOpenGetError();
		gchar *s;
		if(error_msg != NULL)
			s = g_strdup_printf(
"Unable to open the image:\n\
\n\
    %s\n\
\n\
%s",
				path,
				error_msg
			);
		else
			s = g_strdup_printf(
"Unable to open the image:\n\
\n\
    %s",
				path
			);
		PRINT_ERROR_TO_DESKTOP(s)
		g_free(s);
	}

	for(i = 0; i < nframes; i++)
		g_free(rgba_list[i]);
	g_free(rgba_list);
	g_free(delay_list);
	g_free(creator);
	g_free(title);
	g_free(author);
	g_free(comments);

	return(status);
}

/*
 *	Opens the image and displays it tiled on the desktop.
 *
 *	The path specifies the full path to the image to be printed.
 *
 *	The frame_num specifies the index of the image's frame to be
 *	displayed. If frame_num is -1 then the default frame will be
 *	displayed.
 *
 *	The bg_color specifies the background color in 4 bytes RGBA
 *	format.
 *
 *	The quality specifies the image data to window rendering
 *	dither level. Where 2 is maximum dithering, 1 is normal
 *	dithering, and 0 is no dithering.
 */
static gint iv_open_to_root_tiled(
	const gchar *path,
	const gint frame_num,		/* Can be -1 for unspecified */
	guint8 *bg_color,		/* 4 bytes in RGBA format */
	const gint quality		/* Quality, 0 to 2 */
)
{
	const gint	bpp = 4;		/* RGBA */
	gint		i,
					status,
					nframes,
					width, height,
					bpl,
					x, y,
					base_width, base_height;
	gchar		*creator,
					*title,
					*author,
					*comments;
	guint8		**rgba_list,
					lbg_color[4];
	gulong		*delay_list,
					modified_time_sec;

	/* Use the default background color? */
	if(bg_color == NULL)
	{
		bg_color = lbg_color;
		bg_color[0] = 0x00;
		bg_color[1] = 0x00;
		bg_color[2] = 0x00;
		bg_color[3] = 0xff;
	}

	/* Open the image from file to RGBA */
	status = ImgFileOpenRGBA(
		path,
		&width, &height,
		&bpl,
		&rgba_list,
		&delay_list,
		&nframes,
		bg_color,				/* 4 bytes RGBA (will be modified) */
		&x, &y,
		&base_width, &base_height,
		&creator, &title, &author, &comments,
		&modified_time_sec,			/* Modified time in seconds
											 * since EPOCH GMT */
		0xFF,				/* Default alpha value */
		NULL, NULL				/* No progress callback */
	);

	/* Image opened successfully? */
	if((status == 0) && (rgba_list != NULL) && (nframes > 0))
	{
		const gint s_frame_num = (frame_num > -1) ?
			MIN(frame_num, (nframes - 1)) : 0;
		guint8 *rgba = rgba_list[s_frame_num];
		GdkWindow *root_win = gdk_window_get_root();

		/* Display the image on the desktop tiled */
		iv_desktop_put_image_tiled(
			root_win,
			quality,
			rgba,
			width, height,
			bpp, bpl,
			bg_color
		);

		/* Manage any pending events to ensure that the image is
		 * displayed properly before proceeding with additional
		 * operations
		 */
		gtk_events_process();
	}
	else
	{
		/* Failed to open the image, print error message on to the
		 * desktop   
		 */
		const gchar *error_msg = ImgOpenGetError();
		gchar *s;
		if(error_msg != NULL)
			s = g_strdup_printf(
"Unable to open the image:\n\
\n\
    %s\n\
\n\
%s",
				path,
				error_msg
			);
		else
			s = g_strdup_printf(
"Unable to open the image:\n\
\n\
    %s",
				path
			);
		PRINT_ERROR_TO_DESKTOP(s)
		g_free(s);
	}

 	for(i = 0; i < nframes; i++)
		g_free(rgba_list[i]);
 	g_free(rgba_list);
	g_free(delay_list);
	g_free(creator);
	g_free(title);
	g_free(author);
	g_free(comments);

	return(status);
}

/*
 *	Opens the image and queries the user to print it using the
 *	Print Dialog. If the user selects print then the image is
 *	printed.
 *
 *	The path specifies the full path to the image to be printed.
 *
 *	The frame_num specifies the index of the image's frame to
 *	be printed. If frame_num is -1 then the default frame will
 *	be printed.
 *
 *	The bg_color specifies the background color in 4 bytes RGBA
 *	format.
 */
static gint iv_open_print(
	const gchar *path,
	const gint frame_num,		/* Can be -1 for unspecified */
	guint8 *bg_color		/* 4 bytes in RGBA format */
)
{
	const gint	bpp = 4;
	gint		i,
					status,
					nframes,
					width, height,
					bpl,
					x, y,
					base_width, base_height;
	guint8		**rgba_list,
					lbg_color[4];
	gulong		*delay_list,
					modified_time_sec;
	gchar		*creator,
					*title,
					*author,
					*comments;

	/* Need to initialize the confirmation and progress dialogs
	 * for use during this call since they will not be initialzed
	 * in main()
	 */
	ProgressDialogInit();
	CDialogInit();

	/* Use the default background color? */
	if(bg_color == NULL)
	{
		bg_color = lbg_color;
		bg_color[0] = 0x00;
		bg_color[1] = 0x00;
		bg_color[2] = 0x00;
		bg_color[3] = 0xff;
	}

	/* Open the image from file to RGBA */
	status = ImgFileOpenRGBA(
		path,
		&width, &height,
		&bpl,
		&rgba_list,
		&delay_list,
		&nframes,
		bg_color,				/* 4 bytes RGBA (will be modified) */
		&x, &y,
		&base_width, &base_height,
		&creator, &title, &author, &comments,
		&modified_time_sec,			/* Modified time in seconds
											 * since EPOCH GMT */
		0xFF,				/* Default alpha value */
		NULL, NULL				/* No progress callback */
	);
	if((status == 0) && (rgba_list != NULL) && (nframes > 0))
	{
		/* Image opened successfully */
		const gchar *name = (path != NULL) ? g_basename(path) : NULL;
		imgview_image_struct *img;
		print_values_struct *pv;
#ifdef HAVE_LIBENDEAVOUR2
		EDVContext *edv2_ctx = edv_context_new();
#endif

#ifdef HAVE_LIBENDEAVOUR2
		edv_context_init(edv2_ctx, NULL);
#endif

		/* Create a new ImgView image and transfer the opened
		 * image data to it
		 */
		img = ImgViewImageNew(
			width, height,
			bpp, bpl
		);
		if(img != NULL)
		{
			for(i = 0; i < nframes; i++)   
			{
				(void)ImgViewImageAppendFrame(
					img,
					rgba_list[i],
					(delay_list != NULL) ?
						delay_list[i] : 0l
				);
				rgba_list[i] = NULL;
			}
		}

		/* Create and set up the print values */
		pv = PrintValuesNew();
		PrintValuesReset(pv);
#if defined(HAVE_XPRINT)
		pv->flags |= PRINT_VALUE_JOB_TITLE;
		g_free(pv->job_title);
		pv->job_title = g_strdup_printf(
			"%s: %s",
			PROG_NAME_FULL,
			STRISEMPTY(title) ? path : title
		);
#endif
		pv->flags |= PRINT_VALUE_FRAME;
		pv->frame = CLIP(frame_num, 0, (nframes - 1));
		pv->flags |= PRINT_VALUE_OUTPUT_WIDTH | PRINT_VALUE_OUTPUT_HEIGHT;
		pv->output_width = img->width;
		pv->output_height = img->height;

		/* Query the user to print the image and thus get the rest
		 * of the print values from the user
		 */
#ifdef HAVE_LIBENDEAVOUR2
		if(PrintDlgGetResponse(
			img,
			name,
			pv,				/* Print values will be modified */
			bg_color,			/* 4 bytes RGBA */
			NULL,				/* No toplevel GtkWidget */
			edv2_ctx
		))
#else
		if(PrintDlgGetResponse(
			img,
			name,
			pv,				/* Print values will be modified */
			bg_color,			/* 4 bytes RGBA */
			NULL				/* No toplevel GtkWidget */
		))
#endif
		{
			/* Got the user response and print values, now print
			 * the image
			 */
#ifdef HAVE_LIBENDEAVOUR2
			iv_print(
				img,
				pv,
				bg_color,		/* 4 bytes RGBA */
				NULL,		/* No toplevel widget */
				edv2_ctx
			);
#else
			iv_print(
				img,
				pv,
				bg_color,		/* 4 bytes RGBA */
				NULL		/* No toplevel widget */
			);
#endif
		}

		gtk_events_process();

		PrintValuesDelete(pv);

		ImgViewImageDelete(img);

		gtk_events_process();

#ifdef HAVE_LIBENDEAVOUR2
		edv_context_delete(edv2_ctx);
#endif
	}
	else
	{
		/* Failed to open the image */
		const gchar *error_msg = ImgOpenGetError();
		gchar *msg = g_strdup_printf(
"%s:\n\
\n\
    %s",
			error_msg,
			path
		);
		CDialogSetTransientFor(NULL);
		CDialogGetResponse(
			"Open Failed",
			msg,
"Please check if the specified file exists and that\n\
permissions allow the reading of that file.",
			CDIALOG_ICON_WARNING,
			CDIALOG_BTNFLAG_OK | CDIALOG_BTNFLAG_HELP,
			CDIALOG_BTNFLAG_OK
		);
		CDialogSetTransientFor(NULL);
		g_free(msg);
	}

 	for(i = 0; i < nframes; i++)
		g_free(rgba_list[i]);
 	g_free(rgba_list);
	g_free(delay_list);
	g_free(creator);
	g_free(title);
	g_free(author);
	g_free(comments);

	ProgressDialogShutdown();
	CDialogShutdown();

	return(status);
}


/*
 *	Opens the file as a binary or text file in a new Data Viewer.
 *
 *	This is often used as a fallback when iv_viewer_open fails.
 */
static IVDataViewer *iv_data_viewer_new_open_file(
	IVCore *core,
	const gchar *path,
	GtkWidget *toplevel
)
{
	FILE *fp;
	struct stat stat_buf;
	gboolean is_ascii = TRUE;
	const gchar *name;
	gchar *title;
	guint8		*data = NULL,
					*data_ascii_end;
	gulong data_len = 0l;
	IVDataViewer *fv;

	if((core == NULL) || STRISEMPTY(path))
		return(NULL);

	/* Open the file for reading */
	fp = fopen((const char *)path, "rb");
	if(fp == NULL)
	{
		const gint error_code = (gint)errno;
		gchar *msg = g_strdup_printf(
"%s:\n\
\n\
    %s",
			g_strerror(error_code),
			path
		);
		CDialogSetTransientFor(toplevel);
		CDialogGetResponse(
			"Open Failed",
			msg,
			NULL,
			CDIALOG_ICON_WARNING,
			CDIALOG_BTNFLAG_OK,
			CDIALOG_BTNFLAG_OK
		);
		g_free(msg);
		CDialogSetTransientFor(NULL);
		return(NULL);
	}

	/* Get the file's statistics */
	if(!fstat(fileno(fp), &stat_buf))
	{
		/* Allocate data buffer */
		data_len = (gulong)stat_buf.st_size;
		data = (data_len > 0l) ?
			(guint8 *)g_malloc(data_len * sizeof(guint8)) : NULL;
		if(data != NULL)
		{
#ifdef _WIN32
			const gulong blksize = 1024l;
#else
			const gulong blksize = MAX(stat_buf.st_blksize, 1);
#endif
			gint bytes_read;
			guint8	*ptr = data,
					*end = ptr + data_len;

			/* Begin loading data to the buffer */
			while(ptr < end)
			{
				bytes_read = fread(
					ptr, sizeof(guint8),
					MIN(blksize, (gulong)(end - ptr)), fp
				);
				if(bytes_read <= 0)
					break;

				ptr += bytes_read;
			}
		}
	}

	/* Close the file */
	(void)fclose(fp);

	/* Check file format to see if it is ASCII or binary */
	data_ascii_end = NULL;
	if(data_len > 0l)
	{
		guint8	*ptr = data,
					*end = ptr + data_len;
		while(ptr < end)
		{
			if(!isascii(*ptr))
			{
				is_ascii = FALSE;
			}
			if(*ptr == '\0')
			{
				data_ascii_end = ptr;
				is_ascii = FALSE;
				break;
			}
			ptr++;
		}
	}
	if(data_ascii_end == NULL)
		data_ascii_end = data + data_len;

	/* Get the name */
	name = g_basename(path);

	/* Format the title */
	title = g_strdup_printf(
		"%s: %s (%ld bytes)",
		(core->title != NULL) ?
			core->title : PROG_NAME_FULL,
		name,
		data_len
	);

	/* Create the Data Viewer */
	fv = iv_data_viewer_new(
		core,
		core->geometry_flags,
		core->geometry
	);
	if(fv != NULL)
	{
		GtkWidget *w = fv->toplevel;
		tview_struct *tv = fv->tv;
		hview_struct *hv = fv->hv;

		gtk_window_set_title(GTK_WINDOW(w), title);

		TViewSetBusy(tv, TRUE);
		HViewSetBusy(hv, TRUE);

		if(is_ascii)
		{
			gtk_toggle_button_set_active(
				GTK_TOGGLE_BUTTON(fv->text_tb), TRUE
			);
			TViewOpenData(tv, data, data_len);
			HViewOpenData(hv, data, data_len);
		}
		else
		{
			gtk_toggle_button_set_active(
				GTK_TOGGLE_BUTTON(fv->hex_tb), TRUE
			);
			TViewOpenData(tv, data, data_ascii_end - data);
			HViewOpenData(hv, data, data_len);
		}

		TViewSetBusy(tv, FALSE);
		HViewSetBusy(hv, FALSE);
	}

	g_free(data);
	g_free(title);

	return(fv);
}

/*
 *	Checks of there are changes on the Image Viewer and queries
 *	the user to save those changes.
 *
 *	This should be called whenever the Image Viewer is about to
 *	discard the image data.
 *
 *	Warning: If iv->processing is TRUE then This function will not
 *	be able to save.
 *
 *	A return value of TRUE means that the calling function can
 *	continue, otherwise FALSE indicates that the Image VIewer
 *	should not perform the operation.
 */
static gboolean iv_viewer_check_query_save_changes(IVImageViewer *v)
{
	gint response;
	GtkWidget *toplevel;
	imgview_struct *iv = (v != NULL) ? v->iv : NULL;
	if(iv == NULL)
		return(TRUE);

	if(CDialogIsQuery())
		return(FALSE);

	/* No changes that need to be saved? */
	if(!v->has_changes)
		return(TRUE);

	/* Get the ImgView's toplevel widget if it is a GtkWindow */
	if(ImgViewIsToplevelWindow(iv))
		toplevel = ImgViewGetToplevelWidget(iv);
	else
		toplevel = NULL;

	/* Query user to save changes before closing? */
	CDialogSetTransientFor(toplevel);
	response = CDialogGetResponse(
#if defined(PROG_LANGUAGE_SPANISH)
"?Salva Los Cambios?",
"?Los datos contienen los cambios que tienen no mas\n\
fue salvados, usted quiere salvar estos cambios?",
#elif defined(PROG_LANGUAGE_FRENCH)
"Epargne Des Changements?",
"Les donne'es contiennent des changements qui pas\n\
encore ont e'te' e'pargne's, vous voulez e'pargner\n\
ces changements?",
#elif defined(PROG_LANGUAGE_GERMAN)
"Auber Anderungen?",
"Enthalt die daten anderungen die nicht noch sind\n\
gespart worden, sie wollen sparen diese anderungen?",
#elif defined(PROG_LANGUAGE_ITALIAN)
"Risparmia I Cambiamenti?",
"I dati contiene dei cambiamenti che non sono stato\n\
ancora risparmiati, lei vuole risparmiare questi\n\
cambiamenti?",
#elif defined(PROG_LANGUAGE_DUTCH)
"Behalve Veranderingen?",
"Bevatten de gegevens veranderingen die niet nog zijn\n\
gered worden, u wil redden deze veranderingen?",
#elif defined(PROG_LANGUAGE_PORTUGUESE)
"Exceto Mudancas?",
"Os dados contem mudancas que nao mas foi poupado,\n\
quer poupar estas mudancas?",
#elif defined(PROG_LANGUAGE_NORWEGIAN)
"Untatt Forandringer?",
"Dataene inneholder forandringer som ikke enna\n\
spart, gjr De sparer disse forandringene?",
#else
"Save Changes?",
"The data contains changes that have not been\n\
saved, do you want to save these changes?",
#endif
		NULL,
		CDIALOG_ICON_WARNING,
		CDIALOG_BTNFLAG_YES | CDIALOG_BTNFLAG_NO | CDIALOG_BTNFLAG_CANCEL,
		CDIALOG_BTNFLAG_YES
	);
	CDialogSetTransientFor(NULL);
	switch(response)
	{
	  case CDIALOG_RESPONSE_CANCEL:
		return(FALSE);	/* Cancel, do not continue */
		break;

	  case CDIALOG_RESPONSE_NO:
		return(TRUE);	/* No, discard changes */
		break;

	  default:		/* Yes, save the image data */
		/* Call the save image callback to prompt the user to save
		 * the image data
		 */
		iv_viewer_save_cb(v->save_as_mi, v);

		/* Check if the save failed by checking if v->has_changes
		 * is still TRUE
		 */
		return(v->has_changes ? FALSE : TRUE);
		break;
	}
}


/*
 *	Image Viewer image changed callback.
 *
 *	This is called whenever the image viewer has changed its image
 *	data from its own side invoked by the user.
 */
static void iv_imgview_changed_cb(
	imgview_struct *iv,
	imgview_image_struct *image,
	gpointer data
)
{
	gboolean size_changed = FALSE;
	gint		width,
					height;
	IVOptionFlags options;
	IVInfo *info;
	IVCore *core;
	IVImageViewer *v = IV_IMAGE_VIEWER(data);
	if((iv == NULL) || (image == NULL) || (v == NULL))
		return;

	info = &v->info;
	core = v->core;
	options = core->options;

	/* Update image information on viewer structure with the 
	 * information from the given image
	 */
	width = image->width;
	height = image->height;
	if((info->width != width) || (info->height != height))
		size_changed = TRUE;
	v->has_changes = TRUE;
	info->width = width;
	info->height = height;

	/* Resize the ImgView's view widget as needed */
	if((options & IV_OPTION_RESIZE_ON_OPEN) && size_changed)
	{
		iv_viewer_resize(
			v,
			width, height,
			TRUE				/* Keep visible on desktop */
		);
		ImgViewZoomOneToOne(iv);
	}

	/* Update the ImgView's title */
	iv_viewer_update_title(v, NULL);

	/* Update Image Viewer's widgets */
	iv_viewer_update(v);
}

/*
 *	Set view background color callback.
 */
static void iv_viewer_set_bg_color_cb(GtkWidget *widget, gpointer data)
{
	gboolean response;
	GtkWidget *toplevel;
	csd_color_struct start_color, *color_rtn = NULL;
	imgview_image_struct *img;
	imgview_struct *iv;
	IVCore *core;
	IVImageViewer *v = IV_IMAGE_VIEWER(data);
	if((v == NULL) || CSDIsQuery())
		return;

	iv = v->iv;
	core = v->core;
	if(v->processing || (iv == NULL) || (core == NULL))
		return;

	/* Get the ImgView's toplevel widget if it is a GtkWindow */
	if(ImgViewIsToplevelWindow(iv))
		toplevel = ImgViewGetToplevelWidget(iv);
	else
		toplevel = NULL;

	/* Set up initial color */
	start_color.r = (gfloat)v->bg_color[0] / (gfloat)0xff;
	start_color.g = (gfloat)v->bg_color[1] / (gfloat)0xff;
	start_color.b = (gfloat)v->bg_color[2] / (gfloat)0xff;
	start_color.a = (gfloat)v->bg_color[3] / (gfloat)0xff;

	ImgViewSetBusy(iv, TRUE);
	v->processing = TRUE;
	v->stop_count = 0;

	/* Prompt user to select new color */
	CSDSetTransientFor(toplevel);
	response = CSDGetResponse(
#if defined(PROG_LANGUAGE_SPANISH)
		"Color Fijo De Fondo",
		"Conjunto", "Cancele",
#elif defined(PROG_LANGUAGE_FRENCH)
		"Regler La Couleur De Fond",
		"Serie", "Annuler",
#elif defined(PROG_LANGUAGE_GERMAN)
		"Feste Hintergrundfarbe",
		"Satz", "Heben",
#elif defined(PROG_LANGUAGE_ITALIAN)
		"Regolare Il Colore Di Fondo",
		"Serie", "Annullar",
#elif defined(PROG_LANGUAGE_DUTCH)
		"De Vaste Achtergrondkleur",
		"Het Stel", "Annuler",
#elif defined(PROG_LANGUAGE_PORTUGUESE)
		"Cor Fixa De Fundo",
		"Jogo", "Cancela",
#elif defined(PROG_LANGUAGE_NORWEGIAN)
		"Fast BakgrunnFarge",
		"Sett", "Kanseler",
#else
		"Set Background Color",
		"Set", "Cancel",
#endif
		&start_color, &color_rtn,
		NULL, NULL
	);
	CSDSetTransientFor(NULL);

	if(response)
	{
		if(color_rtn != NULL)
		{
			/* Got user response, set new background color */
			gint i;
			GdkColor c[5];

			for(i = 0; i < 5; i++)
			{
				GDK_COLOR_SET_COEFF(
					&(c[i]),
					color_rtn->r, color_rtn->g, color_rtn->b
				);
			}
			ImgViewSetViewBG(iv, c);

			/* Record new background color on view structure */
			v->bg_color[0] = (guint8)(color_rtn->r * 0xff);
			v->bg_color[1] = (guint8)(color_rtn->g * 0xff);
			v->bg_color[2] = (guint8)(color_rtn->b * 0xff);
			v->bg_color[3] = (guint8)(color_rtn->a * 0xff);

			/* Notify change if an image was loaded */
			img = ImgViewGetImage(iv);
			if(img != NULL)
				iv_imgview_changed_cb(
					iv,			/* ImgView */
					img,			/* Image */
					v			/* Data */
				);
		}
	}

	v->processing = FALSE;
	ImgViewSetBusy(iv, FALSE);
}

/*
 *	Resize callback.
 */
static void iv_viewer_resize_cb(GtkWidget *widget, gpointer data)
{
	gboolean was_playing;
	gint src_width, src_height, src_bpl, src_bpp;
	const gchar *path, *name;
	imgview_format format;
	const guint8 *src_data;
	gint tar_width, tar_height;
	GtkWidget *toplevel;
	IVInfo *info;
	imgview_struct *iv;
	IVCore *core;
	IVImageViewer *v = IV_IMAGE_VIEWER(data);
	if(v == NULL)
		return;

	iv = v->iv;
	core = v->core;
	if(v->processing || (iv == NULL) || (core == NULL))
		return;

	was_playing = ImgViewIsPlaying(iv);

	/* Get the ImgView's toplevel widget if it is a GtkWindow */
	if(ImgViewIsToplevelWindow(iv))
		toplevel = ImgViewGetToplevelWidget(iv);
	else
		toplevel = NULL;

	/* Get the loaded image's data */
	src_data = ImgViewGetImageData(
		iv, ImgViewGetCurrentFrame(iv),
		&src_width, &src_height, &src_bpp, &src_bpl, &format
	);
	if(src_data == NULL)
		return;

	ImgViewSetBusy(iv, TRUE);
	v->processing = TRUE;
	v->stop_count = 0;

	info = &v->info;
	path = info->path;
	name = (path != NULL) ? g_basename(path) : NULL;

	/* Set initial size values */
	tar_width = src_width;
	tar_height = src_height;

	/* Map the Resize Dialog to query the user to resize */
	if(iv_resize(
		toplevel,
		iv,
		name,
		&tar_width, &tar_height,
		&core->resize_maintain_aspect
	))
	{
		imgview_image_struct *tar_img = ImgViewGetImage(iv);

		if(was_playing)
			ImgViewPlay(iv);

		/* Notify change */
		iv_imgview_changed_cb(
			iv,			/* ImgView */
			tar_img,		/* Image */
			v			/* Data */
		);
	}

	v->processing = FALSE;
	ImgViewSetBusy(iv, FALSE);
}

/*
 *	Color Levels callback.
 */
static void iv_viewer_color_levels_cb(GtkWidget *widget, gpointer data)
{
	gboolean was_playing;
	const gchar *path, *name;
	GtkWidget *toplevel;
	imgview_struct *iv;
	IVInfo *info;
	IVCore *core;
	IVImageViewer *v = IV_IMAGE_VIEWER(data);
	if(v == NULL)
		return;

	iv = v->iv;
	core = v->core;
	if(v->processing || (iv == NULL) || (core == NULL))
		return;

	was_playing = ImgViewIsPlaying(iv);
	if(was_playing)
		ImgViewPause(iv);

	/* Get the ImgView's toplevel widget if it is a GtkWindow */
	if(ImgViewIsToplevelWindow(iv))
		toplevel = ImgViewGetToplevelWidget(iv);
	else
		toplevel = NULL;


	ImgViewSetBusy(iv, TRUE);
	v->processing = TRUE;
	v->stop_count = 0;

	info = &v->info;
	path = info->path;
	name = (path != NULL) ? g_basename(path) : NULL;

	/* Map the Color Levels Dialog and query the user to modify
	 * color levels
	 */
	if(iv_color_levels_query(
		toplevel,
		iv,
		name,
		&core->color_levels_all_frames
	))
	{
		imgview_image_struct *img = ImgViewGetImage(iv);
		if(img != NULL)
		{
			/* Notify change */
			iv_imgview_changed_cb(
				iv,			/* ImgView */
				img,		/* Image */
				v			/* Data */
			);
		}
	}

	v->processing = FALSE;
	ImgViewSetBusy(iv, FALSE);

	if(was_playing)
		ImgViewPlay(iv);
}

/*
 *	Add Text callback.
 */
static void iv_viewer_add_text_cb(GtkWidget *widget, gpointer data)
{
	gboolean was_playing;
	const gchar *path, *name;
	GtkWidget *toplevel;
	imgview_struct *iv;
	IVInfo *info;
	IVCore *core;
	IVImageViewer *v = IV_IMAGE_VIEWER(data);
	if(v == NULL)
		return;

	iv = v->iv;
	core = v->core;
	if(v->processing || (iv == NULL) || (core == NULL))
		return;

	/* If the image is playing then it needs to be stopped */
	was_playing = ImgViewIsPlaying(iv);
	if(was_playing)
		ImgViewPause(iv);

	/* Get the ImgView's toplevel widget if it is a GtkWindow */
	if(ImgViewIsToplevelWindow(iv))
		toplevel = ImgViewGetToplevelWidget(iv);
	else
		toplevel = NULL;

/*	ImgViewSetBusy(iv, TRUE); */
	v->processing = TRUE;
	v->stop_count = 0;

	info = &v->info;
	path = info->path;
	name = (path != NULL) ? g_basename(path) : NULL;

	/* Map the Text Dialog and query user to add text */
	if(iv_add_text(
		toplevel,
		iv,
		name,
		&core->add_text_font_name,
		&core->add_text_color,
		&core->add_text_outline,
		&core->add_text_outline_color,
		&core->add_text_opacity,
		&core->add_text_all_frames
	))
	{
		imgview_image_struct *img = ImgViewGetImage(iv);

		/* Notify change */
		iv_imgview_changed_cb(
			iv,			/* ImgView */
			img,			/* Image */
			v			/* Data */
		);
	}

	v->processing = FALSE;
/*	ImgViewSetBusy(iv, FALSE); */

	if(was_playing) 
		ImgViewPlay(iv); 
}

/*
 *	Edit Header callback.
 */
static void iv_viewer_header_cb(GtkWidget *widget, gpointer data)
{
	const gchar *path, *name;
	GtkWidget *toplevel;
	IVInfo *info;
	imgview_frame_struct *frame;
	imgview_image_struct *img;
	imgview_struct *iv;
	IVImageViewer *v = IV_IMAGE_VIEWER(data);
	if(v == NULL)
		return;

	info = &v->info;
	iv = v->iv;
	if(v->processing || (iv == NULL))
		return;

	/* Get the current image and the current frame */
	img = ImgViewGetImage(iv);
	frame = ImgViewImageGetFrame(img, ImgViewGetCurrentFrame(iv));
	if(frame == NULL)
		frame = ImgViewImageGetFrame(img, 0);
	if(frame == NULL)
		return;

	ImgViewSetBusy(iv, TRUE);
	v->processing = TRUE;
	v->stop_count = 0;

	path = info->path;
	name = (path != NULL) ? g_basename(path) : NULL;

	/* Get the ImgView's toplevel widget if it is a GtkWindow */
	if(ImgViewIsToplevelWindow(iv))
		toplevel = ImgViewGetToplevelWidget(iv);
	else
		toplevel = NULL;

	/* Query user to edit header */
	if(iv_edit_header(
		toplevel,
		name,
		frame->data,
		img->width, img->height, img->bpl,
		img->nframes,
		v->bg_color,
		&info->title,
		&info->author,
		&info->comments,
		&info->x, &info->y,
		&info->base_width, &info->base_height
	))
	{
		/* Mark that we have changes */
		v->has_changes = TRUE;

		/* Update the ImgView's title */
		iv_viewer_update_title(v, NULL);

		/* Update Image Viewer's widgets */
		iv_viewer_update(v);
	}

	v->processing = FALSE;
	ImgViewSetBusy(iv, FALSE);
}

/*
 *	Edit Frames callback.
 */
static void iv_viewer_frames_cb(GtkWidget *widget, gpointer data)
{
	const gchar *path, *name;
	GtkWidget *toplevel;
	IVInfo *info;
	imgview_struct *iv;
	IVImageViewer *v = IV_IMAGE_VIEWER(data);
	if(v == NULL)
		return;

	iv = v->iv;
	if(v->processing || (iv == NULL))
		return;

	ImgViewSetBusy(iv, TRUE);
	v->processing = TRUE;
	v->stop_count = 0;

	info = &v->info;
	path = info->path;
	name = (path != NULL) ? g_basename(path) : NULL;

	/* Get the ImgView's toplevel widget if it is a GtkWindow */
	if(ImgViewIsToplevelWindow(iv))
		toplevel = ImgViewGetToplevelWidget(iv);
	else
		toplevel = NULL;

	/* Query user to edit frames */
	if(iv_edit_frames(
		toplevel,
		iv,
		name
	))
	{
		const gint nframes = ImgViewGetTotalFrames(iv);
		imgview_image_struct *img = ImgViewGetImage(iv);

		/* Mark that we have changes */
		v->has_changes = TRUE;

		/* Update the number of frames */
		info->nframes = nframes;

		/* Update the number of loops */
		info->nloops = (nframes > 1) ? -1 : 0;

		/* Update the play time */
		info->play_time_ms = 0l;
		if(img != NULL)
		{
			GList *glist;
			imgview_frame_struct *frame;
			for(glist = img->frames_list;
				glist != NULL;
				glist = g_list_next(glist)
			)
			{
				frame = IMGVIEW_FRAME(glist->data);
				if(frame == NULL)
					continue;
				info->play_time_ms += frame->delay;
			}
		}

		/* Update the total play time */
		if(info->play_time_ms > 0l)
		{
			/* Infinite? */
			if(info->nloops < 0)
				info->total_play_time_ms = (gulong)-1;
			else
				info->total_play_time_ms = info->nloops * info->play_time_ms;
		}
		else
			info->total_play_time_ms = 0l;

		/* Update the ImgView's title */
		iv_viewer_update_title(v, NULL);

		/* Update Image Viewer's widgets */
		iv_viewer_update(v);
	}

	v->processing = FALSE;
	ImgViewSetBusy(iv, FALSE);
}

#if !defined(_WIN32)
/*
 *	Screenshot callback.
 */
static void iv_viewer_screenshot_cb(GtkWidget *widget, gpointer data)
{
	imgview_struct *iv;
	IVImageViewer *v = IV_IMAGE_VIEWER(data);
	if(v == NULL)
		return;

	if(v->processing)
		return;

	iv = v->iv;
	if(iv == NULL)
		return;

	ImgViewSetBusy(iv, TRUE);
	v->processing = TRUE;
	v->stop_count = 0;

	iv_viewer_screenshot(v);

	v->processing = FALSE;
	ImgViewSetBusy(iv, FALSE);
}
#endif


/*
 *	ImgIO open image progress callback.
 */
static gint iv_open_progress_cb(
	gpointer progress_data,
	const gint i, const gint n,
	const gint width, const gint height,
	const gint bpl, const gint bpp,
	const guint8 *image_data
)
{
	gboolean image_changed;
	gfloat          progress_coeff,
					progress_coeff_change;
	const guint8 *rgba;
	imgview_struct *iv;
	imgview_image_struct *img;
	IVOptionFlags options;
	IVImageViewer *v;
	IVImgIOCBData *d = IV_IMGIO_CB(progress_data);
	if(d == NULL)
		return(FALSE);

	if(bpp == 4)
		rgba = image_data;
	else
		rgba = NULL;

	options = d->options;

	/* Get the Image Viewer */
	v = d->viewer;
	if(v == NULL)
		return(TRUE);

	if(v->stop_count > 0)
		return(FALSE);

	iv = v->iv;

	/* Calculate the progress coefficient, use -1.0 if it is
	 * not known
	 */
	if(n > 0l)
		progress_coeff = (gfloat)i / (gfloat)n;
	else
		progress_coeff = -1.0f;

	if((d->last_progress_coeff >= 0.0f) && (progress_coeff >= 0.0f))
		progress_coeff_change = progress_coeff - d->last_progress_coeff;
	else
		progress_coeff_change = 0.0f;

	img = ImgViewGetImage(iv);


/* Update the ImgView's background color if it has changed */
#define UPDATE_BG_COLOR	{		\
 if(memcmp(d->last_bg_color, v->bg_color, sizeof(d->last_bg_color))) { \
  gint i;				\
  GdkColor c[5];			\
  for(i = 0; i < 5; i++) {		\
   GDK_COLOR_SET_BYTE(			\
    &(c[i]),				\
    v->bg_color[0],			\
    v->bg_color[1],			\
    v->bg_color[2]			\
   );					\
  }					\
  ImgViewSetViewBG(iv, c);		\
									\
  /* Record the new background color */	\
  (void)memcpy(				\
   d->last_bg_color,			\
   v->bg_color,				\
   sizeof(d->last_bg_color)		\
  );					\
 }					\
}


	/* If the ImgView image has not yet been set for this progress
	 * or the RGBA image data geometry has changed then set
	 * image_changed to TRUE
	 *
	 * ImgView image set for this progress?
	 */
	if(d->img != NULL)
	{
		if((img != NULL) && (rgba != NULL))
		{
			if((img->width != width) || (img->height != height))
				image_changed = TRUE;
			else
				image_changed = FALSE;
		}
		else
		{
			image_changed = FALSE;
		}
	}
	else
	{
		if(rgba != NULL)
			image_changed = TRUE;
		else
			image_changed = FALSE;
	}
	if(image_changed)
	{
		/* The RGBA image has specified for the first time or its
		 * geometry has changed, recreate the ImgView image and
		 * copy the specified RGBA image to it
		 */

		/* Update the ImgView's title */
		iv_viewer_update_title(v, d->path);

		/* Automatically resize and reposition the Image Viewer when
		 * opening a new image?
		 */
		if(options & IV_OPTION_RESIZE_ON_OPEN)
		{
			iv_viewer_resize(
				v,
				width, height,
				TRUE			/* Keep visible on desktop */
			);
			/* Mark that the Image Viewer has already been resized
			 * so that it will not get automatically resized again
			 * just after opening
			 */
			options &= ~IV_OPTION_RESIZE_ON_OPEN;
			d->options = options;
		}

		/* Create a new ImgView image */
		d->img = img = ImgViewImageNew(
			width, height,
			bpp, bpl
		);
		if(img != NULL)
		{
			/* Copy the specified RGBA image data to the new
			 * ImgView image
			 */
			(void)ImgViewImageAppendFrame(
				img,
				g_memdup(
					rgba,
					bpl * height * sizeof(guint8)
				),
				0l                          /* No delay */
			);

			/* Set the new image to the ImgView */
			if(options & IV_OPTION_ZOOM_TO_FIT_ON_OPEN)
				ImgViewSetImageToFit(
					iv,
					img			/* Transfered */
				);
			else
				ImgViewSetImage(
					iv,
					img			/* Transfered */
				);
/*		img = NULL; */

			/* Update the background color if it has changed */
			UPDATE_BG_COLOR

			/* Consider recreating the ImgView image as an update */
/*		d->last_update_ms = edv_time_ms(); */
		}
	}
	else if(img != NULL)
	{
		const gfloat must_redraw_every_coeff_change = 0.2f;
/*	    const gulong        must_redraw_delay_ms = 1000l;
							cur_time_ms = edv_time_ms();
 */

		/* If the specified image geometry matches the current
		 * ImgView image's geometry then update/copy the
		 * specified image data to the ImgView's image
		 */
		if(( /* ((cur_time_ms - d->last_update_ms) >= must_redraw_delay_ms) || */
			(progress_coeff_change >= must_redraw_every_coeff_change)) &&
		   (rgba != NULL) && (img->width == width) &&
		   (img->height == height) && (img->bpp == bpp) &&
		   (img->bpl == bpl)
		)
		{
			/* Get the last frame of the ImgView's image */
			imgview_frame_struct *frame = ImgViewImageGetFrame(
				img,
				img->nframes - 1
			);
			if(frame != NULL)
			{
				/* Copy the specified RGBA image data to this frame
				 * and tell the ImgView to redraw
				 */
				(void)memcpy(
					frame->data,
					rgba,
					img->bpl * img->height * sizeof(guint8)
				);

				/* Update the background color if it has changed */
				UPDATE_BG_COLOR

				/* Queue the ImgView to redraw, which may not
				 * occure immediately if the CPU usage is too
				 * intense
				 */
				ImgViewQueueDrawView(iv);
			}
/*		d->last_update_ms = cur_time_ms; */
			d->last_progress_coeff = progress_coeff;
		}
	}

#undef UPDATE_BG_COLOR

	/* Update the progress bar */
	if(progress_coeff > 0.0f)
		ImgViewProgressUpdate(iv, progress_coeff, TRUE);

	return(TRUE);
}

/*
 *	ImgIO save image progress callback.
 */
static gint iv_save_progress_cb(
	gpointer progress_data,
	const gint i, const gint n,
	const gint width, const gint height,
	const gint bpl, const gint bpp,
	const guint8 *image_data
)
{
	IVImageViewer *v;
	IVImgIOCBData *cb_data = IV_IMGIO_CB(progress_data);
	if(cb_data == NULL)
		return(FALSE);

	/* Get the Viewer */
	v = cb_data->viewer;
	if(v != NULL)
	{
		if(v->stop_count > 0)
			return(FALSE);

		/* Update the progress bar */
		if(n > 0)
			ImgViewProgressUpdate(
				v->iv,
				(gfloat)i / (gfloat)n,
				TRUE
			);
	}

	return(TRUE);
}


/*
 *	Open Image callback.
 */
static void iv_viewer_open_image_cb(GtkWidget *widget, gpointer data)
{
	const gchar *imgio_file_types_list[] = IMAGE_FORMAT_FILE_TYPES_LIST;
	gboolean response;
	gint		i,
					npaths = 0,
					nftypes = 0;
	gchar **paths_list = NULL;
	fb_type_struct	*ftype_rtn = NULL,
					**ftypes_list = NULL;
	GtkWidget *toplevel;
	imgview_struct *iv;
	IVCore *core;
	IVImageViewer *v = IV_IMAGE_VIEWER(data);
	if((v == NULL) || FileBrowserIsQuery())
		return;

	if(v->processing)
		return;

	iv = v->iv;
	core = v->core;

	ImgViewSetBusy(iv, TRUE);

	/* Get the ImgView's toplevel widget if it is a GtkWindow */
	if(ImgViewIsToplevelWindow(iv))
		toplevel = ImgViewGetToplevelWidget(iv);
	else
		toplevel = NULL;

	/* Check if there are any unsaved changes and if so then
	 * query the user to save those changes
	 */
	if(!iv_viewer_check_query_save_changes(v))
	{
		ImgViewSetBusy(iv, FALSE);
		return;
	}

	/* Get the current directory */
	g_free(core->current_directory_path);
	core->current_directory_path = STRDUP(g_get_current_dir());

	/* Create the file types list */
	FileBrowserTypeListNew(
		&ftypes_list, &nftypes,
		"*.*",
		"All Images"
	);
	for(i = 0; imgio_file_types_list[i] != NULL; i += 2)
		FileBrowserTypeListNew(
			&ftypes_list, &nftypes,
			imgio_file_types_list[i],	/* Extensions */
			imgio_file_types_list[i + 1]	/* Name/description */
		);

	/* Query user to open an image file */
	FileBrowserSetTransientFor(toplevel);
	response = FileBrowserGetResponse(
#if defined(PROG_LANGUAGE_SPANISH)
		"Abra La Imagen",
		"Abra", "Cancele",
#elif defined(PROG_LANGUAGE_FRENCH)
		"Image Ouverte",
		"Ouvrir", "Annuler",
#elif defined(PROG_LANGUAGE_GERMAN)
		"Offenes Bildnis",
		"Offen", "Heben",
#elif defined(PROG_LANGUAGE_ITALIAN)
		"Aprire L'Immagine",
		"Aprire", "Annullar",
#elif defined(PROG_LANGUAGE_DUTCH)
		"Open Beeld",
		"Open", "Annuler",
#elif defined(PROG_LANGUAGE_PORTUGUESE)
		"Abra Imagem",
		"Abra", "Cancela",
#elif defined(PROG_LANGUAGE_NORWEGIAN)
		"Apn Avbilde",
		"Apn", "Kanseler",
#else
		"Open Image",
		"Open", "Cancel",
#endif
		core->current_directory_path,
		ftypes_list, nftypes,
		&paths_list, &npaths,
		&ftype_rtn
	);
	FileBrowserSetTransientFor(NULL);

	ImgViewSetBusy(iv, FALSE);

	/* Got image to open? */
	if(response)
	{
		gint i;
		gchar	*path,
					*parent;

		for(i = 0; i < npaths; i++)
		{
			path = STRDUP(paths_list[i]);
			if(path == NULL)
				continue;

			/* Set the current directory */
			parent = g_dirname(path);
			if(parent != NULL)
			{
				g_free(core->current_directory_path);
				core->current_directory_path = g_strdup(parent);
				(void)chdir((const char *)parent);
				g_free(parent);
			}

			/* First image? */
			if(i == 0)
			{
				/* Use the specified Image Viewer to open the
				 * first image
				 */

				ImgViewSetBusy(iv, TRUE);
				v->processing = TRUE;
				v->stop_count = 0;

				iv_viewer_open(
					v,
					path,
					-1
				);

				v->processing = FALSE;
				ImgViewSetBusy(iv, FALSE);
			}
			else
			{
				/* Create a new Image Viewer and open subsequent
				 * images
				 */
				IVImageViewer *v2 = iv_viewer_new(
					core,
					core->options,
					iv->quality,
					core->bg_color_set ? core->bg_color : NULL,
					core->geometry_flags,
					core->geometry,
					FALSE			/* Do not load title */
				);
				if(v2 == NULL)
				{
					g_free(path);
					break;
				}

				ImgViewSetBusy(v2->iv, TRUE);
				v2->processing = TRUE;
				v2->stop_count = 0;

				/* Open the image to the new Image Viewer */
				iv_viewer_open(
					v2,
					path,
					-1
				);

				v2->processing = FALSE;
				ImgViewSetBusy(v2->iv, FALSE);
			}

			g_free(path);
		}
	}

	/* Delete the file types list */
	FileBrowserDeleteTypeList(ftypes_list, nftypes);
}

/*
 *	Open Other callback.
 *
 *	Opens a file and displays it on the Data Viewer.
 */
static void iv_viewer_open_other_cb(GtkWidget *widget, gpointer data)
{
	gboolean response;
	gint		npaths = 0,
					nftypes = 0;
	gchar **paths_list = NULL;
	fb_type_struct	*ftype_rtn = NULL,
					**ftypes_list = NULL;
	GtkWidget *toplevel;
	imgview_struct *iv;
	IVCore *core;
	IVImageViewer *v = IV_IMAGE_VIEWER(data);
	if((v == NULL) || FileBrowserIsQuery())
		return;

	if(v->processing)
		return;

	iv = v->iv;
	core = v->core;
	if((iv == NULL) || (core == NULL))
		return;

	ImgViewSetBusy(iv, TRUE);

	/* Get the ImgView's toplevel widget if it is a GtkWindow */
	if(ImgViewIsToplevelWindow(iv))
		toplevel = ImgViewGetToplevelWidget(iv);
	else
		toplevel = NULL;

	/* Get the current directory */
	g_free(core->current_directory_path);
	core->current_directory_path = STRDUP(g_get_current_dir());

	/* Create the file types list */
	FileBrowserTypeListNew(
		&ftypes_list, &nftypes,
		"*.*",
		"All Files"
	);

	/* Query user to open a file */
	FileBrowserSetTransientFor(toplevel);
	response = FileBrowserGetResponse(
#if defined(PROG_LANGUAGE_SPANISH)
		"Abra El Archivo",
		"Abra", "Cancele",
#elif defined(PROG_LANGUAGE_FRENCH)
		"Dossier Ouvert",
		"Ouvrir", "Annuler",
#elif defined(PROG_LANGUAGE_GERMAN)
		"Offene Akte",
		"Offen", "Heben",
#elif defined(PROG_LANGUAGE_ITALIAN) 
		"Aprire Il File",
		"Aprire", "Annullar",
#elif defined(PROG_LANGUAGE_DUTCH)
		"Het Open Bestand",
		"Open", "Annuler",
#elif defined(PROG_LANGUAGE_PORTUGUESE)
		"Abra Arquivo",
		"Abra", "Cancela",
#elif defined(PROG_LANGUAGE_NORWEGIAN)
		"Apn Arkiv",
		"Apn", "Kanseler",
#else
		"Open File",
		"Open", "Cancel",
#endif
		core->current_directory_path,
		ftypes_list, nftypes,
		&paths_list, &npaths,
		&ftype_rtn
	);
	FileBrowserSetTransientFor(NULL);

	ImgViewSetBusy(iv, FALSE);

	if(response)
	{
		gint i;
		gchar	*path,
					*parent;

		for(i = 0; i < npaths; i++)
		{
			path = STRDUP(paths_list[i]);
			if(path == NULL)
				continue;

			/* Set the current directory */
			parent = g_dirname(path);
			if(parent != NULL)
			{
				g_free(core->current_directory_path);
				core->current_directory_path = g_strdup(parent);
				(void)chdir(parent);
				g_free(parent);
			}

			/* Open file in new Data Viewer */
			iv_data_viewer_new_open_file(
				core,
				path,
				toplevel
			);

			g_free(path);
		}
	}

	/* Delete the file types list */
	FileBrowserDeleteTypeList(ftypes_list, nftypes);
}

/*
 *	Save Image callback.
 */
static void iv_viewer_save_cb(GtkWidget *widget, gpointer data)
{
	const gchar *imgio_file_types_list[] = IMAGE_FORMAT_FILE_TYPES_LIST;
	gboolean response;
	gint		i,
					npaths = 0,
					nftypes = 0;
	gchar **paths_list = NULL;
	fb_type_struct	*ftype_rtn = NULL,
					**ftypes_list = NULL;
	GtkWidget *toplevel;
	imgview_struct *iv;
	IVCore *core;
	IVImageViewer *v = IV_IMAGE_VIEWER(data);
	if((v == NULL) || FileBrowserIsQuery())
		return;

	iv = v->iv;
	core = v->core;
	if(v->processing || (iv == NULL) || (core == NULL))
		return;

	if(!ImgViewIsLoaded(iv))
		return;

	ImgViewSetBusy(iv, TRUE);

	/* Get the current directory */
	g_free(core->current_directory_path);
	core->current_directory_path = STRDUP(g_get_current_dir());

	/* Create the file types list */
	FileBrowserTypeListNew(
		&ftypes_list, &nftypes,
		"*.*",
		"Automatically detect format"
	);
	for(i = 0; imgio_file_types_list[i] != NULL; i += 2)
		FileBrowserTypeListNew(
			&ftypes_list, &nftypes,
			imgio_file_types_list[i],	/* Extensions */
			imgio_file_types_list[i + 1]	/* Name/description */
		);

	/* Get the ImgView's toplevel widget if it is a GtkWindow */
	if(ImgViewIsToplevelWindow(iv))
		toplevel = ImgViewGetToplevelWidget(iv);
	else
		toplevel = NULL;

	/* Query the user to save the image as */
	FileBrowserSetTransientFor(toplevel);
	response = FileBrowserGetResponse(
#if defined(PROG_LANGUAGE_SPANISH)
		"Salve La Imagen Como",
		"Salve", "Cancele",
#elif defined(PROG_LANGUAGE_FRENCH)
		"Epargner L'Image Comme",
		"Epargner", "Annuler",
#elif defined(PROG_LANGUAGE_GERMAN)
		"AuBer Blidnis Als",
		"Sparen", "Heben",
#elif defined(PROG_LANGUAGE_ITALIAN)
		"Risparmiare L'Immagine Come",
		"Risparm", "Annullar",
#elif defined(PROG_LANGUAGE_DUTCH)
		"Behalve Beeld Als",
		"Red", "Annuler",
#elif defined(PROG_LANGUAGE_PORTUGUESE)
		"Exceto Imagem Como",
		"Poupe", "Cancela",
#elif defined(PROG_LANGUAGE_NORWEGIAN)
		"Untatt Avbilde Som",
		"Untatt", "Kanseler",
#else
		"Save Image As",
		"Save", "Cancel",
#endif

		core->current_directory_path,
		ftypes_list, nftypes,
		&paths_list, &npaths,
		&ftype_rtn
	);
	FileBrowserSetTransientFor(NULL);

	ImgViewSetBusy(iv, FALSE);

	if(response && (npaths > 0))
	{
		/* Copy the list of paths so if any changes to the
		 * response paths during the saves will not cause
		 * problems
		 */
		gchar	*path,
					*extensions;
		GList	*glist,
					*paths_list2 = NULL;
		for(i = 0; i < npaths; i++)
			paths_list2 = g_list_append(
				paths_list2,
				STRDUP(paths_list[i])
			);

		/* Copy the extensions */
		extensions = (ftype_rtn != NULL) ?
			STRDUP(ftype_rtn->extensions) : g_strdup("*.*");

		/* Save each image */
		for(glist = paths_list2;
		    glist != NULL;
		    glist = g_list_next(glist)
		)
		{
			path = (gchar *)glist->data;
			if(!STRISEMPTY(path))
		    {
				/* Set the current directory */
				gchar *parent = g_dirname(path);
				if(parent != NULL)
				{
					g_free(core->current_directory_path);
					core->current_directory_path = g_strdup(parent);
				    (void)chdir(parent);
				    g_free(parent);
				}

				ImgViewSetBusy(iv, TRUE);
				v->processing = TRUE;
				v->stop_count = 0;

				/* Save this image */
				iv_viewer_save(
					v,
					path,
					extensions
				);

				v->processing = FALSE;
				ImgViewSetBusy(iv, FALSE);
			}
		}

		g_free(extensions);

		g_list_foreach(paths_list2, (GFunc)g_free, NULL);
		g_list_free(paths_list2);
	}

	/* Delete the file types list */
	FileBrowserDeleteTypeList(ftypes_list, nftypes);

	/* Reset the file browser listing due to change in objects */
	FileBrowserReset();
}

/*
 *	Display On Root Centered callbacks.
 */
static void iv_viewer_display_on_root_centered_cb(
	GtkWidget *widget, gpointer data
)
{
	const guint8 *data_rgba;
	gint		width, height,
					bpp, bpl;
	imgview_format format;
	GdkWindow *root_win;
	IVImageViewer *v = IV_IMAGE_VIEWER(data);
	imgview_struct *iv = (v != NULL) ? v->iv : NULL;
	if(iv == NULL)
		return;

	iv = v->iv;
	root_win = gdk_window_get_root();
	if(v->processing || (iv == NULL) || (root_win == NULL))
		return;

	ImgViewSetBusy(iv, TRUE);
	v->processing = TRUE;
	v->stop_count = 0;

	/* Get the image data of the image's current frame */
	data_rgba = ImgViewGetImageData(
		iv, ImgViewGetCurrentFrame(iv),
		&width, &height, &bpp, &bpl, &format
	);

	/* Display the image data to the desktop */
	iv_desktop_put_image_centered(
		root_win,
		iv->quality,
		data_rgba,
		width, height, bpp, bpl,
		v->bg_color
	);

	v->processing = FALSE;
	ImgViewSetBusy(iv, FALSE);
}

/*
 *	Display On Root To Fit callback.
 */
static void iv_viewer_display_on_root_tofit_cb(
	GtkWidget *widget, gpointer data
)
{
	const guint8 *data_rgba;
	gint width, height, bpp, bpl;
	imgview_format format;
	GdkWindow *root_win;
	IVImageViewer *v = IV_IMAGE_VIEWER(data);
	imgview_struct *iv = (v != NULL) ? v->iv : NULL;
	if(iv == NULL)
		return;

	iv = v->iv;
	root_win = gdk_window_get_root();
	if(v->processing || (iv == NULL) || (root_win == NULL))
		return;

	ImgViewSetBusy(iv, TRUE);
	v->processing = TRUE;
	v->stop_count = 0;

	/* Get the image data of the image's current frame */
	data_rgba = ImgViewGetImageData(
		iv, ImgViewGetCurrentFrame(iv),
		&width, &height, &bpp, &bpl, &format
	);

	/* Display the image data to the desktop */
	iv_desktop_put_image_tofit(
		root_win,
		iv->quality,
		data_rgba,
		width, height, bpp, bpl,
		v->bg_color
	);

	v->processing = FALSE;
	ImgViewSetBusy(iv, FALSE);
}

/*
 *	Display On Root To Cover callback.
 */
static void iv_viewer_display_on_root_tocover_cb(
	GtkWidget *widget, gpointer data
)
{
	const guint8 *data_rgba;
	gint width, height, bpp, bpl;
	imgview_format format;
	GdkWindow *root_win;
	IVImageViewer *v = IV_IMAGE_VIEWER(data);
	imgview_struct *iv = (v != NULL) ? v->iv : NULL;
	if(iv == NULL)
		return;

	iv = v->iv;
	root_win = gdk_window_get_root();
	if(v->processing || (iv == NULL) || (root_win == NULL))
		return;

	ImgViewSetBusy(iv, TRUE);
	v->processing = TRUE;
	v->stop_count = 0;

	/* Get the image data of the image's current frame */
	data_rgba = ImgViewGetImageData(
		iv, ImgViewGetCurrentFrame(iv),
		&width, &height, &bpp, &bpl, &format
	);

	/* Display the image data to the desktop */
	iv_desktop_put_image_tocover(
		root_win,
		iv->quality,
		data_rgba,
		width, height, bpp, bpl,
		v->bg_color
	);

	v->processing = FALSE;
	ImgViewSetBusy(iv, FALSE);
}

/*
 *	Display On Root Tiled callback.
 */
static void iv_viewer_display_on_root_tiled_cb(
	GtkWidget *widget, gpointer data
)
{
	const guint8 *data_rgba;
	gint width, height, bpp, bpl;
	imgview_format format;
	GdkWindow *root_win;
	IVImageViewer *v = IV_IMAGE_VIEWER(data);
	imgview_struct *iv = (v != NULL) ? v->iv : NULL;
	if(iv == NULL)
		return;

	iv = v->iv;
	root_win = gdk_window_get_root();
	if(v->processing || (iv == NULL) || (root_win == NULL))
		return;

	ImgViewSetBusy(iv, TRUE);
	v->processing = TRUE;
	v->stop_count = 0;

	/* Get the image data of the image's current frame */
	data_rgba = ImgViewGetImageData(
		iv, ImgViewGetCurrentFrame(iv),
		&width, &height, &bpp, &bpl, &format
	);

	/* Display the image data to the desktop */
	iv_desktop_put_image_tiled(
		root_win,
		iv->quality,
		data_rgba,
		width, height, bpp, bpl,
		v->bg_color
	);

	v->processing = FALSE;
	ImgViewSetBusy(iv, FALSE);
}

/*
 *	Clear image callback.
 */
static void iv_viewer_clear_cb(GtkWidget *widget, gpointer data)
{
	imgview_struct *iv;
	IVImageViewer *v = IV_IMAGE_VIEWER(data);
	if(v == NULL)
		return;

	iv = v->iv;
	if(v->processing || (iv == NULL))
		return;

	ImgViewSetBusy(iv, TRUE);

	/* Check if there are any unsaved changes and if so then
	 * query the user to save those changes
	 */
	if(!iv_viewer_check_query_save_changes(v))
	{
		ImgViewSetBusy(iv, FALSE);
		return;
	}

	/* Begin clearing image */

	/* Unload image on image viewer */
	ImgViewClear(iv);

	/* Begin resetting information values on viewer structure */
	v->has_changes = FALSE;

	/* Delete image info values */
	iv_clear_info(&v->info);

	/* Update the ImgView's title */
	iv_viewer_update_title(v, NULL);

	/* Update Image Viewer's widgets */
	iv_viewer_update(v);

	ImgViewSetBusy(iv, FALSE);
}

/*
 *	Image information callback.
 */
static void iv_viewer_information_cb(GtkWidget *widget, gpointer data)
{
	gint		width, height,
					bpp, bpl,
					nframes,
					alpha_channel,
					depth = 0;
	guint vis_id = 0x00000000;
	const gchar	*path,
					*ext_ptr,
					*name_ptr,
					*dpy_name,
					*vis_name = NULL;
	const guint8 *rgba;
	GtkWidget *w, *toplevel;
	imgview_format format;
	imgview_struct *iv;
	IVInfo *info;
	IVCore *core;
	IVImageViewer *v = IV_IMAGE_VIEWER(data);
	if(v == NULL)
		return;

	info = &v->info;
	iv = v->iv;
	core = v->core;
	if((iv == NULL) || (core == NULL))
		return;

	/* Get the ImgView's toplevel widget if it is a GtkWindow */
	if(ImgViewIsToplevelWindow(iv))
		toplevel = ImgViewGetToplevelWidget(iv);
	else
		toplevel = NULL;

	/* Get viewer widget */
	w = (GtkWidget *)ImgViewGetViewWidget(iv);
	if((w != NULL) ? (w->window != NULL) : FALSE)
	{
		GdkWindow *window = w->window;
		if(window != NULL)
		{
			GdkVisual *vis = gdk_window_get_visual(window);

			/* Get Visual information */
			if(vis != NULL)
			{
				vis_id = (guint)vis;
				switch(vis->type)
				{
				  case GDK_VISUAL_STATIC_GRAY:
					vis_name = "StaticGray";
					break;
				  case GDK_VISUAL_GRAYSCALE:
					vis_name = "GreyScale";
					break;
				  case GDK_VISUAL_STATIC_COLOR:
					vis_name = "StaticColor";
					break;
				  case GDK_VISUAL_PSEUDO_COLOR:
					vis_name = "PseudoColor";
					break;
				  case GDK_VISUAL_TRUE_COLOR:
					vis_name = "TrueColor";
					break;
				  case GDK_VISUAL_DIRECT_COLOR:
					vis_name = "DirectColor";
					break;
				}
				depth = vis->depth;
			}
		}
	}

	/* Get the current image data */
	rgba = ImgViewGetImageData(
		iv, ImgViewGetCurrentFrame(iv),
		&width, &height, &bpp, &bpl, &format
	);

	nframes = ImgViewGetTotalFrames(iv);

	/* Get the ImgView's alpha channel info */
	if((rgba != NULL) &&
		((format == IMGVIEW_FORMAT_RGBA) ||
		 (format == IMGVIEW_FORMAT_GREYSCALEA32)
		)
	)
	{
		/* There is an allocated alpha channel, but check if
		 * it has defined data
		 */
		alpha_channel = (iv->alpha_flags & IMGVIEW_ALPHA_DEFINED) ?
			2 : 1;
	}
	else
	{
		alpha_channel = 0;
	}

	/* Get other information about the image */

	/* Path */
	path = info->path;

	/* Extension */
	ext_ptr = (path != NULL) ?
		(const gchar *)strrchr((const char *)path, '.') : NULL;

	/* Get file's name without parent path */
	name_ptr = (path != NULL) ? g_basename(path) : NULL;
	if(name_ptr == NULL)
		name_ptr = path;

	/* Get display name */
	dpy_name = (const gchar *)gdk_get_display();

	/* Create a new Image Info Dialog to display the newly obtained
	 * image information
	 */
	if(TRUE)
	{
		GList *glist;
		ImgInfoDlg *d = img_info_dlg_new(toplevel);
		img_info_dlg_set_values(
			d,
#ifdef HAVE_LIBENDEAVOUR2
			core->edv2_ctx,
#endif
			rgba,
			width, height,
			bpp, bpl,
			nframes,
			info->nloops,
			info->play_time_ms,
			info->total_play_time_ms,
			alpha_channel,
			v->bg_color,			/* 4 bytes RGBA */
			info->x, info->y,
			info->base_width, info->base_height,
			v->has_changes,
			name_ptr,
			ext_ptr,
			info->file_size,		/* File size in bytes */
			info->time_modified,		/* Last modified time in
											 * seconds since EPOCH */
			info->title,
			info->author,
			info->creator,
			info->comments,
			dpy_name,			/* X Display Address */
			vis_name,			/* X Visual Type */
			depth				/* X Depth In Bits Per Pixel */
		);
		img_info_dlg_map(d);

		/* Record window */
		for(glist = core->imginfodlg;
			glist != NULL;
			glist = g_list_next(glist)
		)
		{
			if(glist->data == NULL)
			{
				glist->data = d;
				break;
			}
		}
		if(glist == NULL)
			core->imginfodlg = g_list_append(
				core->imginfodlg,
				d
			);
	}
}

/*
 *	Fullscreen callback.
 */
static void iv_viewer_fullscreen_cb(GtkWidget *widget, gpointer data)
{
	gboolean was_playing;
	gint i;
	guint8 **rgba_list;
	gulong *delay_list;
	GList *glist;
	GtkWidget *toplevel;
	imgview_frame_struct *frame;
	imgview_image_struct *img;
	imgview_struct *iv;
	IVCore *core;
	IVImageViewer *v = IV_IMAGE_VIEWER(data);
	if(v == NULL)
		return;

	iv = v->iv;
	core = v->core;
	if((iv == NULL) || (core == NULL))
		return;

	/* Get the ImgView's toplevel widget if it is a GtkWindow */
	if(ImgViewIsToplevelWindow(iv))
		toplevel = ImgViewGetToplevelWidget(iv);
	else
		toplevel = NULL;

	/* Get the current image */
	img = ImgViewGetImage(iv);
	if(img == NULL)
		return;

	rgba_list = (guint8 **)g_malloc0(img->nframes * sizeof(guint8 *));
	delay_list = (gulong *)g_malloc0(img->nframes * sizeof(gulong));
	if((rgba_list == NULL) || (delay_list == NULL))
	{
		g_free(rgba_list);
		g_free(delay_list);
		return;
	}
	for(i = 0, glist = img->frames_list;
		glist != NULL;
		i++, glist = g_list_next(glist)
	)
	{
		frame = IMGVIEW_FRAME(glist->data);
		if(frame == NULL)
			continue;

		rgba_list[i] = frame->data;
		delay_list[i] = frame->delay;
	}

	/* Pause while displaying full screen */
	was_playing = ImgViewIsPlaying(iv);
	if(was_playing)
		ImgViewPause(iv);

	/* Display the image full screen and wait for user reponse */
	FSImgNew(
		rgba_list, delay_list,
		img->nframes,
		img->width, img->height, img->bpl,
		v->bg_color,		/* 4 bytes RGBA */
		iv->quality,		/* Quality, 0 to 2 */
		TRUE,			/* Block until user response */
		toplevel
	);

	g_free(rgba_list);
	g_free(delay_list);

	if(was_playing)
		ImgViewPlay(iv);
}

#if !defined(_WIN32)
/*
 *	Print callback.
 */
static void iv_viewer_print_cb(GtkWidget *widget, gpointer data)
{
	gboolean was_playing;
	gint frame_num;
	const gchar	*path,
					*name;
	GtkWidget *toplevel;
	print_values_struct *pv;
	imgview_struct *iv;
	imgview_image_struct *img;
	IVInfo *info;
	IVCore *core;
	IVImageViewer *v = IV_IMAGE_VIEWER(data);
	if(v == NULL)
		return;

	iv = v->iv;
	core = v->core;
	if(v->processing || (iv == NULL) || (core == NULL))
		return;

	was_playing = ImgViewIsPlaying(iv);
	if(was_playing)
		ImgViewPause(iv);

	/* Get the ImgView's toplevel widget if it is a GtkWindow */
	if(ImgViewIsToplevelWindow(iv))
		toplevel = ImgViewGetToplevelWidget(iv);
	else
		toplevel = NULL;

	info = &v->info;
	path = info->path;
	name = (path != NULL) ? g_basename(path) : NULL;

	/* Get the image that we want to print */
	img = ImgViewGetImage(iv);
	if(img == NULL)
		return;
	frame_num = ImgViewGetCurrentFrame(iv);

	/* Get and set up the print values */
	pv = core->print_values;
#if defined(HAVE_XPRINT)
	pv->flags |= PRINT_VALUE_JOB_TITLE;
	g_free(pv->job_title);
	pv->job_title = g_strdup_printf(
		"%s: %s",
		PROG_NAME_FULL,
		STRISEMPTY(info->title) ?
			(STRISEMPTY(path) ? "Untitled" : path) : info->title
	);
#endif
	pv->flags |= PRINT_VALUE_FRAME;
	pv->frame = frame_num;
	pv->flags |= PRINT_VALUE_OUTPUT_WIDTH | PRINT_VALUE_OUTPUT_HEIGHT;
	pv->output_width = img->width;
	pv->output_height = img->height;

	/* Query user to print image */
#ifdef HAVE_LIBENDEAVOUR2
	if(PrintDlgGetResponse(
		img,			/* Image */
		name,			/* File name */
		pv,				/* Print values, will be modified */
		v->bg_color,		/* 4 bytes RGBA */
		toplevel,
		core->edv2_ctx
	))
#else
	if(PrintDlgGetResponse(
		img,			/* Image */
		name,			/* File name */
		pv,				/* Print values, will be modified */
		v->bg_color,		/* 4 bytes RGBA */
		toplevel
	))
#endif
	{
		ImgViewSetBusy(iv, TRUE);
		v->processing = TRUE;
		v->stop_count = 0;

		/* Print */
#ifdef HAVE_LIBENDEAVOUR2
		iv_print(
			img,
			pv,				/* Print values will be modified */
			v->bg_color,			/* Background color RGBA */
			toplevel,
			core->edv2_ctx
		);
#else
		iv_print(
			img,
			pv,				/* Print values will be modified */
			v->bg_color,			/* Background color RGBA */
			toplevel
		);
#endif

		v->processing = FALSE;
		ImgViewSetBusy(iv, FALSE);
	}

	if(was_playing)
		ImgViewPlay(iv);
}
#endif	/* !_WIN32 */

/*
 *	New Image Viewer callback.
 */
static void iv_viewer_new_viewer_cb(GtkWidget *widget, gpointer data)
{
	imgview_struct *iv;
	IVImageViewer *v2;
	IVCore *core;
	IVImageViewer *v = IV_IMAGE_VIEWER(data);
	if(v == NULL)
		return;

	if(v->processing)
		return;

	iv = v->iv;
	core = v->core;

	/* Create a new Image Viewer */
	v2 = iv_viewer_new(
		core,
		core->options,
		iv->quality,
		core->bg_color_set ? core->bg_color : NULL,
		core->geometry_flags,
		core->geometry,
		TRUE				/* Always load a title when
											 * creating a new Image Viewer */
	);

	/* Update the original Image Viewer's widgets */
	iv_viewer_update(v);
}


/*
 *	Updates the Image Viewer's ImgView toplevel GtkWindow title.
 *
 *	If new_path is not NULL then it will be used as the new file
 *	name instead of v->path.
 */
static void iv_viewer_update_title(
	IVImageViewer *v,
	const gchar *new_path
)
{
	const gchar *path, *name;
	gchar *title;
	GtkWidget *w;
	IVInfo *info;
	IVCore *core;
	imgview_struct *iv = (v != NULL) ? v->iv : NULL;
	if(iv == NULL)
		return;

	info = &v->info;
	core = v->core;
	if((core == NULL))
		return;

	/* Get the ImgView's toplevel widget if it is a GtkWindow */
	if(ImgViewIsToplevelWindow(iv))
		w = ImgViewGetToplevelWidget(iv);
	else
		w = NULL;
	if(w == NULL)
		return;

	/* Get the new path if it is specified or else get the
	 * current path from the info
	 */
	path = (new_path != NULL) ? new_path : info->path;

	/* Get the name portion of the path */
	name = (path != NULL) ? g_basename(path) : NULL;

	/* Format the title */
	if(!STRISEMPTY(path))
		title = g_strdup_printf(
			"%s: %s %s",
			(core->title != NULL) ?
				core->title : PROG_NAME_FULL,
			name,
			(v->has_changes) ? IV_HAS_CHANGES_STR : ""
		);
	else
		title = g_strdup_printf(
			"%s: Untitled #%i %s",
			(core->title != NULL) ?
				core->title : PROG_NAME_FULL,
			core->untitled_count,
			(v->has_changes) ? IV_HAS_CHANGES_STR : ""
		);

	/* Set title */
	gtk_window_set_title(GTK_WINDOW(w), title);

	g_free(title);
}

/*
 *	Updates the Image Viewer's widgets to reflect current values.
 *
 *	Does not update the ImgView's widgets.
 */
static void iv_viewer_update(IVImageViewer *v)
{
	gboolean sensitive;
	imgview_struct *iv = (v != NULL) ? v->iv : NULL;
	if(iv == NULL)
		return;

	sensitive = ImgViewIsLoaded(iv);
/*	GTK_WIDGET_SET_SENSITIVE(v->edit_mi, sensitive); */
	GTK_WIDGET_SET_SENSITIVE(v->resize_mi, sensitive);
	GTK_WIDGET_SET_SENSITIVE(v->color_levels_mi, sensitive);
	GTK_WIDGET_SET_SENSITIVE(v->add_text_mi, sensitive);
	GTK_WIDGET_SET_SENSITIVE(v->edit_header_mi, sensitive);
	GTK_WIDGET_SET_SENSITIVE(v->edit_frames_mi, sensitive);
	GTK_WIDGET_SET_SENSITIVE(v->save_as_mi, sensitive);
	GTK_WIDGET_SET_SENSITIVE(v->fullscreen_mi, sensitive);
	GTK_WIDGET_SET_SENSITIVE(v->desktop_mi, sensitive);
	GTK_WIDGET_SET_SENSITIVE(v->clear_mi, sensitive);
	GTK_WIDGET_SET_SENSITIVE(v->print_mi, sensitive);
}

/*
 *	Resizes the Image Viewer.
 *
 *	The v specifies the Image Viewer.
 *
 *	The width and height specifies the size to resize the Image
 *	Viewer to.
 *
 *	If keep_visible_on_desktop is TRUE then the position and
 *	the width and height may be automatically adjusted to keep
 *	the Image Viewer visible on the desktop.
 */
static void iv_viewer_resize(
	IVImageViewer *v,
	const gint width, const gint height,
	const gboolean keep_visible_on_desktop
)
{
	const gint	wm_frame_width = 8,	/* Assumed values */
					wm_title_height = 20;
	gint		_width = width,
					_height = height,
					left_pad, right_pad,
					top_pad, bottom_pad;
	GdkWindow *window, *root_win;
	GtkWidget *w, *toplevel;
	imgview_struct *iv = (v != NULL) ? v->iv : NULL;
	if((iv == NULL) || (_width <= 0) || (_height <= 0))
		return;

	/* Get the ImgView's toplevel widget if it is a GtkWindow */
	if(ImgViewIsToplevelWindow(iv))
		toplevel = ImgViewGetToplevelWidget(iv);
	else
		toplevel = NULL;
	if(toplevel == NULL)
		return;

	/* If the Image Viewer does not need to be visible on the
	 * desktop then just set the size explicitly here
	 */
	if(!keep_visible_on_desktop)
	{
		GtkWidget *w = (GtkWidget *)ImgViewGetViewWidget(iv);
		if(w != NULL)
		{
			gtk_widget_set_usize(w, _width, _height);
			gtk_widget_queue_resize(w);
		}
		gtk_widget_queue_resize(toplevel);
		return;
	}

	/* Calculate the paddings from the ImgView's view GtkWidget
	 * to the edge of the ImgView's toplevel GtkWindow
	 *
	 * All padding values are absolute values
	 */

	/* Left padding */
	left_pad = wm_frame_width;

	/* Right padding */
	right_pad = wm_frame_width;
	w = iv->vscrollbar;
	if(w != NULL)
		right_pad += 2 + w->allocation.width;

	/* Top padding */
	top_pad = wm_frame_width + wm_title_height;
	w = IMGVIEW_TOOL_BAR_MAPPED(iv) ?
		iv->tool_bar_toplevel : NULL;
	if(w != NULL)
		top_pad += 2 + w->allocation.height;

	/* Bottom padding */
	bottom_pad = wm_frame_width;
	w = iv->hscrollbar;
	if(w != NULL)
		bottom_pad += 2 + w->allocation.height;
	w = IMGVIEW_STATUS_BAR_MAPPED(iv) ?
		iv->status_bar_toplevel : NULL;
	if(w != NULL)
		bottom_pad += 2 + w->allocation.height;

	/* Get the ImgView's view GtkWidget */
	w = (GtkWidget *)ImgViewGetViewWidget(iv);
	if(w == NULL)
		return;

	/* Get the ImgView toplevel's GdkWindow and desktop's
	 * GdkWindow
	 */
	window = toplevel->window;
	root_win = gdk_window_get_root();

	/* If we got the root GdkWindow then make sure that the entire
	 * ImgView does not exceed the desktop's size if possible
	 */
	if(root_win != NULL)
	{
		const gint	toplevel_width = _width + left_pad + right_pad,
					toplevel_height = _height + top_pad + bottom_pad;
		gint	root_width, root_height;

		gdk_window_get_size(root_win, &root_width, &root_height);

		if(toplevel_width > root_width)
			_width = root_width - left_pad - right_pad;
		if(toplevel_height > root_height)
			_height = root_height - top_pad - bottom_pad;
		if((_width <= 0) || (_height <= 0))
			return;
	}
	/* Resize the ImgView's view and toplevel GtkWidgets */
	gtk_widget_set_usize(w, _width, _height);
	gtk_widget_queue_resize(w);
	gtk_widget_queue_resize(toplevel);

	/* Adjust the position to keep the image on the desktop
	 *
	 * Since the toplevel's size is not updated yet, we need to
	 * calculate it based on the view and other widgets' size
	 * then adjust the toplevel's position as needed
	 */
	if((window != NULL) && (root_win != NULL))
	{
		gboolean need_move = FALSE;
		gint	x, y,
					root_width, root_height;

		gdk_window_get_root_origin(window, &x, &y);
		gdk_window_get_size(root_win, &root_width, &root_height);

		if((x + (_width + right_pad)) > root_width)
		{
			x = root_width - (_width + right_pad);
			need_move = TRUE;
		}
		if(x < left_pad)
		{
			x = left_pad;
			need_move = TRUE;
		}

		if((y + (_height + bottom_pad)) > root_height)
		{
			y = root_height - (_height + bottom_pad);
			need_move = TRUE;
		}
		if(y < top_pad)
		{
			y = top_pad;
			need_move = TRUE;
		}

		if(need_move)
		{
			/* Convert view position to toplevel position */
			gint	toplevel_x = x,
					toplevel_y = y;
			GtkWidget *w = IMGVIEW_TOOL_BAR_MAPPED(iv) ?
				iv->tool_bar_toplevel : NULL;
			if(w != NULL)
				toplevel_y -= 2 + w->allocation.height;

			gtk_widget_set_uposition(
				toplevel,
				toplevel_x, toplevel_y
			);
		}
	}
}


/*
 *	Deletes and resets the Image Viewer's info.
 */
static void iv_clear_info(IVInfo *info)
{
	if(info == NULL)
		return;

	g_free(info->path);
	g_free(info->creator);
	g_free(info->title);
	g_free(info->author);
	g_free(info->comments);

	(void)memset(info, 0x00, sizeof(IVInfo));
}


/*
 *	Parses the parameter and value from an argument, returning a
 *	statically allocated string or a string pointing within arg for
 *	both the parm_rtn and val_rtn.
 *
 *	The given arg will not be modified.
 */
static void get_args_value(
	char *arg, char **parm_rtn, char **val_rtn
)
{
	gchar *s;
	static gchar parm_str[256];


	if(parm_rtn != NULL)
		*parm_rtn = "";
	if(val_rtn != NULL)
		*val_rtn = "";

	if(arg == NULL)
		return;

	/* If first char is not a '-' or '+' then skip as there is
	 * no parameter or value.
	 */
	if((*arg != '-') && (*arg != '+'))
		return;

	/* Seek past initial '-' or '+' characters */
	while((*arg == '-') || (*arg == '+'))
		arg++;

	/* Get parameter and store it in our parm_str */
	strncpy(parm_str, arg, sizeof(parm_str));
	parm_str[sizeof(parm_str) - 1] = '\0';

	s = strchr(parm_str, '=');
	if(s != NULL)
		*s = '\0';
	s = strchr(parm_str, '-');
	if(s != NULL)
		*s = '\0';

	/* Update parm return */
	if(parm_rtn != NULL)
		*parm_rtn = parm_str;


	/* Get value */
	s = strchr(arg, '=');
	if(s == NULL)
		s = strchr(arg, '-');
	if(s == NULL)
	{
		if(val_rtn != NULL)
			*val_rtn = "";
	}
	else
	{
		if(val_rtn != NULL)
			*val_rtn = s + 1;
	}
}

/*
 *	Returns TRUE if the given value is true, accepted values are
 *	"yes" "y" "on" "active" "enabled" or any non-zero value. Any
 *	other value will return FALSE.
 */
static gboolean stristrue(const gchar *s)
{
	gint c;

	if(s == NULL)
		return(FALSE);

	c = (int)(*s);
	if((c == 'Y') || (c == 'y'))
		return(TRUE);

	if(!g_strcasecmp(s, "on"))
		return(TRUE);

	if(!g_strcasecmp(s, "active"))
		return(TRUE);

	if(!g_strcasecmp(s, "enabled"))
		return(TRUE);

	c = ATOI(s);
	if(c != 0)
		return(TRUE);

	return(FALSE);
}


#if defined(_WIN32)
int WINAPI WinMain(
	HINSTANCE hInstance,		// Instance
	HINSTANCE hPrevInstance,	// Previous Instance
	LPSTR lpCmdLine,		// Command Line Parameters
	int nCmdShow			// Window Show State
)
#else
int main(int argc, char *argv[])
#endif
{
#ifdef _WIN32
	/* Need to explode arguments for Win32 */
	gint argc;
	gchar **argv = g_strsplit(lpCmdLine, " ", -1);
#endif
	gint rtn_val = 0;
	gboolean	initialized_gtk = FALSE,
					initialized_dialogs = FALSE,
					startup_screenshot = FALSE;
	IVOptionFlags options =	IV_OPTIONS_DEFAULT;
	gint		quality = 1,		/* 0 to 2 */
					startup_frame_num = -1;
	guint8		bg_color[4] = { 0x00, 0x00, 0x00, 0xff };
	gboolean	bg_color_set = FALSE;
	gint		imlib_fallback = 1;	/* 0 or 1 */
	const gchar	*fullscreen_path = NULL,
					*display_on_root_centered_path = NULL,
					*display_on_root_tofit_path = NULL,
					*display_on_root_tocover_path = NULL,
					*display_on_root_tiled_path = NULL,
					*print_path = NULL;
	gchar		*font_name = NULL,
					*wmname = NULL,
					*wmclass = NULL,
					*title = NULL;
	GdkGeometryFlags geometry_flags = 0;
	GdkRectangle	*geometry = NULL;

	gint i;
	GList *file_list = NULL;
	IVCore *core;

	/* Reset the main core pointer */
	iv_core = NULL;

	/* Set the system signal callbacks */
#ifdef SIGHUP
	signal(SIGHUP, iv_signal_cb);
#endif
#ifdef SIGINT
	signal(SIGINT, iv_signal_cb);
#endif
#ifdef SIGTERM
	signal(SIGTERM, iv_signal_cb);
#endif
#ifdef SIGQUIT
	signal(SIGQUIT, iv_signal_cb);
#endif
#ifdef SIGSEGV
	signal(SIGSEGV, iv_signal_cb);
#endif
#ifdef SIGPIPE
	signal(SIGPIPE, iv_signal_cb);
#endif


#ifdef _WIN32
	/* Count arguments */
	for(argc = 0; argv[argc] != NULL; argc++);
#endif                                              


/* Deletes the values obtained from the command line */
#define DELETE_FETCHED_CMD_LINE_VALUES	{	\
 g_free(font_name);				\
 g_free(wmname);				\
 g_free(wmclass);				\
 g_free(title);					\
 g_free(geometry);				\
											\
 g_list_foreach(file_list, (GFunc)g_free, NULL);\
 g_list_free(file_list);			\
 file_list = NULL;				\
}

	/* Parse arguments for help and version only */
	for(i = 1; i < argc; i++)
	{
		gchar *parm, *val;
		gchar *arg = (gchar *)argv[i];
		if(arg == NULL)
			continue;

		get_args_value(arg, &parm, &val);

		/* Help? */
		if(!g_strcasecmp(parm, "help") ||
		   !g_strcasecmp(parm, "h") ||
		   !g_strcasecmp(parm, "?")
		)
		{
#if defined(_WIN32)
			g_print("Usage: iv [files]");
#else
			g_print(PROG_USAGE);
#endif
			DELETE_FETCHED_CMD_LINE_VALUES;
			return(rtn_val);
		}
		/* Version? */
		else if(!g_strcasecmp(parm, "version"))
		{
			gint	major,
					minor,
					release;

			/* This program's version and copyright */
			g_print(
				"%s Version %s\n%s",
				PROG_NAME_FULL, PROG_VERSION,
				PROG_COPYRIGHT
			);
			g_print("\nSupported Libraries:\n");

#if defined(HAVE_X) && defined(HAVE_XF86_VIDMODE)
			g_print(
"        XF86VidMode\n"
			);
#endif
#if defined(HAVE_IMLIB) && defined(IMLIB_VERSION)
			g_print(
"        Imlib Version " IMLIB_VERSION "\n"
			);
#endif
#if defined(HAVE_IMLIB2) && defined(IMLIB2_VERSION)
			g_print(
"        Imlib2 Version " IMLIB2_VERSION "\n"
			);
#endif
#ifdef GTK_VERSION
			g_print(
"        GTK+ Version " GTK_VERSION "\n"
			);
#endif
#ifdef HAVE_LIBGIF
			ImgGIFVersion(&major, &minor, &release);
			g_print(
"        Uncompressed GIF Library Version %i.%i.%i\n",
				major, minor, release
			);
#endif
#ifdef HAVE_LIBJPEG
			ImgJPEGVersion(&major, &minor, &release);
			g_print(
"        JPEG Library Version %i.%i.%i\n",
				major, minor, release
			);
#endif
#ifdef HAVE_LIBMNG
			ImgMNGVersion(&major, &minor, &release);
			g_print(
"        MNG Library Version %i.%i.%i\n",
				major, minor, release
			);
#endif
#ifdef HAVE_LIBPNG
			ImgPNGVersion(&major, &minor, &release);
			g_print(
"        PNG Library Version %i.%i.%i\n",
				major, minor, release
			);
#endif
#ifdef HAVE_LIBTIFF
			ImgTIFFVersion(&major, &minor, &release);
			g_print(
"        TIFF Library Version %i.%i.%i\n",
				major, minor, release
			);
#endif
#ifdef HAVE_LIBXPM
			ImgXPMVersion(&major, &minor, &release);
			g_print(
"        XPM Library Version %i.%i.%i\n",
				major, minor, release
			);
#endif            
			DELETE_FETCHED_CMD_LINE_VALUES;
			return(rtn_val);
		}
	}

	/* Initialize GTK as needed */
	if(!initialized_gtk)
	{
		if(!gtk_init_check(&argc, &argv))
		{
			g_printerr("Unable to initialize GTK.\n");
			DELETE_FETCHED_CMD_LINE_VALUES;
			rtn_val = 1;
			return(rtn_val);
		}
		initialized_gtk = TRUE;

		/* Initialize GDK RGB buffers */
		gdk_rgb_init();
	}


	/* Parse arguments */
	for(i = 1; i < argc; i++)
	{
		gchar *parm, *val;
		gchar *arg = (gchar *)argv[i];
		if(arg == NULL)
			continue;

		get_args_value(arg, &parm, &val);

		/* Enable */
		if(!g_strcasecmp(parm, "enable") ||
		   !g_strcasecmp(parm, "with")
		)
		{
			if(!g_strcasecmp(val, "show_tooltips"))
				options |= IV_OPTION_SHOW_TOOLTIPS;
			else if(!g_strcasecmp(val, "show_tool_bar"))
				options |= IV_OPTION_SHOW_TOOL_BAR;
			else if(!g_strcasecmp(val, "show_values"))
				options |= IV_OPTION_SHOW_VALUES;
			else if(!g_strcasecmp(val, "show_status_bar"))
				options |= IV_OPTION_SHOW_STATUS_BAR;
			else if(!g_strcasecmp(val, "show_image_on_wm_icon"))
				options |= IV_OPTION_SHOW_IMAGE_ON_WM_ICON;
			else if(!g_strcasecmp(val, "resize_on_open"))
				options |= IV_OPTION_RESIZE_ON_OPEN;
			else if(!g_strcasecmp(val, "zoom_to_fit"))
				options |= IV_OPTION_ZOOM_TO_FIT_ON_OPEN;
		}
		/* Disable */
		else if(!g_strcasecmp(parm, "disable") ||
				!g_strcasecmp(parm, "without")
		)
		{
			if(!g_strcasecmp(val, "show_tooltips"))
				options &= ~IV_OPTION_SHOW_TOOLTIPS;
			else if(!g_strcasecmp(val, "show_tool_bar"))
				options &= ~IV_OPTION_SHOW_TOOL_BAR;
			else if(!g_strcasecmp(val, "show_values"))
				options &= ~IV_OPTION_SHOW_VALUES;
			else if(!g_strcasecmp(val, "show_status_bar"))
				options &= ~IV_OPTION_SHOW_STATUS_BAR;
			else if(!g_strcasecmp(val, "show_image_on_wm_icon"))
				options &= ~IV_OPTION_SHOW_IMAGE_ON_WM_ICON;
			else if(!g_strcasecmp(val, "resize_on_open"))
				options &= ~IV_OPTION_RESIZE_ON_OPEN;
			else if(!g_strcasecmp(val, "zoom_to_fit"))
				options &= ~IV_OPTION_ZOOM_TO_FIT_ON_OPEN;
		}
		/* Startup Frame Index */
		else if(!g_strcasecmp(parm, "frame") ||
				!g_strcasecmp(parm, "f")
		)
		{
			if(STRISEMPTY(val))
			{
				i++;
				if(i < argc)
				{
					val = argv[i];
				}
				else
				{
					val = "";
					g_printerr(
						"%s: Requires argument.\n",
						parm
					);
				}
			}
			if(!STRISEMPTY(val))
			{
				startup_frame_num = ATOI(val);
			}
		}
		/* Do Not Automatically Resize On Open */
		else if(!g_strcasecmp(parm, "r"))
		{
			options &= ~IV_OPTION_RESIZE_ON_OPEN;
		}
		/* Automatically Zoom On Open */
		else if(!g_strcasecmp(parm, "z"))
		{
			options |= IV_OPTION_ZOOM_TO_FIT_ON_OPEN;
			/* Must disable resize on open */
			options &= ~IV_OPTION_RESIZE_ON_OPEN;
		}
		/* Show/Hide Tooltips */
		else if(!g_strcasecmp(parm, "tooltips"))
		{
			if(stristrue(val))
				options |= IV_OPTION_SHOW_TOOLTIPS;
			else
				options &= ~IV_OPTION_SHOW_TOOLTIPS;
		}
		/* Hide Tooltips */
		else if(!g_strcasecmp(parm, "m"))
		{
			options &= ~IV_OPTION_SHOW_TOOLTIPS;
		}
		/* Show/Hide Tool Bar */
		else if(!g_strcasecmp(parm, "tool_bar") ||
		        !g_strcasecmp(parm, "toolbar")
			   )
		{
			if(stristrue(val))
				options |= IV_OPTION_SHOW_TOOL_BAR;
			else
				options &= ~IV_OPTION_SHOW_TOOL_BAR;
		}
		else if(!g_strcasecmp(parm, "t"))
		{
			options &= ~IV_OPTION_SHOW_TOOL_BAR;
		}
		/* Show/Hide Values */
		else if(!g_strcasecmp(parm, "show_values"))
		{
			if(stristrue(val))
				options |= IV_OPTION_SHOW_VALUES;
			else
				options &= ~IV_OPTION_SHOW_VALUES;
		}
		else if(!g_strcasecmp(parm, "v"))
		{
			options &= ~IV_OPTION_SHOW_VALUES;
		}
		/* Show/Hide Status Bar */
		else if(!g_strcasecmp(parm, "show_status_bar") ||
				!g_strcasecmp(parm, "show_progress_bar")
		)
		{
			if(stristrue(val))
				options |= IV_OPTION_SHOW_STATUS_BAR;
			else
				options &= ~IV_OPTION_SHOW_STATUS_BAR;
		}
		else if(!g_strcasecmp(parm, "s"))
		{
			options &= ~IV_OPTION_SHOW_STATUS_BAR;
		}
		/* Show Image On WM Icon */
		else if(!g_strcasecmp(parm, "imageonwmicon"))
		{
			if(stristrue(val))
				options |= IV_OPTION_SHOW_IMAGE_ON_WM_ICON;
			else
				options &= ~IV_OPTION_SHOW_IMAGE_ON_WM_ICON;
		}
		else if(!g_strcasecmp(parm, "i"))
		{
			options |= IV_OPTION_SHOW_IMAGE_ON_WM_ICON;
		}
		/* Quality */
		else if(!g_strcasecmp(parm, "q"))
		{
			/* Value from 0 to 2 (2 being best/slowest) */
			quality = CLIP(ATOI(val), 0, 2);
		}
		/* Background Color */
		else if(!g_strcasecmp(parm, "bg") ||
				!g_strcasecmp(parm, "bg_color") ||
				!g_strcasecmp(parm, "bgcolor")
		)
		{
			/* Need to get the argument from the next argument? */
			if(STRISEMPTY(val))
			{
				i++;
				if(i < argc)
				{
					val = argv[i];
				}
				else
				{
					val = "";
					g_printerr(
						"%s: Requires argument.\n",
						parm
					);
				}
			}
			/* Argument specified? */
			if(!STRISEMPTY(val))
			{
				GdkColor c;
				if(gdk_color_parse(val, &c))
				{
					bg_color[0] = (guint8)(c.red >> 8);
					bg_color[1] = (guint8)(c.green >> 8);
					bg_color[2] = (guint8)(c.blue >> 8);
					bg_color[3] = 0xFF;
					bg_color_set = TRUE;
				}
				else
				{
					g_printerr(
"%s: Unable to parse color \"%s\".\n",
						parm, val
					);
				}
			}
		}
		/* Imlib Fallback */
		else if(!g_strcasecmp(parm, "b"))
		{
			/* Value of 0 or 1 */
			imlib_fallback = CLIP(ATOI(val), 0, 1);
		}
		/* Automatically Hide IV when viewing Non-Image files */
		else if(!g_strcasecmp(parm, "auto_hide_iv_nonimage") ||
				!g_strcasecmp(parm, "autohideivnonimage") ||
				!g_strcasecmp(parm, "k")
		)
		{
			options |= IV_OPTION_HIDE_IV_NON_IMAGE;
		}
		/* Display Simple Non-graphical Title On Startup */
		else if(!g_strcasecmp(parm, "simple_title") ||
				!g_strcasecmp(parm, "simpletitle")
		)
		{
			options |= IV_OPTION_SIMPLE_TITLE;
		}
		/* Screenshot On Startup */
		else if(!g_strcasecmp(parm, "screenshot") ||
				!g_strcasecmp(parm, "grab") ||
				!g_strcasecmp(parm, "g")
		)
		{
			startup_screenshot = TRUE;
		}
		/* Fullscreen */
		else if(!g_strcasecmp(parm, "fullscreen") ||
				!g_strcasecmp(parm, "fs")
		)
		{
			/* Need to get argument from next argument? */
			if(STRISEMPTY(val))
			{
				i++;
				if(i < argc)
				{
					val = argv[i];
				}
				else
				{
					val = "";
					g_printerr(
						"%s: Requires argument.\n",
						parm 
					);
				}
			}
			/* Argument specified? */
			if(!STRISEMPTY(val))
			{
				fullscreen_path = val;
			}
		}
		/* Display Image On Desktop */
		else if(!g_strcasecmp(parm, "root") ||
				!g_strcasecmp(parm, "desktop")
		)
		{
			/* Need to get argument from next argument? */
			if(STRISEMPTY(val))
			{
				i++;
				if(i < argc)
				{
					val = argv[i];
				}
				else
				{
					val = "";
					g_printerr(
						"%s: Requires argument.\n",
						parm
					);
				}
			}
			/* Argument specified? */
			if(!STRISEMPTY(val))
			{
				if(strstr(arg, "centered") ||
				   strstr(arg, "center")
				)
					display_on_root_centered_path = val;
				else if(strstr(arg, "tofit"))
					display_on_root_tofit_path = val;
				else if(strstr(arg, "tocover"))
					display_on_root_tocover_path = val;
				else if(strstr(arg, "tiled") ||
						strstr(arg, "tile")
				)
					display_on_root_tiled_path = val;
			}
		}
		/* Print */
		else if(!g_strcasecmp(parm, "print"))
		{
			/* Need to get argument from next argument? */
			if(STRISEMPTY(val))
			{
				i++;
				if(i < argc)
				{
					val = argv[i];
				}
				else
				{
					val = "";
					g_printerr(
						"%s: Requires argument.\n",
						parm
					);
				}
			}
			/* Argument specified? */
			if(!STRISEMPTY(val))
			{
				print_path = val;
			}
		}
		/* Font */
		else if(!g_strcasecmp(parm, "fontname") ||
				!g_strcasecmp(parm, "font") ||
				!g_strcasecmp(parm, "fn")
		)
		{
			/* Need to get argument from next argument? */
			if(STRISEMPTY(val))
			{
				i++;
				if(i < argc)
				{
					val = argv[i];
				}
				else
				{
					val = "";
					g_printerr(
						"%s: Requires argument.\n",
						parm
					);
				}
			}
			/* Argument specified? */
			if(!STRISEMPTY(val))
			{
				g_free(font_name);
				font_name = STRDUP(val);
			}
		}
		/* WM Name */
		else if(!g_strcasecmp(parm, "wmname"))
		{
			/* Need to get argument from next argument? */
			if(STRISEMPTY(val))
			{
				i++;
				if(i < argc)
				{
					val = argv[i];
				}
				else
				{
					val = "";
					g_printerr(
						"%s: Requires argument.\n",
						parm
					);
				}
			}
			/* Argument specified? */
			if(!STRISEMPTY(val))
			{
				g_free(wmname);
				wmname = STRDUP(val);
			}
		}
		/* WM Class */
		else if(!g_strcasecmp(parm, "wmclass"))
		{
			/* Need to get argument from next argument? */
			if(STRISEMPTY(val))
			{
				i++;
				if(i < argc)
				{
					val = argv[i];
				}
				else
				{
					val = "";
					g_printerr(
						"%s: Requires argument.\n",
						parm   
					);
				}
			}
			/* Argument specified? */
			if(!STRISEMPTY(val))
			{
				g_free(wmclass);
				wmclass = STRDUP(val);
			}
		}
		/* Title */
		else if(!g_strcasecmp(parm, "title"))
		{
			/* Need to get argument from next argument? */
			if(STRISEMPTY(val))
			{
				i++;
				if(i < argc)
				{
					val = argv[i];
				}
				else 
				{
					val = "";
					g_printerr(
"%s: Requires argument.\n",
						parm
					);
				}
			}
			/* Argument specified? */
			if(!STRISEMPTY(val))
			{
				g_free(title);
				title = STRDUP(val);
			}
		}
		/* Geometry */
		else if(!g_strcasecmp(parm, "geometry"))
		{
			/* Need to get argument from next argument? */
			if(STRISEMPTY(val))
			{
				i++;
				if(i < argc)
				{
				    val = argv[i];
				}
				else
				{
					val = "";
					g_printerr(
						"%s: Requires argument.\n",
						parm
					);
				}
			}
			/* Argument specified? */
			if(!STRISEMPTY(val))
			{
				gint	x, y,
							width, height;
				GdkGeometryFlags flags = gdk_parse_geometry(
					val,
					&x, &y,
					&width, &height
				);
				if(flags != 0x00000000)
				{
					geometry_flags |= flags;
					if(geometry == NULL)
						geometry = (GdkRectangle *)g_malloc0(
							sizeof(GdkRectangle)
						);
					geometry->x = (flags & GDK_GEOMETRY_X) ? x : 0;
					geometry->y = (flags & GDK_GEOMETRY_Y) ? y : 0;
					geometry->width =
						(flags & GDK_GEOMETRY_WIDTH) ? width : 0;
					geometry->height =
						(flags & GDK_GEOMETRY_HEIGHT) ? height : 0;

					/* Setting the geometry with a size requires
					 * that we need to disable auto resize on open
					 */
					if((flags & GDK_GEOMETRY_WIDTH) ||
					   (flags & GDK_GEOMETRY_HEIGHT)
					)
						options &= ~IV_OPTION_RESIZE_ON_OPEN;
				}
			}
		}

		/* File name (check if arg is not a toggle)? */
		else if((*arg != '-') &&
				(*arg != '+')
		)
		{
			/* Add this argument as a file in the list of files
			 * to open on startup
			 */
			file_list = g_list_append(file_list, STRDUP(arg));
		}
		/* Unsupported argument */
		else
		{
			g_printerr(
"%s: Unsupported argument.\n",
				parm
			);
			DELETE_FETCHED_CMD_LINE_VALUES;
			rtn_val = 2;
			return(rtn_val);
		}
	}


	/* Create the core */
	core = IV_CORE(g_malloc0(sizeof(IVCore)));
	if(core == NULL)
	{
		DELETE_FETCHED_CMD_LINE_VALUES;
		rtn_val = 3;
		return(rtn_val);
	}

#undef DELETE_FETCHED_CMD_LINE_VALUES

	/* Reset the configuration values on the core */
	iv_core_reset(core);

	core->current_directory_path = STRDUP(g_get_current_dir());
	core->iv_viewer = NULL;
	core->iv_fviewer = NULL;
	core->imginfodlg = NULL;
	core->close_all_windows = FALSE;
	core->untitled_count = 0;
	core->grab_cur = gdk_cursor_new(GDK_CROSSHAIR);
	core->grab_window = NULL;

	/* Options */
	core->options = options;
	core->quality = quality;
	(void)memcpy(
		core->bg_color,
		bg_color,
		sizeof(bg_color)
	);
	core->bg_color_set = bg_color_set;
	core->font_name = font_name;
	core->wmname = wmname;
	core->wmclass = wmclass;
	core->title = title;
	core->geometry_flags = geometry_flags;
	core->geometry = geometry;

	core->imlib_fallback = imlib_fallback;

#ifdef HAVE_LIBENDEAVOUR2
	/* Initialize the Endeavour 2 API context */
	core->edv2_ctx = edv_context_new();
	edv_context_init(core->edv2_ctx, NULL);
#endif	/* HAVE_LIBENDEAVOUR2 */


	/* Set the main core pointer */
	iv_core = core;

/* Deletes the IV Core, all other resources, and returns */
#define CLEANUP_RETURN(_status_)	{	\
											\
 /* Delete the core */				\
 iv_core = NULL;				\
 if(core != NULL) {				\
  iv_core_delete(core);				\
 }						\
											\
 /* Shutdown the dialogs */			\
 if(initialized_dialogs) {			\
  FSDShutdown();				\
  CSDShutdown();				\
  PDialogShutdown();				\
  FileBrowserShutdown();			\
  ProgressDialogShutdown();			\
  CDialogShutdown();				\
  FPromptShutdown();				\
 }						\
											\
 /* Delete the list of startup files */		\
 if(file_list != NULL) {			\
  g_list_foreach(file_list, (GFunc)g_free, NULL); \
  g_list_free(file_list);			\
 }						\
											\
 return(_status_);				\
}


#if defined(HAVE_IMLIB)
	/* Initialize Imlib */
	core->imlib_h = Imlib_init(gdk_display_get_current());
	if(core->imlib_h != NULL)
	{
			(void)ImgImlibSetHandle(core->imlib_h);
	}
	else
	{
		g_printerr("Unable to initialize Imlib.\n");
	}
#endif	/* HAVE_IMLIB */


	/* Check and handle Special Startup cases first */

	/* Fullscreen */
	if(!STRISEMPTY(fullscreen_path))
	{
		const gint status = iv_open_fullscreen(
			fullscreen_path,
			startup_frame_num,
			core->bg_color,
			quality
		);
		rtn_val = ABSOLUTE_VALUE(status);
		CLEANUP_RETURN(rtn_val);
	}
	/* Display image centered on the desktop and exit? */
	else if(!STRISEMPTY(display_on_root_centered_path))
	{
		const gint status = iv_open_to_root_centered(
			display_on_root_centered_path,
			startup_frame_num,
			core->bg_color,
			quality
		);
		rtn_val = ABSOLUTE_VALUE(status);
		CLEANUP_RETURN(rtn_val);
	}
	/* Display image to fit on the desktop and exit? */
	else if(!STRISEMPTY(display_on_root_tofit_path))
	{
		const gint status = iv_open_to_root_tofit(
			display_on_root_tofit_path,
			startup_frame_num,
			core->bg_color,
			quality
		);
		rtn_val = ABSOLUTE_VALUE(status);
		CLEANUP_RETURN(rtn_val);
	}
	/* Display image to cover the desktop and exit? */
	else if(!STRISEMPTY(display_on_root_tocover_path))
	{
		const gint status = iv_open_to_root_tocover(
			display_on_root_tocover_path,
			startup_frame_num,
			core->bg_color,
			quality
		);
		rtn_val = ABSOLUTE_VALUE(status);
		CLEANUP_RETURN(rtn_val);
	}
	/* Display image tiled on the desktop and exit? */
	else if(!STRISEMPTY(display_on_root_tiled_path))
	{
		const gint status = iv_open_to_root_tiled(
			display_on_root_tiled_path,
			startup_frame_num,
			core->bg_color,
			quality
		);
		rtn_val = ABSOLUTE_VALUE(status);
		CLEANUP_RETURN(rtn_val);
	}
	/* Print? */
	else if(!STRISEMPTY(print_path))
	{
		const gint status = iv_open_print(
			print_path,
			startup_frame_num,
			core->bg_color
		);
		rtn_val = ABSOLUTE_VALUE(status);
		CLEANUP_RETURN(rtn_val);
	}


	/* If this point is reached then there were no Special Startup
	 * cases, so continue with Standard Startup
	 */

	/* Update the global tooltips state */
	GUISetGlobalTipsState((core->options & IV_OPTION_SHOW_TOOLTIPS) ? TRUE : FALSE);

	/* Initialize the dialogs */
	if(!initialized_dialogs)
	{
		FPromptInit(); 
		CDialogInit();
		ProgressDialogInit();
		FileBrowserInit();
		PDialogInit();
		CSDInit();
		FSDInit();
		initialized_dialogs = TRUE;

		/* Set the file selector callbacks */
		FileBrowserSetObjectCreatedCB(
			iv_file_selector_created_cb, core
		);
		FileBrowserSetObjectModifiedCB(
			iv_file_selector_modified_cb, core
		);
		FileBrowserSetObjectDeletedCB(
			iv_file_selector_deleted_cb, core
		);
	}

	/* Create the Image Viewer(s) */
	i = (gint)g_list_length(file_list);
	if(i > 0)
	{
		/* One or more files to open on startup, create an Image
		 * Viewer for each file in the startup files list
		 */
		const gchar *path;
		GList *glist;
		imgview_struct *iv;
		IVImageViewer *v;

		/* Warn if there are more than 25 objects to open */
		if(i > 25)
		{
			gchar *msg = g_strdup_printf(
"There are %i objects that have been specified\n\
to be opened, are you sure you want to open all\n\
of them?",
				i
			);
			CDialogSetTransientFor(NULL);
			CDialogGetResponse(
				"Confirm Open Multiple Objects",
				msg,
				NULL,
				CDIALOG_ICON_WARNING,
				CDIALOG_BTNFLAG_OK,
				CDIALOG_BTNFLAG_OK
			);
			CDialogSetTransientFor(NULL);
			g_free(msg);
			rtn_val = 4;
			CLEANUP_RETURN(rtn_val);
		}

#if !defined(_WIN32)
		/* Warn if opening objects and doing screenshot at startup */
		if(startup_screenshot)
		{
			CDialogSetTransientFor(NULL);
			CDialogGetResponse(
				"Screenshot Failed",
"Unable to take screenshot and open objects at\n\
startup. You may only specify one of these\n\
options at startup.",
				NULL,
				CDIALOG_ICON_WARNING,
				CDIALOG_BTNFLAG_OK,
				CDIALOG_BTNFLAG_OK
			);
			CDialogSetTransientFor(NULL);
			rtn_val = 2;
			CLEANUP_RETURN(rtn_val);
		}
#endif	/* Not _WIN32 */

		/* Create an Image Viewer and open each object */
		for(glist = file_list; glist != NULL; glist = g_list_next(glist))
		{
			path = (const gchar *)glist->data;
			if(STRISEMPTY(path))
				continue;

			/* Create a new Image Viewer */
			v = iv_viewer_new(
				core,
				core->options,
				quality,
				bg_color_set ? bg_color : NULL,
				geometry_flags,
				geometry,
				FALSE			/* Do not load title */
			);
			if(v == NULL)
				break;

			iv = v->iv;

			/* Update the current directory if the specified path
			 * is a full path
			 */
			if(g_path_is_absolute(path))
			{
				gchar *parent = g_dirname(path);
				if(parent != NULL)
				{
					g_free(core->current_directory_path);
					core->current_directory_path = g_strdup(parent);
					(void)chdir(parent);
					g_free(parent);
				}
			}

			ImgViewSetBusy(iv, TRUE);
			v->processing = TRUE;
			v->stop_count = 0;

			/* Open the image to the new Image Viewer */
			iv_viewer_open(
				v,
				path,
				startup_frame_num
			);

			v->processing = FALSE;
			ImgViewSetBusy(iv, FALSE);
 	    }
	}
	else
	{
		/* No files to open on startup, create one Image Viewer
		 * and have it display the title image
		 */
		IVImageViewer *v = iv_viewer_new(
			core,
			core->options,
			quality,
			bg_color_set ? bg_color : NULL,
			geometry_flags,
			geometry,
			TRUE				/* Display title image */
		);
		if(v != NULL)
		{
#if !defined(_WIN32)
			/* Map the Screenshot Dialog on startup? */
			if(startup_screenshot)
			{
				imgview_struct *iv = v->iv;

				ImgViewSetBusy(iv, TRUE);
				v->processing = TRUE;
				v->stop_count = 0;

				iv_viewer_screenshot(v);

				v->processing = FALSE;
				ImgViewSetBusy(iv, FALSE);
			}
#endif	/* !_WIN32 */

		}
	}

	/* Set the main timeout callback */
	core->main_timeout_id = gtk_timeout_add(
		1000l,
		iv_main_timeout_cb, core
	);

	/* Enter the main GTK+ loop */
	gtk_main();

#if defined(HAVE_IMLIB)
	if(core->imlib_h != NULL)
	{
		(void)ImgImlibSetHandle(NULL);
		core->imlib_h = NULL;
	}
#endif	/* HAVE_IMLIB */

	CLEANUP_RETURN(rtn_val);
#undef CLEANUP_RETURN
}
